;NSIS Modern User Interface version 1.70
;Start Menu Folder Selection Example Script
;Written by Joost Verburg

;--------------------------------
;Include Modern UI

  !include "MUI.nsh"

;--------------------------------
;General

  ;Name and file
  Name "Colliberty Easy Publisher"
  OutFile "CollibertyEasyPublisher_1820.exe"

  ;Default installation folder
  InstallDir "$PROGRAMFILES\Colliberty Easy Publisher"

  ;Get installation folder from registry if available
  ; InstallDirRegKey HKLM "Software\Zope Corporation\Zope\2.7.2-0" "Home"

  ReserveFile "python23.dll"
  ; ReserveFile "test.py"

  ShowInstDetails show

;--------------------------------
;Variables

  Var MUI_TEMP
  Var STARTMENU_FOLDER

  Var ProductName
  Var ProductVersion
  Var ProductShortName
  Var ZopeName
  Var ZopeVersion
  Var ZopeRegKey
  Var ZOPEHOME
  Var MKZOPEINSTANCE
  Var PYTHONPATH

Function .onInit
    ;Extract Install Options files
    ;$PLUGINSDIR will automatically be removed when the installer closes
    InitPluginsDir
    File "/oname=$PLUGINSDIR\python23.dll" "python23.dll"

    StrCpy $ProductName "Colliberty Easy Publisher"
    StrCpy $ProductShortName "Easy Publisher"
    StrCpy $ProductVersion "1.8.20"

    StrCpy $ZopeName "Zope"
    StrCpy $ZopeVersion "2.7.2"
    StrCpy $ZopeRegKey "Software\Zope Corporation\Zope\2.7.2-0"


    ReadRegStr $ZOPEHOME HKLM  "$ZopeRegKey" "Home"


    StrCmp $ZOPEHOME "" 0 ZopeFound
       MessageBox MB_OK "Couldn't find a installed version of $ZopeName $ZopeVersion. \
            Please install $ZopeName version $ZopeVersion before installing $ProductName $ProductVersion."
       Abort ; causes installer to quit.
    ZopeFound:

    GetFullPathName $PYTHONPATH "$ZOPEHOME\bin\python.exe"
    GetFullPathName $MKZOPEINSTANCE "$ZOPEHOME\bin\mkzopeinstance.py"

    StrCmp $MKZOPEINSTANCE "" 0 InstanceScriptFound
       MessageBox MB_OK "Couldn't find the Make Instance Script from Zope 2.7.2. The installation aborts."
       Abort ; causes installer to quit.
    InstanceScriptFound:

    ; nsPython::execFile $MKZOPEINSTANCE

    ExecWait "$PYTHONPATH $MKZOPEINSTANCE"
    ; ExecShell $PYTHONPATH $MKZOPEINSTANCE SW_SHOWNORMAL

    Pop $0

    MessageBox MB_OK "MKZOPEINSTANCE: $0"


FunctionEnd


;--------------------------------
;Interface Settings

  !define MUI_ABORTWARNING

;--------------------------------
;Pages

  !insertmacro MUI_PAGE_LICENSE "${NSISDIR}\License.txt"
  !insertmacro MUI_PAGE_COMPONENTS
  !insertmacro MUI_PAGE_DIRECTORY

  ;Start Menu Folder Page Configuration
  !define MUI_STARTMENUPAGE_REGISTRY_ROOT "HKLM"
  !define MUI_STARTMENUPAGE_REGISTRY_KEY "Software\Colliberty Easy Publisher"
  !define MUI_STARTMENUPAGE_REGISTRY_VALUENAME "Start Menu Folder"

  !insertmacro MUI_PAGE_STARTMENU Application $STARTMENU_FOLDER

  !insertmacro MUI_PAGE_INSTFILES

  !insertmacro MUI_UNPAGE_CONFIRM
  !insertmacro MUI_UNPAGE_INSTFILES

;--------------------------------
;Languages

  !insertmacro MUI_LANGUAGE "English"

;--------------------------------
;Installer Sections

Section "Dummy Section" SecDummy

  SetOutPath "$INSTDIR"

  ;ADD YOUR OWN FILES HERE...

  ;Store installation folder
  WriteRegStr HKLM "Software\Colliberty Easy Publisher" "" $INSTDIR

  ;Create uninstaller
  WriteUninstaller "$INSTDIR\Uninstall.exe"

  !insertmacro MUI_STARTMENU_WRITE_BEGIN Application

    ;Create shortcuts
    CreateDirectory "$SMPROGRAMS\$STARTMENU_FOLDER"
    CreateShortCut "$SMPROGRAMS\$STARTMENU_FOLDER\Uninstall.lnk" "$INSTDIR\Uninstall.exe"

  !insertmacro MUI_STARTMENU_WRITE_END

SectionEnd

;--------------------------------
;Descriptions

  ;Language strings
  LangString DESC_SecDummy ${LANG_ENGLISH} "A test section."

  ;Assign language strings to sections
  !insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
    !insertmacro MUI_DESCRIPTION_TEXT ${SecDummy} $(DESC_SecDummy)
  !insertmacro MUI_FUNCTION_DESCRIPTION_END

;--------------------------------
;Uninstaller Section

Section "Uninstall"

  ;ADD YOUR OWN FILES HERE...

  Delete "$INSTDIR\Uninstall.exe"

  RMDir "$INSTDIR"

  !insertmacro MUI_STARTMENU_GETFOLDER Application $MUI_TEMP

  Delete "$SMPROGRAMS\$MUI_TEMP\Uninstall.lnk"

  ;Delete empty start menu parent diretories
  StrCpy $MUI_TEMP "$SMPROGRAMS\$MUI_TEMP"

  startMenuDeleteLoop:
    RMDir $MUI_TEMP
    GetFullPathName $MUI_TEMP "$MUI_TEMP\.."

    IfErrors startMenuDeleteLoopDone

    StrCmp $MUI_TEMP $SMPROGRAMS startMenuDeleteLoopDone startMenuDeleteLoop
  startMenuDeleteLoopDone:

  DeleteRegKey /ifempty HKLM "Software\Colliberty Easy Publisher"

SectionEnd