# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyApplication/MultiCatalog.py,v 1.1 2004/06/28 10:04:08 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

import Globals
from Globals import DTMLFile
from AccessControl import ClassSecurityInfo
from types import StringType, ListType, TupleType

from Products.ZCatalog.Catalog import CatalogError


from Products.ZCatalog.CatalogBrains import AbstractCatalogBrain
from Products.EasyCore.UpgradeSupport.Utils import upgrade_indexes
from Products.EasyCore.ZCatalog.ProtectedCatalog import ProtectedCatalog

from Products.EasyApplication.EasySkinnable import DRAFT_VER


from Products.EasyCore import MLOG_TAB


ManageZCatalogEntries='Manage ZCatalog Entries'

class AbstractMultiBrain:

    def getLanguage(self):
        """Get the language for this record"""
        return self.aq_parent.getlanguage(self.data_record_id_)

    def getPath(self):
        """Get the physical path for this record"""
        return self.aq_parent.getpath(self.data_record_id_)

    def getURL(self):
        """Try to generate a URL for this record"""
        try:
            return self.REQUEST.physicalPathToURL(self.getPath())
        except:
            return self.getPath()

    def getUID(self):
        """Get the UID for this record"""
        return self.aq_parent.getuid(self.data_record_id_)




class MultiCatalog(ProtectedCatalog):
    """Catalog extension where each object can be indexed many times"""

    security = ClassSecurityInfo()

    security.declarePrivate('resolve_path')
    def resolve_path(self, path):
        """
        Attempt to resolve a url into an object in the Zope
        namespace. The url may be absolute or a catalog path
        style url. If no object is found, None is returned.
        No exceptions are raised.
        """
        if type(path) in (ListType,TupleType): path=path[0]
        try: return self.unrestrictedTraverse(path)
        except: pass

    security.declarePrivate('resolve_url')
    def resolve_url(self, path, REQUEST):
        """
        Attempt to resolve a url into an object in the Zope
        namespace. The url may be absolute or a catalog path
        style url. If no object is found, None is returned.
        No exceptions are raised.
        """
        if type(path) in (ListType,TupleType): path=path[0]
        script=REQUEST.script
        if path.find(script) != 0:
            path='%s/%s' % (script, path)
        try: return REQUEST.resolve_url(path)
        except: pass


    security.declarePrivate('catalog_object')
    def catalog_object(self, obj, uid=None, idxs=[]):
        """ wrapper around catalog """
        if isinstance(uid, TupleType):
            old_language = obj.getCurrentLanguage(default=None)
            obj.changeCurrentVersion(DRAFT_VER, uid[1])
            self._catalog.catalogObject(obj, uid, None, idxs)
            obj.changeCurrentVersion(DRAFT_VER, old_language)
        else:
            old_language = obj.getCurrentLanguage(default=None)
            url = obj.physicalURL
            for language in obj.getLanguages():
                obj.changeCurrentVersion(DRAFT_VER, language)

                r=self._catalog.catalogObject(obj, (url, language), None, idxs)
            obj.changeCurrentVersion(DRAFT_VER, old_language)


        # None passed in to catalogObject as third argument indicates
        # that we shouldn't try to commit subtransactions within any
        # indexing code.  We throw away the result of the call to
        # catalogObject (which is a word count), because it's
        # worthless to us here.

        if self.threshold is not None:
            # figure out whether or not to commit a subtransaction.
            t = id(get_transaction())
            if t != self._v_transaction:
                self._v_total = 0
            self._v_transaction = t
            self._v_total = self._v_total + 1
            # increment the _v_total counter for this thread only and get
            # a reference to the current transaction.
            # the _v_total counter is zeroed if we notice that we're in
            # a different transaction than the last one that came by.
            # self.threshold represents the number of times that
            # catalog_object needs to be called in order for the catalog
            # to commit a subtransaction.  The semantics here mean that
            # we should commit a subtransaction if our threshhold is
            # exceeded within the boundaries of the current transaction.
            if self._v_total > self.threshold:
                get_transaction().commit(1)
                self._p_jar.cacheFullSweep(3)
                self._v_total = 0

    def uncatalog_object(self, uid):
        """ wrapper around catalog """
        if isinstance(uid, TupleType):
            self._catalog.uncatalogObject(uid)
        else:
            old_language = obj.getCurrentLanguage(default=None)
            url = obj.physicalURL
            for language in obj.getLanguages():
                obj.changeCurrentVersion(DRAFT_VER, language)
                r=self._catalog.uncatalogObject((url, language))
            obj.changeCurrentVersion(DRAFT_VER, old_language)

    security.declareProtected('Search ZCatalog', 'getpath')
    def getpath(self, rid):
        """Return the path to a cataloged object given a 'data_record_id_'
        """
        path=self._catalog.paths[rid]
        if type(path) in (ListType,TupleType): path=path[0]
        return path

    security.declareProtected('Search ZCatalog', 'getlanguage')
    def getlanguage(self, rid):
        """Return the path to a cataloged object given a 'data_record_id_'
        """
        path=self._catalog.paths[rid]
        if type(path) in (ListType,TupleType): path=path[1]
        return path

    security.declareProtected('Search ZCatalog', 'getuid')
    def getuid(self, rid):
        """Return the uid (or path) use to a cataloged object given a 'data_record_id_'
        """
        path=self._catalog.paths[rid]
        return path


    security.declareProtected('Search ZCatalog', 'getrid')
    def getrid(self, path, default=None):
        """Return 'data_record_id_' the to a cataloged object given a 'path'
        """
        if type(path) in (ListType,TupleType): path=path[0]
        return self._catalog.uids.get(path, default)

    security.declareProtected(ManageZCatalogEntries,'manage_catalogView')
    manage_catalogView = DTMLFile('dtml/catalogView',globals())

    security.declareProtected('Search ZCatalog', 'searchResults')
    def searchResults(self, REQUEST=None, used=None, **kw):
        """
        Search the catalog according to the ZTables search interface.
        Search terms can be passed in the REQUEST or as keyword
        arguments.
        """
        #set AbstractMultiBrain
        self._catalog.useBrains(AbstractMultiBrain)

        return apply(self._catalog.searchResults, (REQUEST,used), kw)

    __call__=searchResults

    def manage_catalogObject(self, REQUEST, RESPONSE, URL1, urls=[]):
        """ index Zope object(s) that 'urls' point to """

        for url in urls:
            lang, url = url.split(':',1)
            obj = self.resolve_path(url)
            if not obj:
                obj = self.resolve_url(url, REQUEST)
            if obj is not None:
                self.catalog_object(obj, (url, lang))

        return self.manage_catalogView(self,REQUEST,manage_tabs_message='Object Cataloged')


    def manage_uncatalogObject(self, REQUEST, RESPONSE, URL1, urls=None):
        """ removes Zope object(s) 'urls' from catalog """

        for url in urls:
            lang, url = url.split(':',1)
            self.uncatalog_object((url, lang))

        return self.manage_catalogView(self,REQUEST,manage_tabs_message='Object Uncataloged')

    def refreshCatalog(self, clear=0):
        """ re-index everything we can find """

        cat = self._catalog
        paths = cat.paths.values()
        if clear:
            paths = tuple(paths)
            cat.clear()

        for p in paths:
            obj = self.resolve_path(p)
            if not obj:
                obj = self.resolve_url(p, self.REQUEST)
            if obj is not None:
                self.catalog_object(obj, p)

    def clearCatalog(self):
        self._catalog.clear()



    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        #base=aq_base(self)
        ob_ver=getattr(self,'_object_version', 0.0)
        try: ob_ver=float(ob_ver)
        except: ob_ver=1.7

        changed, messages=upgrade_indexes(self, changed)


        self._object_version = 1.75

        if REQUEST is not None:
            return "<h1>%s</h1>%s"%(changed and 'Changed' or 'Not Changed', ''.join(map(lambda x: '<p>%s</p>'%x, messages)),)
        return changed, messages




Globals.InitializeClass(MultiCatalog)
