# coding: iso-8859-1
# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyCore/Utils/Tutils.py,v 1.2 2004/08/21 07:50:39 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

# This file contains generic nice-to-have routines.

import string, os
from re import compile
from OFS.ObjectManager import checkValidId, BadRequestException
from Acquisition import aq_base, aq_inner, aq_parent, aq_chain
from zLOG import LOG

# This maps a content type to an internal type.
CONTENTTYPEMAP = { 'image/pjpeg':   'jpeg',     #progressive jpeg
                   'image/jpeg':    'jpeg',
                   'image/bmp':     'bmp',
                   'image/png':     'png',
                   'image/gif':     'gif',
                   'image/tif':     'tiff',
                 }

# This contains info on each internal type. (display-name, extension, content_type)
EXTENTIONCONTENTTYPEMAP =  { 'jpeg':     'image/jpeg',
                             'jpg':      'image/jpeg',
                             'gif':      'image/gif',
                             'bmp':      'image/bmp',
                             'png':      'image/png',
                             'tiff':     'image/tif',
                             'tif':      'image/tif',
                           }

# This map show possible conversions from each internal type:
CONVERSIONMAP = { 'jpeg': ('png', 'tiff', 'bmp'),
                  'gif':  ('jpeg', 'png', 'tiff', 'bmp'),
                  'bmp':  ('jpeg', 'png', 'tiff',),
                  'png':  ('jpeg', 'tiff', 'bmp'),
                  'tiff': ('jpeg', 'png', 'bmp',),
                }

def xlate2url( url, allowucase=1 ):
    """Takes an url and translates it into something that makes up a good url"""

    url=str(url)
    url=url.strip()

    bad_chars =  ' +?&,;()[]{}/ݟ'
    good_chars = '_____________AAAAAAaaaaaaCcEEEEEeeeeeIIIIiiiiNnOOOOOOooooooSssUUUUuuuuYYyyZz'
    TRANSMAP = string.maketrans(bad_chars, good_chars)

    # make sure url is a string, and remove all funky characters according to TRANSMAP:
    result = string.translate(url, TRANSMAP)
    while len(result) > 0 and result[0]=='_': result=result[1:]
    result=result.strip()
    if not allowucase: return string.lower(result)
    return result

pattern = compile("^([0-9a-z_&.+-]+!)*[0-9a-z_&.+-]+@(([0-9a-z]([0-9a-z-]*[0-9a-z])?\.)+[a-z]{2,3}|([0-9]{1,3}\.){3}[0-9]{1,3})$")
def validateEmailAddress(email):
    return pattern.search(string.lower(email))

def composeEmailAddress(name,email):
    if validateEmailAddress(email):
        if name != '':
            return "%s <%s>" % (name,email)
        else:
            return email

def distinct(alist):    # Fastest order preserving
    set = {}
    return [set.setdefault(e,e) for e in alist if e not in set.keys()]


def distinct_fast(alist):    # Fastest without order preserving
    set = {}
    map(set.setdefault, alist, [])
    return set.keys()

def isValidId(container, id, allow_dup=0):
    try:
        checkValidId(container, id, allow_dup)
        if id in container.objectIds():
            raise BadRequestException, ('The id "%s" is invalid--'
                                        'it is already in use.' % id)
        return 1, None
    except BadRequestException, e:
        return 0, e

def split_filename(id):
    """Splits filename in to body and suffix"""
    split_id=id.split('.')
    if len(split_id)>1:
        id_body, id_ext='.'.join(split_id[:-1]), split_id[-1]
    else:
        id_body, id_ext = id, ''
    return id_body, id_ext

def strip_extension(id):
    return split_filename(id)[0]

def getValidId(container, id, allow_dup=0):
    id = xlate2url(id)
    while not getattr(aq_base(container),'isAnObjectManager',1) and not container == aq_parent(container):
        container=aq_parent(container)
    id_body, id_ext = split_filename(id)
    count=1
    valid, e = isValidId(container, id, allow_dup)
    while not valid and count<20:
        id=id_body + ('_%s' % count)
        if id_ext: id=id+"."+id_ext
        count+=1
        valid, e = isValidId(container, id, allow_dup)
    if count==20:
        raise BadRequestException, e
    return id

def getValidHighestId(container, id):
    id = xlate2url(id)
    while not getattr(aq_base(container),'isAnObjectManager',1) and not container == aq_parent(container):
        container=aq_parent(container)
    id_body, id_ext = split_filename(id)
    count=1
    old_id=None
    valid, e = isValidId(container, id)
    while not valid and count<20:
        old_id=id
        id=id_body + ('_%s' % count)
        if id_ext: id=id+"."+id_ext
        count+=1
        valid, e = isValidId(container, id)
    if count==20:
        raise BadRequestException, e
    return old_id

def filename2id_title(filename, id='', title='', cut_extension=0, allowucase=0):
    """Splits filename to id and title."""
    filename = filename[max(string.rfind(filename,'/'),
                    string.rfind(filename,'\\'),
                    string.rfind(filename,':')
                    )+1:]

    filename_body, filename_ext = os.path.splitext(filename)

    if not id:
        if cut_extension:
            id = filename_body
        else:
            id = filename
        if not allowucase:
            id = id.lower()

    id=xlate2url(id)

    if not title:
        title_id = filename_body.replace('.', ' ')
        title_id = title_id.replace('_', ' ')
        title_id = title_id.replace('-', ' ')
        title = string.capitalize(title_id)


    return id, title



