
import re
from types import StringType
import string

from xml.dom import Node
from xml.dom import NotSupportedErr, NamespaceErr
from xml.dom import implementation
from xml.dom import EMPTY_NAMESPACE, XMLNS_NAMESPACE
from xml.dom.html import HTML_4_TRANSITIONAL_INLINE
from xml.xpath import Context, Evaluate, Compile
from Products.ParsedXML.DOM.Traversal import NodeFilter, TreeWalker

from TextPrinter import TextVisitor
from xml.dom.ext.Printer import PrintWalker
from StringIO import StringIO


from xml.dom.ext import StripXml, Print, PrettyPrint

import difflib

#From objects
N_TYPE     = 0 # node's type
N_NAME     = 1 # node's label (to process xpath)
N_VALUE    = 2 # node's value
N_CHILDS   = 3 # nodes's childs list
N_PARENT   = 4 # node's parent
N_ISSUE    = 5 # node's total issue number
N_XNUM     = 6 # to compute node's xpath
NSIZE = 7 #number of items in a list which represent a node

# NODE TYPES
#NT_SYST = 0 # SYSTEM node (added by parser) /!\ depricated
NT_NODE = 1 # ELEMENT node
NT_ATTN = 2 # ATTRIBUTE NAME node
NT_ATTV = 3 # ATTRIBUTE VALUE node
NT_TEXT = 4 # TEXT node
NT_COMM = 5 # COMMENT node
NT_ROOT = 6 # root node
NODES_TYPES=('NT','NN','AN','AV','T','C', 'R') # for printing


TYPE_NAME = {
        Node.ATTRIBUTE_NODE:                'Attribute',
        Node.CDATA_SECTION_NODE:            'CDATA Section',
        Node.COMMENT_NODE:                  'Comment',
        Node.DOCUMENT_FRAGMENT_NODE:        'Document Fragment',
        Node.DOCUMENT_NODE:                 'Document',
        Node.DOCUMENT_TYPE_NODE:            'DocumentType',
        Node.ELEMENT_NODE:                  'Element',
        Node.ENTITY_NODE:                   'Entity',
        Node.ENTITY_REFERENCE_NODE:         'Entity Reference',
        Node.NOTATION_NODE:                 'Notation',
        Node.PROCESSING_INSTRUCTION_NODE:   'Processing Instruction',
        Node.TEXT_NODE:                     'Text',
   }

def _getXPath(node, append=None, pcnode=None):
    path=None
    if append is None:
        path=[]
        append=path.append
    append(node)
    parent=node.parentNode
    if parent is not None:
        _getXPath(parent, append, node)
    elif node.nodeType==node.ATTRIBUTE_NODE:
        parent=node.ownerElement
        if parent is not None:
            _getXPath(parent, append, node)
        else:
            raise "Invalid ownerElement for Attribute Node"

    if path is not None:
        path.reverse()
        return path

def getXPath(node):
    node=normalizeCompNode(node)
    xpath=_getXPath(node)
    xstr=""
    if len(xpath):
        for node in xpath[1:]:
            if node.nodeType==Node.ELEMENT_NODE:
                xstr=xstr+"/"+node.nodeName
                position=offsetByType(node)
                if position:
                    xstr=xstr+"[%i]"%position
            elif node.nodeType==Node.TEXT_NODE:
                xstr=xstr+"/"+"text()"
                position=offsetByType(node)
                if position:
                    xstr=xstr+"[%i]"%position
            elif node.nodeType==Node.ATTRIBUTE_NODE:
                xstr=xstr+"/@"+node.nodeName
    return xstr


def getDepth(node):
    node=normalizeCompNode(node)
    count=0
    pnode=node.parentNode
    while pnode:
        count+=1
        pnode=pnode.parentNode
    return count

def offset(node):
    node=normalizeCompNode(node)
    count=1
    parent=node.parentNode
    if parent is not None and node is not None:
        for child in list(parent.childNodes):
            if child is node:
                return count
            count=count+1
    return 0

def offsetByType(node):
    node=normalizeCompNode(node)
    count=1
    parent=node.parentNode
    if parent is not None and node is not None:
        for child in list(getElementsByType(parent, node.nodeType)):
            if child is node:
                return count
            count=count+1
    return 0

def pushContainerNode(new_node, container):
    for cnode in list(container.childNodes):
        new_node.appendChild(cnode)
    container.appendChild(new_node)

def addContainer(new_node, node):
    parent=node.parentNode
    parent.replaceChild(new_node, node)
    new_node.appendChild(node)


def NormalizeXPath(xpath): #ChooseDOM
    if xpath[0:8]=='/HTML[2]':
        return 2, '/HTML[1]'+xpath[8:]
    return 1, xpath

def isXPath(xpath):
    if type(xpath)!=StringType or len(xpath)==0 or xpath[0]!='/':
        return 0
    return 1

UNKNOWN=0
INLINE_NODE=1
TEXT_NODE=2
BLOCK_NODE=3
PRINCIPAL_BLOCK=4
WHITE_SPACE=5

category_labels={
    UNKNOWN:'Unknown',
    INLINE_NODE:'Inline',
    TEXT_NODE:'Text',
    BLOCK_NODE:'Block Box',
    PRINCIPAL_BLOCK:'Principal Block Box',
    WHITE_SPACE:'White Space',

}


def nodeCategory(node):
    node=normalizeCompNode(node)
    if isInline(node):
        if isTextNode(node):
            if isWhitespaceOnly(node):
                return WHITE_SPACE
            return TEXT_NODE
        return INLINE_NODE
    elif isBlock(node):
        if isPrincipalBlock(node):
            return PRINCIPAL_BLOCK
        return BLOCK_NODE
    return UNKNOWN

def nodeCategoryLabel(node):
    return category_labels.get(nodeCategory(node), 'Unknown')


def isInline(node):
    node=normalizeCompNode(node)
    if node.nodeType!=Node.ELEMENT_NODE:
        return 1
    if node.nodeName in HTML_4_TRANSITIONAL_INLINE:
        return 1
    return 0

def isBlock(node):
    node=normalizeCompNode(node)
    if node.nodeType!=Node.ELEMENT_NODE:
        return 0
    if node.nodeName not in HTML_4_TRANSITIONAL_INLINE:
        return 1
    return 0

def isPrincipalBlock(node):
    node=normalizeCompNode(node)
    if isBlock(node):
        for child in node.childNodes[:]:
            if isBlock(child):
                return 1
    return 0

def isNonPrincipalBlock(node):
    node=normalizeCompNode(node)
    if isBlock(node):
        for child in node.childNodes[:]:
            if isBlock(child):
                return 0
        return 1
    return 0

def isTextNode(node):
    node=normalizeCompNode(node)
    return node.nodeType==Node.TEXT_NODE

def isWhitespaceOnly(textnode):
    textnode=normalizeCompNode(textnode)
    if textnode.nodeType!=Node.TEXT_NODE:
        return 0
    if len(textnode.nodeValue) == 0:
        return 1
    if textnode.nodeValue.strip():
        return 0
    return 1

def isNodesHomogeneous(node_list):
    node_list=node_list[:]
    if len(node_list):
        parent=node_list[0].parentNode

        for child in node_list[1:]:
            if child.parentNode is not parent:
                return 0
    return 1

#Render

def renderAttributes(node):
    node=normalizeCompNode(node)
    attrs=''
    for attribute in node.attributes:
        attrs=attrs + ' %s="%s"' % (attribute.name, attribute.value)
    return attrs

def renderText(node, strip=1):
    if isNonPrincipalBlock(node):
        io=StringIO()
        pw=PrintWalker(TextVisitor(io), node)
        pw.run()
        io.pos=0
        return io.read()
    return ''

#XPath

def adaptXPath(path):
    if path!='/':
        return "/DOC[1]"+path
    return "/DOC[1]"

def Evaluate2(xpath, dom1, dom2):
    choise, xpath= NormalizeXPath(xpath)
    if choise==2:
        return Evaluate(xpath, dom2)
    return Evaluate(xpath, dom1)


def cloneDOM(html, namespage=EMPTY_NAMESPACE, qualifiedname=None, doctype=None):
    dom = implementation.createDocument(namespage, qualifiedname, doctype)
    html_node=dom.importNode(html.firstChild, 1)
    dom.appendChild(html_node)
    return dom

def renameNode(n, namespaceURI=EMPTY_NAMESPACE, name=None, dom=None):
    if dom and n.ownerDocument is not dom:
        raise WrongDocumentErr(
            "cannot rename nodes from other documents;\n"
            "expected %s,\nfound %s" % (dom, n.ownerDocument))
    if n.nodeType not in (Node.ELEMENT_NODE, Node.ATTRIBUTE_NODE):
        raise NotSupportedErr(
            "renameNode() only applies to element and attribute nodes")
    if name is None: name=n.nodeName #Do nothing
    if namespaceURI != EMPTY_NAMESPACE:
        if ':' in name:
            prefix, localName = name.split(':', 1)
            if (  prefix == "xmlns"
                  and namespaceURI != XMLNS_NAMESPACE):
                raise NamespaceErr(
                    "illegal use of 'xmlns' prefix")
        else:
            if (  name == "xmlns"
                  and namespaceURI != XMLNS_NAMESPACE
                  and n.nodeType == Node.ATTRIBUTE_NODE):
                raise NamespaceErr(
                    "illegal use of the 'xmlns' attribute")
            prefix = None
            localName = name
    else:
        prefix = None
        localName = None
    if n.nodeType == Node.ATTRIBUTE_NODE:
        element = n.ownerElement
        if element is not None:
            element.removeAttributeNode(n)
    else:
        element = None
    # avoid __setattr__
    d = n.__dict__
    d['prefix'] = prefix
    d['localName'] = localName
    d['namespaceURI'] = namespaceURI
    d['nodeName'] = name
    if n.nodeType == Node.ELEMENT_NODE:
        d['tagName'] = name
    else:
        # attribute node
        d['name'] = name
        if element is not None:
            element.setAttributeNode(n)
    # It's not clear from a semantic perspective whether we should
    # call the user data handlers for the NODE_RENAMED event since
    # we're re-using the existing node.  The draft spec has been
    # interpreted as meaning "no, don't call the handler unless a
    # new node is created."
    n.__dict__=d
    return n

def insertParent(node_list, new_node):
    #node_list must be homogeneous
    if isNodesHomogeneous(node_list):
        pass
    raise "Not implemented"

#def backupOldXPaths(dom):
#    walker = TreeWalker(dom.documentElement, NodeFilter.SHOW_ALL, None, 1)
#    while 1:
#        current_node=walker.currentNode
#        current_node.old_xpath = getXPath(current_node)
#        next = walker.nextNode()
#        if next is None: break

#def createXPathIndex(dom):
#    index={}
#    walker = TreeWalker(dom.documentElement, NodeFilter.SHOW_ALL, None, 1)
#    while 1:
#        current_node=walker.currentNode
#        current_node.old_xpath = getXPath(current_node)
#        index[current_node.old_xpath]=current_node
#        next = walker.nextNode()
#        if next is None: break
#    return index

def getElementsByType(node, typeId):
    node=normalizeCompNode(node)
    return filter(lambda x, typeId=typeId: x.nodeType==typeId, node.childNodes)


def splitTextNodes(dom, splitter=re.compile('(\s*)')):
    walker = TreeWalker(dom.documentElement, NodeFilter.SHOW_ALL, None, 1)
    while 1:
        current_node=walker.currentNode
        current_node.isWhiteSpace=isWhitespaceOnly(current_node)
        if current_node.nodeType==Node.TEXT_NODE and not current_node.isWhiteSpace:
            value=current_node.nodeValue
            values=filter(None, splitter.split(value))
            for value in values:
                new_node=dom.createTextNode(value)
                new_node.isWhiteSpace=isWhitespaceOnly(new_node)
                current_node.parentNode.insertBefore(new_node, current_node)
            current_node.parentNode.removeChild(current_node)
            walker.currentNode=new_node
        next = walker.nextNode()
        if next is None: break
    return dom



isComp=lambda x: getattr(x, '__is_comp__', 0)

def normalizeCompNode(node):
    if getattr(node, '__is_comp__', 0):
        return node.node
    return node

def normalizeCompNodes(nodes):
    nnodes=[]
    for node in nodes[:]:
        if getattr(node, '__is_comp__', 0):
            node=node.node
        nnodes.append(node)
    return nnodes


class CompNodeController:
    def __init__(self, label=''):
        self.label=label

class CompNode:

    __is_comp__=1

    def __init__(self, node, controller):
        self.node=node
        self.node.CompNode=self
        self.controller=controller

    def __cmp__(self, snode):
        return cmp(self.repr(), snode.repr())

    def __hash__(self):
        return hash(self.repr())

    def repr(self):
        node=self.node
        if node.nodeType==Node.TEXT_NODE:
            return node.nodeType, node.nodeName, node.nodeValue, renderText(node)
        elif node.nodeType==Node.ELEMENT_NODE:
            return node.nodeType, node.nodeName, renderAttributes(node), renderText(node)
        else:
            return node.nodeType, node.nodeName, node.nodeValue, renderText(node)

    def xml(self):
        node=self.node
        if node.nodeType==Node.TEXT_NODE:
            return node.nodeValue
        elif node.nodeType==Node.ELEMENT_NODE:
            return "<%s%s>" % (node.nodeName,renderAttributes(node))
        else:
            return "%s, %s, %s" % (node.nodeName, node.nodeValue, node.nodeType)

# SequentiateHelpers

# Sequentiate

def Sequentiate(dom):
    controller=CompNodeController("Sequentiate")
    seq=[]
    append=seq.append
    counter=0
    walker = TreeWalker(dom.documentElement, NodeFilter.SHOW_ALL, None, 1)
    while 1:
        node=walker.currentNode
        node.index=counter
        cnode=CompNode(walker.currentNode, controller)
        cnode.index=counter
        counter=counter+1
        next = walker.nextNode()
        if next is None: break
    return seq


def SequentiateSimple(dom):
    controller=CompNodeController("Sequentiate")
    seq=[]
    append=seq.append
    counter=0
    walker = TreeWalker(dom.documentElement, NodeFilter.SHOW_ALL, None, 1)
    while 1:
        if isBlock(walker.currentNode): #isNonPrincipalBlock(walker.currentNode) or isPrincipalBlock(walker.currentNode):
            node=walker.currentNode
            node.index=counter
            cnode=CompNode(walker.currentNode, controller)
            cnode.index=counter
            append(cnode)
            counter=counter+1
        next = walker.nextNode()
        if next is None: break
    return seq

def DOMDumper(dom):
    walker = TreeWalker(dom.documentElement, NodeFilter.SHOW_ALL, None, 1)
    while 1:
        node=walker.currentNode
        print node.nodeType, node.nodeName, node.nodeValue
        next = walker.nextNode()
        if next is None: break
    return dom


def SimpleTreeWalker(node, do_before, do_after, input, output):
    do_before(node, input, output)
    if node.nodeType==Node.ELEMENT_NODE:
        for child in node.childNodes[:]:
            SimpleTreeWalker(child, do_before, do_after, input, output)
    do_after(node, input, output)

# ProcessHelpers

def getInsertPoint(anode, bnode):
    #if isInline(anode):
    #    print "INLINE", anode
    #if isBlock(anode):
    #    print "BLOCK", anode
    if isPrincipalBlock(anode):
        print "PRINCIPALBLOCK", anode
    return anode


def countBlockInList(nodelist):
    count=0
    for node in nodelist:
        if isBlock(node):
            count+=1
    return count

def isChildsDeletedOld(node):
    node=normalizeCompNode(node)
    delete_childs=1
    if node.nodeType==Node.ELEMENT_NODE:
        for child in node.childNodes[:]:
            delete_childs=delete_childs and isChildsDeleted(child)
    node.deleteTree=not not delete_childs
    return delete_childs

def isChildsDeleted(node):
    node=normalizeCompNode(node)

    input={}
    output={'count':0, 'delete_childs':1}

    def do_before(node, input, output):
        output['delete_childs'] = output['delete_childs'] and getattr(node, 'deleteNode', 0)

    def do_after(node, input, output):
        if output['delete_childs']:
            node.deleteTree=1

    SimpleTreeWalker(node, do_before, do_after, input, output)


def nodeInChildren(node, container, recursive=1):
    node=normalizeCompNode(node)
    container=normalizeCompNode(container)
    for child in container.childNodes[:]:
        if node is child:
            return 1
        if child.nodeType==Node.ELEMENT_NODE and recursive:
            return nodeInChildren(node, child, recursive)
    return 0

def createNodeListToAdd(compnodelist, dom_node):

    add_list=[]
    count=0
    length=len(compnodelist)

    if len(compnodelist):
        last_tree=None
        while count <length:
            child=compnodelist[count]
            if last_tree is None or not nodeInChildren(child.node, last_tree):
                node=dom_node.node.ownerDocument.importNode(child.node, 1)
                add_list.append(node)
                last_tree=child.node
            count+=1

    return add_list




def ProcessReplace2(seq1, alo, ahi, seq2, blo, bhi):
    pass



def getPreviousMatchingDepth(seq2, blo, depth, default=None):
    obj=default
    for j in xrange(blo-1, 0, -1):
        last_depth=getDepth(seq2[j])
        if last_depth==depth:
            obj=seq2[j]
            break
    return obj

def Process(seq1, seq2, label=''):

    last_match=None
    differ=difflib.SequenceMatcher(lambda x: x.node.isWhiteSpace, seq1, seq2)
    print "="*55
    print label
    #connect equals
    oldnode1=None
    oldnode2=None
    for tag, alo, ahi, blo, bhi in differ.get_opcodes():
        if tag == 'replace':
            print '?', tag, map(lambda x: x.xml(), seq1[alo:ahi]), map(lambda x: x.xml(), seq2[blo:bhi]), alo, ahi, blo, bhi

            ProcessReplace2(seq1, alo, ahi, seq2, blo, bhi)

            #for i in xrange(0, ahi-alo):
            #    ai=alo+i
            #    bi=blo+i
            #    anode=seq1[ai].node
            #    bnode=seq2[bi].node

            #    anode.replaceNode=bnode
            #    bnode.replaceNode=anode
            #    if anode.nodeType==Node.ELEMENT_NODE and bnode.nodeType==Node.ELEMENT_NODE:
            #        if anode.nodeName!=bnode.nodeName:
            #            anode.newTagName=bnode.nodeName
            #            bnode.oldTagName=anode.nodeName
            #            #print "New tagname %s=>%s"% (anode.nodeName, bnode.nodeName)
            #        elif renderAttributes(anode)!=renderAttributes(bnode):
            #            anode.newAttributes=bnode.attributes
            #            bnode.oldAttributes=anode.attributes
            #            #print "Attributes Changed: '%s' => '%s'"% (renderAttributes(anode), renderAttributes(bnode))
            #    elif anode.nodeType==Node.TEXT_NODE and bnode.nodeType==Node.TEXT_NODE:
            #            anode.newText=bnode.nodeValue
            #            bnode.oldText=anode.nodeValue
            pass

        elif tag == 'delete':
            print '-', tag, map(lambda x: x.xml(), seq1[alo:ahi]), map(lambda x: x.xml(), seq2[blo:bhi]), alo, ahi, blo, bhi

            for i in xrange(0, ahi-alo):
                seq1[alo+i].node.deleteNode=1
            for i in xrange(0, ahi-alo):
                isChildsDeleted(seq1[alo+i])


        elif tag == 'insert':
            print '+', tag, seq1[alo].node, map(lambda x: x.xml(), seq2[blo:bhi]), alo, ahi, blo, bhi
            current_depth_1=getDepth(seq1[alo])
            current_depth_2=getDepth(seq2[blo])
            ##print
            ##print "\t", "current_depth", current_depth_1, current_depth_2
            #?insert_node=seq1[alo]
            #?for j in xrange(blo, bhi):
            #?    print "\t..", getDepth(seq2[j])
            #?    ##current_depth_2

            #?previous=getPreviousMatchingDepth(seq2, blo, current_depth_2, None)
            #?print "previous", previous, previous.node

            #?if last_match and blo!=last_match[3]:
            #?    print "NO MATCH, blo!=last_match[3]", blo, last_match[3]

            #if last_match:
            #    last_j=last_match[3]
            #    #print "\t", "last_j", last_j
            #    #print "\t", "last_match", last_match
            #    obj=None
            #    ##for j in xrange(last_match[3]-1, 0, -1):
            #    for j in xrange(blo-1, 0, -1):
            #        last_depth=getDepth(seq2[j])
            #        ##print "\t\t", j, last_depth, current_depth_2
            #        if last_depth==current_depth_2:
            #            obj=seq2[j]
            #            ##print "\t\tobj:", obj
            #            ##print "\t\tobj.node:", getattr(obj,'node', '--')
            #            ##print "\t\tobj.node.equalNode:", getattr(obj,'node', 0) and getattr(obj.node,'equalNode', '==')
            #            break


            print

            #eq_before=last_match
            #eq_after=seq1[alo]
            #print "eq_before",eq_before
            #print "eq_after",eq_after

            innode=seq1[alo]
            parentNode=innode.node.parentNode
            new_nodes=seq2[blo:bhi]
            innode.node.insertBeforeList=createNodeListToAdd(new_nodes, innode)


        elif tag == 'equal':
            print '=', tag, map(lambda x: x.xml(), seq1[alo:ahi]), map(lambda x: x.xml(), seq1[alo:ahi]), alo, ahi, blo, bhi
            last_match = (alo, ahi, blo, bhi)
            for i in xrange(0, ahi-alo):
                ai=alo+i
                bi=blo+i
                anode=seq1[ai].node
                bnode=seq2[bi].node
                anode.equalNode=bnode
                bnode.equalNode=anode

        else:
            self.fail("Tag '%s' doesn't exist." % tag)
        ###oldnode1=seq1[ahi-1]#???
        ###oldnode2=seq2[bhi-1]#???

    print



def insertNode(newnode, insert_point, parent, owner, tagName='INS'):
    newnode=owner.importNode(newnode,1)
    ins=owner.createElement(tagName)
    ins.appendChild(newnode)
    parent.insertBefore(ins, insert_point)

def deleteNode(deleted_object, owner, tagName='DEL'):
    isChildsDeleted(deleted_object)
    newnode=owner.createElement(tagName)
    deleted_object.parentNode.replaceChild(newnode, deleted_object)
    newnode.appendChild(deleted_object)

def isChild(child, parent):
    return child.parentNode is parent

def ProcessSimple(seq1, seq2, label=''):

    last_match=None
    differ=difflib.SequenceMatcher(lambda x: x.node.isWhiteSpace, seq1, seq2)

    owner=seq1[0].node.ownerDocument

    for tag, alo, ahi, blo, bhi in differ.get_opcodes():
        if tag == 'replace':
            ###print '?', tag, map(lambda x: x.xml(), seq1[alo:ahi]), map(lambda x: x.xml(), seq2[blo:bhi]), alo, ahi, blo, bhi

            insert_point=seq1[alo].node
            parent=insert_point.parentNode
            current=None

            for newline in seq2[blo:bhi]:
               if not current or newline.node.parentNode is not current:
                    insertNode(newline.node, insert_point, parent, owner, 'INS')
                    current=newline.node

                #newnode=owner.importNode(newline.node,1)
                #ins=owner.createElement('REP')
                #ins.appendChild(newnode)
                #parent.insertBefore(ins, insert_point)

            for delline in seq1[alo:ahi]:
                deleted_object=delline.node
                #parent=deleted_object.parentNode
                newnode=owner.createElement('DEL')
                deleted_object.parentNode.replaceChild(newnode, deleted_object)
                newnode.appendChild(deleted_object)

        elif tag == 'delete':
            ###print '-', tag, map(lambda x: x.xml(), seq1[alo:ahi]), map(lambda x: x.xml(), seq2[blo:bhi]), alo, ahi, blo, bhi

            #for i in xrange(0, ahi-alo):
            #    seq1[alo+i].node.deleteNode=1
            #    print "seq1[alo+i].node", seq1[alo+i].node.nodeName, seq1[alo+i].node.nodeValue, getattr(seq1[alo+i].node,'deleteNode', 0)

            #for i in xrange(0, ahi-alo):
            #    isChildsDeleted(seq1[alo+i].node)
            #    print "seq1[alo+i].node tree", seq1[alo+i].node.nodeName, seq1[alo+i].node.nodeValue, getattr(seq1[alo+i].node,'deleteTree', 0)

            for delline in seq1[alo:ahi]:
                deleted_object=delline.node
                deleteNode(deleted_object, owner)



        elif tag == 'insert':
            ###print '+', tag, seq1[alo].node, map(lambda x: x.xml(), seq2[blo:bhi]), alo, ahi, blo, bhi
            insert_point=seq1[alo].node
            parent=insert_point.parentNode
            current=None

            for newline in seq2[blo:bhi]:
                #print "newline.node", newline.node
                if not current or newline.node.parentNode is not current:
                    insertNode(newline.node, insert_point, parent, owner)
                    current=newline.node

                #child.parentNode is parent

        elif tag == 'equal':
            ###print '=', tag, map(lambda x: x.xml(), seq1[alo:ahi]), map(lambda x: x.xml(), seq1[alo:ahi]), alo, ahi, blo, bhi
            last_match = (alo, ahi, blo, bhi)
            for i in xrange(0, ahi-alo):
                ai=alo+i
                bi=blo+i
                anode=seq1[ai].node
                bnode=seq2[bi].node
                anode.equalNode=bnode
                bnode.equalNode=anode

        else:
            self.fail("Tag '%s' doesn't exist." % tag)

    print


def DiffFormat(dom):
    walker = TreeWalker(dom.documentElement, NodeFilter.SHOW_ALL, None, 1)
    while 1:
        node=walker.currentNode
        if getattr(node, 'deleteTree', 0):
            print "- ", node.nodeType, node.nodeName, node.nodeValue and node.nodeValue.strip()
        elif getattr(node, 'deleteNode', 0):
            print "- ", node.nodeType, node.nodeName, node.nodeValue and node.nodeValue.strip()
        elif getattr(node, 'insertBeforeList', 0):
            for node in getattr(node, 'insertBeforeList', []):
                print "+ ", node.nodeType, node.nodeName, node.nodeValue and node.nodeValue.strip()
        else:
            print "= ", node.nodeType, node.nodeName, node.nodeValue and node.nodeValue.strip()

        next = walker.nextNode()
        if next is None: break

def DiffFormatSimple(dom):
    walker = TreeWalker(dom.documentElement, NodeFilter.SHOW_ALL, None, 1)
    print "=-"*40
    while 1:
        node=walker.currentNode
        #if isNonPrincipalBlock(node) or node.nodeName in ('DEL', 'INS'): #node.nodeName not in ('HTML', 'BODY'):
        if isBlock(node) or node.nodeName in ('DEL', 'INS'): #node.nodeName not in ('HTML', 'BODY'):
            if getattr(node, 'deleteTree', 0):
                #print "- ", node.nodeType, node.nodeName, node.nodeValue and node.nodeValue.strip()
                Print(node)
                print
            elif getattr(node, 'deleteNode', 0):
                #print "- ", node.nodeType, node.nodeName, node.nodeValue and node.nodeValue.strip()
                Print(node)
                print
            elif getattr(node, 'insertBeforeList', 0):
                for node in getattr(node, 'insertBeforeList', []):
                    #print "+ ", node.nodeType, node.nodeName, node.nodeValue and node.nodeValue.strip()
                    Print(node)
                    print
            else:
                #print "= ", node.nodeType, node.nodeName, node.nodeValue and node.nodeValue.strip()
                Print(node)
                print
            next = walker.nextSibling()
        else:
            next = walker.nextNode()
        if next is None: break
    print "=-"*40
