# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyCore/ZCatalog/ProtectedCatalog.py,v 1.3 2004/09/13 20:52:02 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__ = """EasyCore.ZCatalog.ProtectedCatalog"""
__version__='$Revision: 1.3 $'[11:-2]

from types import ListType, TupleType
from Products.ZCatalog.ZCatalog import ZCatalog
from Products.ZCatalog.Catalog import CatalogError

from Products.EasyCore import MLOG_TAB


class ProtectedCatalog(ZCatalog):

    #Protected Indexes

    __init__=ZCatalog.__init__
    manage_options=ZCatalog.manage_options

    _undeletable_indexes=()
    _undeletable_columns=()

    def reindexIndex(self,name,REQUEST=None):
        paths = tuple(self._catalog.paths.values())
        for p in paths:
            obj = self.resolve_path(p)
            if not obj and REQUEST is not None:
                obj = self.resolve_url(p, REQUEST)
            if obj is not None:
                self.catalog_object(obj, p, idxs=[name])


    def addIndex(self, name, type, extra=None, undeletable=0):
        if undeletable:
            self._undeletable_indexes=self._undeletable_indexes+(name,)
        indexes=self.indexes()
        if name in indexes:
            if getattr(self._catalog.indexes[name],'meta_type', None)==type:
                return
            self.delIndex(name, force=1, dont_raise=1)
        ZCatalog.addIndex(self, name, type, extra=extra)
        REQ=getattr(self, 'REQUEST', None)
        self.reindexIndex(name,REQ)

    def delIndex(self, name, force=0, dont_raise=0):
        if name in self._undeletable_indexes and not force:
            if dont_raise: return
            raise CatalogError, "Index %s is undeletable." % name
        if name in self.indexes():
            ZCatalog.delIndex(self, name)

    def addColumn(self, name, default_value=None, undeletable=0):
        if undeletable:
            self._undeletable_columns=self._undeletable_columns+(name,)
        schema=self.schema()
        if name in schema:
            self.delColumn(name, force=1, dont_raise=1)
        ZCatalog.addColumn(self, name, default_value=default_value)
        if name in self.indexes():
            REQ=getattr(self, 'REQUEST', None)
            self.reindexIndex(name,REQ)

    def delColumn(self, name, force=0, dont_raise=0):
        if name in self._undeletable_columns and not force:
            if dont_raise: return
            raise CatalogError, "Column %s is undeletable." % name
        if name in self.schema():
            ZCatalog.delColumn(self, name)


    def getUniqueValuesFor(self, query={}, unique_index=None, group_by=None, output_ids=None):
        groups={}
        result = self.searchResults(query)
        if group_by:
            for brain in result:
                gname=getattr(brain, str(group_by), group_by)
                uv=groups.get(gname, {})
                value=getattr(brain, str(unique_index), unique_index)
                if type(value) not in (ListType, TupleType): value=[value]
                uv.update(uv.fromkeys(value)) #, brains some way
                groups[gname]=uv
            for key in groups.keys():
                groups[key]=groups[key].keys()
            return groups
        uniqueValues={}
        for brain in result:
            value=getattr(brain, str(unique_index), unique_index)
            if type(value) not in (ListType, TupleType): value=[value]
            uniqueValues.update(uniqueValues.fromkeys(value)) #, brains some way
        #if output_ids:
        #    for key, value in uniqueValues.items():
        #        output=()
        #        for oid in output_ids:
        #            output=getattr(value, oid
        #        #output_ids
        #        #return map(lambda x, o_ids=output_ids: , uniqueValues.items())
        #        pass
        return uniqueValues.keys()
