

from Interface import Attribute, Base

class EasyObjectLinkable(Base):
    """

    """

    _object_links = Attribute( '_object_links',
            """
            Maps link names to a list of valid links.

            Paths in the list of links should never be invalid.

            _object_links -- a mapping (dictionary) with link names
                (tuple of local name and remote name) as keys and
                a list of paths as value.
            """ )

    _link_policys = Attribute( '_link_policys',
          """
          Maps link names to link policy object (EasyLinkPolicy interface).



          _link_policys -- consist of a mapping (dictionary)
          with link_names (tuples) as keys and instances that
          implement the EasyLinkPolicy interface as values.

          """ )

    def createLink( self, link_name, path, policy=None):
        """
        Creates a link between the current object and a remote object.
        Both objects must implement this interface.

        link_name -- is a string or a tuple of two strings the local link name
            and the remote link name. If a single string is used a remove link
            name will automatically be create  called 'reverse_'+local_link.

        path -- is the traversable path to the remove object which we want
            to create a link too (and from)

        policy -- is a instance of a EasyLinkPolicy class
            that the object tries to set as the LinkPolicy for
            this link_name.
            If there already exists a policy for this link_name
            and the policy instaces doesn't match an exception
            will be raised (??here we might want to add a argument that
            lets createLink fail graciously if the policys don't match??)

        returns -- If creation was succesful None is returned, othervise
            is a object implementing the LinkAssessment interface returned.

        (Note there must be a way for the link creation to be rolled back if
        the initiating object fails to create the link, or if the remote object
        fails.)
        """

    def verifyLinkCreation( self, link_name, path ):
        """
        Is called by the remote object that initiated the link creation and how
        wants to create a link between it and this object to verify that this
        object allowes the link to be created and setting up the reverse link
        to the other object.

        link_name -- is a string or a tuple of two strings the local link name
            and the remote link name. If a single string is used a remove link
            name will automatically be create  called 'reverse_'+local_link.

        path -- is the traversable path back to the initiating object, needs
            to be valid for the link to be created

        returns -- 1 if the link was othervise 0.

        """

    def deleteLink( self, link_name, path):
        """
        Deletes a link object after checking policies on both sides.

        link_name -- is a string or a tuple of two strings the local link name
            and the remote link name. If a single string is used a remove link
            name will automatically be create  called 'reverse_'+local_link.

        path -- is the traversable path back to the initiating object, needs
            to be valid for the link to be created

        return -- None
        """

    def deleteReference(self, link_name, path):
        """
        Deletes the local part of the link if there is a
        remote object corresonding to the path argument.

        link_name -- is a string or a tuple of two strings the local link name
            and the remote link name. If a single string is used a remove link
            name will automatically be create called 'reverse_'+local_link.

        path -- is the traversable path back to the initiating object, needs
            to be valid for the link to be deleted

        return -- None if deletion was allowed else a object implementing the
            LinkAssessment interface returned.

        raise -- LinkPathError exception if the remove object can be found.
        """

    def deleteAllLinkByLinkName(self, link_name, suppress_exception=0):
        """
        Deletes all links matching the link_name.

        link_name -- is a string or a tuple of two strings the local link name
            and the remote link name. If a single string is used a remove link
            name will automatically be create  called 'reverse_'+local_link.

        returns -- a list of assessment where deletion failed.
            Assessments will carry additional the attributes:
                assessment.full_name
                assessment.reverse_link
                assessment.path
            An empty list is returned if all links where deleted.
        """

    def getPaths(self, link_name, suppress_exception=0):
        """
        Gets the paths of a link.

        If the link is empty but defined in _link_policys an empty
        list is returned, othervice a LinkNameError exception
        is raised.

        suppress_exception -- controls if an exception should be
            raise in case the link doesn't exists and is not
            defined in _link_policys

            If suppsuppress_exception is set to 1 an empty list []
            will be returned.

            Othervice the LinkNameError exception will be raised.

        """

    def hasLink(self, link_name):
        """
        Check if a link exists, including empty links that
        has been defined in _link_policys.
        """

    def isLinkEmpty(self, link_name, suppress_exception=0):
        """
        Check if a link is empty.
        If the link doesn't exists in either _link_policys
        or _object_links an exception is raise.

        suppress_exception -- controls if an exception should be
            raise in case the link neighter exists in _link_policys
            not _object_links. If ssuppsuppress_exception is 1
            an exception will not be raise and 1 will be returned.

            Othervice the LinkNameError exception will be raised.
        """

    def getLinkIds(self):
        """
        Returns all active link_names including links
        defined in _link_policys even if the are empty.
        """

    def getLocalLinkIds(self):
        """
        Returns the local part of all links including links
        defined in _link_policys even if the are empty.
        """

    def getLinkItems(self, link_name):
        """
        Returns a list of (link_name, path) for one link, or an empty list.

        Links defined in _link_policys are also returned even if
        they are empty.

        link_name -- the link to return items for.
        """

    def getAllLinkItems(self):
        """
        Returns a list of all (link_name, paths)

        Links defined in _link_policys are also returned even if
        they are empty.
        """

    def getLinkValues(self, link_name):
        """
        Returns a the value for link_name which is a list of path or an empty list.

        Links defined in _link_policys are also returned even if
        they are empty.
        """

    def getAllLinkValues(self):
        """
        Returns a list of all values (list of paths) of all links.

        Links defined in _link_policys are also returned even if
        they are empty.
        """

class SortableLinks(Base):
    """

    """

    def isSortable( self, link_name ):
        """
        """

    def moveUp( self, link_name, paths=[] ):
        """
        """

    def moveDown( self, link_name, paths=[] ):
        """
        """

    def moveFirst( self, link_name, paths=[] ):
        """
        """

    def moveLast( self, link_name, paths=[] ):
        """
        """


class EasyLinkPolicy(Base):
    """
    EasyLinkPolicy objects can enforce rules for the link,
    for instance if the link is sorted, if it allows multiple links,
    requirements on base classes, base templates, interfaces or plugins
    required on the object that's being linked to.

    """

    def isLinkCreationAllowed(self, link_name, local_obj, remote_obj):
        """
        Called on the Policy to verify the creation of a link.


        link_name -- is the full link name (tuple of the lenght 2, consisiting
            of a local name and a remote name string).

        local_obj -- is the a reference to the local object.

        remote_obj -- is the a reference to the remove object as retured by
            unrestricted_traverse.

        return -- a 1 if creation is allowed, otherwice 0
        """

    def getLinkCreationAssessment(self, link_name, local_obj, remote_obj):
        """
        Called on the Policy to get the reason why a link can't be created.

        link_name -- is the full link name (tuple of the lenght 2, consisiting
            of a local name and a remote name string).

        local_obj -- is the a reference to the local object.

        remote_obj -- is the a reference to the remove object as retured by
            unrestricted_traverse.

        return -- a LinkAssessment object if the link is not allowed to
                  be created, othervise None is returned
        """


    def isLinkDestructionAllowed(self, link_name, local_obj, remote_obj):
        """
        Called on the Policy to verify the destruction of a link.

        link_name -- is the full link name (tuple of the lenght 2, consisiting
            of a local name and a remote name string).

        local_obj -- is the a reference to the local object.

        remote_obj -- is the a reference to the remove object as retured by
            unrestricted_traverse.

        return -- a 1 if creation is allowed, otherwice 0
        """

    def getLinkDestructionAssessment(self, link_name, local_obj, remote_obj):
        """
        Called on the Policy to get the reason why a link can't be destroyed.

        link_name -- is the full link name (tuple of the lenght 2, consisiting
            of a local name and a remote name string).

        local_obj -- is the a reference to the local object.

        remote_obj -- is the a reference to the remove object as retured by
            unrestricted_traverse.

        return -- a LinkAssessment object if the link is not allowed to
                  be created, othervise None is returned
        """

class LinkAssessment(Base):
    """
    LinkAssessment are objects that describe the reason why a
    link policy failed.

    """
    code = Attribute( 'code', """Returns an integer of the type of assessment. Reserved numbers for internal assessment types are 0-999 (default 0).""")

    #assessment_type = Attribute( 'assessment_type', """Describes the type of assessment. In plain text.""")

    operation = Attribute( 'operation', """Describes the type of operation that was denied. in plain text.""")

    reason = Attribute( 'reason', """Describes the reason why the policy faild to validate in plain text.""")


