# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/BrowserRegistry.py,v 1.1 2004/06/28 10:04:15 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""BrowserRegistry Object
$Colliberty: BrowserRegistry.py,v 1.1 2002/02/20 14:20:00 johanc Exp $"""
__version__='$Revision: 1.1 $'[11:-2]


import Globals
from ExtensionClass import Base
from string import split, strip, replace
import re


SIMPLE = 0
EXPERT = 1
TUPLETYPE = type(())
LISTTYPE = type([])

include_by_functions = {}
exclude_by_functions = {}
include_cache = {}
exclude_cache = {}

class BrowserRegistry:


    def register(self, regexprs, function_ids, mode=SIMPLE):
        global include_cache, exclude_cache
        if type(regexprs) not in (TUPLETYPE, LISTTYPE):
            regexprs = [regexprs]
        if type(function_ids) not in (TUPLETYPE, LISTTYPE):
            function_ids = [function_ids]
        for function_id in function_ids:
            current_regs = include_by_functions.get(function_id, [])
            regs_compiled=[]
            for re_string in regexprs:
                if mode==SIMPLE:
                    re_string = replace(re_string, '.', '\.')
                    re_string = replace(re_string, '*', '.*')
                re_compiled = re.compile(re_string,re.I)
                regs_compiled.append(re_compiled)
            current_regs.append(regs_compiled)
            include_by_functions[function_id] = current_regs

    include = register

    def exclude(self, regexprs, function_ids, mode=SIMPLE):
        global include_cache, exclude_cache
        if type(regexprs) not in (TUPLETYPE, LISTTYPE):
            regexprs = [regexprs]
        if type(function_ids) not in (TUPLETYPE, LISTTYPE):
            function_ids = [function_ids]
        for function_id in function_ids:
            current_regs = exclude_by_functions.get(function_id, [])
            regs_compiled=[]
            for re_string in regexprs:
                if mode==SIMPLE:
                    re_string = replace(re_string, '.', '\.')
                    re_string = replace(re_string, '*', '.*')
                re_compiled = re.compile(re_string,re.I)
                regs_compiled.append(re_compiled)
            current_regs.append(regs_compiled)
            exclude_by_functions[function_id] = current_regs



    def getFunctionIds(self):
        return ua_strings_by_function.keys()

    def isFunctionSupportedFor(self, function_id, uas):
        global include_cache, exclude_cache
        if uas in exclude_cache.get(function_id, []): return 0
        if uas in include_cache.get(function_id, []): return 1
        for current_reg in exclude_by_functions.get(function_id, []):
            res=1
            for re_comp in current_reg:
                if not re_comp.search(uas):
                    res=0
                    break;
            if res:
                cache = exclude_cache.get(function_id, [])
                cache.append(uas)
                exclude_cache[function_id] = cache
                return 0
        for current_reg in include_by_functions.get(function_id, []):
            res=1
            for re_comp in current_reg:
                if not re_comp.search(uas):
                    res=0
                    break;
            if res:
                cache = include_cache.get(function_id, [])
                cache.append(uas)
                include_cache[function_id] = cache
                return 1
        cache = exclude_cache.get(function_id, [])
        cache.append(uas)
        exclude_cache[function_id] = cache
        return 0


    def clear_cache(self):
        global include_cache, exclude_cache
        inc_cache = include_cache
        ext_cache = exclude_cache
        include_cache = {}
        exclude_cache = {}
        for funcid in inc_cache.keys():
            for uas in inc_cache[funcid]:
                self.isFunctionSupportedFor(funcid, uas)
        for funcid in ext_cache.keys():
            for uas in ext_cache[funcid]:
                self.isFunctionSupportedFor(funcid, uas)

BrowserRegistry = BrowserRegistry()

