# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/EasyEditor.py,v 1.17 2004/10/06 18:08:28 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__ = """EasyEditor"""
__version__='$Revision: 1.17 $'[11:-2]



import Globals
from Globals import DTMLFile, MessageDialog
from os import linesep
import sys
from Acquisition import aq_base, aq_chain
import string

from ZODB.PersistentMapping import PersistentMapping
from OFS.Folder import Folder

from Products.EasyCore.Utils.Utils import spacify
from Products.EasyCore.Utils.HTMLEntity import htmlconverterbase
from Products.EasyCore.UpgradeSupport.Utils import upgradeProperties
from Products.EasyCore.UpgradeSupport.Utils import upgrade_indexes

from Products.EasyEditor.EasyTemplates.EasyTemplates import EasyTemplate
from Products.EasyEditor.EasyTemplates.EasyTemplateBase import EasyViewContainer
from Products.EasyEditor.EasyTemplates.EasyViews import EasyPlugins, EasyViews, EasyRegions, DTMLView, default_dtml

from Products.EasyEditor.EditableRegions.EditableRegions import EditableRegion
from Products.EasyEditor.Styles.StylesManage import StylesManage
from Products.EasyEditor.Features.Features import EasyFeatures

from EasyPublisher import EasyPublisherBase
from Products.OrderedFolder.OrderedFolder import OrderedFolder

from BrowserFolder.BrowserFolder import BrowserAware
from AccessControl import ClassSecurityInfo
from Products.EasyCore import EasyCorePermissions

from Products.EasyCore.VirtualPath import VirtualPathBase
from Products.EasyCore.Utils.StrippingParser import StrippingParser
from Products.EasyCore.EasyEditorConf import EASYEDITORID
from Products.EasyCore.EasyEditorConf import DEFAULTTEMPLATEID, DEFAULTVIEWNAME
from Products.EasyCore.EasyEditorConf import DEFAULTSTORAGETYPE, DEFAULTVIEWTITLE
from Products.EasyCore.EasyEditorConf import DEFAULTCONTENT, DEFAULTCONTENTSTORAGEID
from Products.EasyCore.EasyEditorConf import DEFAULTSTORAGECONTENT
from Products.EasyCore.EasyEditorConf import ACQUIRETEMPLATE, ACQUIREVIEW
from Products.EasyCore.EasyTypeRegistry import EasyTypeRegistry

from Products.EasyEditor.EasyEditorRegistry import EasyStorage, ViewDefaults, ViewDefaultsByMetaType, EasyTemplates, EasyTemplateHelpers

from Products.EasyCore.ZCatalog.ProtectedCatalog import ProtectedCatalog

from Products.CMFCore.SkinsContainer import SkinsContainer

from zLOG import LOG, WARNING, ERROR

from Products.EasyCore import MLOG_TAB

_marker=[]

DEFAULT_INDEXES = {}
REQUIRED_INDEXES = {
          'id':'FieldIndex'
        , 'title':'TextIndex'
        , 'bobobase_modification_time':'FieldIndex'
        , 'meta_type':'FieldIndex'
        , 'creation_date':'FieldIndex'
        , 'last_modified_by':'FieldIndex'
        , 'author':'FieldIndex'
        , 'physicalPath':'FieldIndex'
        , 'virtualPath':'FieldIndex'
        , 'version':'FieldIndex'
        , 'release':'FieldIndex'
        , 'navigation_aware':'FieldIndex'
        , 'show_template_types':'KeywordIndex'
        , 'allow_template_types':'KeywordIndex'
        , 'use_easy_class':'FieldIndex'
        , 'show_meta_types':'KeywordIndex'
        , 'allow_meta_types':'KeywordIndex'
        , 'exported_attributes':'KeywordIndex'
        #, 'exported_views':'KeywordIndex'
        #, 'exported_methods':'KeywordIndex'
        #, 'exported_sheets':'KeywordIndex'
}
DEPRECATED_INDEXES=('id','title','meta_type','bobobase_modification_time','path')

DEFAULT_COLUMNS = ()
REQUIRED_COLUMNS = (
          'id'
        , 'title'
        , 'bobobase_modification_time'
        , 'meta_type'
        , 'creation_date'
        , 'last_modified_by'
        , 'author'
        , 'physicalPath'
        , 'virtualPath'
        , 'version'
        , 'release'
        , 'navigation_aware'
        , 'show_template_types'
        , 'allow_template_types'
        , 'use_easy_class'
        , 'show_meta_types'
        , 'allow_meta_types'
        , 'exported_attributes'
        #, 'exported_views'
        #, 'exported_methods'
        #, 'exported_sheets'
)
DEPRECATED_COLUMNS=('summary', 'bobobase_modification_time', 'meta_type', 'title', 'id')


class EasyEditor(EasyViewContainer, htmlconverterbase, StylesManage, ProtectedCatalog,
                VirtualPathBase, SkinsContainer, EasyPublisherBase, BrowserAware, Folder):
    """A EasyEditor Object """

    security = ClassSecurityInfo()

    meta_type = 'EasyEditor'

    misc = 'misc_/EasyEditor'

    isEditor = 1
    skey_default='position'
    possible_obj_sort_columns=['id','meta_type', 'get_size', 'position']
    _template_index=None
    hidden_attributes=[]
    hidden_meta_types=[]
    __static_exports__=['getExportedIdsFor'] + StylesManage.__static_exports__



    _properties=(
            {'id':'title', 'type': 'string', 'label':'Title'},
            {'id':'skey_default', 'label':'Default Object Sorting', 'type':'selection', 'mode':'w','select_variable':'possible_obj_sort_columns'},
            {'id':'hidden_attributes', 'label':'Hidden Attributes', 'type':'lines', 'mode':'w'},
            {'id':'hidden_meta_types', 'label':'Hidden Meta Types', 'type':'lines', 'mode':'w'},

        )

    selected_meta_types = ('DTML Method','Page Template','Script (Python)','EasyFile','EasyImage','EasyFolder')

    def __init__(self, id, title, vocab_id=None, container=None):
        ProtectedCatalog.__init__(self, id, title, vocab_id, container)
        self.default_view = {'_view': DEFAULTVIEWNAME }
        self._dirty_editor = []

        #Init Protected Catalog
        for id in DEPRECATED_INDEXES:
            self.delIndex(id, force=1, dont_raise=1)
        for id, type in REQUIRED_INDEXES.items():
            self.addIndex(id, type, undeletable=1)
        for id, type in DEFAULT_INDEXES.items():
            self.addIndex(id, type)

        for id in DEPRECATED_COLUMNS:
            self.delColumn(id, force=1, dont_raise=1)
        for id in REQUIRED_COLUMNS:
            self.addColumn(id, undeletable=1)
        for id in DEFAULT_COLUMNS:
            self.addColumn(id)

    manage_options = (
            {'label':'Content', 'action':'manage_main'},
            {'label':'Default View', 'action':'manage_default_view_form'},
            {'label':'Show Exported', 'action':'manage_show_all_exported'},
    ) + ProtectedCatalog.manage_options[1:] + Folder.manage_options[-1:]

    security.declarePrivate('all_meta_types')
    def all_meta_types(self):
        inherited_all_meta_types = EasyEditor.inheritedAttribute('all_meta_types')
        easyplugins_meta_types = ()
        for views in EasyPlugins.values():
            easyplugins_meta_types = easyplugins_meta_types + tuple(views)
        easytemplates_meta_types = ()
        for templates in EasyTemplates.values():
            easytemplates_meta_types = easytemplates_meta_types + tuple(templates)
        for templatehelper in EasyTemplateHelpers.values():
            easytemplates_meta_types = easytemplates_meta_types + tuple(templatehelper)
        if callable(inherited_all_meta_types):
            all=inherited_all_meta_types(self)
        else:
            all=inherited_all_meta_types
        return easyplugins_meta_types+easytemplates_meta_types + tuple(all)


    def categorized_meta_types(self):
        gmt=self.filtered_meta_types()
        groups={}
        TEMPLATE_RELATED=['DTML Method', 'Script (Python)', 'Page Template','Controller Page Template', 'Controller Python Script', 'Controller Validator']
        templates=()
        for mt in gmt:
            mt=mt.copy()
            category=mt.get('category', None)
            name=mt.get('name', None)
            if name:
                mt['name']=spacify(name)
            if category is None: category='Zope'
            if category=='Easy Templates' or name in TEMPLATE_RELATED:
                templates=templates+(mt,)
            else:
                group=groups.get(category,())
                group=group+(mt,)
                groups[category]=group
        return [{'group':'','items':templates, }, ]+map(lambda x: {'group':x[0],'items':x[1], }, groups.items())



    security.declareProtected('View', 'easyeditorclient')
    easyeditorclient = DTMLFile('EasyEditorClient/EasyEditor', globals())

    security.declareProtected('View', 'urlmodule')
    urlmodule = DTMLFile('EasyEditorClient/URLModule', globals())

    security.declareProtected('View', 'pastemodule')
    pastemodule = DTMLFile('EasyEditorClient/PasteModule', globals())

    security.declareProtected('View', 'helpermodule')
    helpermodule = DTMLFile('EasyEditorClient/HelperModule', globals())

    security.declareProtected('View', 'initmodule')
    initmodule = DTMLFile('EasyEditorClient/InitModule', globals())

    security.declareProtected('View', 'dialogmodule')
    dialogmodule = DTMLFile('EasyEditorClient/DialogModule', globals())

    security.declareProtected('View', 'savemodule')
    savemodule = DTMLFile('EasyEditorClient/SaveModule', globals())

    security.declareProtected('View', 'keypressmodule')
    keypressmodule = DTMLFile('EasyEditorClient/KeypressModule', globals())

    security.declareProtected('View', 'debugmodule')
    debugmodule = DTMLFile('EasyEditorClient/DebugModule', globals())

    security.declareProtected('View', 'unittestmodule')
    unittestmodule = DTMLFile('EasyEditorClient/UnittestModule', globals())

    security.declareProtected('View', 'XMLPickleModule')
    XMLPickleModule = DTMLFile('EasyEditorClient/XMLPickleModule', globals())

    security.declareProtected('View', 'DHTMLUtilityModule')
    DHTMLUtilityModule = DTMLFile('EasyEditorClient/DHTMLUtilityModule', globals())

    security.declareProtected('View', 'easyeditorJS')
    easyeditorJS = DTMLFile('EasyEditorClient/easyeditorJS', globals())

    security.declareProtected('View', 'easyeditorCSS')
    easyeditorCSS = DTMLFile('EasyEditorClient/easyeditorCSS', globals())


    security.declarePrivate('aq_editor')
    def aq_editor(self):
        return self

    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'manage_default_view_form')
    manage_default_view_form = DTMLFile('dtml/manage_default_view_form', globals())

    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'manage_default_view')
    def manage_default_view(self, default_views=[], REQUEST=None):
        """Manage the name of default view"""
        new_default_view = {}
        for record in default_views:
            new_default_view[record.viewattr] = record.viewdefault
        self.default_view = new_default_view
        if REQUEST is not None:
            manage_tabs_message= """Default Views has been changed"""
            REQUEST.set('manage_tabs_message',manage_tabs_message)
            REQUEST.set('management_view','Default View')
            return self.manage_default_view_form(self,REQUEST)

    # Handle invalidation of Template Editor cache
    def setEditorDirty(self):
        self._dirty_editor = []

    def isTemplateDirty(self,templateid):
        return templateid in self._dirty_editor

    def clearDirtyTemplate(self,templateid):
        if not templateid in self._dirty_editor:
            self._dirty_editor.append(templateid)


    security.declareProtected(EasyCorePermissions.EasyDocumentChange, 'manage_editor')
    def manage_editor(self, document, templateid, document_url='', REQUEST=None):
        """Edit a document with a template"""
        template = self.getTemplate(templateid)
        if template == None:
            return MessageDialog(
                title  ="Template doesn't exist",
                message= "The template associated with this document doesn't exist. You need to change the associated template.",
                action ='manage_workspace')

        # the context is only the editor and the template (for the editorclient)
        REQUEST.set('templateid',templateid)
        REQUEST.set('viewmode','editor')
        REQUEST.set('here',document)
        REQUEST.set('document_url', document_url)
        REQUEST.set('template',template)
        REQUEST.set('editor',self)
        viewname = None
        if REQUEST.has_key('viewid'): viewname = REQUEST['viewid']
        if viewname is None or not template.hasView(viewname):
            viewname = template.getDefaultEditViewId()
        REQUEST.set('viewid',viewname)
        return template.manage_editor(self, document, REQUEST=REQUEST)


    #from Products.EasyCore.EasyTypeRegistry import EasyTypeRegistry
    security.declarePublic('getEasyEditableMetaTypes')
    def getEasyEditableMetaTypes(self):
        return EasyTypeRegistry.getEasyEditableMetaTypes()

    security.declarePublic('getAvailableTypes')
    def getAvailableTypes(self):
        return self.templateMap()
        return self.templateIds()

    # DEFAULT VIEWS AND VIEW ATTRIBUTE NAMES
    def getDefaultView(self, view_type=None, default = None):
        if view_type is None: view_type = '_view'
        viewname = self.default_view.get(view_type, None)
        if self.default_view.has_key(view_type):
            return self.default_view.get(view_type)
        if ViewDefaults.has_key(view_type):
            return ViewDefaults.get(view_type)
        if default is not None: return default
        return DEFAULTVIEWNAME

    def hasDefaultViewFor(self, viewattr):
        return self.default_view.has_key(viewattr)

    def defaultviews(self):
        return ViewDefaults.items()

    # TEMPLATES

    def hasTemplates(self):
        return not not len(self.objectIds(spec=EasyTemplates.keys()))

    def hasTemplate(self,templateid):
        templates = self.objectIds(spec=EasyTemplates.keys())
        return templateid in templates

    def getTemplate(self,templateid, raise_exc=0, default=None):
        template = getattr(self, templateid, None)
        if template is not None and template.meta_type in EasyTemplates.keys():
            return template
        return self.getDefaultTemplate(default=default)

    def getDefaultTemplateId(self):
        templates=self.objectIds(spec=EasyTemplates.keys())
        if templates:
            return templates[0]
        raise AttributeError, "No Template in EasyEditor"

    def getDefaultTemplate(self, default=None):
        templates=self.objectValues(spec=EasyTemplates.keys())
        if templates:
            return templates[0]
        return default


    def templateIds(self,spec=None):
         return self.objectIds(spec=EasyTemplates.keys())

    def templateValues(self,spec=None):
        return self.objectValues(spec=EasyTemplates.keys())

    def templateItems(self,spec=None):
        return self.objectItems(spec=EasyTemplates.keys())

    def templateMap(self):
        return map(lambda x: (x.id, x.title or x.id ), self.objectValues(spec=EasyTemplates.keys()))

    # FEATURES
    def hasFeature(self,featureid):
        return EasyFeatures.has_key(featureid)

    def getFeature(self,featureid):
        return EasyFeatures.get(featureid,None)

    def featureIds(self,spec=None):
        return EasyFeatures.keys()

    def featureValues(self,spec=None):
        return filter(lambda x, spec=spec: spec == None or x.feature_type in spec, EasyFeatures.values())

    def featureItems(self,spec=None):
        return EasyFeatures.items()

    def featureMap(self):
        return EasyFeatures.items()

    # STORAGE
    def hasStorage(self,storageid):
        return EasyStorage.has_key(storageid)

    #def getStorage(self,storageid,default=None):
    #    return EasyStorage.get(storageid, default)

    def storageIds(self,spec=None):
        return EasyStorage.keys()

    def storageValues(self,spec=None):
        return EasyStorage.values()

    def storageItems(self,spec=None):
        return  EasyStorage.items()

    def getStorageByStorageType(self):
         storages=EasyStorage.values()
         grouped={}
         keys=EasyStorage.keys()
         keys.sort()
         for storage in map(EasyStorage.get, keys):
            storage_group=storage.storage_group
            group=grouped.get(storage_group, [])
            group.append( (storage.id, storage) )
            grouped[storage_group] = group
         grouped_keys=grouped.keys()
         grouped_keys.sort()
         return map(lambda key,grouped=grouped: (key, grouped.get(key)), grouped_keys)


    # Helpers

    security.declarePublic('striphtml')
    def striphtml(self, s):
        parser = StrippingParser()
        parser.feed(s)
        parser.close()
        return string.join(string.split(parser.result))


    # webDAV support

    def addNewTemplate(self, id, title='', REQUEST=None,RESPONSE=None):
        if REQUEST:
            title = REQUEST.get('title', '')
        template=EasyTemplate(id, title)
        self._setObject(id, template)
        template = self._getOb(id)
        default=DTMLView(DEFAULTVIEWNAME, DEFAULTVIEWTITLE, 1, source_string=default_dtml)
        template._setObject(DEFAULTVIEWNAME, default)
        content = EditableRegion(DEFAULTCONTENT
                                , DEFAULTSTORAGETYPE
                                , DEFAULTSTORAGECONTENT
                                , 1
                                , '#FFFFFF'
                                , '#FFCCCC'
                                , DEFAULTCONTENTSTORAGEID
                                )

        template._setObject(DEFAULTCONTENT, content)

    def MKCOL_handler(self,id,REQUEST=None,RESPONSE=None):
        """
            Handle WebDAV MKCOL.
        """
        self.addNewTemplate(id, REQUEST=REQUEST,RESPONSE=RESPONSE)


    def PUT_factory( self, name, typ, body ):
        """
            Dispatcher for PUT requests to non-existent IDs.  Returns
            an object of the appropriate type (or None, if we don't
            know what to do).
        """
        try:
            if name and name.endswith('.pt'):
                from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
                ob = ZopePageTemplate(name, body, content_type=typ)
            elif typ in ('text/html', 'text/xml', 'text/plain'):
                from OFS.DTMLMethod import DTMLMethod
                ob = DTMLMethod( '', __name__=name )
            elif typ[:6]=='image/':
                from Products.EasyImage.EasyImage import EasyImage
                ob=EasyImage(name, body, '', content_type=typ)
            else:
                from Products.EasyFile.EasyFile import EasyFile
                ob = EasyFile(name, body, '', content_type=typ)
        except:
            import sys
            exc = sys.exc_info()
            LOG('EasyEditable', ERROR, "Can't create sub object [%s]. %s (%s)" % (typ,exc[0],exc[1]), error=exc)
            return None
        return ob



    #CMF Skintool Interface

    security.declareProtected(EasyCorePermissions.AccessContentsInformation, 'getSkinPath')
    def getSkinPath(self, templateid):
        '''
        Converts a skin name to a skin path.
        '''
        #Should return a hierachical skin_path (maybe with inserts of custom skin folders)
        #But it will do for now
        return templateid

    security.declareProtected(EasyCorePermissions.AccessContentsInformation, 'getDefaultSkin')
    def getDefaultSkin(self):
        '''
        Returns the default skin name.
        '''
        return self.getDefaultTemplateId()


    security.declarePrivate('getSkinByName')
    def getSkinByName(self, name):
        path = self.getSkinPath(name)
        if path is None:
            return None
        return self.getSkinByPath(path)


    security.declarePrivate('getSkinByPath')
    def getSkinByPath(self, path, raise_exc=0):
        '''
        Returns a skin at the given path.  A skin path is of the format:
        "some/path, some/other/path, ..."  The first part has precedence.
        '''
        baseself = aq_base(self)
        #skinob = aq_parent(aq_parent(self)) #This would probably fix the "can't access REQUEST" bug.
        skinob = baseself
        parts = list(string.split(path, ','))
        parts.reverse()
        for part_path in parts:
            partob = baseself
            for name in string.split(string.strip(part_path), '/'):
                if name == '':
                    continue
                if name[:1] == '_':
                    # Not allowed.
                    partob = None
                    if raise_exc:
                        raise SkinPathError('Underscores are not allowed')
                    break
                # Allow acquisition tricks.
                partob = getattr(partob, name, None)
                if partob is None:
                    # Not found.  Cancel the search.
                    if raise_exc:
                        raise SkinPathError('Name not found: %s' % part_path)
                    break
            if partob is not None:
                # Now partob has containment and context.
                # Build the final skinob by creating an object
                # that puts the former skinob in the context
                # of the new skinob.
                skinob = partob.__of__(skinob)
        return skinob



    #Catalog Overrides
    def catalog_object(self, obj, uid=None, idxs=None, update_metadata=1):
        ProtectedCatalog.catalog_object(self, obj, uid=uid, idxs=idxs, update_metadata=update_metadata)
        if getattr(aq_base(obj), 'isTemplate', 0) and hasattr(aq_base(obj), 'exported_attributes'):
            tid=obj.getTemplateId()
            if self._template_index is None:
                self._template_index=PersistentMapping()
            self._template_index[tid]=obj.exported_attributes()

    def uncatalog_object(self, uid):
        ProtectedCatalog.catalog_object(self, obj, uid=uid, idxs=idxs, update_metadata=update_metadata)
        if getattr(aq_base(obj), 'isTemplate', 0)=='' and hasattr(aq_base(obj), 'exported_attributes'):
            if self._template_index is None:
                self._template_index=PersistentMapping()
            tid=obj.getTemplateId()
            if self._template_index.has_key(tid):
                del self._template_index[tid]

    def getExportedIdsFor(self, temlateid, include_local=1):
        template_index=[]
        if self._template_index is not None:
            template_index=list(self._template_index.get(temlateid, []))
        if include_local:
            return template_index+self.exported_attributes()+self.__static_exports__
        return template_index+self.__static_exports__

    security.declarePublic('exported_attributes')
    def exported_attributes(self):
        return self.filtered_object_ids() + self.filtered_property_ids()

    security.declarePrivate('filtered_object_ids')
    def filtered_object_ids(self):
        """\
        Returned object ids for all objects but Regions
        and objects in hidden_attributes
        """
        spec=list(self.hidden_meta_types)
        spec.append('Easy Template')

        set=[]
        for ob in self._objects:
            if ob['meta_type'] not in spec and ob['id'] not in self.hidden_attributes:
                set.append(ob['id'])
        return set

    security.declarePrivate('filtered_property_ids')
    def filtered_property_ids(self):
        """\
        Returned object ids for all objects but names
        defined in the property hidden_attributes.
        """
        set=[]
        for prop in self._properties:
            if prop['id'] not in self.hidden_attributes:
                set.append(prop['id'])
        return set

    manage_show_all_exported = DTMLFile('dtml/manage_show_all_exported', globals())


    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        if type(getattr(aq_base(self), '__ac_local_roles__'))==type({}):
            del self.__ac_local_roles__
            changed=1

        if filter( lambda x: x['meta_type'] == 'simple item', self._objects):
            new_object_list = ()
            for subobj in self._objects:
                if subobj['meta_type'] == 'simple item': subobj['meta_type'] = 'CSSStyle'
                new_object_list = new_object_list + (subobj,)
            self._objects = new_object_list
            changed=1

        if not hasattr(self, '_catalog'):
            from Products.ZCatalog.Catalog import Catalog
            self.vocabulary = None
            self.vocab_id = ''
            self.threshold = 10000
            self._v_total = 0
            self._catalog = Catalog()

        upgradeProperties(self, {'title':'','skey_default':'position',
                                    'hidden_attributes':[],
                                    'hidden_meta_types':[],
                                    }, ('_is_transparent',))

        changed, messages=upgrade_indexes(self, changed, DEFAULT_INDEXES, DEPRECATED_INDEXES, REQUIRED_INDEXES, DEFAULT_COLUMNS, DEPRECATED_COLUMNS, REQUIRED_COLUMNS)

        if getattr(aq_base(self),'_object_version', None)!='1.8': self._object_version = '1.8'

        if REQUEST is not None:
            return "<h1>Done</h1>"
        return changed

Globals.InitializeClass(EasyEditor)


def manage_addEasyEditor(self,id="", REQUEST=None):
    """Add Easy Editor"""

    id = EASYEDITORID

    if hasattr(self.aq_base,id):
        EASYEDITOREXISTS = 'There already exists a EasyEditor in folder %s' % (self.id)
        if REQUEST is not None:
            return MessageDialog(
                title  ='Easy Editor Exists',
                message= EASYEDITOREXISTS,
                action ='manage_main')
        return EASYEDITOREXISTS
    ob=EasyEditor(id=id,title=id)
    self._setObject(id, ob)
    editor = self._getOb(id)


    if REQUEST is not None:
        manage_tabs_message= """%s has been added.""" % (id,)
        REQUEST.set('manage_tabs_message',manage_tabs_message)
        return self.manage_main(self,REQUEST)

manage_addEasyEditorForm = manage_addEasyEditor



def initialize(context):
    """Initialize EasyEditor Product"""


    context.registerClass (
            EasyEditor,
            permission=EasyCorePermissions.EasyEditorAdd,
            icon='icons/EasyEditor.gif',
            constructors=(manage_addEasyEditor,
                          manage_addEasyEditorForm),
            meta_type = EasyEditor.meta_type,
            category='Easy Publisher Management',
            )




