# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/EasyFolder.py,v 1.5 2004/09/28 10:04:50 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyFolder Object

$Colliberty: EasyFolder.py,v 1.8 2001/08/21 11:58:00 johanc Exp $"""
__version__='$Revision: 1.5 $'[11:-2]


import sys
import Globals
from Globals import DTMLFile
import DateTime
from Acquisition import aq_base
from AccessControl import getSecurityManager
from AccessControl import ClassSecurityInfo
from Products.EasyCore import EasyCorePermissions
from Products.EasyCore.ManageOptionBase import ManageOptionBase

from OFS.Folder import Folder

from string import join

from Products.EasyCore.UpgradeSupport.Utils import upgradeProperties
from Products.EasyCore.EasyEditorConf import EASYEDITORID, DEFAULTTEMPLATEID,DEFAULTVIEWNAME, DEFAULTVIEWATTR, ACQUIRETEMPLATE, ACQUIREVIEW
from Products.EasyNavigation.EasyNavigatable import Navigatable
from Products.EasyWorkflow.EasyWorkflowBase import NullDate
from Products.EasyDublinCore.EasyDublinCore import EasyDublinCore
from Products.EasyEditor.EasyWebDAVSupport import EasyWebDAVSupport
from Products.EasyWorkflow.EasySync import EasySyncAware


from zLOG import LOG, ERROR



class EasyFolder(ManageOptionBase, EasyWebDAVSupport, Navigatable, EasySyncAware, EasyDublinCore, Folder):
    """A EasyDocument Object """

    security = ClassSecurityInfo()

    _auto_add_to_navigation = 0

    meta_type = 'EasyFolder'

    use_manage_options = ((Folder.manage_options[:1], None, Folder.manage_options[1:]),) + \
                        Navigatable.use_manage_options

    _properties = Navigatable._properties + EasyDublinCore._properties_without_content+()

    def PrincipiaSearchSource(self):
        #Very temporary search source for text indexes
        #This method should be made to look for attributes,
        #properties and subobjects defined for the object that
        #should be included in the PrincipiaSearchSource.
        result = self.title + ' ' + self.description + ' ' + join(self.category,' ')
        return result

    def __init__(self, id
                , template = None
                , title=''
                , navtitle=''
                , description=''
                , language=''
                , category=()
                ):
        EasySyncAware.__init__(self) #added to include Sync activation!! 03/01/05
        EasyDublinCore.__init__(self
                                , title=title
                                , description=description
                                , language=language
                                , category=category
                                )
        Navigatable.__init__(self, title, navtitle)
        self.id = id
        self.title = title
        self.template = template


    # webDAV support

    def addNewDocument(self, id
                        , template = None
                        , title=''
                        , navtitle=''
                        , description=''
                        , language=''
                        , category=()
                        , subject=()
                        , effective_date=NullDate
                        , expiration_date=NullDate
                        , format='text/html'
                        , add_to_navigation=None
                        , REQUEST=None,RESPONSE=None
                    ):
        if REQUEST:
            title = REQUEST.get('title', '')
        if REQUEST:
            navtitle = REQUEST.get('navtitle', '')
        try:
            folder=EasyFolder(id
                            , template=template
                            , title=title
                            , navtitle=navtitle
                            , description=description
                            , language=language
                            , category=category
                        )
            self._setObject(id, folder)
        except:
            import sys
            exc = sys.exc_info()
            LOG('EasyFolder', ERROR, "Can't create sub document. %s (%s)" % exc[:2], error=exc)

    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        if getattr(self,'_object_version', None)!='1.7': self._object_version = '1.7'
        if not hasattr(aq_base(self),'_object_links'):
            self._object_links = {} ; changed=1
        upgradeProperties(self, {'navigation_title':''}, ('skey_default',))
        if REQUEST is not None:
            return "<h1>Done</h1>"
        return changed

    #EVENT TRIGGERS
    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        Navigatable.manage_afterAdd(self, item, container)
        EasySyncAware.manage_afterAdd(self, item, container)

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        Navigatable.manage_beforeDelete(self, item, container)
        EasySyncAware.manage_beforeDelete(self, item, container)

    security.declarePrivate('manage_afterClone')
    def manage_afterClone(self, item):
        Navigatable.manage_afterClone(self, item)
        EasySyncAware.manage_afterClone(self, item)



Globals.InitializeClass(EasyFolder)


manage_addEasyFolderForm=DTMLFile('dtml/manage_addEasyFolderForm', globals())

def manage_addEasyFolder(self, id
                        , template = None
                        , title=''
                        , navtitle=''
                        , description=''
                        , language=''
                        , category=()
                        , subject=()
                        , effective_date=NullDate
                        , expiration_date=NullDate
                        , format='text/html'
                        , add_to_navigation=None
                        , REQUEST=None):
    """Add EasyFolder"""

    ob=EasyFolder(    id
                    , template=template
                    , title=title
                    , navtitle=navtitle
                    , description=description
                    , language=language
                    , category=category
                    )
    self._setObject(id, ob)
    ob = self._getOb(id) # get object in context

    if REQUEST is not None:
        return self.manage_main(self,REQUEST)


def initialize(context):
    """Initialize EasyFolder Product"""

    context.registerClass (
            EasyFolder,
            permission=EasyCorePermissions.EasyDocumentAdd,
            icon='icons/EasyFolder.gif',
            constructors=(manage_addEasyFolderForm,
                          manage_addEasyFolder),
            meta_type = EasyFolder.meta_type,
            category='Easy Publisher',
            )


