# coding: iso-8859-1
# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/EasyTemplates/EasySchema.py,v 1.3 2004/09/16 08:56:54 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasySchema

$Colliberty: EasySchema.py,v 1.8 2001/11/18 12:00:00 johanc Exp $"""
__version__='$Revision: 1.3 $'[11:-2]


import Globals
from Globals import HTML, DTMLFile
from types import ListType, TupleType
from string import join, strip, replace
from urllib import quote
from copy import copy

from Acquisition import aq_base, aq_chain, aq_parent

from OFS.DTMLMethod import DTMLMethod
from OFS.Folder import Folder
from AccessControl import ClassSecurityInfo
from ZODB.PersistentMapping import PersistentMapping

from Products.EasyCore import EasyCorePermissions
from Products.EasyCore.Utils.HTMLFormHelpers import checkableTableRows
from Products.EasyCore.Utils.Javascript import escape_javascript_text, escape_javascript_id
from Products.EasyCore.Utils.Utils import spacify
from Products.EasyCore.UpgradeSupport.Utils import upgradeProperties
from Products.EasyCore.VirtualPath import VirtualPathBase

from Products.EasyCatalog.EasyCatalogAware import EasyCatalogAwareObjectAndPropertyManager, EasyCatalogAware
from Products.EasyEditor.EasyEditorClient.EasyEditorConstants import *
from Products.EasyEditor.Features.Features import getEasyFeatureButtonGroupsOrdered, getEasyFeatureDropDownGroupsOrdered
from Products.EasyEditor.EasyTemplates.EasyViews import EasyPlugins, EasyViews, EasyRegions, DTMLView

#from Products.OrderedFolder.OrderedFolder import OrderedFolder
from Products.OrderedFolder.SubObjectTypeSupport import SubObjectTypeSupport

from zLOG import LOG, ERROR
from Products.EasyCore import MLOG_TAB

# Check cache environment variable
import os
SUPPRESS_CACHE = os.environ.has_key('SUPPRESS_CACHE')



class EasySchema(EasyCatalogAwareObjectAndPropertyManager, EasyCatalogAware, Folder, VirtualPathBase):
    """Mixin class for easy template objects"""

    security = ClassSecurityInfo()

    meta_type = 'Easy Schema'

    title = ''
    isTemplate = 1
    navigation_aware=0
    selected_meta_types = ('DTML Method','Page Template','File','Image','Folder','Script (Python)')

    show_template_types=False
    allow_template_types=()
    show_meta_types=False
    allow_meta_types=()
    use_easy_class='EasyDocument'
    disable_editor=0
    edit_option_name='HTML'
    edit_view_first=0
    hidden_attributes=[]
    hidden_meta_types=[]

    __static_exports__=['getRegionIds']

    #skey_default='position'
    possible_obj_sort_columns=['id','meta_type', 'get_size', 'position']

    __catalogs__=('EasyEditor',)

    _properties = (
            {'id':'title', 'label':'Title', 'type':'string', 'mode':'w'},
            {'id':'navigation_aware', 'label':'NavigationAware', 'type':'boolean', 'mode':'w'},
            #{'id':'skey_default', 'label':'Default Object Sorting', 'type':'selection', 'mode':'w', 'select_variable':'possible_obj_sort_columns'},
            {'id':'show_template_types', 'label':'Filter Template Types', 'type':'boolean', 'mode':'w', },
            {'id':'allow_template_types', 'label':'Allowed Template Types', 'type':'multiple selection', 'mode':'w', 'select_variable':'getAvailableTypes'},
            {'id':'use_easy_class', 'label':'Use Base Class', 'type':'selection', 'mode':'w', 'select_variable':'getEasyEditableMetaTypes'},
            {'id':'show_meta_types', 'label':'Filter Meta Types', 'type':'boolean', 'mode':'w', },
            {'id':'allow_meta_types', 'label':'Allowed Meta Types', 'type':'multiple selection', 'mode':'w', 'select_variable':'inherited_meta_types'},
            {'id':'disable_editor', 'label':'Disable Editor', 'type':'boolean', 'mode':'w'},
            {'id':'edit_option_name', 'label':'Edit Option Name', 'type':'string', 'mode':'w'},
            {'id':'edit_view_first', 'label':'Edit Option First', 'type':'boolean', 'mode':'w'},
            {'id':'hidden_attributes', 'label':'Hidden Attributes', 'type':'lines', 'mode':'w'},
            {'id':'hidden_meta_types', 'label':'Hidden Meta Types', 'type':'lines', 'mode':'w'},

        )


    manage_options_configurations =(
        #{'label':'Show Exported', 'action':'manage_show_exported'},
    )

    manage_options = (Folder.manage_options[0],) + manage_options_configurations + \
        Folder.manage_options[1:] + SubObjectTypeSupport.manage_options



    def getIcon(self):
        try:
            if hasattr(self, 'custom_icon'):
                return self.custom_icon.absolute_url()
            return self.icon
        except:
            return self.icon


    def getMetaTypes(self):
        return self.allow_meta_types

    def getTemplateTypes(self):
        templates=[]
        for id in self.allow_template_types:
            templates.append(self.getTemplate(id))
        return templates

    security.declarePublic('inherited_meta_types')
    def inherited_meta_types(self):
        all_meta_types=EasySchema.inheritedAttribute('all_meta_types')
        if callable(all_meta_types): all_meta_types=all_meta_types(self)
        all_meta_types=map(lambda x: x['name'], all_meta_types)
        return all_meta_types

    security.declarePublic('all_meta_types')
    def all_meta_types(self):
        easyregions_meta_types = ()
        for regions in EasyRegions.values():
            easyregions_meta_types = easyregions_meta_types + tuple(regions)
        return easyregions_meta_types

    def categorized_meta_types(self):
        gmt=self.filtered_meta_types()
        groups={}
        TEMPLATE_RELATED=['DTML Method', 'Script (Python)', 'Page Template','Controller Page Template', 'Controller Python Script', 'Controller Validator']
        templates=()
        for mt in gmt:
            mt=mt.copy()
            category=mt.get('category', None)
            name=mt.get('name', None)
            if name:
                mt['name']=spacify(name)
            if category is None: category='Zope'
            if category=='Easy Templates' or name in TEMPLATE_RELATED:
                templates=templates+(mt,)
            else:
                group=groups.get(category,())
                group=group+(mt,)
                groups[category]=group
        return [{'group':'','items':templates, }, ]+map(lambda x: {'group':x[0],'items':x[1], }, groups.items())


    security.declareProtected(EasyCorePermissions.ViewManagementScreens,'easy_manage_tabs')
    def easy_manage_tabs(self,easy_manage_options, REQUEST=None):
        #if callable(self.manage_options): manage_options = self.manage_options()
        #else: manage_options = self.manage_options
        result = ''
        for option in easy_manage_options:
            option_tag = HTML("""
                <BUTTON
                    id="&dtml-action;"
                    class=menubutton
                    onClick="
                        <dtml-if "_.has_key('actionhandler')">
                            this.blur();
                            &dtml-actionhandler;
                            window.event.cancelBubble=true;
                        <dtml-else>
                            this.blur();
                            doSave('&dtml-action;','&dtml-label;',1);
                            window.event.cancelBubble=true;
                        </dtml-if>"
                    onMouseOver="
                        menumouseOverButton(this);
                        <dtml-if "_.has_key('actionhandler')">
                            glb_temp_window_status = window.status;
                            window.status='&dtml-label;';
                        <dtml-else>
                            glb_temp_window_status = window.status;
                            window.status=' &dtml-label;';
                        </dtml-if>"
                    "
                    onMouseOut="mouseOffButton(this);window.status=glb_temp_window_status;"
                    <dtml-if "_.has_key('accesskey')">
                    accesskey="&dtml-accesskey;"</dtml-if>
                    >&dtml-label;</BUTTON>
                """,globals())(self,option)
            result = result + option_tag + '\n'
        return result

    def __init__(self, id, title):
        self.id = id
        self.title = title


    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        if getattr(aq_base(self),'_object_version', None)!='1.8': self._object_version = '1.8' ; changed=1
        upgradeProperties(self, {}, ('skey_default',))
        if REQUEST is not None:
            return "<h1>Done</h1><p>Changed: %s</p>" % str(changed)
        return changed

Globals.InitializeClass(EasySchema)



manage_add_EasySchemaForm = DTMLFile('dtml/manage_add_EasySchemaForm', globals())

def manage_add_EasySchema(self,id, title='', submit=None, REQUEST=None):
    """Add an Easy Schema"""
    ob = EasySchema(id=id,title=title)
    id=self._setObject(id, ob)

    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')



def initialize(context):
    """Initialize EasySchema Product"""

    context.registerEasyViewClass(
            EasySchema,
            meta_type = EasySchema.meta_type,
            permission = EasyCorePermissions.EasyTemplateAdd,
            constructors = (manage_add_EasySchemaForm, manage_add_EasySchema),
            icon='icons/EasySchema.gif',
            category='Easy Templates',
        )




