# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/EasyTemplates/EasyTemplateBase.py,v 1.2 2004/10/06 18:08:28 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__='''EasyTemplateBase
$Id: EasyTemplateBase.py,v 1.2 2004/10/06 18:08:28 johanc Exp $'''
__version__='$Revision: 1.2 $'[11:-2]

import Globals
from Acquisition import aq_base, aq_parent, aq_inner
from AccessControl import ClassSecurityInfo
from Globals import DTMLFile
from BTrees.OOBTree import OOSet

from Products.EasyCore import EasyCorePermissions
from Products.EasyEditor.EasyTemplates.EasyViews import EasyViews, EasyRegions


from Products.EasyCore import MLOG_TAB


class EasyTemplateBase:
    """Initialize the EasyTemplates product."""



class EasyViewContainer:
    """API for listing and accessing views."""
    
    #VIEWS
    security = ClassSecurityInfo()

    def viewObjectIds(self, edit=None):
        set=[]
        if edit:
            for ob in self._objects:
                if (ob.has_key('edit') and ob.has_key('view')) or (getattr(self._getOb(ob['id']), 'isView' , 0) and getattr(self._getOb(ob['id']), 'isEditable' , 0)):
                    set.append(ob['id'])
        else:
            for ob in self._objects:
                if ob.has_key('view') or getattr(self._getOb(ob['id']), 'isView' , 0):
                    set.append(ob['id'])
        return set

    def viewObjectValues(self, edit=None):
        # Returns a list of actual subobjects of the current object.
        # If 'spec' is specified, returns only objects whose meta_type
        # match 'spec'.
        return [ self._getOb(id) for id in self.viewObjectIds(edit=edit) ]

    def getInheritedViewIds(self, edit=None):
        ids=OOSet()
        for base_template in self.getParentTemplates():
            ids.update(base_template.viewObjectIds(edit=edit))
        return tuple(ids)

    def getInheritedObjectValues(self, edit=None):
        ids=OOSet()
        views=[]
        for base_template in self.getParentTemplates():
            new_ids=base_template.viewObjectIds(edit=edit)
            for new_id in new_ids:
                if new_id not in ids:
                    try:
                        views.append(base_template.getView(new_id))
                        ids.update([new_id])                    
                    except:
                        pass
        return views
        
    def toogleViewObjects(self, view_ids=[], set_view=1, set_edit=None):
        """Toogles views or edit views in the _object tulpe."""    
        objects=[]
        for obj in self._objects:
            if obj['id'] in view_ids:
                if set_view: 
                    if obj.has_key('view'):
                        del obj['view']
                    else:
                        obj['view']=1
                if set_edit: 
                    if obj.has_key('edit'):
                        del obj['edit']
                    else:
                        obj['edit']=1
            objects.append(obj)
        self._objects=tuple(objects)
   

    manage_main=DTMLFile('dtml/manage_main', globals())
    
    security.declareProtected(EasyCorePermissions.EasyTemplateChange,'manage_toogleAsViews')
    def manage_toogleAsViews(self, ids=[], REQUEST=None):
        """Manage views through the web"""
        self.toogleViewObjects(view_ids=ids, set_view=1, set_edit=0)
        management_view='Contents'
        manage_tabs_message="Toogled %s as views." % ', '.join(ids)
        if REQUEST:
            return self.manage_main(self, REQUEST, manage_tabs_message=manage_tabs_message, management_view=management_view)
    
    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'manage_toogleAsEditViews')
    def manage_toogleAsEditViews(self, ids=[], REQUEST=None):
        """Manage edit views through the web"""
        self.toogleViewObjects(view_ids=ids, set_view=0, set_edit=1)
        management_view='Contents'
        manage_tabs_message="Toogled %s as edit views." % ', '.join(ids)
        if REQUEST:
            return self.manage_main(self, REQUEST, manage_tabs_message=manage_tabs_message, management_view=management_view)
    
    security.declarePublic('getDefaultEditViewId')
    def getDefaultEditViewId(self):
        """Returns the id of the first EditView found."""
        for view in self.getInheritedObjectValues(edit=1):
            if view.isEditable:
                return (callable(view.id) and view.id() or view.id)
        # there must be a better exception, or we need to create one
        raise AttributeError, "No valid Default Edit View"  
        
    security.declarePublic('hasView')
    def hasView(self,viewid):
        views = self.getInheritedViewIds()
        return viewid in views

    security.declarePublic('getView')
    def getView(self, viewid):
        view = getattr(self, viewid, None)
        if view is not None:
            return view
        raise AttributeError, "No View named '%s'" % viewid

    security.declarePublic('viewIds')
    def viewIds(self,spec=None,isEditable=None):
        return self.getInheritedViewIds(edit=isEditable)

    security.declarePublic('viewValues')
    def viewValues(self,spec=None, isEditable=None):
        """Deprecated   """
        return self.getInheritedObjectValues(edit=isEditable)
        

    security.declarePublic('viewMap')
    def viewMap(self,spec=None,isEditable=None):
        return map(lambda x: ( (callable(x.id) and x.id() or x.id), x.title + ' [' + (callable(x.id) and x.id() or x.id) + ']'), self.viewValues(spec=spec, isEditable=isEditable))


    #TEMPLATE INHERITANCE
    def getParentTemplates(self, templates=None):
        if templates is None: templates=[]
        templates.append(self)
        if not getattr(aq_base(self), 'isEditor', 0):
            parent=aq_parent(aq_inner(self))
            parent.getParentTemplates(templates=templates)
        return templates
        

    
Globals.InitializeClass(EasyViewContainer)