# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/EasyTemplates/EasyTemplatePlugins.py,v 1.3 2004/08/28 18:03:34 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__='''EasyTemplatePluginBase
$Id: EasyTemplatePlugins.py,v 1.3 2004/08/28 18:03:34 johanc Exp $'''
__version__='$Revision: 1.3 $'[11:-2]

import Globals
from Globals import HTML, DTMLFile
from ExtensionClass import Base
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_chain, aq_parent, Implicit, ImplicitAcquisitionWrapper
from OFS.SimpleItem import SimpleItem

from Products.EasyCore import MLOG_TAB

from Products.EasyCore import EasyCorePermissions

from zLOG import LOG, ERROR
from Products.EasyCore import MLOG_TAB


#try:
#    superGetAttr = Base.__getattr__
#except:
#    try:
#        superGetAttr = Base.inheritedAttribute('__getattr__')
#    except:
#        superGetAttr = None

class EasyTemplatePluginBase(SimpleItem):
    """A plugin class for EasyTemplates."""

    security = ClassSecurityInfo()

    meta_type = 'Easy Template Plugin Base'
    title = ''
    isTemplatePlugin = 1

    __exported_attributes__=()

    manage_options = (
        {'label': 'Refresh',
         'action': 'manage_main',
         },
        ) + SimpleItem.manage_options

    def __init__(self):
        pass

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        """Register Plugin AfterAdd"""
        try:
            container.registerPlugin(self)
        except:
            pass

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        """Remove Plugin Registration"""
        try:
            container.unregisterPlugin(self)
        except:
            pass

    def getAttributes(self):
        return self.__exported_attributes__

    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'manage_main')
    _manage_main=DTMLFile('dtml/manage_template_plugin', globals())
    def manage_main(self, REQUEST=None, refresh=None):
        """\
        Refresh the plug-in if refresh is set.
        """
        if refresh:
            aq_parent(self).unregisterPlugin(self)
            aq_parent(self).registerPlugin(self)

        if REQUEST:
            return self._manage_main(self, REQUEST=REQUEST)


Globals.InitializeClass(EasyTemplatePluginBase)

class MethodWrapper(Implicit):

    def __init__(self, method, mapping=None):
        self.__dict__['_method'] = method
        self.globals=mapping

    def __call__(self, *arg, **kw):
        return apply(self.__dict__['_method'], (self,) + arg, kw)

    #class MW(ExtensionClass.Base):
    #def __init__(self, meth): self.__dict__[methodattr]=meth

    def __of__(self, parent):
        #w_self = ImplicitAcquisitionWrapper(aq_base(self), parent)
        return ImplicitAcquisitionWrapper(aq_base(self), parent)

class TestPlugin(EasyTemplatePluginBase):

    meta_type = 'Test Plugin'
    title = 'Test Plugin'
    id='TestPlugin'

    __exported_attributes__=('testConstant', 'testMethod2Wrapped', 'testMethodWrapped')

    testConstant="This is a testMethod2Wrapped:Constant."

    def testMethod(self, value=5, REQUEST=None):
        """Callable Method"""
        result=7*value
        if REQUEST:
            return """<h1>TestPlugin:testMethod: %s</h1>""" % str(result)
        return """<h1>%s::%s: %s</h1>""" % (self.__class__.__name__, "testMethod", str(result))
    testMethodWrapped=MethodWrapper(testMethod, globals())


    def testMethod2(self, value=5, REQUEST=None):
        """Callable Method"""
        result=29*value
        if REQUEST:
            return """<h1>testMethod2: %s</h1>""" % str(result)
        return """<h1>%s::%s: %s</h1>""" % (self.__class__.__name__, "testMethod2", str(result))
    testMethod2Wrapped=MethodWrapper(testMethod2, globals())


def manage_add_TestPlugin(self,REQUEST=None):
    """Add a TestPlugin"""

    ob = TestPlugin()
    id=self._setObject(ob.id, ob)

    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        REQUEST.RESPONSE.redirect(u+'/manage_main')


class TestPlugin2(EasyTemplatePluginBase):

    meta_type = 'Test Plugin II'
    title = 'Test Plugin II'
    id='TestPluginII'

    __exported_attributes__=('test2Constant', 'testMethodWrapped', 'test2Method2Wrapped')

    test2Constant="This is a TestPlugin2::test2Constant."

    def testMethod(self, value=5, REQUEST=None):
        """Callable Method"""
        result=7*value
        if REQUEST:
            return """<h1>TestPlugin2::testMethod: %s</h1>""" % str(result)
        return """<h1>%s::%s: %s</h1>""" % (self.__class__.__name__, "testMethod", str(result))
    testMethodWrapped=MethodWrapper(testMethod, globals())


    def test2Method2(self, value=5, REQUEST=None):
        """Callable Method"""
        result=29*value
        if REQUEST:
            return """<h1>TestPlugin2::test2Method2: %s</h1>""" % str(result)
        return """<h1>%s::%s: %s</h1>""" % (self.__class__.__name__, "test2Method2", str(result))
    test2Method2Wrapped=MethodWrapper(test2Method2, globals())


def manage_add_TestPlugin2(self,REQUEST=None):
    """Add a TestPlugin2"""

    ob = TestPlugin2()
    id=self._setObject(ob.id, ob)

    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        REQUEST.RESPONSE.redirect(u+'/manage_main')


def initialize(context):
    """Initialize StyleManagers Product"""

    context.registerPluginClass(
            TestPlugin,
            meta_type = TestPlugin.meta_type,
            permission = EasyCorePermissions.EasyTemplateAdd,
            constructors = (manage_add_TestPlugin, manage_add_TestPlugin),
            icon='icons/TestPlugin.gif',
            category='Easy Templates',
        )

    context.registerPluginClass(
            TestPlugin2,
            meta_type = TestPlugin2.meta_type,
            permission = EasyCorePermissions.EasyTemplateAdd,
            constructors = (manage_add_TestPlugin2, manage_add_TestPlugin2),
            icon='icons/TestPlugin.gif',
            category='Easy Templates',
        )



