# coding: iso-8859-1
# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/EasyTemplates/EasyTemplates.py,v 1.22 2004/10/06 18:08:28 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyTemplates

$Colliberty: EasyTemplates.py,v 1.8 2001/11/18 12:00:00 johanc Exp $"""
__version__='$Revision: 1.22 $'[11:-2]


import Globals
from Globals import HTML, DTMLFile
from types import ListType, TupleType
from string import join, strip, replace
from urllib import quote
from copy import copy

from Acquisition import aq_base, aq_parent, aq_inner

from OFS.DTMLMethod import DTMLMethod
from OFS.Folder import Folder
from AccessControl import ClassSecurityInfo
from ZODB.PersistentMapping import PersistentMapping

from Products.EasyCore import EasyCorePermissions
from Products.EasyCore.Utils.HTMLFormHelpers import checkableTableRows
from Products.EasyCore.Utils.Javascript import escape_javascript_text, escape_javascript_id
from Products.EasyCore.Utils.Utils import spacify
from Products.EasyCore.UpgradeSupport.Utils import upgradeProperties
from Products.EasyCore.VirtualPath import VirtualPathBase

from Products.EasyCatalog.EasyCatalogAware import EasyCatalogAwareObjectAndPropertyManager, EasyCatalogAware
from Products.EasyPresentationHelpers.EasyParameterTracker import EasyParameterTracker
from Products.EasyEditor.EasyEditorClient.EasyEditorConstants import *
from Products.EasyEditor.Features.Features import getEasyFeatureButtonGroupsOrdered, getEasyFeatureDropDownGroupsOrdered
from Products.EasyEditor.EasyTemplates.EasyViews import EasyPlugins, EasyViews, EasyRegions, DTMLView
from Products.EasyEditor.EasyTemplates.EasyTemplateBase import EasyViewContainer

#from Products.OrderedFolder.OrderedFolder import OrderedFolder
from Products.OrderedFolder.SubObjectTypeSupport import SubObjectTypeSupport

from zLOG import LOG, ERROR
from Products.EasyCore import MLOG_TAB

# Check cache environment variable
import os
SUPPRESS_CACHE = os.environ.has_key('SUPPRESS_CACHE')


# superGetAttr is assigned to whatever ObjectManager.__getattr__
# used to do.
from OFS.ObjectManager import ObjectManager
try:
    superGetAttr = ObjectManager.__getattr__
except:
    try:
        superGetAttr = ObjectManager.inheritedAttribute('__getattr__')
    except:
        superGetAttr = None



class EasyTemplate(EasyViewContainer, EasyCatalogAwareObjectAndPropertyManager, EasyCatalogAware, 
                   Folder, EasyParameterTracker, VirtualPathBase):
    """Mixin class for easy template objects"""

    security = ClassSecurityInfo()

    meta_type = 'Easy Template'

    title = ''
    isTemplate = 1
    navigation_aware=0
    selected_meta_types = ('DTML Method','Page Template','File','Image','Folder','Script (Python)')

    show_template_types=False
    allow_template_types=()
    show_meta_types=False
    allow_meta_types=()
    use_easy_class='EasyDocument'
    disable_editor=0
    edit_option_name='HTML'
    edit_view_first=0
    hidden_attributes=[]
    hidden_meta_types=[]

    __static_exports__=['getRegionIds']

    skey_default='position'
    possible_obj_sort_columns=['id','meta_type', 'get_size', 'position']

    __catalogs__=('EasyEditor',)

    _properties = (
            {'id':'title', 'label':'Title', 'type':'string', 'mode':'w'},
            {'id':'navigation_aware', 'label':'NavigationAware', 'type':'boolean', 'mode':'w'},
            {'id':'skey_default', 'label':'Default Object Sorting', 'type':'selection', 'mode':'w', 'select_variable':'possible_obj_sort_columns'},
            {'id':'show_template_types', 'label':'Filter Template Types', 'type':'boolean', 'mode':'w', },
            {'id':'allow_template_types', 'label':'Allowed Template Types', 'type':'multiple selection', 'mode':'w', 'select_variable':'getAvailableTypes'},
            {'id':'use_easy_class', 'label':'Use Base Class', 'type':'selection', 'mode':'w', 'select_variable':'getEasyEditableMetaTypes'},
            {'id':'show_meta_types', 'label':'Filter Meta Types', 'type':'boolean', 'mode':'w', },
            {'id':'allow_meta_types', 'label':'Allowed Meta Types', 'type':'multiple selection', 'mode':'w', 'select_variable':'inherited_meta_types'},
            {'id':'disable_editor', 'label':'Disable Editor', 'type':'boolean', 'mode':'w'},
            {'id':'edit_option_name', 'label':'Edit Option Name', 'type':'string', 'mode':'w'},
            {'id':'edit_view_first', 'label':'Edit Option First', 'type':'boolean', 'mode':'w'},
            {'id':'hidden_attributes', 'label':'Hidden Attributes', 'type':'lines', 'mode':'w'},
            {'id':'hidden_meta_types', 'label':'Hidden Meta Types', 'type':'lines', 'mode':'w'},
        )


    manage_options_configurations =(
        {'label':'Styles', 'action':'manage_styles'},
        {'label':'Cache', 'action':'manage_invalidateCacheHTML'},
        {'label':'Show Exported', 'action':'manage_show_exported'},
    )

    manage_options = (Folder.manage_options[0],) + manage_options_configurations + \
        Folder.manage_options[1:] + SubObjectTypeSupport.manage_options + \
        EasyParameterTracker.manage_options


    def getIcon(self):
        try:
            if hasattr(self, 'custom_icon'):
                return self.custom_icon.absolute_url()
            return self.icon
        except:
            return self.icon


    def getMetaTypes(self):
        return self.allow_meta_types

    def getTemplateTypes(self):
        templates=[]
        for id in self.allow_template_types:
            templates.append(self.getTemplate(id))
        return templates

    security.declarePublic('inherited_meta_types')
    def inherited_meta_types(self):
        all_meta_types=EasyTemplate.inheritedAttribute('all_meta_types')
        if callable(all_meta_types): all_meta_types=all_meta_types(self)
        all_meta_types=map(lambda x: x['name'], all_meta_types)
        return all_meta_types

    security.declarePublic('all_meta_types')
    def all_meta_types(self):
        inherited_all_meta_types = EasyTemplate.inheritedAttribute('all_meta_types')
        easyplugins_meta_types = ()
        for views in EasyPlugins.values():
            easyplugins_meta_types = easyplugins_meta_types + tuple(views)
        easyviews_meta_types = ()
        for views in EasyViews.values():
            easyviews_meta_types = easyviews_meta_types + tuple(views)
        easyregions_meta_types = ()
        for regions in EasyRegions.values():
            easyregions_meta_types = easyregions_meta_types + tuple(regions)
        if callable(inherited_all_meta_types):
            all=inherited_all_meta_types(self)
        else:
            all=inherited_all_meta_types
        return easyplugins_meta_types+easyviews_meta_types + easyregions_meta_types + tuple(all)

    def categorized_meta_types(self):
        gmt=self.filtered_meta_types()
        groups={}
        TEMPLATE_RELATED=['DTML Method', 'Script (Python)', 'Page Template','Controller Page Template', 'Controller Python Script', 'Controller Validator']
        templates=()
        for mt in gmt:
            mt=mt.copy()
            category=mt.get('category', None)
            name=mt.get('name', None)
            if name:
                mt['name']=spacify(name)
            if category is None: category='Zope'
            if category=='Easy Templates' or name in TEMPLATE_RELATED:
                templates=templates+(mt,)
            else:
                group=groups.get(category,())
                group=group+(mt,)
                groups[category]=group
        return [{'group':'','items':templates, }, ]+map(lambda x: {'group':x[0],'items':x[1], }, groups.items())


    security.declareProtected(EasyCorePermissions.ViewManagementScreens,'easy_manage_tabs')
    def easy_manage_tabs(self,easy_manage_options, REQUEST=None):
        #if callable(self.manage_options): manage_options = self.manage_options()
        #else: manage_options = self.manage_options
        result = ''
        for option in easy_manage_options:
            option_tag = HTML("""
                <BUTTON
                    id="&dtml-action;"
                    class=menubutton
                    onClick="
                        <dtml-if "_.has_key('actionhandler')">
                            this.blur();
                            &dtml-actionhandler;
                            window.event.cancelBubble=true;
                        <dtml-else>
                            this.blur();
                            doSave('&dtml-action;','&dtml-label;',1);
                            window.event.cancelBubble=true;
                        </dtml-if>"
                    onMouseOver="
                        menumouseOverButton(this);
                        <dtml-if "_.has_key('actionhandler')">
                            glb_temp_window_status = window.status;
                            window.status='&dtml-label;';
                        <dtml-else>
                            glb_temp_window_status = window.status;
                            window.status=' &dtml-label;';
                        </dtml-if>"
                    "
                    onMouseOut="mouseOffButton(this);window.status=glb_temp_window_status;"
                    <dtml-if "_.has_key('accesskey')">
                    accesskey="&dtml-accesskey;"</dtml-if>
                    >&dtml-label;</BUTTON>
                """,globals())(self,option)
            result = result + option_tag + '\n'
        return result

    def __init__(self, id, title, navigation_aware=1):
        self.id = id
        self.title = title
        self.styles = []
        self._dirty_editor = 1
        self._cache_editor = None
        self.navigation_aware=navigation_aware


    def initcontent(self, ob, REQUEST=None):
        #DEPRECATED, I don't think so???
        for region in self.regionValues():
            region.initcontent(ob, REQUEST=REQUEST)

    security.declareProtected(EasyCorePermissions.EasyTemplateChange,'manage_editable')
    def manage_editable(self, document, editor, REQUEST=None):
        """Gets the document in a editable template"""
        context = aq_base(self).__of__(aq_parent(document))
        context = aq_base(document).__of__(context)
        return aq_base(self).__call__(client=context,REQUEST=REQUEST)

    def manage_editor(self, editor, document, document_url='', REQUEST=None):
        if REQUEST.has_key('debug') or not hasattr(self,'_dirty_editor') or not hasattr(self,'_v_cache_editor') or not hasattr(self,'_cache_editor_src'):
            self._dirty_editor = 1
            self._v_cache_editor = None
            self._cache_editor_src = None
        if self._dirty_editor or self._v_cache_editor is None or self._cache_editor_src is None or REQUEST.has_key('nocache') or SUPPRESS_CACHE==1:
            context = aq_parent(document)
            context = aq_base(editor).__of__(context)
            context = aq_base(self).__of__(context)
            context = aq_base(document).__of__(context)
            easy_manage_options = list(manage_commands) + document.filtered_manage_options()
            self._cache_editor_src = context.easyeditorclient(easy_manage_options=easy_manage_options, document_url=document_url, REQUEST=REQUEST)
            self._v_cache_editor = HTML(self._cache_editor_src, globals())
            self._dirty_editor = 0
        return self._v_cache_editor(self, REQUEST)

    def aq_template(self):
        return self

    def getTemplateId(self):
        #should return the relative URL from EasyEditor, but
        #first the whole API needs to be changed.
        return self.id

    #STYLES
    security.declareProtected(EasyCorePermissions.EasyTemplateChange,'manage_styles')
    manage_styles = DTMLFile('dtml/manage_styles',globals())

    security.declareProtected(EasyCorePermissions.EasyTemplateChange,'manage_edit_styles')
    def manage_edit_styles(self,styles=[],REQUEST=None):
        """\
        Manage seleced style definitions.
        """
        if type(styles)==type(''): styles = [styles]
        self.styles = styles
        self.setEditorDirty()
        if REQUEST:
            return self.manage_styles(self,REQUEST,manage_tabs_message="The styles selection was updated.",management_view='Styles')

    security.declarePublic('getDefaultStyle')
    def getDefaultStyle(self):
        return "p"

    security.declarePublic('hasStyleId')
    def hasStyleId(self,styleid):
        return styleid in self.styles

    security.declarePublic('styleIds')
    def styleIds(self,spec=None):
        return self.styles

    security.declarePublic('styleValues')
    def styleValues(self,spec=None):
        return self.styleValuesFor(self.styles)

    security.declarePublic('styleItems')
    def styleItems(self,spec=None):
        return self.styleItemsFor(self.styles)

    security.declarePublic('styleMap')
    def styleMap(self):
        return self.styleMapFor(self.styles)


    #REGIONS

    security.declarePublic('getRegionStorageIds')
    def getRegionStorageIds(self):
        return map(lambda region: region.storagecontainerid, self.objectValues(spec=EasyRegions.keys()))

    security.declarePublic('getRegionIds')
    def getRegionIds(self):
        return self.objectIds(spec=EasyRegions.keys())

    security.declarePrivate('checkRegionStorageId')
    def checkRegionStorageId(self, rsid):
        rsid = strip(rsid)
        if rsid in self.getRegionStorageIds():
            raise 'StorageKeyError', StorageKeyExists
        return rsid

    security.declarePublic('hasRegion')
    def hasRegion(self,regionid):
        regions = self.getRegionIds()
        return regionid in regions

    security.declarePublic('getRegion')
    def getRegion(self,regionid):
        region=getattr(self, regionid, None)
        if region: return region
        #Else look for a matching storageregion(???)
        LOG("DEPRECATED",INFO, "EasyTemplate:getRegion uses storagecontainerid to locate region. This is deprecated since ")
        region = filter(lambda x,rsid=regionid: x.storagecontainerid==rsid, self.regionValues())
        return region[0]

    security.declarePublic('regionIds')
    def regionIds(self, spec=None, ids=None, isSearchable=None):
        if spec is None: spec = EasyRegions.keys()
        if isSearchable:
            if type(ids) in (ListType, TupleType):
                return map(lambda x: x.id, filter(lambda region, ids=ids: region.isSearchable and region.id in ids, self.objectValues(spec=spec)))
            return map(lambda x: x.id, filter(lambda region: region.isSearchable, self.objectValues(spec=spec)))
        if type(ids) in (ListType, TupleType):
            return filter(lambda x, ids=ids: x in ids, self.objectIds(spec=spec))
        return self.objectIds(spec=spec)

    security.declarePublic('regionValues')
    def regionValues(self,spec=None, ids=None, isSearchable=None):
        if spec is None: spec = EasyRegions.keys()
        if isSearchable:
            if type(ids) in (ListType, TupleType):
                return filter(lambda region: region.isSearchable, self.objectValues(spec=spec))
            return filter(lambda region, ids=ids: region.isSearchable and region.id in ids, self.objectValues(spec=spec))
        if type(ids) in (ListType, TupleType):
            return filter(lambda region, ids=ids: region.id in ids, self.objectValues(spec=spec))
        return self.objectValues(spec=spec)

    security.declarePublic('regionItems')
    def regionItems(self,spec=None, ids=None, isSearchable=None):
        if spec is None: spec = EasyRegions.keys()
        if isSearchable:
            if type(ids) in (ListType, TupleType):
                return filter(lambda (key, region), ids=ids: region.isSearchable and id in ids, self.objectItems(spec=spec))
            return filter(lambda (key, region): region.isSearchable, self.objectItems(spec=spec))
        if type(ids) in (ListType, TupleType):
            return filter(lambda (key, region), ids=ids: key in ids, self.objectItems(spec=spec))
        return self.objectItems(spec=spec)

    security.declarePublic('regionMap')
    def regionMap(self):
        return map(lambda x: ((callable(x.id) and x.id() or x.id), x.title, x.storagecontainerid, x.isSearchable), self.regionValues())


    #STORAGE
    security.declarePrivate('manage_save')
    def manage_save(self, editable_region_map=[], meta_data_map=[], editablecontainer=None, REQUEST=None):
        if type(editable_region_map) not in (TupleType, ListType): editable_region_map=[editable_region_map]
        if type(meta_data_map) not in (TupleType, ListType): meta_data_map=[meta_data_map]
        if editablecontainer is not None and REQUEST is not None:
            for region_id in editable_region_map:
                if REQUEST.has_key(region_id) and REQUEST[region_id]!=NOT_INITIALIZED:
                    if self.hasRegion(region_id):
                        self.getRegion(region_id).manage_save(editablecontainer, REQUEST[region_id], REQUEST=REQUEST)
            for metadata_id in meta_data_map:
                if REQUEST.has_key(metadata_id):
                    setattr(editablecontainer, metadataId,REQUEST[metadata_id])
            editablecontainer.reindex_object()

    security.declarePrivate('manage_load')
    def manage_load(self, name, editablecontainer, default=None, REQUEST=None):
        if hasattr(aq_base(self), name):
            region = getattr(aq_base(self), name)
            return region.manage_load(editablecontainer, default=default, REQUEST=REQUEST)
        return default


        

    #CACHE MANAGEMENT

    security.declarePublic('setEditorDirty')
    def setEditorDirty(self):
        self._dirty_editor = 1

    security.declarePublic('isEditorDirty')
    def isEditorDirty(self):
        return self._dirty_editor

    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'manage_invalidateCacheHTML')
    manage_invalidateCacheHTML = DTMLFile('dtml/manage_invalidateCacheHTML',globals())

    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'manage_invalidateCache')
    def manage_invalidateCache(self, client=None, REQUEST=None):
        """Reset the cache so the Editor cache of this template rerenders"""
        self.setEditorDirty()
        if REQUEST is not None:
            manage_tabs_message="The Editor cache has been invalidated"
            management_view="Cache"
            return self.manage_invalidateCacheHTML(management_view=management_view, manage_tabs_message=manage_tabs_message, client=client, REQUEST=REQUEST)


    # RENDER JS COMPONENTS

    security.declarePublic('renderjsInitRegions')
    def renderjsInitRegions(self, client=None, REQUEST=None):
        result = ''
        for region in self.regionValues():
            default=escape_javascript_text(region.getDefault())
            result = result + renderjsInitRegionsTemplate % { 'regionid': region.id, 'highlight_bgcolor': region.highlight_bgcolor or '#FFFFCC', 'default': default, 'not_initialized': NOT_INITIALIZED}
        return result

    security.declarePublic('renderRegion')
    def renderRegion(self, regionid, REQUEST, style=''):
        if self.hasRegion(regionid):
            region = self.getRegion(regionid)
            renderRegion = region.manage_load(aq_base(REQUEST['here']), default="##ERROR", REQUEST=REQUEST)
            return """<div id="%(regionid)s" style="%(style)s">%(region)s</div>""" % {'regionid':regionid,'region':renderRegion,'style':style}
        else:
            return '<div id="%(regionid)s" style="font:bold 10px verdana;">Region %(regionid)s not installed</div>"' % {'regionid':regionid }

    security.declarePublic('renderStylesSelectionList')
    def renderStylesSelectionList(self,REQUEST=None):
        return checkableTableRows(name='styles',activekeys = self.styles, items=self.aq_parent.styleMap(),REQUEST=REQUEST)

    security.declarePublic('renderjsStyles')
    def renderjsStyles(self,client=None,REQUEST=None):
        return ''

    security.declarePublic('renderjsRegionFormTags')
    def renderjsRegionFormTags(self, charset_tag='UTF-8', REQUEST=None):
        REQUEST=getattr(self, "REQUEST", REQUEST)
        result = ''
        renderjsRegionFormTagsTemplate = """\
          <INPUT type="hidden" name="editable_region_map:list" value="%(id)s" >
          <INPUT id="region_%(id)s" type="hidden" name="%(id)s:%(charset_tag)s%(type)s" value="%(not_initialized)s">
        """
        for region in self.regionValues():
            value = region.manage_load(aq_base(REQUEST['here']),default=NOT_INITIALIZED, REQUEST=REQUEST)
            result = result + renderjsRegionFormTagsTemplate % {'id':region.id, 'not_initialized': value, 'type': region.getType(), 'charset_tag':charset_tag}
        return result

    security.declarePublic('renderjsRegionCopyToFields')
    def renderjsRegionCopyToFields(self, client=None, REQUEST=None):
        result = ''
        for regionid in self.getRegionIds():
            result = result + renderjsRegionCopyToFieldsTemplate % {'regionid': regionid}
        return result

    security.declarePublic('renderjsButton')
    def renderjsButton(self, client=None, REQUEST=None):
        button_groups = []
        sep = """<img src="/%s/button_seperator.gif" style="width:4;height:20;border:0;" align=absbottom>""" % (self.misc,)
        for button_group in getEasyFeatureButtonGroupsOrdered():
            sub_result = ""
            for feature in button_group.getButtonsOrdered():
                render = feature.__of__(self).renderjsButton(client,REQUEST)
                if render: sub_result = sub_result + render
            if sub_result: button_groups.append(sub_result)
        return join(button_groups,sep)

    security.declarePublic('renderjsButtonHandler')
    def renderjsButtonHandler(self,client=None,REQUEST=None):
        result = ""
        for feature in self.featureValues(spec=[UI_BUTTON]):
            if feature.renderjsButtonHandler:
                render = feature.__of__(self).renderjsButtonHandler(client,REQUEST)
                if render:
                    result = result + render
        return result

    security.declarePublic('renderjsDropDown')
    def renderjsDropDown(self,client=None,REQUEST=None):
        if REQUEST is None: REQUEST=self.REQUEST
        dropdown_groups = []
        sep = """<img src="/%s/button_seperator.gif" style="width:4;height:20;border:0;" align=absbottom>""" % (self.misc,)
        # "document" is a workaround for "Parent of Editor bug, 2002-11-14"
        #THIS IS SUPPOSED TO SOLVE SOME BUG??? document=REQUEST.get('here', None)
        for dropdown_group in getEasyFeatureDropDownGroupsOrdered():
            sub_result = ""
            for feature in dropdown_group.getDropDownsOrdered():
                render = feature.__of__(self).renderjsDropDown(client,REQUEST)
                                    #GOES WITH THE BUG ABOVE: (document)
                if render: sub_result = sub_result + render
            if sub_result: dropdown_groups.append(sub_result)
        return join(dropdown_groups,sep)

    security.declarePublic('renderjsDropDownHandler')
    def renderjsDropDownHandler(self,client=None,REQUEST=None):
        result = ""
        for feature in self.featureValues(spec=[UI_SELECT]):
            if hasattr(feature,'renderjsDropDownHandler') and feature.renderjsDropDownHandler:
                render = feature.__of__(self).renderjsDropDownHandler(client,REQUEST)
                if render:
                    result = result + render
        return result

    security.declarePublic('renderjsStyleSelector')
    def renderjsStyleSelector(self,client=None,REQUEST=None):
        result = ""
        for feature in self.featureValues(spec=[UI_SELECT]):
            if hasattr(feature,'renderjsStyleSelector') and feature.renderjsStyleSelector:
                render = feature.__of__(self).renderjsStyleSelector(client,REQUEST)
                if render:
                    result = result + render
        return result

    security.declarePublic('renderjsStyleSelectorHandler')
    def renderjsStyleSelectorHandler(self,client=None,REQUEST=None):
        result = ""
        for feature in self.featureValues(spec=[UI_SELECT]):
            if hasattr(feature,'renderjsStyleSelectorHandler') and feature.renderjsStyleSelectorHandler:
                render = feature.__of__(self).renderjsStyleSelectorHandler(client,REQUEST)
                if render:
                    result = result + render
        return result

    security.declarePublic('renderjsMenuButton')
    def renderjsMenuButton(self,client=None,REQUEST=None):
        result = ""
        for feature in self.renderjsMenuButton(spec=[UI_SELECT]):
            if hasattr(feature,'renderjsMenuButton') and feature.renderjsMenuButton:
                render = feature.__of__(self).renderjsMenuButton(client,REQUEST)
                if render:
                    result = result + render
        return result

    security.declarePublic('renderjsMenuButtonHandler')
    def renderjsMenuButtonHandler(self,client=None,REQUEST=None):
        result = ""
        for feature in self.featureValues(spec=[UI_SELECT]):
            if hasattr(feature,'renderjsMenuButtonHandler') and feature.renderjsMenuButtonHandler:
                render = feature.__of__(self).renderjsMenuButtonHandler(client,REQUEST)
                if render:
                    result = result + render
        return result

    security.declarePublic('renderjsEventHandler')
    def renderjsEventHandler(self,client=None,REQUEST=None):
        result = ""
        for feature in self.featureValues():
            if hasattr(feature,'renderjsEventHandler') and feature.renderjsEventHandler:
                render = feature.__of__(self).renderjsEventHandler(client,REQUEST)
                if render:
                    result = result + render
        return result

    security.declarePublic('renderjsHelperMethod')
    def renderjsHelperMethod(self,client=None,REQUEST=None):
        result = ""
        for feature in self.featureValues():
            if hasattr(feature,'renderjsHelperMethod') and feature.renderjsHelperMethod:
                render = feature.__of__(self).renderjsHelperMethod(client,REQUEST)
                if render:
                    result = result + render
        return result

    security.declarePublic('renderjsUpdateApperenceCall')
    def renderjsUpdateApperenceCall(self, client=None, REQUEST=None):
        result = ""
        for feature in self.featureValues(spec=[UI_BUTTON,UI_SELECT]):
            if hasattr(feature,'renderjsUpdateApperenceCall') and feature.renderjsUpdateApperenceCall:
                render = feature.__of__(self).renderjsUpdateApperenceCall(client,REQUEST)
                if render:
                    result = result + render
            elif hasattr(feature,'renderjsUpdateApperenceCall') and feature.renderjsUpdateApperenceCall:
                result = result + """    %s("document.getElementById('%s')",%s,%s);\n""" %  \
                   (feature.updateAppearenceHandler,
                    feature.elementid or "''",
                    feature.command or "''",
                    feature.usesLatched)
        return result

    security.declarePublic('renderjsUpdateApperenceHandler')
    def renderjsUpdateApperenceHandler(self,client=None, REQUEST=None):
        result = ""
        for feature in self.featureValues(spec=[UI_BUTTON,UI_SELECT]):
            if hasattr(feature,'renderjsUpdateApperenceHandler') and feature.renderjsUpdateApperenceHandler:
                render = feature.__of__(self).renderjsUpdateApperenceHandler(client,REQUEST)
                if render:
                    result = result + render
        return result

    security.declarePublic('renderjsDataIslandsDTML')
    def renderjsDataIslandsDTML(self,client=None, REQUEST=None):
        return """<dtml-var "renderjsDataIslands(_.None,_)">"""

    security.declarePublic('renderjsDataIslands')
    def renderjsDataIslands(self,client=None, REQUEST=None):
        result = ""
        for feature in self.featureValues():
            if hasattr(feature,'renderjsDataIslands') and feature.renderjsDataIslands:
                render = feature.__of__(self).renderjsDataIslands(client,REQUEST)
                if render:
                    result = result + render
        return result

    security.declarePublic('renderSelectionTemplates')
    def renderSelectionTemplates(self,client=None, REQUEST=None):
        return ""

    security.declarePublic('renderjsConstantsDTML')
    def renderjsConstantsDTML(self,client=None, REQUEST=None):
        return """<dtml-var "renderjsConstants(_.None,_)">"""

    security.declarePublic('renderjsConstants')
    def renderjsConstants(self,client=None, REQUEST=None):
        result = ""
        for feature in self.featureValues():
            if hasattr(feature,'renderjsConstants') and feature.renderjsConstants:
                render = feature.__of__(self).renderjsConstants(client,REQUEST)
                if render:
                    result = result + render
        return result

    security.declarePublic('renderTitleDTML')
    def renderTitleDTML(self,client=None, REQUEST=None):
        return """<dtml-var "here.title_and_id()" missing="No Title or ID">"""

    security.declarePublic('renderjsInitEditorDTML')
    def renderjsInitEditorDTML(self,client=None, REQUEST=None):
        return """<dtml-var "renderjsInitEditor(_.None,_)">"""

    security.declarePublic('renderURLConstantsDTML')
    def renderURLConstantsDTML(self,client=None, REQUEST=None):
        result = """
                QUERY_STRING = "<dtml-var QUERY_STRING>";
                EDITOR_URL = "<dtml-var URL1>";
                EDITOR_PATH = "<dtml-var URLPATH1>";
                DOCUMENT_URL = "<dtml-var URL1>";
                DOCUMENT_PATH = "<dtml-var URLPATH1>";
                DOCUMENT_VIEW_URL = "<dtml-var URL1>";
                DOCUMENT_VIEW_PATH = "<dtml-var URLPATH1>";
                DOCUMENT_PATH_LENGTH = <dtml-var "_.len(URLPATH1)">;
                HOST_PART = "<dtml-var "BASE0">";
                HOST_PART_LENGTH = <dtml-var "_.len(BASE0)">;
                ROOT_PATH = "<dtml-var BASEPATH0>";
                EASY_NAVIGATOR_URL = "<dtml-var "EasyNavigation.virtualURL">";
                <dtml-if "_.has_key('EasyResources')">EASY_RESOURCES_URL = "<dtml-var "EasyResources.virtualURL">";</dtml-if>
            """
        return result

    security.declarePublic('renderEditorIFrameDTML')
    def renderEditorIFrameDTML(self,client=None, REQUEST=None):
        result = """<iframe
                id="editor"
                src="<dtml-var URL1>/manage_editable"
                style="
                    position:absolute;
                    width:100%;
                    z-index:2;
                    visibility:hidden;
                "
                onLoad="initContent();"
            ></iframe>
        """
        return result

    security.declarePublic('renderjsInitEditor')
    def renderjsInitEditor(self,client=None, REQUEST=None):
        result = ""
        for feature in self.featureValues():
            if hasattr(feature,'renderjsInitEditor') and feature.renderjsInitEditor:
                render = feature.__of__(self).renderjsInitEditor(client,REQUEST)
                if render:
                    result = result + render
        return result


    #Plugin Support
    ##ToDo: add sorting mechanism/policy for plugin attributes that overlap.
    __plugin_attributes__=None

    def registerPlugin(self, plugin):
        if self.__plugin_attributes__ is None:
            self.__plugin_attributes__ = PersistentMapping()

        p_id=plugin.getId()
        Missing=[]
        for attr in plugin.getAttributes():
            plugins=self.__plugin_attributes__.get(attr, Missing)
            if plugins is Missing:
                self.__plugin_attributes__[attr] = [p_id]
            elif p_id not in plugins:
                plugins.insert(0, p_id)
                self.__plugin_attributes__[attr] = plugins
            else:
                plugins.remove(p_id)
                plugins.insert(0, p_id)
                self.__plugin_attributes__[attr] = plugins
            #No dublets
        self.index_object()

    def unregisterPlugin(self, plugin):
        if self.__plugin_attributes__ is None:
            self.__plugin_attributes__ = PersistentMapping()
        p_id=plugin.getId()
        for attr in plugin.getAttributes():
            plugins=self.__plugin_attributes__.get(attr, None)
            if plugins and p_id in plugins:
                plugins.remove(p_id)
                self.__plugin_attributes__[attr] = plugins
        self.index_object()

    def __getattr__(self, name):
        if self.__plugin_attributes__ and len(self.__plugin_attributes__) and self.__plugin_attributes__.has_key(name):
            plugins=self.__plugin_attributes__[name]
            if plugins:
                obj = self.__dict__.get(plugins[0])
                return aq_base(getattr(obj, name))
        if superGetAttr is None:
            raise AttributeError, name
        return superGetAttr(self, name)

    security.declarePublic('getPluginIds')
    def getPluginIds(self):
        return self.__plugin_attributes__.keys()

    security.declarePublic('exported_attributes')
    def exported_attributes(self):
        ids=self.filtered_object_ids() + self.filtered_property_ids()+self.__static_exports__
        if self.__plugin_attributes__:
            ids = ids + self.__plugin_attributes__.keys()
        ids.sort()
        return ids

    security.declarePrivate('filtered_object_ids')
    def filtered_object_ids(self):
        """\
        Returned object ids for all objects but Regions
        and objects in hidden_attributes
        """
        spec=list(self.hidden_meta_types)
        spec.append('EditableRegion')

        set=[]
        for ob in self._objects:
            if ob['meta_type'] not in spec and ob['id'] not in self.hidden_attributes:
                set.append(ob['id'])
        return set

    security.declarePrivate('filtered_property_ids')
    def filtered_property_ids(self):
        """\
        Returned object ids for all objects but names
        defined in the property hidden_attributes.
        """
        set=[]
        for prop in self._properties:
            if prop['id'] not in self.hidden_attributes:
                set.append(prop['id'])
        return set

    def getStaticExports(self):
        return self.__static_exports__

    manage_show_exported = DTMLFile('dtml/manage_show_exported', globals())


    # webDAV support

    def MKCOL_handler(self,id,REQUEST=None,RESPONSE=None):
        """
            Handle WebDAV MKCOL.
        """
        #self.addNewTemplate(id, REQUEST=REQUEST,RESPONSE=RESPONSE)

    def PUT_factory( self, name, typ, body ):
        """
            Dispatcher for PUT requests to non-existent IDs.  Returns
            an object of the appropriate type (or None, if we don't
            know what to do).
        """
        try:
            if name and name.endswith('.pt'):
                from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate
                ob = ZopePageTemplate(name, body, content_type=typ)
            elif typ in ('text/html', 'text/xml', 'text/plain'):
                from OFS.DTMLMethod import DTMLMethod
                ob = DTMLMethod( '', __name__=name )
            elif typ[:6]=='image/':
                from Products.EasyImage.EasyImage import EasyImage
                ob=EasyImage(name, body, '', content_type=typ)
            else:
                from Products.EasyFile.EasyFile import EasyFile
                ob = EasyFile(name, body, '', content_type=typ)
        except:
            import sys
            exc = sys.exc_info()
            LOG('EasyEditable', ERROR, "Can't create sub object [%s]. %s (%s)" % (typ,exc[0],exc[1]), error=exc)
            return None
        return ob

    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0, force=0):
        """ """
        if type(getattr(aq_base(self), '__ac_local_roles__'))==type({}):
            del self.__ac_local_roles__
            changed=1

        upgradeProperties(self, {'navigation_aware':1,
                                 'add_to_menus':'',
                                 'title':'',
                                 'skey_default':'position',
                                 'show_template_types':False,
                                 'allow_template_types':(),
                                 'show_meta_types':False,
                                 'allow_meta_types':(),
                                 'use_easy_class':'EasyDocument',
                                 'disable_editor':0,
                                 'edit_option_name':'HTML',
                                 'edit_view_first':0,
                                 'hidden_attributes':[],
                                 'hidden_meta_types':[],
                                }, ('_is_transparent',))

        if REQUEST is not None:
            return "<h1>Done</h1>"
        return changed

Globals.InitializeClass(EasyTemplate)


manage_commands =  \
        (
            {
                'label': 'Save',
                'action': 'manage_editor',
                'actionhandler': "doSave('manage_editor','Editor');",
                'accesskey': 's',
            },
        )


temp_styles = (
                ('heading1',('H1',)),
                ('bodytext',('P','PRE','LI')),
                ('ingress',('P','PRE'))
              )


NOT_INITIALIZED = 'NOT_INITIALIZED'
renderjsInitRegionsTemplate = """
initRegion('%(regionid)s', '%(highlight_bgcolor)s', '%(default)s', '%(not_initialized)s');
"""


renderjsRegionCopyToFieldsTemplate = """\
    if (edited_document.getElementById('%(regionid)s').tagName=='div') {
        getElementById('%(regionid)s').value=edited_document.getElementById('%(regionid)s').innerHTML;
    }
"""

renderjsStylesAddTemplate = """\
    cssstyles["%(styleid)s"] = new Array(%(stylearray)s);
"""


manage_add_EasyTemplateForm = DTMLFile('dtml/manage_add_EasyTemplateForm', globals())

def manage_add_EasyTemplate(self,id, title='', submit=None, navigation_aware=0, REQUEST=None):
    """Add a Default Template"""
    ob = EasyTemplate(id=id,title=title, navigation_aware=navigation_aware)
    id=self._setObject(id, ob)
    template = self._getOb(id)


    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')



def initialize(context):
    """Initialize StyleManagers Product"""

    context.registerEasyTemplateClass(
            EasyTemplate,
            meta_type = EasyTemplate.meta_type,
            permission = EasyCorePermissions.EasyTemplateAdd,
            constructors = (manage_add_EasyTemplateForm, manage_add_EasyTemplate),
            icon='icons/EasyTemplate.gif',
            category='Easy Templates',
        )


class DTMLView(DTMLView):
    """The old DTMLView Object. This recreates the objects with the new DTMLView objects."""
    pass



