# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/EditableRegions/EditableRegions.py,v 1.9 2004/09/14 12:17:07 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__ = """EasyTemplates Editable Regions"""
__version__='$Revision: 1.9 $'[11:-2]



import Globals, sys
from Globals import DTMLFile
from OFS.SimpleItem import SimpleItem
from urllib import quote
from Acquisition import aq_base, aq_parent, aq_chain, ImplicitAcquisitionWrapper
from Products.EasyEditor.EasyEditorRegistry import EasyStorage, EasyRegionTypes
from AccessControl import ClassSecurityInfo
from Products.EasyCore import EasyCorePermissions
from string import strip
import DateTime

from OFS.Cache import Cacheable
from webdav.Lockable import ResourceLockedError
from webdav.WriteLockInterface import WriteLockInterface
from OFS.History import html_diff, Historical
from Products.EasyCore.PropertyTextParser import PropertyTextParser, PropertyTextGenerator
import sys, traceback
from types import DictType

try:
    from mx.Tidy import *
except:
    tidy=None

from Products.EasyCore import MLOG_TAB

_marker=[]


class EditableRegion(SimpleItem, Cacheable, Historical):
    """A EditableRegion"""

    security = ClassSecurityInfo()

    __region_kind__ = 'EditableRegion'

    meta_type = "EditableRegion"
    _v_errors=''

    isPrincipiaFolderish = 1
    isSearchable = 0
    acquisition_blocks_on_creation=0
    content_type="text/plain"
    instuction=''
    show_instuction=0
    label=None
    use_tidy=0
    #region_type=EasyStringType()

    #Tidy Configuration
    tidy_options={
        'add_xml_decl':0,
        'add_xml_space':0,
        'assume_xml_procins':0,
        'break_before_br':0,
        'clean':0,
        'drop_empty_paras':1,
        'drop_font_tags':0,
        'enclose_block_text':0,
        'fix_backslash':0,
        'fix_bad_comments':0,
        'gnu_emacs':0,
        'hide_endtags':0,
        'indent_attributes':0,
        'input_xml':0,
        'fix_backslash':1,

        'fix_backslash':1,
        'fix_bad_comments':1,
        'gnu_emacs':0,
        'hide_endtags':0,
        'indent_attributes':0,
        'input_xml':0,
        'literal_attributes':0,
        'logical_emphasis':0,
        'numeric_entities':0,
        'output_error':1,
        'output_markup':1,
        'output_xhtml':0,
        'output_xml':0,
        'quiet':0,
        'quote_ampersand':1,
        'quote_marks':0,
        'quote_nbsp':1,
        'raw':0,
        'show_warnings':0,
        'tidy_mark':0,
        'uppercase_attributes':0,
        'uppercase_tags':0,
        'word_2000':0,
        'wrap_asp':1,
        'wrap_attributes':0,
        'wrap_jste':1,
        'wrap_php':1,
        'wrap_script_literals':0,
        'wrap_sections':1,
        'indent_spaces':2,
        'tab_size':4,
        'wrap':72,
        'alt_text':'',
        'indent':'no', #"yes", "no" or "auto"
        'char_encoding':'ascii',
    }

    __implements__ = (WriteLockInterface,)

    _hidden_properties = (
                            {   'id':'storagecontainerid',
                                'label':'StorageContainerID',
                                'type': 'string',
                            },
                            {   'id':'label',
                                'label':'Label',
                                'type': 'string',
                            },
                            {   'id':'storage',
                                'label':'Storage',
                                'type': 'string',
                            },
                            {   'id':'default',
                                'label':'Default',
                                'multiline':1,
                                'type': 'text',
                            },
                            {   'id':'isSearchable',
                                'label':'Is Searchable',
                                'type': 'boolean',
                            },
                            {   'id':'bgcolor',
                                'label':'Background Color',
                                'type': 'string',
                            },
                            {   'id':'highlight_bgcolor',
                                'label':'Highlight Color',
                                'type': 'string',
                            },
                            {   'id':'acquisition_blocks_on_creation',
                                'label':'Acquisition Blocks on Creation',
                                'type': 'boolean',
                            },
                            {   'id':'instuction',
                                'multiline':1,
                                'label':'Instruction/Description',
                                'type': 'text',
                            },
                        )


    regions_manage_options = (
                                {'label':'Edit',   'action':'manage_main'},
                                {'label':'HTML Tidy',   'action':'manage_tidy'},
                             ) + SimpleItem.manage_options

    def __init__(self, id, storage,  default, isSearchable, bgcolor, highlight_bgcolor, storagecontainerid=None, acquisition_blocks_on_creation=0):
        self.id = id
        self.label=id
        if storagecontainerid is None:
            self.storagecontainerid = id
        else:
            self.storagecontainerid = storagecontainerid
        #self.region_type = region_type ##region_type,
        self.storage = storage
        self.setDefault(default)
        self.isSearchable = isSearchable
        self.bgcolor = bgcolor
        self.highlight_bgcolor = highlight_bgcolor
        self.acquisition_blocks_on_creation=acquisition_blocks_on_creation
        self._last_changed=DateTime.now()

    def initcontent(self, editablecontainer, REQUEST=None):
        self.getStorage().manage_save(editablecontainer, self, REQUEST=REQUEST)

    def manage_options(self):
        return self.regions_manage_options


    #MANAGE STORAGE


    def getStorage(self):
        return ImplicitAcquisitionWrapper(EasyStorage[self.storage], self)

    def getStorageContainerId(self):
        return self.storagecontainerid

    def lastChanged(self):
        return self._last_changed

    def setDirty(self):
        self._last_changed=DateTime.now()
        if hasattr(self,'setEditorDirty') and callable(self.setEditorDirty):
            self.setEditorDirty()

    def hasChanged(self, editablecontainer):
        return self.getStorage().hasChanged(editablecontainer, self)

    # MANAGE EDITABLE REGION

    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'manage_main')
    manage_main = DTMLFile('dtml/manage_main_EditableRegion', globals())


    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'manage_edit')
    def manage_edit(self
                    , label, storage, bgcolor, highlight_bgcolor
                    , show_instuction=0, instuction='', isSearchable=0
                    , storagecontainerid=None, acquisition_blocks_on_creation=0
                    , REQUEST=None
                    ):
        """\
        Change the storage for this Editable Region
        """
        self.label=label
        self.show_instuction=show_instuction
        self.instuction=instuction
        #if region_type:
        #    self.region_type = region_type
        if storage:
            self.storage = storage
        if storagecontainerid and storagecontainerid!=self.storagecontainerid:
            storagecontainerid = self.checkRegionStorageId(storagecontainerid)
            self.storagecontainerid = storagecontainerid
        elif storagecontainerid=='' and self.storagecontainerid!=self.id:
            try:
                storagecontainerid = self.checkRegionStorageId(self.id)
                self.storagecontainerid = storagecontainerid
            except:
                return self.manage_main(self,REQUEST,management_view='Edit',management_message="ERROR")
        self.bgcolor = bgcolor
        self.highlight_bgcolor = highlight_bgcolor
        self.isSearchable=isSearchable
        self.acquisition_blocks_on_creation=acquisition_blocks_on_creation
        self.manage_save_storage_conf(REQUEST=REQUEST)
        self.manage_save_default_conf(REQUEST=REQUEST)
        self.setDirty()
        if REQUEST is not None:
            return self.manage_main(self,REQUEST,management_view='Edit')

    _manage_tidy = DTMLFile('dtml/manage_tidy_EditableRegion', globals())
    save_changes="Save Changes"
    restore_defaults="Restore Defaults"

    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'manage_tidy')
    def manage_tidy(self, use_tidy=None, tidy_options=None, action=None, REQUEST=None):
        """Save Tidy configuration."""

        if action==self.restore_defaults:
            self.tidy_options=self.__class__.tidy_options

        elif action==self.save_changes:
            if tidy_options:
                self.tidy_options.update(tidy_options)

        if use_tidy is not None:
            self.use_tidy =use_tidy

        if REQUEST is not None:
            return self._manage_tidy(self,REQUEST=REQUEST,management_view='HTML Tidy',**self.tidy_options)

    security.declarePrivate('manage_initilized')
    def manage_initilized(self, editablecontainer, REQUEST=None):
        self.getStorage().manage_initilized(editablecontainer, self, REQUEST=REQUEST)

    security.declarePrivate('manage_load')
    def manage_load(self, editablecontainer, default=_marker, REQUEST=None):
        if default is _marker:
            default = self.getDefault()
        data=self.getStorage().manage_load(editablecontainer, self, default, REQUEST=REQUEST)
        return data

    security.declarePrivate('manage_save')
    def manage_save(self, editablecontainer, value=None, REQUEST=None):
        if self.use_tidy and tidy:
            (nerrors, nwarnings, value, errordata)=tidy(value, **self.tidy_options)
            body_begin=value.find('<body')
            if body_begin:
                text_begin=value.find('>', body_begin)
                if text_begin:
                    text_end=value.find('</body', text_begin)
                    if text_end:
                        value=value[text_begin+1:text_end]
                    else:
                        value=value[text_begin+1:]
        self.getStorage().manage_save(editablecontainer, self, value, REQUEST=REQUEST)

    security.declarePrivate('manage_compare')
    def manage_compare(self, editablecontainer, obj, REQUEST=None):
        return self.getStorage().manage_compare(editablecontainer, self, obj, REQUEST=REQUEST)

    security.declarePrivate('manage_copy')
    def manage_copy(self, editablecontainer, parent, REQUEST=None):
        self.getStorage().manage_copy(editablecontainer, self, parent, REQUEST=REQUEST)

    security.declarePrivate('manage_save_storage_conf')
    def manage_save_storage_conf(self, REQUEST=None):
        storage=self.getStorage()
        self.setDirty()
        return storage.manage_save_storage_conf(self, REQUEST=REQUEST)

    security.declarePrivate('manage_view_storage_conf')
    def manage_view_storage_conf(self, REQUEST=None):
        storage=self.getStorage()
        return storage.manage_view_storage_conf(self, REQUEST=REQUEST)

    security.declarePrivate('manage_view_default_conf')
    def manage_view_default_conf(self, REQUEST=None):
        storage=self.getStorage()
        self.setDirty()
        return storage.manage_view_default_conf(self, REQUEST=REQUEST)

    security.declarePrivate('manage_save_default_conf')
    def manage_save_default_conf(self, REQUEST=None):
        storage=self.getStorage()
        return storage.manage_save_default_conf(self, REQUEST=REQUEST)

    security.declareProtected(EasyCorePermissions.AccessContentsInformation, 'getDefault')
    def getDefault(self):
        storage=self.getStorage()
        return storage.getDefault(self)

    security.declarePrivate('setDefault')
    def setDefault(self, value):
        storage=self.getStorage()
        return storage.setDefault(self, value)

    security.declareProtected(EasyCorePermissions.AccessContentsInformation, 'getConfProperty')
    def getConfProperty(self, id, default=None):
        """\
        Returns a property (attribute) that may or may not
        be manage by the storage. Because the properties
        used by the storage to configure a region is dynamic
        this falls back on:
          1. default value in storage,
          2. default value supplied by called (or None).
        """

        value=getattr(self, id, _marker)
        if value is _marker:
            storage=self.getStorage()
            value=storage.getConfProperty(value, default)
        return value

    def getTemplate(self):
        """\
        Returns the regions template, what that might meen in the future
        isn't really clear. It could be the containing template (as implemented
        here) or a template futher down the acquisition path.
        """
        return aq_parent(self)

    #?##Public Interface
    #?#security.declareProtected(EasyCorePermissions.AccessContentsInformation, 'getValue')
    #?#def getValue(self, editablecontainer, default=_marker):
    #?#    return self.manage_load(editablecontainer, default, REQUEST=REQUEST) #No REQUEST

    #?#security.declareProtected(EasyCorePermissions.AccessContentsInformation, 'setValue')
    #?#def setValue(self, editablecontainer, value=None):
    #?#    self.manage_save(editablecontainer, default, REQUEST=REQUEST) #No Request


    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        EditableRegion.inheritedAttribute('manage_afterAdd')(self, item, container)
        self.setDirty()


    security.declarePrivate('render_html_view')
    def render_html_view(self, editablecontainer, **kw):
        storage=self.getStorage()
        if hasattr(aq_base(storage), 'render_html_view'):
            return apply(storage.render_html_view, (editablecontainer, self), kw)
        return '<p>No Storage HTML View</p>'

    security.declarePrivate('do_html_view')
    def do_html_view(self, editablecontainer, **kw):
        storage=self.getStorage()
        if hasattr(aq_base(storage), 'do_html_view'):
            return apply(storage.do_html_view, (editablecontainer, self), kw)
        return '<p>No Storage HTML View</p>'



    security.declarePrivate('getStorageType')
    def getStorageType(self):
        storage=self.getStorage()
        return storage.getStorageType()


    def read(self):
        error=''
        try:
            generator = PropertyTextGenerator()
            generator.renderFromInstance(self, self._hidden_properties)
            if self._v_errors:
                return "%s\n%s" %(self._v_errors, generator.getCode())
            return generator.getCode()
        except:
            try:
                error="# ERROR: %s: %s\n# \n" % sys.exc_info()[:2]
                generator_errors='\n'.join(map(lambda err: "# " + str(err), generator.getErrors()))
                if self._v_errors:
                    return error + "%s\n%s" % (self._v_errors, generator_errors)
                return local_errors
            except:
                pass
        return error + self._v_errors

    def munge(self, body):
        self._v_errors = ''
        try:
            parser = PropertyTextParser()
            parser.parseToInstance(self, self._hidden_properties, body)
        except:
            error="# ERROR: %s: %s\n# \n" % sys.exc_info()[:2]
            self._v_errors = error + '\n'.join(map(lambda err: "# " + str(err), parser.getErrors()))


    #FTP Support
    security.declareProtected('View management screens', 'document_src')
    def document_src(self, REQUEST=None, RESPONSE=None):
        """Return unprocessed document source."""
        if RESPONSE is not None:
            RESPONSE.setHeader('Content-Type', self.content_type)
        return self.read()

    ## Protocol handlers
    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'PUT')
    def PUT(self, REQUEST, RESPONSE):
        """Handle HTTP PUT requests."""
        self.dav__init(REQUEST, RESPONSE)
        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        body=REQUEST.get('BODY', '')
        self.munge(body)
        self.setDirty()
        self.ZCacheable_invalidate()
        RESPONSE.setStatus(204)
        return RESPONSE

    security.declareProtected(EasyCorePermissions.EasyTemplateChange, 'manage_FTPget')
    def manage_FTPget(self):
        "Get source for FTP download"
        return self.read()

    def get_size(self):
        return len(self.read())

    def content_type(self):
        return self.content_type

    def getType(self):
        return self.getStorage().getType()


    security.declarePublic('index_html')
    def index_html(self, client=None, REQUEST=None):
        """Render default content."""
        return self.getDefault()

    __call__=index_html

    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        if type(getattr(self.aq_base, '__ac_local_roles__'))==type({}):
            del self.__ac_local_roles__
            changed = 1
        if getattr(self,'_object_version', None)!='1.7': self._object_version = '1.7'
        if REQUEST is not None:
            return "<h1>Done</h1>"
        return changed

Globals.InitializeClass(EditableRegion)

manage_add_EditableRegionForm = DTMLFile('dtml/manage_add_EditableRegionForm', globals())

def manage_add_EditableRegion(self, id, storage, bgcolor, highlight_bgcolor, isSearchable=0, submit='', default='', storagecontainerid=None, REQUEST=None):
    """Add a Editable Region"""
    if hasattr(self,'checkRegionStorageId'):
        storagecontainerid = self.checkRegionStorageId(storagecontainerid)
    else:
        raise "Can't checkRegionStorageId"
    id = strip(id)
    ob = EditableRegion(id, storage,default,isSearchable, bgcolor, highlight_bgcolor,storagecontainerid)
    id=self._setObject(id, ob)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')


def initialize(context):
    """Initialize EditableRegion Product"""


    context.registerEasyRegionClass(
            EditableRegion,
            meta_type = EditableRegion.meta_type,
            permission = EasyCorePermissions.EasyTemplateAdd,
            constructors = (manage_add_EditableRegionForm, manage_add_EditableRegion),
            icon='icons/EditableRegion.gif',
            category='Easy Templates',
        )



