# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/Storage/ListTypeStorages.py,v 1.4 2004/09/09 13:25:45 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#


__doc__="""PropertyStorage

$Colliberty: PropertyStorage.py,v 1.4.37 2001/10/12 18:08:00 johanc Exp $"""
__version__='$Revision: 1.4 $'[11:-2]


import Globals
from Globals import HTML
from PropertyStorage import PropertyStorage
from ZPublisher.Converters import type_converters
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from types import StringType, ListType, TupleType
from string import atoi, capitalize

from Products.EasyCore.Utils.Utils import unix_line_endings

from Products.EasyCore import MLOG_TAB

_marker=[]

POSSIBLE_LINES='possible_'

def tuplify(value_list):
    if type(value_list) not in (ListType, TupleType): value_list=[value_list]
    new_list=[]
    add=new_list.append
    for value in value_list:
        if type(value) not in (ListType,TupleType):
            add((value,value))
    return new_list



class ListTypeStorage(PropertyStorage):

    value_type='list'
    storage_type="List"

    def getDefault(self, region):
        return list(getattr(region, 'default', []))

    def setDefault(self, region, value):
        return setattr(region, 'default', list(value))



class LinesPropertyStorage(ListTypeStorage):
    """A LinesPropertyStorage Object """

    property_type = 'lines'
    storage_type="Lines"
    value_type='list'

    def render_html_view(self, editablecontainer, region, REQUEST=None):
        lines=self.manage_load(editablecontainer, region, self.getDefault(region), REQUEST=REQUEST)
        if lines and type(lines)!=StringType:
            lines = '\n'.join(lines)
        return self.do_html_view(editablecontainer, region, id=region_key, value=lines)

    def do_html_view(self, editablecontainer, region, defaults=None, **kw):
        rows_max=getattr(region, 'property_rows_max', self.property_rows_max)
        rows_min=getattr(region, 'property_rows_min', self.property_rows_min)
        rows_add=getattr(region, 'property_rows_add', self.property_rows_add)
        cols=getattr(region,'property_cols', self.property_cols)
        rows=len(kw['value'].split('\n'))+rows_add
        rows=rows>rows_max and rows_max or rows
        rows=rows<rows_min and rows_min or rows
        if defaults is None:
            defaults={'cols':cols,'rows':rows}
        defaults.update(kw)
        return """\
            <textarea
                name="%(id)s:lines"
                wrap="off"
                cols="%(cols)s"
                style="width:100%%">%(value)s</textarea>""" % defaults


    def manage_view_default_conf(self, region, REQUEST=None):
        return """<textarea name="%s" cols="%s" rows="%s" style="width:100%">%s</textarea>""" % ('default', '50', '10', '\n'.join(self.getDefault(region)))

    def manage_save_default_conf(self, region, REQUEST=None):
        self.setDefault(region, REQUEST.get('default', getattr(region, 'default', '')))

    def getDefault(self, region):
        return list(getattr(region, 'default', []))

    def setDefault(self, region, value):
        value=unix_line_endings(value)
        value=value.split('\n')
        if value==['']: value=[]
        setattr(region, 'default', value)


Globals.InitializeClass(LinesPropertyStorage)


class TokensPropertyStorage(ListTypeStorage):
    """A TokensPropertyStorage Object """

    property_type = 'tokens'
    storage_type="Tokens"
    value_type='list'


    def render_html_view(self, editablecontainer, region, REQUEST=None):
        tokens=self.manage_load(editablecontainer, region, self.getDefault(region), REQUEST=REQUEST)
        if type(tokens)!=StringType:
            tokens = ' '.join(tokens)
        return self.do_html_view(editablecontainer, region, id=region.id, value=tokens)


    def do_html_view(self, editablecontainer, region, defaults=None, **kw):
        cols=50
        if REQUEST is not None and REQUEST.has_key('dtpref_cols'):
            dtpref_cols=REQUEST.dtpref_cols
            if type(dtpref_cols) is StringType and dtpref_cols[-1]=='%': dtpref_cols=dtpref_cols[:-1]
            cols=max(atoi(dtpref_cols)-15,20)
        if defaults is None:
            defaults={'size':cols,}
        defaults.update(kw)
        return """<input type="text" name="%(id)s:tokens" value="%(value)s" size="%(size)s">""" % defaults


    def manage_view_default_conf(self, region, REQUEST=None):
        return """<input type="string" name="%s" width="%s" value="%s"/>""" % ('default', '40', ' '.join(self.getDefault(region)))

    def manage_save_default_conf(self, region, REQUEST=None):
        self.setDefault(region, REQUEST.get('default', getattr(region, 'default', '')))

    def getDefault(self, region):
        return list(getattr(region, 'default', []))

    def setDefault(self, region, value):
        value=value.split('\n')
        if value==['']: value=[]
        setattr(region, 'default', value)


Globals.InitializeClass(TokensPropertyStorage)


def initialize(context):
    """Initialize PropertyStorage"""

    context.registerEasyStorage(LinesPropertyStorage, 'LinesPropertyStorage')
    context.registerEasyStorage(TokensPropertyStorage, 'TokensPropertyStorage')


