# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/Storage/NumberPropertyStorages.py,v 1.3 2004/09/09 13:25:45 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#


__doc__="""PropertyStorage

$Colliberty: PropertyStorage.py,v 1.4.37 2001/10/12 18:08:00 johanc Exp $"""
__version__='$Revision: 1.3 $'[11:-2]


import Globals
from Globals import HTML
from PropertyStorage import PropertyStorage
from ZPublisher.Converters import type_converters
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from types import StringType, ListType, TupleType
from string import atoi, capitalize

from Products.EasyCore import MLOG_TAB

_marker=[]

POSSIBLE_LINES='possible_'

class BooleanPropertyStorage(PropertyStorage):
    """A BooleanPropertyStorage Object """

    property_type = 'boolean'
    storage_type="Boolean"
    value_type='boolean'

    def render_html_view(self, editablecontainer, region, REQUEST=None):
        checked=self.manage_load(editablecontainer, region, self.getDefault(region), REQUEST=REQUEST) and 'checked' or ''
        return self.do_html_view(editablecontainer, region, id=region.id, checked=checked)

    def do_html_view(self, editablecontainer, region, defaults=None, **kw):
        if defaults is None:
            defaults={'checked':'',}
        defaults.update(kw)
        return """<input type="checkbox" name="%(id)s:boolean" value="1" %(checked)s>""" % defaults



Globals.InitializeClass(BooleanPropertyStorage)


class IntPropertyStorage(PropertyStorage):
    """A IntPropertyStorage Object """

    property_type = 'int'
    storage_type="Integer"
    value_type='int'

    def render_html_view(self, editablecontainer, region, REQUEST=None):
        value=self.manage_load(editablecontainer, region, self.getDefault(region), REQUEST=REQUEST)
        return self.do_html_view(editablecontainer, region, id=region.id, value=value)

    def do_html_view(self, editablecontainer, region, defaults=None, **kw):
        if defaults is None:
            defaults={'size':'8',}
        defaults.update(kw)
        return """<input type="text" name="%(id)s:int" value="%(value)s" size="%(size)s">""" % defaults


Globals.InitializeClass(IntPropertyStorage)


class FloatPropertyStorage(PropertyStorage):
    """A FloatPropertyStorage Object """

    property_type = 'float'
    storage_type="Float"
    value_type='float'

    def render_html_view(self, editablecontainer, region, REQUEST=None):
        value = self.manage_load(editablecontainer, region, self.getDefault(region), REQUEST=REQUEST)
        return self.do_html_view(editablecontainer, region, id=region.id, value=value)

    def do_html_view(self, editablecontainer, region, defaults=None, **kw):
        if defaults is None:
            defaults={'size':'8',}
        defaults.update(kw)
        return """<input type="text" name="%(id)s:float" value="%(value)s" size="%(size)s">""" % defaults


Globals.InitializeClass(FloatPropertyStorage)


class LongPropertyStorage(PropertyStorage):
    """A LongPropertyStorage Object """

    property_type = 'long'
    storage_type="Long Integer"
    value_type='long'

    def render_html_view(self, editablecontainer, region, REQUEST=None):
        value = self.manage_load(editablecontainer, region, self.getDefault(region), REQUEST=REQUEST)
        return self.do_html_view(editablecontainer, region, id=region.id, value=value)

    def do_html_view(self, editablecontainer, region, defaults=None, **kw):
        if defaults is None:
            defaults={'size':'8',}
        defaults.update(kw)
        return """<input type="text" name="%(id)s:long" value="%(value)s" size="%(size)s">""" % defaults




Globals.InitializeClass(LongPropertyStorage)


def initialize(context):
    """Initialize PropertyStorage"""

    context.registerEasyStorage(IntPropertyStorage,     'IntPropertyStorage')
    context.registerEasyStorage(FloatPropertyStorage,   'FloatPropertyStorage')
    context.registerEasyStorage(LongPropertyStorage,   'LongPropertyStorage')
    context.registerEasyStorage(BooleanPropertyStorage, 'BooleanPropertyStorage')

