# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/Storage/SelectionPropertyStorage.py,v 1.10 2004/09/13 20:54:43 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#


__doc__="""PropertyStorage

$Colliberty: PropertyStorage.py,v 1.4.37 2001/10/12 18:08:00 johanc Exp $"""
__version__='$Revision: 1.10 $'[11:-2]


import Globals
from Globals import HTML, DTMLFile
from PropertyStorage import PropertyStorage
from ListTypeStorages import tuplify
from ZPublisher.Converters import type_converters
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from types import StringType, ListType, TupleType
from string import atoi, capitalize

from Products.EasyCore.Utils.Utils import SelectList

from Products.EasyCore import MLOG_TAB

_marker=[]

POSSIBLE_LINES='possible_'




class SelectionPropertyStorage(PropertyStorage):
    """A SelectionPropertyStorage Object """

    property_type = 'selection'
    storage_type="SelectionPropertyStorage"
    value_type='string'
    do_show_empty_default=''
    select_empty_default=''
    property_select_variable=''
    property_select_list=[]
    property_select_list_sort_on=''
    property_select_list_sort_order='ascending'
    enable_none_selection=0
    none_selection_value='N/A'

    security = ClassSecurityInfo()


    def validityCheck(self, editablecontainer, region):
        prop=self.getPropertyInfoFor(editablecontainer, region)
        return prop and hasattr(editablecontainer, region.storagecontainerid) and\
               prop.has_key('') and hasattr(editablecontainer, prop['select_variable'])
        none_selection_value


    def render_html_view(self, editablecontainer, region, REQUEST=None):

        value=self.manage_load(editablecontainer, region, self.getDefault(region), REQUEST=REQUEST)

        return self.do_html_view(editablecontainer, region
                                , id=region.id
                                , value=value
                                )


    def do_html_view(self, editablecontainer, region, defaults=None, **kw):

        if defaults is None:
            defaults={'size':'5', 'value':'', 'select_list':[] }
        defaults.update(kw)

        select_variable=self.getSelectVariable(region)
        property_select_list=getattr(region, select_variable, None)

        if callable(property_select_list): property_select_list=property_select_list()
        sort_on=getattr(region, 'property_select_list_sort_on', self.property_select_list_sort_on)
        sort_order=getattr(region, 'property_select_list_sort_order', self.property_select_list_sort_order)
        property_select_list=SelectList(property_select_list, sort_on=sort_on, sort_order=sort_order)
        if property_select_list is None:
            property_select_list=getattr(region, 'property_select_list', [])

        defaults['select_list'] = property_select_list
        defaults['enable_none_selection'] = getattr(region, 'enable_none_selection', self.enable_none_selection)
        defaults['none_selection_value'] = getattr(region, 'none_selection_value', self.none_selection_value)

        return self._default_render_html_view(self, **defaults)


    _default_render_html_view=HTML("""\
        <select name="&dtml-id;:string">
            <dtml-if enable_none_selection>
                <option value="&dtml-none_selection_value;" <dtml-var "value==none_selection_value and 'selected' or ''">><dtml-var none_selection_value></option>
            </dtml-if>
            <dtml-in select_list prefix="o">
                <option value="&dtml-o_key;" <dtml-var "value==o_key and 'selected' or ''">><dtml-var o_item></option>
            </dtml-in>
        </select>
        """, globals())


    #def manage_save(self, editablecontainer, region, value=None, REQUEST=None):
    #    PropertyStorage.manage_save(self, editablecontainer, region, value=value, REQUEST=REQUEST)

    _default_property_config=HTML("""\
        <dtml-with "region">
        <table width="100%" cellspacing="0" cellpadding="2">
          <tr><td colspan="2" class="form-label">Mode</td></tr>
          <tr class="EProw-hilite">
            <td><input type="checkbox" name="deletable:int" value="1" <dtml-if "'d' in property_mode">checked</dtml-if>></td>
            <td class="form-label" width="100%">Deletable</td>
          </tr>
          <tr class="EProw-hilite">
            <td><input type="checkbox" name="writtable:int" value="1" <dtml-if "'w' in property_mode">checked</dtml-if>></td>
            <td class="form-label" width="100%">Writable</td>
          </tr>
        </table>
        <table width="100%" cellspacing="0" cellpadding="2">
          <tr class="EProw-hilite">
            <td class="form-label" width="120">Select Variable</td>
            <td><input type="text" name="property_select_variable:string" value="&dtml.missing-property_select_variable;" style="width:100%"></td>
          </tr>
          <tr class="EProw-hilite">
            <td class="form-label" width="120">Sort on:</td>
            <td><select name="property_select_list_sort_on">
                <option value="" <dtml-var "property_select_list_sort_on == '' and 'selected' or ''">>No Sorting</option>
                <option value="key" <dtml-var "property_select_list_sort_on == 'key' and 'selected' or ''">>Key</option>
                <option value="value" <dtml-var "property_select_list_sort_on == 'value' and 'selected' or ''">>Value</option>
                </select>
            </td>
          </tr>
          <tr class="EProw-hilite" width="120">
            <td class="form-label">Sort order:</td>
            <td><select name="property_select_list_sort_order">
                <option value="ascending" <dtml-var "property_select_list_sort_order == 'ascending' and 'selected' or ''">>Ascending</option>
                <option value="descending" <dtml-var "property_select_list_sort_order == 'descending' and 'selected' or ''">>Descending</option>
                </select>
            </td>
          </tr>
        </table>
        <table width="100%" cellspacing="0" cellpadding="2">
          <tr class="EProw-hilite">
            <td><input type="checkbox" name="enable_none_selection:int" value="1" <dtml-if enable_none_selection>checked</dtml-if>></td>
            <td class="form-label" width="100%">Enable None Selection</td>
          </tr>
        </table>
        <table width="100%" cellspacing="0" cellpadding="2">
          <tr class="EProw-hilite">
            <td class="form-label" width="120">None Selection Value</td>
            <td><input type="text" name="none_selection_value:string" value="&dtml-none_selection_value;" style="width:100%"></td>
          </tr>
        </table>
        <table width="100%" cellspacing="0" cellpadding="2">
          <tr>
            <td class="form-label" colspan="2">Static List of Values</td>
          </tr>
          <tr class="EProw-hilite">
            <td  colspan="2">
              <textarea name="property_select_list:lines" rows="6" cols="40" style="width:100%"><dtml-var "'\\n'.join(property_select_list)"></textarea>
            </td>
          </tr>
        </table>
        </dtml-with>
        """, globals())


    def manage_view_storage_conf(self, region, REQUEST=None):
        return self._default_property_config(self, region=region, REQUEST=REQUEST)


    def manage_save_storage_conf(self, region, REQUEST=None):
        property_mode=''
        if REQUEST.get('writtable', 0):
            property_mode=property_mode+'w'
        if REQUEST.get('deletable', 0):
            property_mode=property_mode+'d'
        region.property_mode = property_mode

        region.property_select_variable = REQUEST.get('property_select_variable',getattr(region,'property_select_variable', self.property_select_variable))

        region.property_select_list = REQUEST.get('property_select_list', [])


        region.property_select_list_sort_on = REQUEST.get('property_select_list_sort_on', self.property_select_list_sort_on)
        region.property_select_list_sort_order = REQUEST.get('property_select_list_sort_order', self.property_select_list_sort_order)

        region.enable_none_selection = REQUEST.get('enable_none_selection', 0)
        region.none_selection_value = REQUEST.get('none_selection_value', '')



    def manage_view_default_conf(self, region, REQUEST=None):
        # We need our own implementatiojn here rather that calling render_html_view
        # because we don't know that manage_load will work in all situations.
        # Our value should be the default.
        select_variable=self.getSelectVariable(region)
        property_select_list=getattr(region, select_variable, None)
        if property_select_list is None:
            property_select_list=getattr(region, 'property_select_list', [])
        if callable(property_select_list): property_select_list=property_select_list()

        sort_on=getattr(region, 'property_select_list_sort_on', self.property_select_list_sort_on)
        sort_order=getattr(region, 'property_select_list_sort_order', self.property_select_list_sort_order)
        enable_none_selection=getattr(region, 'enable_none_selection', self.enable_none_selection)
        none_selection_value=getattr(region, 'none_selection_value', self.none_selection_value)


        property_select_list=SelectList(property_select_list, sort_on=sort_on, sort_order=sort_order)
        if property_select_list is None:
            property_select_list=getattr(region, 'property_select_list', [])

        value=self.getDefault(region)
        return self._default_render_html_view(self, REQUEST=REQUEST
                            , value=value
                            , select_list=property_select_list
                            , id='default'
                            , enable_none_selection=enable_none_selection
                            , none_selection_value=none_selection_value
                            )

    def manage_save_default_conf(self, region, REQUEST=None):
        self.setDefault(region, REQUEST.get('default', getattr(region, 'default', '')))

    def getDefault(self, region):
        return str(getattr(region, 'default', ''))

    def setDefault(self, region, value):
        return setattr(region, 'default', str(value))




Globals.InitializeClass(SelectionPropertyStorage)






def initialize(context):
    """Initialize PropertyStorage"""

    context.registerEasyStorage(SelectionPropertyStorage,   'SelectionPropertyStorage')

