# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/Storage/SimpleCollectionStorage.py,v 1.3 2004/09/13 20:54:43 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#
#


__doc__="""CollectionStorage

$Colliberty: CollectionStorage.py,v 1.4.37 2001/10/12 18:08:00 johanc Exp $"""
__version__='$Revision: 1.3 $'[11:-2]


import Globals
from Globals import HTML
from Globals import PersistentMapping
from SelectionPropertyStorage import SelectionPropertyStorage
from ListTypeStorages import ListTypeStorage
from ZPublisher.Converters import type_converters
from AccessControl import ClassSecurityInfo
from Products.EasyCore import EasyCorePermissions
from Acquisition import Implicit, aq_base, aq_chain, aq_parent
from types import StringType, ListType, TupleType
from string import atoi, capitalize
try:
    from ZODB.PersistentList import PersistentList
except ImportError:
    from persistent.list import PersistentList


import StructuredText
from Products.PageTemplates.Expressions import getEngine
from Products.EasyCore.Utils.Utils import SelectList,unix_line_endings
from Products.EasyCore import MLOG_TAB
from Products.EasyCore.EasySortable.EasySortable import EasySortable


_marker=[]

POSSIBLE_LINES='possible_'


class IDict:
    def __init__(self, brain):
        self._brain=brain
    def __getitem__(self, name):
        return getattr(self._brain, name)
    def __setitem__(self, name, value):
        pass
    def copy(self):
        return self

class BrainWrapper(Implicit):
    def __init__(self, id, brain, item):
        self.id=id
        self._brain=brain
        self._item=item

    def __getattr__(self, name):
        try:
            return getattr(aq_base(self._brain), name)
        except:
            value=self._item.get(name, _marker)
            if value is _marker:
                raise AttributeError, name
            return value

    def __str__(self):
        return "BrainWrapper: %s" % (self.id,)

    def __repr__(self):
        return "<BrainWrapper: %s>" % (self.id,)

    def getBrain(self):
        return self

class SimpleCollectionStorage(EasySortable, ListTypeStorage):
    """\
    A SimpleCollectionStorage Object

    Storage a list of a compound set attributes defind by other
    regions in the same template.

    """

    property_type = 'collection'

    storage_type="CollectionStorage"
    value_type='list'

    select_variable=''
    id_tales="id"
    title_tales="$title"
    attribute_template="Id:    $id\nTitle: $title"

    select_list_sort_on='value'
    select_list_sort_order='ascending'
    use_schema=None


    security = ClassSecurityInfo()

    def validityCheck(self, editablecontainer, region):
        return 1

    def render_html_view(self, editablecontainer, region, REQUEST=None):

        sort_on=getattr(region, 'select_list_sort_on', self.select_list_sort_on)
        sort_order=getattr(region, 'select_list_sort_order', self.select_list_sort_order)

        sub_item_ids=self._getSubItemIds(editablecontainer, region, REQUEST=REQUEST)

        value=self.manage_load(editablecontainer, region, self.getDefault(region))

        if type(value) not in (TupleType, ListType):value=[value]

        return self.do_html_view(editablecontainer, region
                                , id=region.id
                                , value=value
                                , sub_item_ids=SelectList(sub_item_ids, sort_on=sort_on, sort_order=sort_order)
                                , id_to_add=region.getStorageContainerId()
                                , items=self.getItemBrains(editablecontainer, region)
                                )

    def do_html_view(self, editablecontainer, region, defaults=None, **kw):
        if defaults is None:
            defaults={'size':'10','region':region,'editablecontainer':editablecontainer}
        defaults.update(kw)
        return self._default_render_html_view(self, **defaults)


    _default_property_config=HTML("""\
        <dtml-with "region">
        <table width="100%" cellspacing="0" cellpadding="2">
          <tr><td colspan="2" class="form-label">Choose Schema</td></tr>
          <tr class="EProw-hilite">
            <td></td>
            <td class="form-label" width="100%">
                <dtml-if possible_schemas>
                    <select name="use_schema">
                    <dtml-in possible_schemas prefix="reg">
                        <option value="&dtml-reg_key;" <dtml-var "reg_key == use_schema and 'selected' or ''">><dtml-var title></option>
                    </dtml-in>
                    </select>
                <dtml-else>
                    <span class="form-text">No more items to add.</span>
                </dtml-if>
            </td>
          </tr>
        </table>
        <table width="100%" cellspacing="0" cellpadding="2">
          <tr class="EProw-hilite">
            <td class="form-label" width="120" style="padding-top:16px;">Sort on:</td>
            <td style="padding-top:16px;"><select name="select_list_sort_on">
                <option value="" <dtml-var "select_list_sort_on == '' and 'selected' or ''">>No Sorting</option>
                <option value="key" <dtml-var "select_list_sort_on == 'key' and 'selected' or ''">>Key</option>
                <option value="value" <dtml-var "select_list_sort_on == 'value' and 'selected' or ''">>Value</option>
                </select>
            </td>
          </tr>
          <tr class="EProw-hilite" width="120">
            <td class="form-label" style="padding-bottom:12px;">Sort order:</td>
            <td style="padding-bottom:12px;"><select name="select_list_sort_order">
                <option value="ascending" <dtml-var "select_list_sort_order == 'ascending' and 'selected' or ''">>Ascending</option>
                <option value="descending" <dtml-var "select_list_sort_order == 'descending' and 'selected' or ''">>Descending</option>
                </select>
            </td>
          </tr>
        </table>

        <table width="100%" cellspacing="0" cellpadding="2">
          <tr><td class="form-label" colspan="2">List of Addable Items</td></tr>
          <tr class="EProw-hilite"><td class="form-text" colspan="2" style="padding-top:12px;"><strong>Name of look up method</strong><br>(The method should return a list of brains.)</strong>
          <br><strong>Syntax:</strong> method_name(ids=None, id_name='id')
          </td></tr>
          <tr class="EProw-hilite">
            <td colspan="2" style="padding-bottom:6px;"><input type="text" name="select_variable:string" value="&dtml.missing-select_variable;" style="width:100%"></td>
          </tr>

          <tr class="EProw-hilite"><td class="form-text" colspan="2" style="padding-top:6px;"><strong>Id:</strong></td></tr>
          <tr class="EProw-hilite">
            <td colspan="2" style="padding-bottom:6px;"><input type="text" name="id_tales:string" value="&dtml.missing-id_tales;" style="width:100%"></td>
          </tr>

          <tr class="EProw-hilite"><td class="form-text" colspan="2" style="padding-top:6px;"><strong>Title (Tales String Syntax):</strong></td></tr>
          <tr class="EProw-hilite">
            <td colspan="2" style="padding-bottom:6px;"><input type="text" name="title_tales:string" value="&dtml.missing-title_tales;" style="width:100%"></td>
          </tr>

          <tr class="EProw-hilite"><td class="form-text" colspan="2" style="padding-top:6px;"><strong>Attribute Template (Tales String Syntax):</strong></td></tr>
          <tr class="EProw-hilite">
            <td colspan="2" style="padding-bottom:6px;">
            <textarea name="attribute_template:text" rows="6" cols="40" style="width:100%"><dtml-var attribute_template></textarea>
            </td>
          </tr>

        </table>

        </dtml-with region>
        """, globals())

    #MANAGE VIEW STORAGE

    def manage_view_storage_conf(self, region, REQUEST):
        template=region.getTemplate()
        regions=template.regionItems()
        possible_schemas=self._getSchemas(self, region, REQUEST=REQUEST)
        return self._default_property_config(self
                , region=region
                , possible_schemas=SelectList(possible_schemas, sort_on='title', sort_order='asc')
                , regions=regions
                , REQUEST=REQUEST)


    def manage_save_storage_conf(self, region, REQUEST=None):
        region.select_list_sort_on = REQUEST.get('select_list_sort_on', self.select_list_sort_on)
        region.select_list_sort_order = REQUEST.get('select_list_sort_order', self.select_list_sort_order )

        region.use_schema = REQUEST.get('use_schema', self.use_schema )


        select_variable = REQUEST.get('select_variable', self.select_variable)
        self.setSelectVariable(region, select_variable)
        region.id_tales = REQUEST.get('id_tales', self.id_tales)
        region.title_tales = REQUEST.get('title_tales', self.title_tales)

        region.attribute_template = REQUEST.get('attribute_template', self.attribute_template)

    def setSelectVariable(self, region, select_variable):
        region.select_variable=select_variable

    def getSelectVariable(self, region):
        return getattr(region, 'select_variable', self.select_variable)

    def _getSchemas(self, editablecontainer, region, REQUEST=None):
        return aq_parent(aq_parent(self)).objectValues(spec='Easy Schema')

    def _getSchema(self, editablecontainer, region, REQUEST=None):
        use_schema=getattr(region, 'use_schema', self.use_schema)
        if use_schema:
            return getattr(aq_parent(aq_parent(self)), use_schema, None)

    def _getSubRegions(self, editablecontainer, region):
        schema=self._getSchema(editablecontainer, region)
        if schema:
            return schema.objectValues(spec='EditableRegion')
        return []

    def _getSubItemIds(self, editablecontainer, region, REQUEST=None):
        if REQUEST is None: REQUEST=self.REQUEST
        session=REQUEST.SESSION
        select_variable = self.getSelectVariable(region)

        id_tales = getattr(region, 'id_tales', self.id_tales)
        title_tales = getattr(region, 'title_tales', self.title_tales)

        sub_item_ids=getattr(region, select_variable, None)
        ids=self._get_ids(editablecontainer, region)
        if callable(sub_item_ids):
            sub_item_ids=sub_item_ids()
        if sub_item_ids:
            sub_item_ids=filter(lambda x, ids=ids: x.id not in ids, sub_item_ids)
        else:
            sub_item_ids=()

        new_list=[]
        for brain in sub_item_ids:
            id = getattr(brain, id_tales, getattr(brain, 'id', None))
            try:
                expr = getEngine().compile("string:"+title_tales)
                title = getEngine().getContext(IDict(brain)).evaluate(expr)
            except:
                title = getattr(brain, 'title', None)
            if id and not title:
                title=id
            elif title and not id:
                id=title
            if id and title:
                new_list.append((id, title))

        return new_list


    def renderAttributeTemplate(self, region, brain):
        attribute_template=getattr(region, 'attribute_template', self.attribute_template)
        expr = getEngine().compile("string:"+attribute_template)
        attributes = getEngine().getContext(IDict(brain)).evaluate(expr)
        st = StructuredText.Basic(attributes)
        doc = StructuredText.DocumentWithImages(st)
        rendered=StructuredText.HTMLWithImages(doc,header=0,level=4)
        return rendered


    def renderSubRegions(self, editablecontainer, region, brain):
        rs_id=region.storagecontainerid
        result='''<input type="hidden" name="sub_%(rs_id)s.id:records" value="%(id)s">''' % {'rs_id':rs_id, 'id':brain.id}
        for s_region in self._getSubRegions(editablecontainer, region):
            field_id='sub_%s.%s:records' % (region.storagecontainerid, s_region.storagecontainerid)
            values_for=self._get_values_for(editablecontainer, region, brain.id, {})
            value=values_for.get(s_region.storagecontainerid, s_region.getDefault())
            rendered=s_region.do_html_view(editablecontainer, id=field_id, value=value)
            result+="""<p class="form-text"><span class="form-label">%(label)s</span> (%(stype)s)
                       %(rendered)s</p>
                  """ % {
                    'label':s_region.label
                  , 'stype':s_region.getStorageType()
                  , 'rendered':rendered
                }
            if s_region.show_instuction:
                result+="""<p class="">%s</p>""" %(s_region.instuction,)
        return result


    def manage_view_default_conf(self, region, REQUEST=None):
        # We need our own implementatiojn here rather that calling render_html_view
        # because we don't know that manage_load will work in all situations.
        # Our value should be the default.

        sort_on=getattr(region, 'select_list_sort_on', self.select_list_sort_on)
        sort_order=getattr(region, 'select_list_sort_order', self.select_list_sort_order)

        sub_item_ids=self._getSubItemIds(self, region, REQUEST=REQUEST)

        value=[]

        return self._default_render_html_view(self, REQUEST=REQUEST
                            , value=value
                            , sub_item_ids=SelectList(sub_item_ids, sort_on=sort_on, sort_order=sort_order)
                            , id_to_add=region.getStorageContainerId()
                            , items=self.getItemBrains(region, region)
                            , region=region
                            , editablecontainer=aq_base(self)
                            )


    _default_render_html_view=HTML("""\
        <div style="margin:12px;padding:6px;border: 2px inset #ccc;background-color:#ddd;">
            <dtml-if sub_item_ids>
                <select name="&dtml-id_to_add;_to_add:string">
                  <dtml-in sub_item_ids prefix="o">
                     <option value="&dtml-o_key;"><dtml-var o_item></option>
                  </dtml-in>
                </select>
                <input type="submit" name="command.:record" value="&dtml-Add;">
            <dtml-else>
                <span class="form-text">No more items to add.</span>
            </dtml-if sub_item_ids>

            <dtml-if items>

            <dtml-in "items" prefix="o">

              <table cellpadding="2" cellspacing="0" border="0"
                class="<dtml-var "o_even and 'EProw-normal' or 'EProw-lolite'">"
                style="width:100%;margin:12px;padding:6px;border: 2px outset #ccc;">
                  <tr>
                  <td>
                    <input type="checkbox" name="selected_ids:list" value="&dtml-id;">
                    <input type="SUBMIT" value="Save">
                    <input type="submit" name="command.&dtml-id;:record" value="&dtml-Remove;">
                    <input type="submit" name="command.&dtml-id;:record" value="&dtml-Top;">
                    <input type="submit" name="command.&dtml-id;:record" value="&dtml-Up;">
                    <input type="submit" name="command.&dtml-id;:record" value="&dtml-Down;">
                    <input type="submit" name="command.&dtml-id;:record" value="&dtml-Bottom;">
                  </td></tr>

                <tr>
                    <td class="form-text" valign="top" style="padding:12px 12px 12px 12px;">
                        <dtml-var "renderAttributeTemplate(region, getBrain())">
                        <dtml-var "renderSubRegions(editablecontainer, region, getBrain())">
                    </td>
                </tr>
              </div>
              </table>
            </dtml-in>


            </dtml-if>
        </div>
        """, globals())

    def manage_save_default_conf(self, region, REQUEST=None):
        self._common_save(region, region, REQUEST=REQUEST)



    # API
    Add="Add"
    Remove="Remove"
    Top="Top"
    Up="Up"
    Down="Down"
    Bottom="Bottom"

    def _reset(self, editablecontainer, region):
        ids_name=self.getStorageNameForIds(editablecontainer, region)
        values_name=self.getStorageNameForValues(editablecontainer, region)
        delattr(editablecontainer, ids_name)
        delattr(editablecontainer, values_name)

    def _add_item_key(self, editablecontainer, region, key):
        values=self._get_values(editablecontainer, region)
        ids=self._get_ids(editablecontainer, region)
        if not values.has_key(key):
            values[key]=PersistentMapping()
        if key not in ids: ids.append(key)


    def _remove_item_keys(self, editablecontainer, region, keys):
        ###self._reset(editablecontainer, region)
        values=self._get_values(editablecontainer, region)
        ids=self._get_ids(editablecontainer, region)
        if type(keys) not in (ListType, TupleType): keys=[keys]
        for key in keys:
            if values.has_key(key):
                del values[key]
            while key in ids:
                ids.remove(key)

    def _move_to_top(self, editablecontainer, region, keys):
        keys=list(keys)
        old_ids=list(self._get_ids(editablecontainer, region))
        new_order=self.moveTop(keys, old_ids)
        ids_name=self.getStorageNameForIds(editablecontainer, region)
        setattr(editablecontainer, ids_name, PersistentList(new_order))

    def _move_up(self, editablecontainer, region, keys, steps=1):
        keys=list(keys)
        old_ids=list(self._get_ids(editablecontainer, region))
        new_order=self.moveUp(keys, old_ids, steps=steps)
        ids_name=self.getStorageNameForIds(editablecontainer, region)
        setattr(editablecontainer, ids_name, PersistentList(new_order))

    def _move_down(self, editablecontainer, region, keys, steps=1):
        keys=list(keys)
        old_ids=list(self._get_ids(editablecontainer, region))
        new_order=self.moveDown(keys, old_ids, steps=steps)
        ids_name=self.getStorageNameForIds(editablecontainer, region)
        setattr(editablecontainer, ids_name, PersistentList(new_order))

    def _move_to_bottom(self, editablecontainer, region, keys):
        keys=list(keys)
        old_ids=list(self._get_ids(editablecontainer, region))
        new_order=self.moveBottom(keys, old_ids)
        ids_name=self.getStorageNameForIds(editablecontainer, region)
        setattr(editablecontainer, ids_name, PersistentList(new_order))

    security.declarePublic('getStorageNameForIds')
    def getStorageNameForIds(self, editablecontainer, region):
        return region.storagecontainerid+'_ids'

    security.declarePublic('getStorageNameForValues')
    def getStorageNameForValues(self, editablecontainer, region):
        return region.storagecontainerid+'_values'


    def _get_ids(self, editablecontainer, region):
        sname=self.getStorageNameForIds(editablecontainer, region)
        ids=getattr(aq_base(editablecontainer), sname, None)
        if ids is None:
            ids=PersistentList()
            setattr(editablecontainer, sname, ids)
        return ids

    def _get_values(self, editablecontainer, region):
        values=None
        values_name=self.getStorageNameForValues(editablecontainer, region)
        if hasattr(aq_base(editablecontainer), values_name):
            values=getattr(aq_base(editablecontainer), values_name)
        if not isinstance( values, PersistentMapping):
            values=PersistentMapping()
        setattr(editablecontainer, values_name, values)
        return values

    def _get_values_for(self, editablecontainer, region, id, default=None):
        values=self._get_values(editablecontainer, region)
        ids=self._get_ids(editablecontainer, region)
        if not values.has_key(id) and id not in ids:
            return default
        if not values.has_key(id) and id in ids:
            values[id]=PersistentMapping()
        return values[id]

    security.declareProtected(EasyCorePermissions.AccessContentsInformation, 'getItemBrains')
    def getItemBrains(self, editablecontainer, region):
        ids=self._get_ids(editablecontainer, region)
        if ids:
            values=self._get_values(editablecontainer, region)
            lookup_method=getattr(region, self.getSelectVariable(region), None)
            if callable(lookup_method):
                id_tales=getattr(region, 'id_tales', self.id_tales)
                brains={}
                for brain in lookup_method(ids=list(ids), id_name=id_tales):
                    id_name=getattr(brain, id_tales, None)
                    if id_name is not None:
                        item=values.get(id_name, None)
                        if item is not None:
                            brains[id_name]=BrainWrapper(id_name, brain, item)
            return map(lambda x, brains=brains: brains[x], ids)
        return []

    def _common_save(self, editablecontainer, region, REQUEST=None):
        #Sometimes editablecontainer can be set to region.
        commands=REQUEST.get('command', None)
        if commands is not None:
            for cid, command in commands.items():
                id_to_add='%s_to_add' % region.getStorageContainerId()
                if command==self.Add:
                    item_key=REQUEST.get(id_to_add, None)
                    if item_key is not None:
                        self._add_item_key(editablecontainer, region, item_key)
                elif command==self.Remove:
                    if not cid:
                        cid = REQUEST.get('selected_ids', None)
                    if cid:
                        self._remove_item_keys(editablecontainer, region, cid)
                elif command==self.Top:
                    if not cid:
                        cid = REQUEST.get('selected_ids', None)
                    else:
                        cid=[cid]
                    if cid:
                        self._move_to_top(editablecontainer, region, cid)
                elif command==self.Up:
                    if not cid:
                        cid = REQUEST.get('selected_ids', None)
                    else:
                        cid=[cid]
                    if cid:
                        self._move_up(editablecontainer, region, cid)
                elif command==self.Down:
                    if not cid:
                        cid = REQUEST.get('selected_ids', None)
                    else:
                        cid=[cid]
                    if cid:
                        self._move_down(editablecontainer, region, cid)
                elif command==self.Bottom:
                    if not cid:
                        cid = REQUEST.get('selected_ids', None)
                    else:
                        cid=[cid]
                    if cid:
                        self._move_to_bottom(editablecontainer, region, cid)


        ids=self._get_ids(editablecontainer, region)
        item_keys=REQUEST.get('sub_%s' % region.storagecontainerid, [])
        for r_item in item_keys:
            values=self._get_values_for(editablecontainer, region, r_item.id, None)
            if values is not None:
                for s_region in self._getSubRegions(editablecontainer, region):
                    values[s_region.storagecontainerid]=r_item[s_region.storagecontainerid]


    def manage_save(self, editablecontainer, region, value=None, REQUEST=None):
        self._common_save(editablecontainer, region, REQUEST=REQUEST)

    def manage_load(self, editablecontainer, region, value=None, REQUEST=None):
         return self.do_get(editablecontainer, region)


    def do_get(self, container, region, default=None):
        return list(self._get_ids(container, region))

    def manage_initilized(self, editablecontainer, region, REQUEST=None):
        pass

    def getDefault(self, region):
        return list(getattr(region, 'default', []))

    def setDefault(self, region, value):
        return setattr(region, 'default', list(value))


Globals.InitializeClass(SimpleCollectionStorage)


def initialize(context):
    """Initialize SimpleCollectionStorage"""

    context.registerEasyStorage(SimpleCollectionStorage,   'SimpleCollectionStorage')


