# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyEditor/Storage/StringPropertyStorages.py,v 1.6 2004/09/13 20:54:43 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#


__doc__="""StringPropertyStorages

$Colliberty: StringPropertyStorages.py,v 1.4.37 2001/10/12 18:08:00 johanc Exp $"""
__version__='$Revision: 1.6 $'[11:-2]


import Globals
from Globals import HTML
from PropertyStorage import PropertyStorage
from ZPublisher.Converters import type_converters
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from types import StringType, ListType, TupleType
from string import atoi, capitalize


from Products.EasyCore import MLOG_TAB



_marker=[]

POSSIBLE_LINES='possible_'


def quotedHTML(text,
               character_entities=(
                   (('&'), '&amp;'),
                   (("<"), '&lt;' ),
                   ((">"), '&gt;' ),
                   (('"'), '&quot;'))): #"
    text=str(text)
    for re,name in character_entities:
        if text.find(re) >= 0: text=name.join(text.split(re))
    return text


class StringPropertyStorage(PropertyStorage):
    """A StringPropertyStorage Object """

    property_type = 'string'
    storage_type="String"
    storage_group="Text"
    property_cols=40
    value_type='string'

    _default_property_config=HTML("""\
        <dtml-with "region">
        <table width="100%" cellspacing="0" cellpadding=2">
          <tr><td colspan="2" class="form-label">Mode</td></tr>
          <tr class="EProw-hilite">
            <td>
              <input type="hidden" name="deletable:int:missing" value="0">
              <input type="checkbox" name="deletable:int" value="1" <dtml-if "'d' in property_mode">checked</dtml-if>>
            </td>
            <td class="form-label" width="100%">Deletable</td>
          </tr>
          <tr class="EProw-hilite">
            <td>
              <input type="hidden" name="writtable:int:missing" value="0">
              <input type="checkbox" name="writtable:int" value="1" <dtml-if "'w' in property_mode">checked</dtml-if>>
            </td>
            <td class="form-label" width="100%">Writable</td>
          </tr>
        </table>
        <table width="100%" cellspacing="0" cellpadding=2">
          <tr><td colspan="2" class="form-label">Size</td></tr>
          <tr class="EProw-hilite">
            <td class="form-label" width="170">Columns</td>
            <td><input type="text" name="property_cols:int" value="&dtml.missing-property_cols;"></td>
          </tr>
        </table>
        </dtml-with region>
        """, globals())

    def render_html_view(self, editablecontainer, region, REQUEST=None):
        value=self.manage_load(editablecontainer, region, self.getDefault(region), REQUEST=REQUEST)
        value=quotedHTML(value)
        return self.do_html_view(editablecontainer, region, id=region.id, value=value)

    def do_html_view(self, editablecontainer, region, defaults=None, **kw):
        REQUEST=getattr(editablecontainer ,'REQUEST', None)
        size=getattr(region,'property_cols', self.property_cols)
        if defaults is None:
            defaults={'size':size,}
        defaults.update(kw)
        return """<input type="text" name="%(id)s:string" value="%(value)s" size="%(size)s">""" % defaults

    def manage_save_storage_conf(self, region, REQUEST=None):
        property_mode=''
        if REQUEST.get('writtable', 0):
            property_mode=property_mode+'w'
        if REQUEST.get('deletable', 0):
            property_mode=property_mode+'d'
        region.property_mode = property_mode

        region.property_cols = REQUEST.get('property_cols',getattr(region,'property_cols', self.property_cols))


Globals.InitializeClass(StringPropertyStorage)




class TextPropertyStorage(StringPropertyStorage):
    """A TextPropertyStorage Object """

    property_type = 'text'
    property_cols=40
    property_rows_max=10
    property_rows_min=10
    property_rows_add=3

    storage_type="Text"
    storage_group="Text"
    value_type='text'

    _default_property_config=HTML("""\
        <dtml-with "region">
        <table width="100%" cellspacing="0" cellpadding="2">
          <tr><td colspan="2" class="form-label">Mode</td></tr>
          <tr class="EProw-hilite">
            <td>
              <input type="hidden" name="deletable:int:missing" value="0">
              <input type="checkbox" name="deletable:int" value="1" <dtml-if "'d' in property_mode">checked</dtml-if>>
            </td>
            <td class="form-label" width="100%">Deletable</td>
          </tr>
          <tr class="EProw-hilite">
            <td>
              <input type="hidden" name="writtable:int:missing" value="0">
              <input type="checkbox" name="writtable:int" value="1" <dtml-if "'w' in property_mode">checked</dtml-if>>
            </td>
            <td class="form-label" width="100%">Writable</td>
          </tr>
        </table>
        <table width="100%" cellspacing="0" cellpadding=2">
          <tr><td colspan="2" class="form-label">Size</td></tr>
          <tr class="EProw-hilite">
            <td class="form-label" width="170">Columns</td>
            <td><input type="text" name="property_cols:int" value="&dtml.missing-property_cols;"></td>
          </tr>
          <tr class="EProw-hilite">
            <td class="form-label" width="170">Rows (max)</td>
            <td><input type="text" name="property_rows_max:int" value="&dtml.missing-property_rows_max;"></td>
          </tr>
          <tr class="EProw-hilite">
            <td class="form-label" width="170">Rows (min)</td>
            <td><input type="text" name="property_rows_min:int" value="&dtml.missing-property_rows_min;"></td>
          </tr>
          <tr class="EProw-hilite">
            <td class="form-label" width="170">Rows (add to lines)</td>
            <td><input type="text" name="property_rows_add:int" value="&dtml.missing-property_rows_add;"></td>
          </tr>
        </table>
        </dtml-with region>
        """, globals())


    def render_html_view(self, editablecontainer, region, REQUEST=None):
        value=str( self.manage_load(editablecontainer, region, self.getDefault(region), REQUEST=REQUEST) )
        return self.do_html_view(editablecontainer, region, id=region.id, value=value)

    def do_html_view(self, editablecontainer, region, defaults=None, **kw):
        rows_max=getattr(region, 'property_rows_max', self.property_rows_max)
        rows_min=getattr(region, 'property_rows_min', self.property_rows_min)
        rows_add=getattr(region, 'property_rows_add', self.property_rows_add)
        cols=getattr(region,'property_cols', self.property_cols)
        rows=len(kw['value'].split('\n'))+rows_add
        rows=rows>rows_max and rows_max or rows
        rows=rows<rows_min and rows_min or rows
        if defaults is None:
            defaults={'cols':cols,'rows':rows}
        defaults.update(kw)
        return """\
            <textarea
                name="%(id)s"
                wrap="off"
                cols="%(cols)s"
                rows="%(rows)s"
                style="width:100%%">%(value)s</textarea>""" % defaults


    def manage_save_storage_conf(self, region, REQUEST=None):
        property_mode=''
        if REQUEST.get('writtable', 0):
            property_mode=property_mode+'w'
        if REQUEST.get('deletable', 0):
            property_mode=property_mode+'d'
        region.property_mode = property_mode

        region.property_cols = REQUEST.get('property_cols',getattr(region,'property_cols', self.property_cols))
        max_v = REQUEST.get('property_rows_max',getattr(region,'property_rows_max', self.property_rows_max))
        min_v = REQUEST.get('property_rows_min',getattr(region,'property_rows_min', self.property_rows_min))
        add_v = REQUEST.get('property_rows_add',getattr(region,'property_rows_add', self.property_rows_add))
        max_v=max(max_v, min_v)
        min_v=min(max_v, min_v)
        add_v=min(min_v, add_v)
        region.property_rows_max = max_v
        region.property_rows_min = min_v
        region.property_rows_add = add_v





Globals.InitializeClass(TextPropertyStorage)

def initialize(context):
    """Initialize PropertyStorage"""

    context.registerEasyStorage(StringPropertyStorage,   'StringPropertyStorage')
    context.registerEasyStorage(TextPropertyStorage,     'TextPropertyStorage')
