# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyFile/EasyExcelFile.py,v 1.1 2004/07/19 11:55:05 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyExcelFile Object

EasyExcelFile.py,v 1.8 2001/09/29 12:00:00 lennart Exp $"""
__version__='$Revision: 1.1 $'[11:-2]


import Globals
import string
import os
from types import TupleType, ListType
from Acquisition import Implicit, aq_base, aq_inner, aq_parent, aq_chain
from Globals import DTMLFile
from webdav.common import rfc1123_date
from DateTime import DateTime
from OFS.Image import File
from Products.EasyCore.EasyCorePermissions import *
from Products.EasyCore import getIconpath, renderIconTag
from Products.EasyCore.Utils.Utils import get_filename
from Products.EasyFile.EasyFile import EasyFile

from OFS.content_types import guess_content_type
from Products.EasyCore.Utils.Tutils import filename2id_title, getValidId

from AccessControl import ClassSecurityInfo

from webdav.WriteLockInterface import WriteLockInterface
from ZPublisher import HTTPRangeSupport
from zLOG import LOG, WARNING
from webdav.Lockable import ResourceLockedError
from OFS.content_types import guess_content_type
from AccessControl.Permissions import access_contents_information, view


from Products.EasyResources.EasyResource import EasyResource
from Products.EasyCore.ManageOptionBase import ManageOptionBase
from Products.EasyDublinCore.EasyDublinCore import EasyDublinCore
from Products.EasyCore.VirtualPath import VirtualPathBase

from MSExcelXMLParser import xml_parser






class EasyExcelFile(ManageOptionBase, EasyResource, EasyDublinCore, VirtualPathBase, File):
    """A EasyExcelFile Object """

    meta_type = 'EasyExcelFile'
    fixed_content_type=1
    content_type='application/excel'

    security = ClassSecurityInfo()

    __catalogs__=('EasyResources',)
    __implements__ = (WriteLockInterface, HTTPRangeSupport.HTTPRangeInterface)

    column_names=[]
    use_first_row=1
    parsed_data=[]


    manage_options_left = ( {'label':'Edit',            'action': 'manage_main' },)
    manage_options_mid = File.manage_options[2:3]+(
                           {'label':'View/Download',   'action': ''            },)
    manage_options_right = File.manage_options[3:]
    use_manage_options = ((manage_options_left, manage_options_mid, manage_options_right),)

    def __init__(self, id, file
                    , title=''
                    , category=()
                    , subject=()
                    , description=''
                    , language=''
                    , content_type='application/excel'
                ):
        EasyDublinCore.__init__(self, category=category
                                    , subject=subject
                                    , description=description
                                    , language=language
                                )
        File.__init__(self, id, title, file, content_type=content_type)



    #def getIcon(self):
    #    if getattr(self, 'disable_dynamic_icons', 0):
    #        return self.icon
    #    return getIconpath(self.content_type, self.icon)

    #security.declareProtected(access_contents_information, 'renderIconTag')
    #def renderIconTag(self, *args, **kw):
    #    return apply(renderIconTag, (self.content_type, self.icon)+args, kw)


    #MetaDate API


    def setContentType(self, content_type=None):
        self.content_type='application/excel'

    def updateContentType(self):
        self.content_type='application/excel'

    security.declareProtected(View, 'useFirstRowAsColumnNames')
    def useFirstRowAsColumnNames(self):
        return self.use_first_row

    security.declareProtected(EasyFileChange, 'manage_main')
    manage_main = DTMLFile('dtml/manage_editEasyExcelFileForm', globals())

    security.declareProtected(EasyFileChange, 'manage_edit')
    def manage_edit(self, title
                    , column_names=None
                    , use_first_row=0
                    , filedata=None
                    , REQUEST=None):
        """\
        Changes the title and content type attributes of the File or Image.
        """

        if self.wl_isLocked():
            raise ResourceLockedError, "File is locked via WebDAV"

        self.title=str(title)

        if filedata is not None:
            self.update_data(filedata, size=len(filedata))
        else:
            self.ZCacheable_invalidate()
            self.parse_file()

        if column_names is not None:
            self.column_names=map(string.strip, column_names)

        self.use_first_row=use_first_row

        if REQUEST is not None:
            message="Saved changes."
            try:    url=self.DestinationURL()
            except: url=REQUEST['URL1']
            REQUEST.RESPONSE.redirect('%s/manage_main?manage_tabs_message=%s' % (url,message))

    security.declareProtected(EasyFileChange, 'manage_upload')
    def manage_upload(self,file='',REQUEST=None):
        """\
        Replaces the current contents of the File or Image object with file.

        The file or images contents are replaced with the contents of 'file'.
        """
        if self.wl_isLocked():
            raise ResourceLockedError, "File is locked via WebDAV"
        data, size = self._read_data(file)

        self.update_data(data, size=size)


        if REQUEST is not None:
            message="Saved changes."
            try:    url=self.DestinationURL()
            except: url=REQUEST['URL1']
            REQUEST.RESPONSE.redirect('%s/manage_main&manage_tabs_message=%s' % (url,message))


    def PUT(self, REQUEST, RESPONSE):
        """\
        Handle HTTP PUT requests
        """
        self.dav__init(REQUEST, RESPONSE)
        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        file=REQUEST['BODYFILE']
        data, size = self._read_data(file)

        self.update_data(data, size=size)


        RESPONSE.setStatus(204)
        return RESPONSE

    #Database API

    def update_data(self, data, content_type=None, size=None):
        File.update_data(self, data, content_type=content_type, size=size)
        self.parse_file()

    security.declarePrivate('parse_ms_excel_xml')
    def parse_file(self):
        """\
        """
        parsed_data = xml_parser(aq_base(self.data.data))
        parsed_data=parsed_data[0]
        if self.use_first_row:
            self.column_names=map(str, parsed_data[0])
            parsed_data=parsed_data[1:]
        self.parsed_data=parsed_data

    def getColumns(self):
        return self.column_names


    #TinyTablePlus Interface

    def cols_text(self):
        raise "Not Implemented"

    def data_text(self):
        raise "Not Implemented"


    def __call__(self, *args, **kargs):
        return self.parsed_data
    getRows = __call__

    def getAllRows(self):
        return self.parsed_data

    def delRows(self, *args, **kargs):
        raise "Not Implemented"

    def delAllRows(self):
        raise "Not Implemented"

    def setRow(self, *args, **kw):
        raise "Not Implemented"


    # SEQUENCE INTERFACE
    # Add different behaviour for interger index
    # and string index.

    def __len__(self):
        return len(self.parsed_data)

    def __getitem__(self, key):
        return self.parsed_data[key]

    def __setitem__(self, key, value):
        self.parsed_data[key]=value
        self._p_changed=1

    def __delitem__(self, key):
        del self.parsed_data[key]
        self._p_changed=1

    def __getslice__(self, low, high):
        return self.parsed_data[low:high]

    def __setslice__(self, low, high, seq):
        self.parsed_data[low:high]=seq
        self._p_changed=1

    def __delslice__(self, low, high):
        del self.parsed_data[low:high]
        self._p_changed=1


    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """\

        """
        if hasattr(aq_base(self),'filename'):
            # Upgrading from 1.4's ExtFile based to 1.5's File based EasyImage
            parent = self.aq_parent
            filename = get_filename( ['var','reposit'], base.filename)
            file = open(filename, 'rb')
            data = file.read()
            file.close()
            manage_addEasyFile=parent.manage_addProduct['EasyFile'].manage_addEasyFile
            manage_addEasyFile(parent, 'tmpimg', data, base.title)
            parent._delObject(self.id)
            parent.manage_renameObject('tmpimg',base.id)
            changed=1

        if getattr(self,'_object_version', None)!='1.7': self._object_version = '1.7'
        if REQUEST is not None:
            return "<h1>Done</h1>"
        return changed

Globals.InitializeClass(EasyExcelFile)



manage_addEasyExcelFileForm=DTMLFile('dtml/manage_addEasyExcelFileForm', globals())

def manage_addEasyExcelFile(self, id, file
                        , title=''
                        , category=()
                        , subject=()
                        , description=''
                        , language=''
                        , REQUEST=None
                        ):
    """
    Add a new EasyExcelFile object.

    Creates a new ExcelFile object 'id' with the contents of 'file'.
    """
    filename=getattr(file, 'filename', id)

    id, title_deprecated = filename2id_title(filename, id, title, cut_extension=0)
    id=getValidId(self, id)
    fileObj=EasyExcelFile(id, file, title
                      , category=category
                      , subject=subject
                      , description=description
                      , language=language
                       )
    self._setObject(id, fileObj)
    fileObj = self._getOb(id)
    fileObj.manage_upload(file, REQUEST=REQUEST)


    if REQUEST is not None:
        try:    url=self.DestinationURL()
        except: url=REQUEST['URL1']
        REQUEST.RESPONSE.redirect('%s/manage_main' % (url,))
