# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyFile/ExcelStorage.py,v 1.1 2004/07/19 11:58:04 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyExcelFile Object

EasyExcelFile.py,v 1.8 2001/09/29 12:00:00 lennart Exp $"""
__version__='$Revision: 1.1 $'[11:-2]


import Globals
from BTrees.IOBTree import IOBTree
from Acquisition import Implicit, aq_base, aq_inner, aq_parent, aq_chain
from types import TupleType, ListType, IntType
from Persistence import Persistent
from Globals import DTMLFile
from OFS.SimpleItem import SimpleItem


class Sheets(SimpleItem):

    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, sheets=[]):
        self._sheets_names={}
        self._sheets=IOBTree()
        for i, sheet_data in enumerate(sheets):
            self._sheets[i]=Sheet(sheet_data)

    def addSheet(self, name=None, sheet_data=None):
        if type(sheet_data) in [ListType, TupleType]:
            sheet_data=Sheet(sheet_data)
        if name is None:
            name=len(self._sheets)
        if type(name)!=IntType:
            key=self._sheets_names.get(name, len(self._sheets))
            self._sheets_names.set(name, key)
            self._p_changed=1
        else: key=name
        self._sheets[key]=Sheet(sheet_data)

    def __len__(self):
        return len(self._sheets)

    def __getitem__(self, key):
        return self._sheets[key]

    def __setitem__(self, key, value):
        self._sheets[key]=value
        #self._p_changed=1

    def __delitem__(self, key):
        del self._sheets[key]
        #self._p_changed=1

Globals.InitializeClass(Sheets)


class Sheet(SimpleItem):

    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self):
        self._rows=IOBTree()

    def addRow(self, row):
        if type(row) in [ListType, TupleType]:
            row=Row(row)
        self._rows.add(row)

    def __len__(self):
        return len(self._rows)

    def __getitem__(self, key):
        return self._rows[key]

    def __setitem__(self, key, value):
        self._rows[key]=value
        #self._p_changed=1

    def __delitem__(self, key):
        del self._rows[key]
        #self._p_changed=1

Globals.InitializeClass(Sheet)



class Row(SimpleItem):

    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, data):
        self._data=IOBTree()
        #Python 2.3
        for i, item in enumerate(data):
            self._data[i]=item

    def __len__(self):
        return len(self._data)

    def __getitem__(self, key):
        return self._data[key]

    def __setitem__(self, key, value):
        self._data[key]=value
        self._p_changed=1

    def __delitem__(self, key):
        del self._data[key]
        self._p_changed=1

Globals.InitializeClass(Row)

