# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyImage/EasyImage.py,v 1.1 2004/06/28 10:05:45 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__="""EasyImage Object"""
__version__='$Revision: 1.1 $'[11:-2]


import Globals
import string
from types import IntType,TupleType, ListType
from OFS.Image import Image, Pdata, getImageInfo
from Globals import DTMLFile
from ComputedAttribute import ComputedAttribute
from Acquisition import aq_base
from Products.EasyWorkflow.EasyWorkflowBase import NullDate
from Products.EasyResources.EasyResource import EasyResource
from Products.EasyWorkflow.EasyWorkflowBase import NullDate
from Products.EasyDublinCore.EasyDublinCore import EasyDublinCore
from Products.EasyCore import getIconpath, renderIconTag
from Products.EasyCore.VirtualPath import VirtualPathBase
from Products.EasyCore.EasyCorePermissions import *
from Products.EasyCore.Utils.Utils import get_filename
from AccessControl.Permissions import access_contents_information, view
from AccessControl import ClassSecurityInfo
from Products.EasyCore.ManageOptionBase import ManageOptionBase

class ImageClassProxy(EasyResource, EasyDublinCore, Image):
    pass

from zLOG import LOG, INFO, WARNING


from webdav.common import rfc1123_date
import os
try:
    import PIL
    import PIL.PngImagePlugin
    import PIL.ImageFile
    PIL.ImageFile.MAXBLOCK = 1000000 # default is 64k
    from StringIO import StringIO
    _PIL_is_present = 1
except:
    _PIL_is_present = 0
    LOG("EasyImage", INFO, "PIL (Python Image Library) is not installed on this system")

# This maps a content type to an internal type.
CONTENTTYPEMAP = { 'image/pjpeg':   'JPEG',     #Progressive JPEG
                   'image/jpeg':    'JPEG',
                   'image/bmp':     'BMP',
                   'image/png':     'PNG',
                   'image/gif':     'GIF',
                   'image/tif':     'TIFF',
                   'image/tiff':    'TIFF',
                 }

# This contains info on each internal type. (display-name, extension, content_type)
TYPEINFOMAP =  { 'JPEG':     { 'name':'JPEG', 'content_type': 'image/jpeg', },
                 'GIF':      { 'name':'GIF',  'content_type': 'image/gif', },
                 'BMP':      { 'name':'BMP',  'content_type': 'image/bmp', },
                 'PNG':      { 'name':'PNG',  'content_type': 'image/png', },
                 'TIFF':     { 'name':'TIFF', 'content_type': 'image/tif', },
               }

# This map show possible conversions from each internal type:
CONVERSIONMAP = { 'JPEG': ('PNG', 'TIFF', 'BMP'),
                  'GIF':  ('JPEG', 'PNG', 'TIFF', 'BMP'),
                  'BMP':  ('JPEG', 'PNG', 'TIFF',),
                  'PNG':  ('JPEG', 'TIFF', 'BMP'),
                  'TIFF': ('JPEG', 'PNG', 'BMP',),
                }

AMBIGUOUS_CONTENT_TYPES=(
      'application/octet-stream'
    , 'text/x-unknown-content-type'
    )


class EasyImage(ManageOptionBase, VirtualPathBase, ImageClassProxy):
    """A EasyImage Object """

    security = ClassSecurityInfo()

    meta_type = 'EasyImage'

    description = ""
    keywords = ""
    category = ""

    __catalogs__=('EasyResources',)

    manage_options_image = ({'label':'Edit',            'action': 'manage_main'       },)\
                            + Image.manage_options[2:3]

    manage_options_convertresize = ({'label':'Convert/Resize',  'action': 'manage_convertResizeForm'},)

    manage_options_inherited =  ({'label':'View/Download',   'action': 'view_image_or_file'}, )\
                                + Image.manage_options[3:]




    if _PIL_is_present == 1:
        #manage_options = manage_options_image + manage_options_convertresize + manage_options_inherited
        use_manage_options = ((manage_options_image, manage_options_convertresize, manage_options_inherited),)
        PIL_is_present = 1
    else:
        #manage_options = manage_options_image + manage_options_inherited
        use_manage_options = ((manage_options_image, None, manage_options_inherited),)
        PIL_is_present = 0


    _properties = EasyDublinCore._properties + \
            (   {'id':'picture_text', 'type': 'text', 'mode': 'w'},
                {'id':'photographer', 'type': 'string', 'mode': 'w'},
                {'id':'publisher', 'type': 'string', 'mode': 'w'}, )

    #Making empty defaults:
    picture_text = ''
    photographer = ''
    publisher = ''

    def __init__(self, id, file, title=''
                     , category=(), subject=(), description='', language=''
                     , picture_text='', photographer='', publisher=''
                     , content_type='', precondition=''
                ):
        EasyDublinCore.__init__(self, category=category
                                    , subject=subject
                                    , description=description
                                    , language=language
                                )
        self.has_preview = 0
        self.orig_data = None
        self.orig_height = self.orig_width = self.orig_size = None
        self.prev_data = None
        self.prev_height = self.prev_width = self.prev_size = None
        Image.__init__(self, id, title, file, content_type=content_type, precondition=precondition)
        self.picture_text=picture_text
        self.photographer=photographer
        self.publisher=publisher

    def _isPrincipiaFolderish(self):
        return getattr(self, 'show_files_in_tree', 0)
    isPrincipiaFolderish= ComputedAttribute(_isPrincipiaFolderish,1)


    def getIcon(self):
        if getattr(self, 'disable_dynamic_icons', 0):
            return self.icon
        return getIconpath(self.content_type, self.icon)

    security.declareProtected(access_contents_information, 'renderIconTag')
    def renderIconTag(self, *args, **kw):
        return apply(renderIconTag, (self.content_type, self.icon)+args, kw)
        #return renderIconTag(self.content_type, self.icon)

    security.declareProtected(access_contents_information, 'getImageInfoPIL')
    def getImageInfoPIL(self, data):
        file = StringIO(str(data))
        im = PIL.Image.open(file)
        size = im.size
        ct = TYPEINFOMAP[im.format]['content_type']
        return ct, size[0], size[1]

    security.declareProtected(EasyImageChange, 'update_data')
    def update_data(self, data, content_type=None, size=None):
        if size is None: size=len(data)

        self.size=size
        self.data=data

        if _PIL_is_present == 0:
                ct, width, height = getImageInfo(data)
        else:
                ct, width, height = self.getImageInfoPIL(data)

        if ct:
            content_type = ct
        if width >= 0 and height >= 0:
            self.width = width
            self.height = height

        # Now we should have the correct content type, or still None
        if content_type is not None: self.content_type = content_type

        self.ZCacheable_invalidate()


    security.declareProtected(EasyImageChange, 'manage_upload')
    def manage_upload(self,file='',REQUEST=None):
        """
        Replaces the current contents of the File or Image object with file.

        The file or images contents are replaced with the contents of 'file'.
        """
        Image.manage_upload(self, file, REQUEST)

        self.orig_data = None
        self.orig_height = self.orig_width = self.orig_size = None
        self.prev_data = None
        self.prev_height = self.prev_width = self.prev_size = None

        if self.has_preview:
            create_prev = 1 # Create a preview if there already is one.
        else: # Check for defaults
            ER = getattr(self, 'EasyResources', None)
            if ER == None:
                create_prev = 1
            else:
                create_prev = getattr(ER, 'default_preview_create', 1)

        if create_prev:
            self.manage_create_prev( ratio=1)
        else:
            self.has_preview = 0

        if REQUEST is not None:
            message = "Image Uploaded"
            return self.manage_main(
                self,
                REQUEST,
                manage_tabs_message=message,
                management_view='Edit')


    security.declareProtected(EasyImageChange, 'manage_create_prev')
    def manage_create_prev(self, maxx=0, maxy=0, ratio=0, REQUEST=None):
        """ Creates the preview."""

        if not _PIL_is_present:
            if REQUEST is not None:
                message = "Image library not installed"
                return self.manage_main(
                    self,
                    REQUEST,
                    manage_tabs_message=message,
                    management_view='Edit')
            return 0

        maxx = int(maxx)
        maxy = int(maxy)

        if maxx == 0 or maxy == 0:
            # Find the default preview setting, or if it doesn't have one use 100
            defmaxx = defmaxy = 0
            ER = getattr(self, 'EasyResources', None)
            if ER != None:
                defmaxy = getattr(ER, 'default_preview_height', 100)
                defmaxx = getattr(ER, 'default_preview_width', 100)

            if maxx == 0: maxx = defmaxx or 100
            if maxy == 0: maxy = defmaxy or 100

        self.prev_data, self.prev_type, self.prev_width, self.prev_height = \
            self.makeimage( maxx, maxy, ratio)
        self.prev_size = len(self.prev_data)
        if self.prev_size <> 0:
            self.has_preview = 1

        if REQUEST is not None:
            message = "Preview created"
            return self.manage_main(
                self,
                REQUEST,
                manage_tabs_message=message,
                management_view='Edit')
        return 1


    security.declareProtected(EasyImageChange, 'manage_del_prev')
    def manage_del_prev(self, REQUEST=None):
        """Deletes the preview image"""
        self.has_preview = 0
        self.prev_size = 0
        self.prev_type = ''
        self.prev_data = None
        self.prev_height = None
        self.prev_width = None

        if REQUEST is not None:
            message = "Preview deleted"
            return self.manage_main(
                self,
                REQUEST,
                manage_tabs_message=message,
                management_view='Edit')


    security.declareProtected(EasyImageChange, 'manage_revert')
    def manage_revert(self, REQUEST=None):
        """Reverts to the original image"""
        self.data = self.orig_data
        self.content_type = self.orig_type
        self.size = self.orig_size
        self.width = self.orig_width
        self.height = self.orig_height
        self.orig_size = 0
        self.orig_type = ''
        self.orig_data = None
        self.orig_height = 0
        self.orig_width = 0

        if REQUEST is not None:
            message = "Reverted to original"
            return self.manage_main(
                self,
                REQUEST,
                manage_tabs_message=message,
                management_view='Edit')


    security.declarePrivate('set_header')
    def set_header(self, REQUEST, RESPONSE, content_type, size):
        # HTTP If-Modified-Since header handling.
        header=REQUEST.get_header('If-Modified-Since', None)
        if header is not None:
            header=string.split(header, ';')[0]
            # Some proxies seem to send invalid date strings for this
            # header. If the date string is not valid, we ignore it
            # rather than raise an error to be generally consistent
            # with common servers such as Apache (which can usually
            # understand the screwy date string as a lucky side effect
            # of the way they parse it).
            try:    mod_since=long(DateTime(header).timeTime())
            except: mod_since=None
            if mod_since is not None:
                if self._p_mtime:
                    last_mod = long(self._p_mtime)
                else:
                    last_mod = long(0)
                if last_mod > 0 and last_mod <= mod_since:
                    # Set header values since apache caching will return Content-Length
                    # of 0 in response if size is not set here
                    RESPONSE.setHeader('Last-Modified', rfc1123_date(self._p_mtime))
                    RESPONSE.setHeader('Content-Type', content_type)
                    RESPONSE.setHeader('Content-Length', size)
                    RESPONSE.setStatus(304)
                    return 304
        return 0


    security.declareProtected(view, 'index_html')
    def index_html(self, REQUEST=None, RESPONSE=None, preview=0, original=0):
        """\
        The default view of the contents of a File or Image.

        Returns the contents of the file or image.  Also, sets the
        Content-Type HTTP header to the objects content type.
        """
        if REQUEST is None: REQUEST=getattr(self, 'REQUEST')
        if RESPONSE is None: RESPONSE=getattr(REQUEST, 'RESPONSE')

        if preview == 1 and self.has_preview:
            ct = self.prev_type
            size = self.prev_size
            data = self.prev_data
        elif original == 1 and self.orig_data <> None:
            ct = self.orig_type
            size = self.orig_size
            data = self.orig_data
        else:
            ct = self.content_type
            size = self.size
            data = self.data

        self.set_header(REQUEST, RESPONSE, ct, size)

        if self.precondition and hasattr(self,self.precondition):
            # Grab whatever precondition was defined and then
            # execute it.  The precondition will raise an exception
            # if something violates its terms.
            c=getattr(self,self.precondition)
            if hasattr(c,'isDocTemp') and c.isDocTemp:
                c(REQUEST['PARENTS'][1],REQUEST)
            else:
                c()

        RESPONSE.setHeader('Last-Modified', rfc1123_date(self._p_mtime))
        RESPONSE.setHeader('Content-Type', ct)
        RESPONSE.setHeader('Content-Length', size)

        # Don't cache the data itself, but provide an opportunity
        # for a cache manager to set response headers.
        self.ZCacheable_set(None)

        if type(data) is type(''): return data

        while data is not None:
            RESPONSE.write(data.data)
            ndata= data.next
            data._p_deactivate()
            data= ndata

        return ''


    security.declareProtected(view, 'tag')
    def tag(self, preview=0, original=0, height=None, width=None, alt=None,
            scale=0, xscale=0, yscale=0, css_class=None, **args):
        """ Generate an HTML IMG tag for this image, with customization."""
        if preview and self.has_preview and self.prev_width and self.prev_height:
            x = self.prev_width
            y = self.prev_height
            u = self.absolute_url() + '/preview'
        elif original and self.orig_width and self.orig_height:
            x = self.orig_width
            y = self.orig_height
            u = self.absolute_url() + '/original'
        else:
            x = self.width
            y = self.height
            u = self.absolute_url()


        if height is None: height=y
        if width is None:  width=x

        # Auto-scaling support
        xdelta = xscale or scale
        ydelta = yscale or scale

        if xdelta and width:
            width =  str(int(round(int(width) * xdelta)))
        if ydelta and height:
            height = str(int(round(int(height) * ydelta)))

        result='<img src="%s"' % (u)

        if alt is None:
            alt=getattr(self, 'title', '')
        result = '%s alt="%s"' % (result, alt)

        if height:
            result = '%s height="%s"' % (result, height)

        if width:
            result = '%s width="%s"' % (result, width)

        if not 'border' in map(string.lower, args.keys()):
            result = '%s border="0"' % result

        if css_class is not None:
            result = '%s class="%s"' % (result, css_class)

        for key in args.keys():
            value = args.get(key)
            result = '%s %s="%s"' % (result, key, value)

        return '%s />' % result


    security.declareProtected(view, 'preview')
    def preview(self, REQUEST=None, RESPONSE=None):
        """ return a preview of the image """
        return self.index_html( REQUEST, RESPONSE, preview=1 )

    security.declareProtected(view, 'preview_tag')
    def preview_tag(self, **kw):
        """Makes an IMG-tag for the preview"""
        kw['preview']=1
        return apply(self.tag, (), kw)

    security.declareProtected(view, 'preview_html')
    preview_html=preview_tag

    security.declareProtected(view, 'original')
    def original(self, REQUEST=None, RESPONSE=None):
        """ return a preview of the image """
        return self.index_html( REQUEST, RESPONSE, original=1 )

    security.declareProtected(view, 'original_tag')
    def original_tag(self, **kw):
        """Makes an IMG-tag for the original"""
        kw['original']=1
        return apply(self.tag, (), kw)

    security.declareProtected(view, 'original_html')
    original_html=original_tag

    #hasOriginal or hasPreview
    security.declareProtected(access_contents_information, 'hasOriginal')
    def hasOriginal(self):
        return self.orig_data is not None

    security.declareProtected(access_contents_information, 'hasPreview')
    def hasPreview(self):
        return self.has_preview

    #Get Image Data
    #
    #Width
    security.declareProtected(access_contents_information, 'getWidth')
    def getWidth(self):
        return self.width

    security.declareProtected(access_contents_information, 'getPreviewWidth')
    def getPreviewWidth(self):
        return self.prev_width

    security.declareProtected(access_contents_information, 'getOriginalWidth')
    def getOriginalWidth(self):
        return self.orig_width

    #Height
    security.declareProtected(access_contents_information, 'getHeight')
    def getHeight(self):
        return self.height

    security.declareProtected(access_contents_information, 'getPreviewHeight')
    def getPreviewHeight(self):
        return self.prev_height

    security.declareProtected(access_contents_information, 'getOriginalHeight')
    def getOriginalHeight(self):
        return self.orig_height

    #AbsoluteURL
    security.declareProtected(access_contents_information, 'getAbsoluteURL')
    def getAbsoluteURL(self):
        return self.absolute_url()

    security.declareProtected(access_contents_information, 'getPreviewAbsoluteURL')
    def getPreviewAbsoluteURL(self):
        return self.absolute_url() + '/preview'

    security.declareProtected(access_contents_information, 'getOriginalAbsoluteURL')
    def getOriginalAbsoluteURL(self):
        return self.absolute_url() + '/original'


    #Event API

    def manage_afterClone(self, item):
        Image.manage_afterClone(self, item)

    def manage_afterAdd(self, item, container):
        self.updateContentType()
        Image.manage_afterAdd(self, item, container)
        if hasattr(self, 'index_object'):
            self.index_object()

    def manage_beforeDelete(self, item, container):
        Image.manage_beforeDelete(self, item, container)
        self.unindex_object()

    def updateContentType(self):
        others=getattr(self, 'ambiguous_content_types', [])
        if type(others) not in (TupleType, ListType): others=[others]
        if self.content_type in AMBIGUOUS_CONTENT_TYPES or\
                self.content_type in others:
            content_type, content_enc=guess_content_type(self.getId(), self.data[:100], None)
            if content_type is not None:
                self.content_type=content_type

    security.declareProtected(EasyImageChange, 'manage_main')
    manage_main = DTMLFile('dtml/manage_editEasyImageForm', globals())

    security.declareProtected(EasyImageChange, 'manage_convertResizeForm')
    manage_convertResizeForm = DTMLFile('dtml/manage_convertResizeEasyImageForm', globals())

    security.declareProtected(EasyImageChange, 'manage_resize_image')
    def manage_resize_image(self, maxx=None, maxy=None, ratio=1, REQUEST=None):
        """TTW: Resizing images"""
        self.check_original()
        message = ""
        if _PIL_is_present == 0:
            if REQUEST is not None:
                message = "Python Imaging Library is not installed. Resizing not supported."
                return self.manage_convertResizeForm(
                    self,
                    REQUEST,
                    manage_tabs_message=message,
                    management_view='Convert/Resize')
        try:  maxx = int(maxx)
        except: maxx = 0
        try:  maxy = int(maxy)
        except: maxy = 0
        if maxx != 0 or maxy != 0:
            self.data, self.content_type, self.width, self.height = self.makeimage(maxx, maxy, ratio, CONTENTTYPEMAP[self.content_type])
            message = "Image Resized."

        if REQUEST is not None:
            return self.manage_convertResizeForm(
                self,
                REQUEST,
                manage_tabs_message=message,
                management_view='Convert/Resize')

    security.declarePrivate('check_original')
    def check_original(self):
        if self.orig_data == None:
            self.orig_data = self.data
            self.orig_type = self.content_type
            self.orig_size = self.size
            self.orig_width = self.width
            self.orig_height = self.height


    security.declarePrivate('makeimage')
    def makeimage(self, maxx=0, maxy=0, ratio=1, ftype='JPEG'):
        """Creates an image from the orginal with specified size."""

        if self.orig_data == None:
            tmpdata = self.data
        else:
            tmpdata = self.orig_data

        file = StringIO(str(tmpdata))

        if maxx <> 0 or maxy <> 0:
            im = PIL.Image.open(file)
            width=im.size[0]
            height=im.size[1]
            if ratio==1 or (maxx==0 and maxy!=height) or (maxy==0 and maxy!=width):            # keep aspect-ratio
                if maxx!=0: aspectx = float(width) / float(maxx)
                else: aspectx = 0
                if maxy!=0: aspecty = float(height) / float(maxy)
                else: aspecty = 0
                if aspectx > aspecty:
                    maxy = float(height) / aspectx
                if aspectx < aspecty:
                    maxx = float(width) / aspecty
            maxx = int(maxx)
            maxy = int(maxy)
            if im.mode!='RGBA': im = im.convert("RGBA")
            im.load()
            if im.size[0]!=maxx or im.size[1]!=maxy:
                im = im.resize((maxx, maxy), PIL.Image.BICUBIC)

            newfile = StringIO()
            im.save(newfile, ftype, optimize=1)
            return newfile.getvalue(), TYPEINFOMAP[ftype]['content_type'], im.size[0], im.size[1]

        return None, '', 0, 0


    security.declarePrivate('get_conversion_types')
    def get_conversion_types(self, content_type=None):

        if content_type == None: content_type = self.content_type
        try:
            filetype = CONTENTTYPEMAP[content_type]
        except:
            filetype = content_type

        conversiontypes = CONVERSIONMAP[filetype]
        res = []
        for ct in conversiontypes:
            res = res + [(ct, TYPEINFOMAP[ct]['name'])]

        return res


    security.declareProtected(EasyImageChange, 'manage_convert_image')
    def manage_convert_image(self, new_format='', REQUEST=None):
        """TTW: Converting images"""
        self.check_original() # Make sure the original is kept.

        if new_format<>'':
            if _PIL_is_present == 0:
                if REQUEST is not None:
                    message = "Python Imaging Library is not installed. Conversions not supported."
                    return self.manage_convertResizeForm(
                        self,
                        REQUEST,
                        manage_tabs_message=message,
                        management_view='Convert/Resize')

            else:
                self.data, self.content_type, self.width, self.height = self.makeimage(self.width, self.height, ratio=0, ftype=new_format)

        if REQUEST is not None:
            message = "Image converted."
            return self.manage_convertResizeForm(
                self,
                REQUEST,
                manage_tabs_message=message,
                management_view='Convert/Resize')


    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        base = aq_base(self)
        # Upgrading from 1.4's ExtImage based to 1.5's Image based EasyImage
        if hasattr(base,'filename'):
            parent = self.aq_parent
            filename = get_filename( ['var','reposit'], base.filename)
            file = open(filename, 'rb')
            data = file.read()
            file.close()
            manage_addEasyImage=parent.manage_addProduct['EasyImage'].manage_addEasyImage
            manage_addEasyImage(parent, 'tmpimg', data, base.title)
            parent._delObject(base.id)
            parent.manage_renameObject('tmpimg', base.id)
            changed=1

        if getattr(self,'_object_version', None)!='1.7': self._object_version = '1.7'
        if REQUEST is not None:
            return "<h1>Done</h1>"
        return changed

Globals.InitializeClass(EasyImage)



