# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyLanguageService/EasyLanguageHelpers.py,v 1.2 2004/09/13 20:54:43 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#
__doc__='''EasyLanguageHelpers.'''
__version__='$Revision: 1.2 $'[11:-2]


from rfc3066_language_codes import *
from types import ListType, TupleType, StringType
from string import lower

def isSequense(seq):
    return type(seq) in (ListType, TupleType)

def uniqueValues(seq):
    d={}
    map(d.setdefault, seq)
    return d.keys()



def lowercase(items):
    return map(lambda id: lower(id), items)

def getLanguageItemsByCodes(lang_codes):
    """
    Returns all languages in lang_codes as (RFC3066, PlainText)
        lang_codes is a list of RFC3066 codes.

    TODO: Language Item should be sorted on the Languages Plain Text
          (language codes are not alphabetically ordered)
    """
    lang_codes=lowercase(lang_codes)
    keys=keysSortedOnItems(rfc_3066)
    lang_codes=filter(lambda x, keys=keys: x in keys, lang_codes)
    return map(lambda key, rfc_3066=rfc_3066: (key, rfc_3066[key]) , lang_codes)

def getAllLanguageIds(filter_codes=[]):
    """\
    Returns all languages as RFC3066 codes
    except for those in filter_codes
    filter_codes is a sequence of RFC3066 code.
    """
    if not isSequense(filter_codes): filter_codes=[]
    filter_codes=lowercase(filter_codes)
    keys=keysSortedOnItems(rfc_3066)
    keys=filter(lambda x, filter_codes=filter_codes: x not in filter_codes, keys)
    #keys.sort()
    return keys

def getAllLanguageValues(filter_codes=[]):
    """\
    Returns all languages as PlainText
    except for those in filter_codes

    filter_codes is a sequence of RFC3066 code.
    """
    if not isSequense(filter_codes): filter_codes=[]
    filter_codes=lowercase(filter_codes)
    keys=keysSortedOnItems(rfc_3066)
    keys=filter(lambda x, filter_codes=filter_codes: x not in filter_codes, keys)
    #keys.sort()
    return map(lambda key, rfc_3066=rfc_3066: rfc_3066[key] , keys)

def getAllLanguageItems(filter_codes=[]):
    """
    Returns all languages as (RFC3066, PlainText) except for the once in filter_codes.

    filter_codes is a sequence of RFC3066 code.
    """
    if not isSequense(filter_codes): filter_codes=[]
    filter_codes=lowercase(filter_codes)
    keys=keysSortedOnItems(rfc_3066)
    keys=filter(lambda x, filter_codes=filter_codes: x not in filter_codes, keys)
    #keys.sort()
    return map(lambda key, rfc_3066=rfc_3066: (key, rfc_3066[key]) , keys)

def getCountryItemsByCodes(country_codes):
    """Returns all languages in country_codes as (ISO639_1, PlainText)"""
    if not isSequense(country_codes): country_codes=[country_codes]
    country_codes=lowercase(country_codes)
    country_codes=filter(lambda code, has=iso_3166_1.has_key: has(code), country_codes)
    return map(lambda code, get=iso_3166_1.get: (code, get(code)), country_codes)

def getAllCountryIds(self, filter_codes=[]):
    """/
    Returns all countries as ISO639_1 codes
    except for those in filter_codes

    filter_codes is a sequence of ISO639_1 code.
    """
    if not isSequense(filter_codes): filter_codes=[filter_codes]
    filter_codes=lowercase(filter_codes)
    country_ids=filter(lambda code, filter_codes=filter_codes: code not in filter_codes, iso_3166_1.keys())
    return country_ids

def getAllCountryValues(self, filter_codes=[]):
    """\
    Returns all countries  as PlainText
    except for those in filter_codes

    filter_codes is a sequence of ISO639_1 code.
    """
    if not isSequense(filter_codes): filter_codes=[filter_codes]
    filter_codes=lowercase(filter_codes)
    country_ids=filter(lambda code, filter_codes=filter_codes: code not in filter_codes, iso_3166_1.keys())
    return map(lambda code, get=iso_3166_1.get: get(code), country_ids)

def getAllCountryItems(filter_codes=[]):
    """
    Returns all languages as (ISO639_1, PlainText) except for the once in filter_codes.

    filter_codes is a sequence of ISO639_1 code.
    """
    if not isSequense(filter_codes): filter_codes=[filter_codes]
    filter_codes=lowercase(filter_codes)
    country_ids=filter(lambda code, filter_codes=filter_codes: code not in filter_codes, iso_3166_1.keys())
    return map(lambda code, get=iso_3166_1.get: (code, get(code)), country_ids)


def getLanguageByRFC3066(code, default='English'):
    """Translates a language code to a human readable form in english.

    If a sequence of language codes is passed, a sequence of the corresponding
    languages is returns.
    """
    get=rfc_3066.get
    if isSequense(code):
        code=lowercase(code)
        return map(lambda x, d=default, get=get: get(x,d), code)
    else:
        return get(code, default)

def getLanguageByISO639_1(code, default='English'):
    """Translates a language code to a human readable form in english.

    If a sequence of language codes is passed, a sequence of the corresponding
    languages is returns.
    """
    get=iso_639_1.get
    if isSequense(code):
        code=lowercase(code)
        return map(lambda x, d=default, get=get: get(x[:2],d), code)
    else:
        return get(code[:2], default)


def getCountryByISO3166_1(code, default='No match'):
    """Translates a language code to a human readable form in english.

    If a sequence of language codes is passed, a sequence of the corresponding
    languages is returns.
    """
    get=iso_3166_1.get
    if isSequense(code):
        code=lowercase(code)
        return map(lambda x, d=default, get=get: get(x,d), code)
    else:
        return get(code, default)

def getRFC3066ByLanguage(language, default='en'):
    """Translates a language in a human readable form to the code.
    """
    # 1. Should be a re.search or something?
    # 2. default not in use!!
    return filter(lambda k,v,language=language: v==language, rfc_3066.items())

def getISO639_1ByLanguage(language, default='en'):
    """Translates a language in a human readable form to the code."""
    # 1. Should be a re.search or something?
    # 2. default not in use!!
    return filter(lambda k,v,language=language: v==language, iso_639_1.items())

def getISO3166_1ByCountry(country, default='en'):
    """Translates a language in a human readable form to the code."""
    # 1. Should be a re.search or something?
    # 2. default not in use!!
    if type(country) in (ListType, TupleType):
        country=country[0]
    codes=filter(lambda x, c=country: x[1].lower()==c.lower(), iso_3166_1.items())
    if codes:
        return codes[0][0]
    return default

def getISO3166_1ByCountryItems(country, default='en'):
    """Translates a list of languages into a list of tuples with (ISO-3166-1, human readable)."""
    # 1. Should be a re.search or something?
    # 2. default not in use!!
    if type(country) not in (ListType, TupleType):
        country=[country]
    country=map(lambda x: x.lower(), country)
    return filter(lambda item, country=country: item[1].lower() in country, iso_3166_1.items())


def getISO639_1ByRFC3066(code):
    """
    Returns the root language class of the language code.

    If a sequence of language codes is passed, a tuple of the corresponding
    languages is returns.
    """
    if isSequense(code):
        code=lowercase(code)
        d={}
        map(d.setdefault, map(lambda x: x[:2], code))
        return d.keys()
    else:
        return code[:2]

def validISO639_1(code):
    """
    Returnes true if this is a valid language code.
    """
    haskey=keysSortedOnItems(iso_639_1)
    if isSequense(code):
        code=lowercase(code)
        return map(lambda x, haskey=haskey: haskey(x), code)
    else:
        return haskey(code)

def validRFC3066(code):
    """
    Returnes true if this is a valid language code.
    """
    haskey=rfc_3066.has_key
    if isSequense(code):
        code=lowercase(code)
        return map(lambda x, haskey=haskey: haskey(x), code)
    else:
        return haskey(code)

def validISO3166_1(code):
    """
    Returnes true if this is a valid language code.
    """
    haskey=iso_3166_1.has_key
    if isSequense(code):
        code=lowercase(code)
        return map(lambda x, haskey=has_key: haskey(x), code)
    else:
        return haskey(code)

