# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyLanguageService/EasyLanguageService.py,v 1.3 2004/09/13 20:54:43 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#


__doc__='''EasyLanguageService product.'''
__version__='$Revision: 1.3 $'[11:-2]

import Globals
from types import ListType, TupleType, StringType
from Globals import DTMLFile
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from ZODB.PersistentMapping import PersistentMapping
from Products.EasyCore.Utils.PersistentList import PersistentList
from AccessControl import Permissions
from EasyLanguageHelpers import *
try: from Products.EasyWorkflow.EasyWorkflowBase import psPUBLISHED
except: psPUBLISHED="Published"

EasyLanguageServiceManage = 'Easy LanguageService Manage'

try:
    from Products.EasyCore.Utils.Tutils import distinct
except:
    def distinct(alist):    # Fastest order preserving
        set = {}
        return [set.setdefault(e,e) for e in alist if e not in set.keys()]

# General Zope permissions
View = Permissions.view
AccessContentsInformation = Permissions.access_contents_information

try:
    from Products.EasyNavigation.EasyNavigation import NavigationBrain
    import Acquisition
    import AccessControl.Role
    from ComputedAttribute import ComputedAttribute


    class LocalNavigationBrain(Acquisition.Implicit, NavigationBrain):

         __allow_access_to_unprotected_subobjects__=1

         def __init__(self, dict=None):
             if type(dict)!=type({}): dict={}
             for key,item in dict.items():
                 setattr(self,key,item)

         def _physicalPath(self):
             af_url=self.absolute_folder_url
             if af_url[-1]=='/': af_url=af_url[:-1]
             if af_url[0]!='/': af_url='/'+af_url
             return tuple(af_url.split('/'))
         physicalPath = ComputedAttribute(_physicalPath, 1)

         def _virtualPath(self):
             root_len=len(self.getVirtualRootPath)
             path=self._physicalPath()
             if len(path)>=root_len:
                 path=('',) + path[root_len:]
             return path
         virtualPath = ComputedAttribute(_virtualPath, 1)

         def _physicalURL(self):
             return self.absolute_folder_url
         physicalURL = ComputedAttribute(_physicalURL, 1)

         def _virtualURL(self):
             vpath=self._virtualPath()
             return '/'.join(vpath) + '/'
         virtualURL = ComputedAttribute(_virtualURL, 1)

         def _virtualParentPath(self):
             return self._virtualPath()[:-1]
         virtualParentPath = ComputedAttribute(_virtualParentPath, 1)

         def _physicalParentPath(self):
             return self._physicalPath()[:-1]
         physicalParentPath = ComputedAttribute(_physicalParentPath, 1)

         def has_key(self, key):
            return hasattr(self, key)


except:
    NavigationBrain=None


EASYLANGUAGESERVICE='EasyLanguageService'
DEFAULT_LANGUAGE='en'

_marker=[]



class LanguageException(Exception):
    pass

class EasyLanguageService(PropertyManager, SimpleItem):
    """
    The EasyLanguageService is use to keep information about
    language codes (RFS3066) and methods for choosing default
    language (from browser default) an make translations to
    human readable forms.

    Only ISO 639_1: 2-letter codes and combination of ISO 639: 2-letter codes
    and ISO 3166-1 country codes are currently supported.
    This is what is supported by Internet Explorer and possibly other browsers.

    EasyLanguageService also helps a web site register supported
    language versions and the languages base url.

    """

    security = ClassSecurityInfo()

    meta_type = EASYLANGUAGESERVICE
    isPrincipiaFolderish=1

    preferred_languages='preferred_languages'

    _properties = PropertyManager._properties + \
            (
                {'id':'preferred_languages', 'type': 'string', 'mode': 'w'},
            )

    manage_options_local =(
        {'label':'LanguageFolders', 'action':'manage_main'},
        {'label':'LanguageMappings', 'action':'manage_mappings'},
    )

    manage_options_property_manager = PropertyManager.manage_options
    manage_options_simple_item = SimpleItem.manage_options

    manage_options = manage_options_local + manage_options_property_manager + manage_options_simple_item

    def __init__(self, id, title=EASYLANGUAGESERVICE):
        self.id=id
        self.title=title
        self.language_folders = PersistentMapping()
        self.language_mappings = PersistentMapping()
        self.language_order = PersistentList()


    #############
    #Query Interface
    #############

    # Helpers
    security.declarePublic('relativeUrl')
    def relativeUrl(self, absolute_url, REQUEST=None):
        if REQUEST is None: REQUEST=self.REQUEST
        root_path=REQUEST.get('VirtualRootPhysicalPath', ())
        root_url='/'.join(root_path)
        return absolute_url[len(root_url):]

    security.declarePublic('findMatches')
    def findMatches(self, list1, list2, default = []):
        """Returns an intersection of two lists"""
        result = []
        for item in list1:
            # add exact matches
            if item in list2 and item not in result:
                result.append(item)
            # If no exact matches, try to match language part only.
            # If language part already exist, ignore for duplicates
            # If an exact match of the longer lang code format exists
            # ignore addition.
            elif item[0:2] in list2 and item[0:2] not in result and item not in result:
                result.append(item[0:2])
        if not result: return default
        return result

    security.declarePublic('filterAllowed')
    def filterAllowed(self, languages=[], default=None):
        """Filter languages that are defined with home pages (or none)"""
        filtered = self.findMatches(languages, self.getDefinedLanguages())
        if not filtered:
            if default is None: return []
            if type(default) not in (TupleType,ListType): default=[default]
            return default
        if type(filtered) not in (TupleType,ListType): default=[filtered]
        return filtered

    security.declarePublic('findBestMatch')
    def findBestMatch(self, languages, REQUEST=None):
        cookie = self.getPreferredLanguageInSession()
        if cookie:
            langs = [cookie]
        else:
            langs = []

        plang = self.getHttpAcceptLanguage(REQUEST)
        dlang = self.getDefinedLanguages()
        langs = langs + plang + dlang

        matches = self.findMatches(langs, languages)
        if not matches:
            return self.getDefaultLanguage(raise_exc=0)

        return matches[0]

    #Language Mapping

    security.declarePublic('getSubstitute')
    def getSubstitute(self, languages=[]):
        """
        Returns a list of language codes where the
        orignal code has been substituted according
        to the current language mapping.
        """
        values=[]
        for lang in languages:
            values.append(self.language_mappings.get(lang, lang))
        return values

    security.declarePublic('getLanguageMappingItems')
    def getLanguageMappingItems(self,spec=None):
        values = {}
        for key,item in self.language_mappings.items():
            new_item=values.get(item, [])
            new_item.append(key)
            values[item]=new_item
        return values.items()

    security.declarePublic('getLanguageMappingKeys')
    def getLanguageMappingKeys(self,spec=None):
        return self.language_mappings.values()

    security.declarePublic('getLanguageMappingUsedKeys')
    def getLanguageMappingUsedKeys(self,spec=[]):
        """Returns all unique ids that has a mapping or is mapped from."""
        result={}
        set=result.setdefault
        if spec:
            map(lambda x, set=set: set(x), spec)
        keys=self.language_mappings.keys()
        values=self.language_mappings.values()
        if keys:
            map(lambda x, set=set: set(x), keys)
        if values:
            map(lambda x, set=set: set(x), values)
        return result.keys()

    security.declarePublic('getLanguageMappedKeys')
    def getLanguageMappedKeys(self):
        """Returns all unique ids that is mapping to another language."""
        return self.language_mappings.keys()


    #Defined Languages

    security.declarePublic('getDefaultLanguage')
    def getDefaultLanguage(self, default=_marker, raise_exc=1):
        """Return first language in defined list."""
        if self.language_order:
            return self.language_order[0]
        if default is not _marker:
            return default
        if raise_exc:
            raise LanguageException('No default language')
        return DEFAULT_LANGUAGE


    security.declarePublic('getDefaultLanguageName')
    def getDefaultLanguageName(self, default=None):
        """Return allowed preferred language."""
        lang_code = self.getDefaultLanguage(None)
        return self.getLanguageByRFC3066(lang_code,default)

    security.declarePublic('getDefinedLanguages')
    def getDefinedLanguages(self):
        """
        Returns a list of language codes of
        the languages the are defined in the service.
        """
        return list(self.language_order)

    security.declarePublic('getHttpAcceptLanguage')
    def getHttpAcceptLanguage(self, REQUEST):
        """
        Returns a list of the browsers HTTP PREFEERED LANGUAGES as
        reported in the REQUEST.
        """
        if REQUEST is None:
            REQUEST = getattr(self, 'REQUEST', None)
        if REQUEST is not None:
            HAL = REQUEST.get('HTTP_ACCEPT_LANGUAGE', '')
            HALs =  HAL.split(',')
            preferred_lang = map(lambda x: x.split(';')[0].strip(), HALs)
            return preferred_lang
        return []


    #Preferred Languages

    security.declarePublic('getPreferredLanguagesAttributeName')
    def getPreferredLanguagesAttributeName(self):
        """Returns the name of the preferred lanugage request attribute."""
        return self.preferred_languages

    security.declarePublic('getPreferredLanguages')
    def getPreferredLanguages(self, use_substitute=1, REQUEST=None):
        """Returns the language code that would be most
        preferred by the user.
        """
        preferred_lang=self.getPreferredLanguageInSession([])
        if preferred_lang==[]:
            preferred_lang = self.getHttpAcceptLanguage(REQUEST)
        if type(preferred_lang) is StringType: preferred_lang = [preferred_lang]
        if use_substitute:
            return self.getSubstitute(preferred_lang)
        return preferred_lang

    security.declarePublic('getAllowedPreferredLanguages')
    def getAllowedPreferredLanguages(self, use_substitute=1, default=None):
        """Return a list of allowed preferred languages or an empty list."""
        #if type(default) not in (TupleType,ListType): default=[default]
        pl=self.getPreferredLanguages(use_substitute)
        langs = self.filterAllowed(pl, default)
        if not langs:
            # No match between defined languages and preferres languages:
            # return first defined
            defined_language = self.getDefinedLanguages()
            if defined_language:
                return defined_language
            if default is None: return []
            if type(default) not in (TupleType,ListType): default=[default]
            return default
        if langs is None: return []
        if type(langs) not in (TupleType,ListType): langs=[langs]
        return langs


    #Current Languages

    security.declarePublic('getCurrentLanguage')
    def getCurrentLanguage(self, use_substitute=1, default=_marker):
        """Return the allowed preferred language, or default if none."""
        langs=self.getAllowedPreferredLanguages(use_substitute=use_substitute,default=None)
        if langs:
            return langs[0]
        if default is _marker:
            raise LanguageException('Could not find suitable language')
        if default is None:
            #Get the default default?
            pass
        return default

    security.declarePublic('getCurrentLanguageName')
    def getCurrentLanguageName(self, use_substitute=1, default=None):
        """Return allowed preferred language as plain text (English)."""
        language=self.getLanguageByRFC3066(self.getCurrentLanguage(use_substitute=use_substitute), None)
        if language is not None:
            return language
        return default

    #NEW API
    security.declarePublic('getLanguageByLocation')
    def getLanguageByLocation(self, path_or_object, default=None):
        if hasattr(path_or_object, 'getPhysicalPath'):
            path_or_object=path_or_object.getPhysicalPath()
        if type(path_or_object) is TupleType:
            for lang, info in self.language_folders.items():
                path=info['absolute_folder_url'].split('/')
                if path_or_object[:len(path)] == path: return lang
        return default

    security.declarePublic('getLanguageFolderByLocation')
    def getLanguageFolderByLocation(self, path_or_object, default=None):
        if hasattr(path_or_object, 'getPhysicalPath'):
            path_or_object=path_or_object.getPhysicalPath()
        if type(path_or_object) is TupleType:
            for lang, info in self.language_folders.items():
                path=info['absolute_folder_url'].split('/')
                if path_or_object[:len(path)] == path:
                    try:
                        return self.restrictedTraverse(info['absolute_folder_url'])
                    except:
                        pass
        return default

    security.declarePublic('getLanguageFolderPathByLocation')
    def getLanguageFolderPathByLocation(self, path_or_object, default=None):
        if hasattr(path_or_object, 'getPhysicalPath'):
            path_or_object=path_or_object.getPhysicalPath()
        if type(path_or_object) is TupleType:
            for lang, info in self.language_folders.items():
                path=info['absolute_folder_url'].split('/')
                if path_or_object[:len(path)] == path: return path
        return default

    security.declarePublic('getLanguageFolderURLByLocation')
    def getLanguageFolderURLByLocation(self, path_or_object, default=None):
        if hasattr(path_or_object, 'getPhysicalPath'):
            path_or_object=path_or_object.getPhysicalPath()
        if type(path_or_object) is TupleType:
            for lang, info in self.language_folders.items():
                path=info['absolute_folder_url'].split('/')
                if path_or_object[:len(path)] == path: return info['absolute_folder_url']
        return default


    #Language Folders

    security.declarePublic('getFolderURL')
    def getFolderURL(self, lang, default=None):
        """Returns the URL associated with a language code"""
        language_folder = self.language_folders.get(lang, default)
        if language_folder:
            af_url = language_folder.get('absolute_folder_url', None)
            if af_url:
                return af_url
        return default

    security.declarePublic('getFolder')
    def getFolder(self, lang, default=None):
        """Returns the folder associated with a language code"""
        try:
            return self.restrictedTraverse(self.getFolderURL(lang))
        except:
            pass
        return default


    security.declarePublic('getLanguageFolderItems')
    def getLanguageFolderItems(self,spec=None):
        """Returns Language Folder itemes in sorted order"""
        return map(lambda key, lf=self.language_folders: (key,lf[key]), self.language_order)

    security.declarePublic('getLanguageFolderKeys')
    def getLanguageFolderKeys(self, spec=[]):
        """Returns Language Folder keys (plus spec) in sorted order"""
        return self.language_order + spec



    security.declarePublic('getCurrentLanguageFolder')
    def getCurrentLanguageFolder(self, use_substitute=1, default=None):
        """Return the current language folder object."""
        return self.getFolder(self.getCurrentLanguage(use_substitute), default)

    security.declarePublic('getCurrentLanguageFolderAbsoluteURL')
    def getCurrentLanguageFolderAbsoluteURL(self, use_substitute=1, default=None):
        """Return the absolute URL to current language folder."""
        lang = self.getCurrentLanguage(use_substitute)
        url = self.getFolderURL(lang)
        if url is None: return default
        if url[-1]!='/': url=url+'/'
        return url

    security.declarePublic('getCurrentLanguageFolderURL')
    def getCurrentLanguageFolderURL(self, use_substitute=1, default=None):
        """Return the virtual URL to current languages folder."""
        url = self.getCurrentLanguageFolderAbsoluteURL( use_substitute, default)
        if url is not None:
            url = self.relativeUrl(url)
            return url
        return default


    # Action Interface

    security.declarePublic('redirect')
    def redirect(self, use_substitute=1, default=None, lock=1):
        """Redirect to the Language Folder, and ignore exceptions that may be raise after this."""
        location=self.getCurrentLanguageFolderURL(use_substitute=use_substitute, default=default)
        if location:
            try: self.REQUEST.RESPONSE.redirect(location, lock=lock)
            except: pass
        if default is not None:
            try: self.REQUEST.RESPONSE.redirect(default, lock=lock)
            except: pass

    # Session Interface
    security.declarePublic('setPreferredLanguageInSession')
    def setPreferredLanguageInSession(self, language=None, REQUEST=None):
        """Sets a cookie with the preferred language."""
        if language and validRFC3066(language):
            sdm = getattr(self, 'session_data_manager', None)
            if sdm:
                sd=sdm.getSessionData(create=1)
                sd.set(self.preferred_languages, language)
        if REQUEST:
            return "<pre>Language set to: '%s' in session variable '%s'.</pre>" % (language,self.preferred_languages)

    security.declarePublic('removePreferredLanguageInSession')
    def removePreferredLanguageInSession(self, REQUEST=None):
        """Removes the cookie with the preferred language."""
        sdm = getattr(self, 'session_data_manager', None)
        if sdm:
            sd=sdm.getSessionData(create=1)
            sd.delete(self.preferred_languages)
        if REQUEST:
            return "<pre>Language deleted in session variable '%s'.</pre>" % (language,self.preferred_languages)


    security.declarePublic('getPreferredLanguageInSession')
    def getPreferredLanguageInSession(self, default=None, REQUEST=None):
        """Gets the cookie with the preferred language or default."""
        sdm = getattr(self, 'session_data_manager', None)
        if sdm:
            sd=sdm.getSessionData(create=1)
            language = sd.get(self.preferred_languages, default)
        else:
            language = default
        if REQUEST:
             return "<pre>Language in session variable '%s' is set to: '%s' (default: '%s').</pre>" % (self.preferred_languages, language, default)
        return language


    # RFC/ISO Query Interface


    # Language Query Interface

    security.declarePublic('getLanguageItemsByCodes')
    def getLanguageItemsByCodes(self, lang_codes):
        """\
        Returns all languages in lang_codes as (rfc3066Code, PlainText)
        """
        return getLanguageItemsByCodes(lang_codes)


    security.declarePublic('getAllLanguageIds')
    def getAllLanguageIds(self, filter_codes=[]):
        """\
        Returns all languages as RFC3066 codes
        except for those in filter_codes

        filter_codes is a sequence of RFC3066 code.
        """
        return getAllLanguageIds(filter_codes=filter_codes)

    security.declarePublic('getAllLanguageValues')
    def getAllLanguageValues(self, filter_codes=[]):
        """\
        Returns all languages as PlainText
        except for those in filter_codes

        filter_codes is a sequence of RFC3066 code.
        """
        return getAllLanguageValues(filter_codes=filter_codes)


    security.declarePublic('getAllLanguageItems')
    def getAllLanguageItems(self, filter_codes=[]):
        """
        Returns all languages as (RFC3066, PlainText) except for the once in filter_codes.

        filter_codes is a sequence of RFC3066 code.
        """
        return getAllLanguageItems(filter_codes=filter_codes)


    security.declarePublic('getCountryItemsByCodes')
    def getCountryItemsByCodes(self, country_codes):
        """Returns all languages in country_codes as (ISO639_1, PlainText)"""
        return getCountryItemsByCodes(country_codes)

    security.declarePublic('getAllCountryIds')
    def getAllCountryIds(self, filter_codes=[]):
        """/
        Returns all countries as ISO639_1 codes
        except for those in filter_codes

        filter_codes is a sequence of ISO639_1 code.
        """
        return getAllCountryIds(filter_codes=filter_codes)

    security.declarePublic('getAllCountryValues')
    def getAllCountryValues(self, filter_codes=[]):
        """\
        Returns all countries  as PlainText
        except for those in filter_codes

        filter_codes is a sequence of ISO639_1 code.
        """
        return getAllCountryValues(filter_codes=filter_codes)

    security.declarePublic('getAllCountries')
    def getAllCountryItems(self, filter_codes=[]):
        """
        Returns all languages as (ISO639_1, PlainText) except for the once in filter_codes.

        filter_codes is a sequence of ISO639_1 code.
        """
        return getAllCountryItems(filter_codes)

    security.declarePublic('getLanguageByRFC3066')
    def getLanguageByRFC3066(self, code, default='English'):
        """Translates a language code to a human readable form in english.

        If a sequence of language codes is passed, a sequence of the corresponding
        languages is returns.
        """
        return getLanguageByRFC3066(code, default)

    security.declarePublic('getLanguageByISO639_1')
    def getLanguageByISO639_1(self, code, default='English'):
        """Translates a language code to a human readable form in english.

        If a sequence of language codes is passed, a sequence of the corresponding
        languages is returns.
        """
        return getLanguageByISO639_1(code, default)

    security.declarePublic('getCountryByISO3166_1')
    def getCountryByISO3166_1(self, code, default='No match'):
        """Translates a language code to a human readable form in english.

        If a sequence of language codes is passed, a sequence of the corresponding
        languages is returns.
        """
        return getCountryByISO3166_1(code, default)

    security.declarePublic('getRFC3066ByLanguage')
    def getRFC3066ByLanguage(self, language, default='en'):
        """Translates a language in a human readable form to the code.
        """
        return getRFC3066ByLanguage(language, default)

    security.declarePublic('getISO639_1ByLanguage')
    def getISO639_1ByLanguage(self, language, default='en'):
        """Translates a language in a human readable form to the code."""
        return getISO639_1ByLanguage(language, default)

    security.declarePublic('getISO3166_1ByCountry')
    def getISO3166_1ByCountry(self, country, default='en'):
        """Translates a language in a human readable form to the code."""
        return getISO3166_1ByCountry(country, default=default)

    security.declarePublic('getISO3166_1ByCountryItems')
    def getISO3166_1ByCountryItems(self, country, default='en'):
        """Translates a language in a human readable form to the code."""
        return getISO3166_1ByCountryItems(country, default=default)

    security.declarePublic('getISO639_1ByRFC3066')
    def getISO639_1ByRFC3066(self, code):
        """
        Returns the root language class of the language code.

        If a sequence of language codes is passed, a tuple of the corresponding
        languages is returns.
        """
        return getISO639_1ByRFC3066(code)

    security.declarePublic('validISO639_1')
    def validISO639_1(self, code):
        """
        Returnes true if this is a valid language code.
        """
        return validISO639_1(code)

    security.declarePublic('validRFC3066')
    def validRFC3066(self, code):
        """
        Returnes true if this is a valid language code.
        """
        return validRFC3066(code)

    security.declarePublic('validISO3166_1')
    def validISO3166_1(self, code):
        """
        Returnes true if this is a valid language code.
        """
        return validISO3166_1(code)


    #Management Interface

    _manage_main = DTMLFile('dtml/manage_main', globals())
    security.declareProtected(EasyLanguageServiceManage, 'manage_main')
    def manage_main(self
                    , selected_language=[]
                    , add_language=None
                    , add_folder_url=None
                    , action='View'
                    , REQUEST=None):
        """Manage Language Folders"""
        management_view='LanguageFolders'
        manage_tabs_message=""
        deleted_languages=[]
        if action=='Delete' and selected_language:
            for key in selected_language:
                del self.language_folders[key]
                while key in self.language_order:
                    self.language_order.remove(key)
            self._p_changed=1
            manage_tabs_message="Language(s) %s [%s] was deleted." % (getLanguageByRFC3066(selected_language), selected_language)
            deleted_languages=selected_language[0:1]
        elif action=='Move Up' and selected_language:

            try:
                pos = self.language_order.index(selected_language[0])
            except:
                pos = len(self.language_order)
            new_order=[]
            if pos>0:
                new_order=self.language_order[0:pos-1]
            new_order.extend(selected_language)
            for langcode in self.language_order:
                if langcode not in new_order:
                    new_order.append(langcode)
            self.language_order=new_order
        elif action=='Move Down' and selected_language:
            new_order=[]
            pos=0
            max_len=len(self.language_order)
            langcode = self.language_order[pos]
            while langcode != selected_language[0] and pos<max_len:
                if langcode not in selected_language:
                    new_order.append(langcode)
                pos=pos+1
                if pos<max_len: langcode = self.language_order[pos]
            next_found=0
            while not next_found and pos<max_len:
                langcode = self.language_order[pos]
                if langcode not in selected_language and langcode not in new_order:
                    next_found = 1
                    new_order.append(langcode)
                pos=pos+1
            new_order.extend(selected_language)
            while pos<max_len:
                langcode = self.language_order[pos]
                if langcode not in selected_language and langcode not in new_order:
                    new_order.append(langcode)
                pos=pos+1
            self.language_order=distinct(new_order)
        elif action=='Move Top' and selected_language:
            new_order=[]
            new_order.extend(selected_language)
            for key in self.language_order:
                if key not in selected_language:
                    new_order.append(key)
            self.language_order=distinct(new_order)
        elif action=='Move Bottom' and selected_language:
            new_order=[]
            for key in self.language_order:
                if key not in selected_language:
                    new_order.append(key)
            new_order.extend(selected_language)
            self.language_order=distinct(new_order)
        elif action=='Add Language' and add_language is not None:
            changed_or_added='added'
            if add_folder_url!='':
                #ADD or CHANGE languages that do defined a language folder.
                root=REQUEST.get('VirtualRootPhysicalPath', ())
                if add_language in self.language_order and self.language_folders.has_key(add_language): changed_or_added='changed'
                if add_folder_url[0]!='/': add_folder_url='/'+add_folder_url
                add_folder_url = '/'.join(root) + add_folder_url
                physical = self.getPhysicalPath()
                parent_url = '/'.join(physical[:-1])
                if len(add_folder_url)>len(parent_url) and parent_url==add_folder_url[:len(parent_url)]:
                    try:
                        self.restrictedTraverse(add_folder_url)
                        d = {
                                'absolute_folder_url':add_folder_url,
                                'rfc3066':add_language,
                                'language':getLanguageByRFC3066(add_language)
                            }
                        self.language_folders[add_language] = d
                        if add_language not in self.language_order:
                            self.language_order.append(add_language)
                            self.language_order=distinct(self.language_order)
                        self._p_changed=1
                        add_language=''
                        add_folder_url=''
                        manage_tabs_message="Language %s was %s with folder %s." % (getLanguageByRFC3066(add_language), changed_or_added, self.relativeUrl(add_folder_url))
                    except:
                        manage_tabs_message="Failed! Invalid url %s, language %s was not %s." % (self.relativeUrl(add_folder_url),getLanguageByRFC3066(add_language),changed_or_added)
                else:
                    manage_tabs_message="Failed! You can't select a folder that isn't under the parent of this service."
            elif add_folder_url=='':
                #ADD or CHANGE languages that don't defined a language folder.
                changed_or_added='added'
                if self.language_folders.has_key(add_language): changed_or_added='changed'
                d = {
                        'absolute_folder_url':add_folder_url,
                        'rfc3066':add_language,
                        'language':getLanguageByRFC3066(add_language)
                    }
                self.language_folders[add_language] = d
                if add_language not in self.language_order:
                    self.language_order.append(add_language)
                    self.language_order=distinct(self.language_order)
                self._p_changed=1
                add_language=''
                add_folder_url=''
                manage_tabs_message="Language %s was %s." % (getLanguageByRFC3066(add_language), changed_or_added)
        return self._manage_main(self, REQUEST=REQUEST
                                , manage_tabs_message=manage_tabs_message
                                , management_view=management_view
                                , deleted_languages=deleted_languages
                                , selected_language=selected_language
                                , add_language=add_language
                                , add_folder_url=add_folder_url or ''
                                )


    _manage_mappings = DTMLFile('dtml/manage_mappings', globals())
    security.declareProtected(EasyLanguageServiceManage, 'manage_mappings')
    def manage_mappings(self
                    , selected_mappings=[]
                    , add_mapping=None
                    , add_languages=[]
                    , action='View'
                    , REQUEST=None):
        """Manage Language Folders"""
        management_view='LanguageFolders'
        manage_tabs_message=""
        deleted_mapping=None
        deleted_languages=None
        keys_in_use = self.getLanguageMappingUsedKeys()

        if action=='Delete' and selected_mappings:
            self.language_mappings
            if len(selected_mappings)==1:
                deleted_mapping=selected_mappings[0]
                deleted_languages=[]
            for item, key in self.language_mappings.items():
                if key in selected_mappings:
                    del self.language_mappings[item]
                    if len(selected_mappings)==1:
                        deleted_languages.append(item)

            manage_tabs_message="Mapping to language(s) %s [%s] was deleted." % (', '.join(getLanguageByRFC3066(selected_mappings)), ', '.join(selected_mappings))

        elif action=='Add Mapping' and add_mapping and add_mapping not in keys_in_use:

            keys_in_use.append(add_mapping)
            add_languages = filter(lambda x,keys_in_use=keys_in_use: x not in keys_in_use, add_languages)
            for lang in add_languages:
                self.language_mappings[lang] = add_mapping

        return self._manage_mappings(self,REQUEST=REQUEST
                                    , manage_tabs_message=manage_tabs_message
                                    , management_view=management_view
                                    , deleted_mapping=deleted_mapping
                                    , deleted_languages=deleted_languages
                                    )




    #Navigation API

    if NavigationBrain:
        security.declarePublic('getMenu')
        def getMenu(self, object_path, sort_on='order', sort_order=None, skip_root=None):
            brains = []
            append=brains.append
            vrpp=self.REQUEST.get('VirtualRootPhysicalPath', ())
            navigation_menu = self.getPhysicalPath()
            if hasattr(object_path,'physicalPath'): object_path=object_path.physicalPath

            LocalNavigationBrain.query=None
            LocalNavigationBrain.object_path=object_path
            LocalNavigationBrain.skip_root=skip_root
            LocalNavigationBrain.getMenu=None
            LocalNavigationBrain.virtual_root_path_of_request=vrpp
            LocalNavigationBrain.NavigationService=self
            LocalNavigationBrain.isAnObjectManager=1
            LocalNavigationBrain.getVirtualRootPath=vrpp
            LocalNavigationBrain.navigation_menus=self.title or self.id

            for key, item in self.getLanguageFolderItems():
                try: obj=self.unrestrictedTraverse(item['absolute_folder_url'])
                except: obj=None
                if obj is None or obj.getPublishState()==psPUBLISHED:
                    brains.append(LocalNavigationBrain(dict={
                              'absolute_folder_url':    item['absolute_folder_url']
                            , 'rfc3066':                item['rfc3066']
                            , 'language':               item['language']
                            , 'navigation_title':       (obj and obj.navigation_title) or item['language']
                            , 'title':                  (obj and obj.title) or item['language']
                            , 'description':            (obj and obj.description) or item['language']
                        }).__of__(self))

            #Post sort
            #def sortmethod( x,y, nav_menu):
            #    return cmp(x.navigation_menus_data[nav_menu][0],
            #               y.navigation_menus_data[nav_menu][0])
            #
            #if sort_on=='order':
            #    menuobjectlist = list(menuobjectlist)
            #    menuobjectlist.sort( lambda x,y, nav_menu=object_path,
            #                     sortmethod=sortmethod: sortmethod(x,y,nav_menu))
            #    if sort_order=='reverse':
            #        menuobjectlist.reverse()

            return brains

Globals.InitializeClass(EasyLanguageService)


def manage_addEasyLanguageService(self, title='', REQUEST=None):
    """Add a EasyLanguageService into a web site."""

    id=EASYLANGUAGESERVICE

    ob=EasyLanguageService(id)
    self._setObject(id,ob)
    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')



def initialize(context):
    """Initialize EasyMailService Product"""


    context.registerClass(
        EasyLanguageService,
        permission=EasyLanguageServiceManage,
        constructors=(manage_addEasyLanguageService,
                      manage_addEasyLanguageService),
        icon='icons/EasyLanguageService.gif',
        category='Easy Publisher Management',
    )



