# coding: iso-8859-1
# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyNavigation/EasyNavigation.py,v 1.6 2004/09/15 14:13:03 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__='''EasyNavigator product: EasyNavigation class.'''
__version__='1.7.5'

import Globals
from string import join, split

from Globals import DTMLFile, HTML
import DateTime
from AccessControl import getSecurityManager
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_chain

from Products.EasyCore.ZCatalog.ProtectedCatalog import ProtectedCatalog

from ExtensionClass import Base

from Products.EasyWorkflow.EasyWorkflowBase import wfPUBLISHED
from Products.EasyCore.UpgradeSupport.Utils import upgrade_indexes
from Products.EasyCore.VirtualPath import VirtualPathBase, mergePaths
from Products.EasyCore.VirtualPath import isAbove, isBellow, isChild, isCurrent, isSibling
from Products.EasyPresentationHelpers.EasyPresentationHelpers import EasyPresentationHelpers
from Products.EasyCore.EasyCorePermissions  import EasyViewAhead, EasyNavigationChange, View, AccessContentsInformation, EasyNavigationServiceAdd
from Products.EasyCore.EasyResourcesConf import RESOURCE_CATALOG_ID
from Products.EasyCore.EasyNavigationConf import NAVIGATION_CATALOG_ID, FILTERDEFAULT, EASYEDITABLEMETATYPES
from ComputedAttribute import ComputedAttribute
from Products.EasyEditor.EasyPublisher import EasyPublisherBase
from Products.EasyWorkflow.EasyWorkflowBase import *

from zLOG import LOG, INFO, ERROR
from Products.EasyCore import MLOG_TAB


EASY_VHM='EasyVHM'


DEFAULT_INDEXES = {}
REQUIRED_INDEXES = {
          'id':'FieldIndex'
        , 'title':'TextIndex'
        , 'bobobase_modification_time':'FieldIndex'
        , 'summary':'TextIndex'
        #, 'path':'PathIndex'
        , 'meta_type':'FieldIndex'
        , 'PrincipiaSearchSource':'TextIndex'
        , 'navigation_menus':'KeywordIndex'
        , 'getWorkflowState':'FieldIndex'
        , 'getWorkflowStateLastChangedDate':'FieldIndex'
        , 'getWorkflowStateLastChangedBy':'FieldIndex'
        , 'creation_date':'FieldIndex'
        , 'effective_date':'FieldIndex'
        , 'expiration_date':'FieldIndex'
        , 'last_modified_by':'FieldIndex'
        , 'author':'FieldIndex'
        , 'getVirtualRootPath':'FieldIndex'
        , 'physicalPath':'FieldIndex'
        , 'virtualPath':'FieldIndex'
        , 'virtualParentPath':'FieldIndex'
        , 'physicalParentPath':'FieldIndex'
        # , 'hasSubDocuments':'FieldIndex'
        , 'category':'KeywordIndex'
        , 'subject':'KeywordIndex'
        , 'version':'FieldIndex'
        , 'release':'FieldIndex'
        , 'getTemplateId':'FieldIndex'
        , 'physicalPathIndex':'PathIndex'
}
DEPRECATED_INDEXES=('id','title','meta_type','bobobase_modification_time','path')

DEFAULT_COLUMNS = ()
REQUIRED_COLUMNS = (
          'id'
        , 'title'
        , 'meta_type'
        , 'bobobase_modification_time'
        , 'summary'
        , 'navigation_title'
        , 'navigation_menus_data'
        , 'navigation_menus'
        , 'creation_date'
        , 'effective_date'
        , 'expiration_date'
        , 'publish_date'
        , 'archive_date'
        , 'getWorkflowState'
        , 'getWorkflowStateLastChangedDate'
        , 'getWorkflowStateLastChangedBy'
        , 'last_modified_by'
        , 'author'
        , 'approved_by'
        , 'icon'
        , 'getVirtualRootPath'
        , 'physicalPath'
        , 'virtualPath'
        , 'physicalURL'
        , 'virtualURL'
        , 'virtualParentPath'
        , 'physicalParentPath'
        , 'hasSubDocuments'
        , 'category'
        , 'subject'
        , 'description'
        , 'version'
        , 'release'
        , 'getTemplateId'
        , 'isAnObjectManager'
        , 'physicalPathIndex'

)
DEPRECATED_COLUMNS=('summary', 'bobobase_modification_time', 'meta_type', 'title', 'id')


class NavigationBrain:

    def _getPhysicalURL(self):
        url = join(self.physicalPath,'/')
        if url=='': url='/'
        # This should work, unless some objects does not like having a slash
        # the end of the URL, but that is unlikely:
        if url[-1]!='/': url=url+'/'
        return url
    physicalURL = ComputedAttribute(_getPhysicalURL, 1)

    def _getVirtualURLXD(self):
        """\
        Returns a virtualURL.
        If the domain that is being accessed doesn't match
        the domain that the menu item is in the URL is remapped
        to include the protocol, domain name, and port.

        XD stand for Cross Domain
        """
        vhm=self.EasyVHM
        if vhm:
            try:
                item_domain=vhm.get_domain(self.physicalPath)
                if self.domain!=item_domain:
                    return vhm.get_site_root_url(self.physicalPath,1) + self.virtualURL
            except:
                pass
        return self.virtualURL
    virtualURLXD = ComputedAttribute(_getVirtualURLXD, 1)

    def isLocalDomain(self):
        """\
        Test if the domain that is being accessed doesn't match
        the domain that the menu item is in.
        """
        vhm = self.EasyVHM
        if vhm:
            try:
                item_domain=vhm.get_domain(self.physicalPath)
                return self.domain==item_domain
            except:
                pass
        return 1

    def checkInPath(self):
        if hasattr(self,'path_in'):
            return self.path_in is not None and self.physicalPath in self.path_in
        return 0

    def __call__(self):
        return self

    def isAbove(self, object_path=None):
        if object_path is None: object_path = self.current_path
        return isAbove(self.physicalPath, object_path)

    def isBellow(self, object_path=None):
        if object_path is None: object_path = self.current_path
        return isBellow(self.physicalPath, object_path)

    def isChild(self, object_path=None):
        if object_path is None: object_path = self.current_path
        return isChild(self.physicalPath, object_path)

    def isCurrent(self, object_path=None):
        if object_path is None: object_path = self.current_path
        return isCurrent(self.physicalPath, object_path)

    def isSibling(self, object_path=None):
        if object_path is None: object_path = self.current_path
        return isSibling(self.physicalPath, object_path)

    #Workflow Methods
    def getPublishState(self):
        if (self.effective_date!=NullDate and self.effective_date>self.now) or (not self.getWorkflowState == wfPUBLISHED):
            return psUNPUBLISHED
        if self.expiration_date!=NullDate and self.expiration_date < self.now:
            return psARCHIVED
        return psPUBLISHED

    def isPublished(self):
        state=self.getWorkflowState
        pstate=self.getPublishState()
        if pstate==psPUBLISHED and state==wfPUBLISHED:
            return 1
        return 0

    def isArchived(self):
        pstate=self.getPublishState()
        if pstate==psARCHIVED:
            return 1
        return 0

    ##ATTRIBUTE SET WHEN CALLED.
    #LocalNavigationBrain.query=query
    #LocalNavigationBrain.object_path=object_path
    #LocalNavigationBrain.skip_root=skip_root
    #LocalNavigationBrain.getMenu=self.getMenu
    #LocalNavigationBrain.current_path=current_path
    #LocalNavigationBrain.here=here
    #LocalNavigationBrain.path_in=path_in
    #LocalNavigationBrain.getNavigationTrail=self.getNavigationTrail
    #LocalNavigationBrain.virtual_root_path_of_request=vrpp
    #LocalNavigationBrain.domain=self.getCurrentDomainName()
    #LocalNavigationBrain.NavigationService=self
    #LocalNavigationBrain.EasyVHM=getattr(self, EASY_VHM, None)


    #def getPublishIcon(self, size=16):
    #    return self.getPublishIconForState(self.getPublishState(), size=size)

    #def getPublishStateMessage(self):
    #    return self.getPublishStateMessageForState(self.getPublishState())

    #def getWorkflowIcon(self, size=16):
    #    return self.getWorkflowIconForState(self.getWorkflowState(), size=size)

    #def getWorkflowStateMessage(self):
    #    return self.getWorkflowMessageForState(self.getWorkflowState())

    #def getWorkflowStateName(self):
    #    return self.getWorkflowNameForState(self.getWorkflowState())



class EasyNavigation(ProtectedCatalog, VirtualPathBase, EasyPresentationHelpers, EasyPublisherBase, Base):
    """ EasyNavigation object
        The Navigation Catalog is a ProtectedCatalog that is extended to work as a
        navigation service. "Navigatable" objects are automatically indexed in the
        EasyNavigation, and functions to get a list of objects that are either
        entries in a menu or entries in a path.
    """

    meta_type = NAVIGATION_CATALOG_ID

    security = ClassSecurityInfo()

    isAnObjectManager=1
    isPrincipiaFolderish=1

    manage_options = (
        {'label': 'Workflow',           # TAB: Contents
         'action': 'manage_workflow',
         'help': ('EasyNavigation','WorkflowOverview.stx')},
        ) + ProtectedCatalog.manage_options


    def __init__(self, id, title='', vocab_id=None, container=None):
        ProtectedCatalog.__init__(self, id, title, vocab_id, container)

        # More default fields and indexes:

        for id in DEPRECATED_INDEXES:
            self.delIndex(id, force=1, dont_raise=1)
        for id, type in REQUIRED_INDEXES.items():
            self.addIndex(id, type, undeletable=1)
        for id, type in DEFAULT_INDEXES.items():
            self.addIndex(id, type)

        for id in DEPRECATED_COLUMNS:
            self.delColumn(id, force=1, dont_raise=1)
        for id in REQUIRED_COLUMNS:
            self.addColumn(id, undeletable=1)
        for id in DEFAULT_COLUMNS:
            self.addColumn(id)


    #security.declareProtected(EasyNavigationChange, 'changeOrderFor')
    #def changeOrderFor(self, navigation_menu, object_list):
    #    """object_list: [{ UID: order }, { UID: order },, ...]"""
    #    for menuentry in object_list:
    #        o = self.restrictedTraverse(menuentry[0])
    #        o.changeOrder( navigation_menu, menuentry[1])

    def _calc_path_in(self, current_path):
        path_in = []
        current = ()
        for pathid in current_path:
            current = current + (pathid,)
            path_in.append(current)
        return path_in

    security.declareProtected(View, 'getMenu')
    def getMenu(self
                    , object_path=None
                    , sort_on='order'
                    , sort_order=None
                    , skip_root=None
                    , query=None
                ):
        """ Returns the list of menuentries for the menu object_path.

        object_path realy should have been called menu_path, because that what it is.

        The current objects path current_path is calculate bellow from
        the here variable set in the REQUEST object when a template/iew is called.
        """
        req = getattr(self, 'REQUEST', None)
        if req: vrpp = req.get('VirtualRootPhysicalPath', ('',))
        else: vrpp = ('',)

        if type(query)!=type({}): query={}

        if object_path is None: object_path = self.getPhysicalPath()
        if sort_on is None: sort_on = 'order'

        here=None
        req=getattr(self, 'REQUEST', None)
        if req is not None: here=getattr(req, 'here', None)
        current_path=object_path
        if here: current_path=here.getPhysicalPath()
        path_in=self._calc_path_in(current_path)

        object_path_list = object_path
        if type(object_path_list)!=type([]): object_path_list = [object_path_list]
        query['navigation_menus'] = {'query': object_path_list }

        #Pre sort
        if sort_on != 'order':
            query['sort-on'] = sort_on
            if sort_order == 'reverse': query['sort-order'] = 'reverse'

        #Search
        class LocalNavigationBrain(NavigationBrain):  pass
        LocalNavigationBrain.query=query
        LocalNavigationBrain.object_path=object_path
        LocalNavigationBrain.skip_root=skip_root
        LocalNavigationBrain.getMenu=self.getMenu
        LocalNavigationBrain.current_path=current_path
        LocalNavigationBrain.here=here
        LocalNavigationBrain.path_in=path_in
        LocalNavigationBrain.getNavigationTrail=self.getNavigationTrail
        LocalNavigationBrain.virtual_root_path_of_request=vrpp
        LocalNavigationBrain.domain=self.getCurrentDomainName()
        LocalNavigationBrain.NavigationService=self
        LocalNavigationBrain.EasyVHM=getattr(self, EASY_VHM, None)
        now = DateTime.now()
        LocalNavigationBrain.now=now

        self._catalog.useBrains(LocalNavigationBrain)

        if not getSecurityManager().checkPermission(EasyViewAhead, self) or hasattr(self,'REQUEST') and hasattr(self.REQUEST,'anonymous'):
            query['getWorkflowState'] = {'query':wfPUBLISHED }
            query['effective_date'] = {'query': now, 'range': 'max'}
            query['expiration_date'] = {'query': now, 'range': 'min'}
            menuobjectlist = self.searchResults(query)
            query['effective_date'] = {'query': ''}
            menuobjectlist = menuobjectlist + self.searchResults(query)
        else:
            query['sort-order'] = sort_order
            query['effective_date'] = {'query': '', 'range': 'max'}
            query['expiration_date'] = {'query': '', 'range': 'min'}
            menuobjectlist = self.searchResults(query)

        if skip_root:
            menuobjectlist = list(menuobjectlist)
            menuobjectlist = menuobjectlist[1:]

        #Post sort
        def sortmethod( x,y, nav_menu):
            #Use default value because sometimes the catalog doesn't return
            #correct values. (This is a intimetent error)
            try:
                return cmp(x.navigation_menus_data.get(nav_menu,(None,))[0],
                           y.navigation_menus_data.get(nav_menu,(None,))[0])
            except:
                LOG("SortMethodError",ERROR, "Error in sortmethod of getMenu in EasyNavigation",
                    """\
                    x:  %s
                    y:  %s
                    nav_menu: %s
                    x.navigation_menus_data.get(nav_menu): %s
                    y.navigation_menus_data.get(nav_menu): %s
                    """ % (
                          getattr(x, 'id', 'No Id')
                        , getattr(y, 'id', 'No Id')
                        , str(nav_menu)
                        , str(x.navigation_menus_data.get(nav_menu, 'Empty'))
                        , str(y.navigation_menus_data.get(nav_menu,'Empty'))
                        )
                )

                return 0

        if sort_on=='order':
            menuobjectlist = list(menuobjectlist)
            menuobjectlist.sort( lambda x,y, nav_menu=object_path,
                             sortmethod=sortmethod: sortmethod(x,y,nav_menu))
            if sort_order=='reverse':
                menuobjectlist.reverse()

        #del LocalNavigationBrain
        del query
        return menuobjectlist


    security.declareProtected(View, 'getMenuMap')
    def getMenuMap(self
                        , object_path=None
                        , current_path=None
                        , path_in=None
                        , sort_on='order'
                        , sort_order=None
                        , submenu=None
                        , skip_root=None
                        , check_in_path=0
                        , traverse_submenus_in_path=1
                        , query=None
                    ):
        """\
        DEPRECATED, It is expensive

        Returns a hierarchical  dictionary of menuentries for the menu object_path.
        """
        if type(query)!=type({}): query={}
        if current_path is None: current_path = object_path
        if path_in is not None and type(path_in) != type([]): path_in = [path_in]
        if path_in is None and check_in_path:
            path_in = []
            current = ()
            for pathid in current_path:
                current = current + (pathid,)
                path_in.append(current)
        menuobjectlist = self.getMenu(object_path=object_path, sort_on=sort_on, sort_order=sort_order, skip_root=skip_root, query=query)
        navigation_mapping = []
        for menuobject in menuobjectlist:
            physicalPath = menuobject.physicalPath
            if path_in is not None and physicalPath in path_in: in_path = 1
            else: in_path = 0
            submenu = None
            hasObjects=0
            isabove = self.isAbove(physicalPath, object_path)
            issibling = self.isSibling(physicalPath, object_path)
            ischild = self.isChild(physicalPath, object_path)
            iscurrent = self.isCurrent(physicalPath, object_path)
            if not isabove and not iscurrent:
                if submenu:
                    submenu = self.getMenuMap(physicalPath, current_path=current_path, sort_on=sort_on, sort_order=sort_order, submenu=submenu-1, query=query)
                    hasObjects=len(submenu)
                elif in_path and traverse_submenus_in_path:
                    submenu = self.getMenuMap(physicalPath, current_path=current_path, path_in=path_in, sort_on=sort_on, sort_order=sort_order, query=query)
                    hasObjects=len(submenu)
                else:
                    hasObjects=len(self.getMenu(physicalPath, query=query))
            navigation_mapping.append({
                        'id': menuobject.id,
                        'title': menuobject.title or menuobject.navigation_title or menuobject.id,
                        'navigation_title': menuobject.navigation_title or menuobject.title or menuobject.id,
                        'physicalPath': physicalPath,
                        'virtualURL': menuobject.virtualURL,
                        'brain': menuobject,
                        'submenu': submenu,
                        'hasSubDocuments': hasObjects and not isabove,
                        'is_current': current_path == physicalPath,
                        'in_path': in_path,
                        'is_child': ischild,
                        'is_above': isabove,
                        'is_sibling': issibling,
                    })
        return navigation_mapping

    security.declareProtected(View, 'getNavigationTrail')
    def getNavigationTrail(self
                                , object_path=None
                                , path_in=None
                                , current_path=None
                                , sort_on=None
                                , sort_order=None
                                , submenu=None
                                , skip_root=None
                                , query=None
                            ):
        """Returns a list of the brain for the hierarcical path of the object
        If 'submenu' is true submenu are retrieved.
        """
        if type(query)!=type({}): query={}
        req = getattr(self, 'REQUEST', None)
        if req: vrpp = req.get('VirtualRootPhysicalPath', ('',))
        else: vrpp = ('',)

        if object_path is None: object_path = self.getPhysicalPath()
        if sort_on is None: sort_on = 'order'

        here=None
        req=getattr(self, 'REQUEST', None)
        if req is not None: here=getattr(req, 'here', None)
        current_path=object_path
        if here: current_path=here.getPhysicalPath()

        if path_in is not None and type(path_in) != type([]): path_in = [path_in]
        if path_in is None: path_in=self._calc_path_in(current_path)

        path_query = []
        path = ()
        for id in object_path:
            path = path + (id,)
            path_query.append(path)

        class LocalNavigationBrain(NavigationBrain):  pass
        LocalNavigationBrain.query=query
        LocalNavigationBrain.object_path=object_path
        LocalNavigationBrain.here=here
        LocalNavigationBrain.path_in=path_in
        LocalNavigationBrain.current_path=current_path
        LocalNavigationBrain.skip_root=skip_root
        LocalNavigationBrain.getMenu=self.getMenu
        LocalNavigationBrain.getNavigationTrail=self.getNavigationTrail
        LocalNavigationBrain.virtual_root_path_of_request=vrpp
        LocalNavigationBrain.domain=self.getCurrentDomainName()
        LocalNavigationBrain.NavigationService=self
        LocalNavigationBrain.EasyVHM=getattr(self, EASY_VHM, None)
        now = DateTime.now()
        LocalNavigationBrain.now=now

        #self._catalog.useBrains(LocalNavigationBrain)

        query['physicalPath'] = {'query':path_query}
        if not getSecurityManager().checkPermission(EasyViewAhead, self) or hasattr(self,'REQUEST') and hasattr(self.REQUEST,'anonymous'):
            query['getWorkflowState'] = {'query':wfPUBLISHED }
            query['effective_date'] = {'query': DateTime.now(), 'range': 'max'}
            query['expiration_date'] = {'query': now, 'range': 'min'}
            # Ugly hack, merging two queries, since Zope 2.5.1 sorts empty strings as higher than numbers
            menuobjectlist = self.searchResults(query)
            query['effective_date'] = {'query': ''}
            menuobjectlist = menuobjectlist + self.searchResults(query)
            menuobjectlist = list(menuobjectlist)
            menuobjectlist.sort( lambda a, b: cmp(a['physicalPath'], b['physicalPath']))
        else:
            menuobjectlist = self.searchResults(query, sort_on='physicalPath')

        if skip_root:
            menuobjectlist = list(menuobjectlist)
            menuobjectlist = menuobjectlist[1:]
        return menuobjectlist

    security.declareProtected(View, 'getNavigationTrailMap')
    def getNavigationTrailMap(self
                                    , object_path=None
                                    , path_in=None
                                    , current_path=None
                                    , sort_on=None
                                    , sort_order=None
                                    , submenu=None
                                    , skip_root=None
                                    , query=None
                                ):
        """\
        DEPRECATED, It is expensive
        Returns a mapping of the brain for the hierarcical path of the object
        If 'submenu' is true submenu are retrieved.
        The result will be a special list of dictionaries.
        """
        if type(query)!=type({}): query={}
        menuobjectlist = self.getNavigationTrail(object_path=object_path, path_in=path_in, current_path=current_path, sort_on=sort_on, sort_order=sort_order, submenu=submenu, skip_root=skip_root, query=query)

        #expensive because it loads the whole object in
        navigation_mapping = []
        for menuobject in menuobjectlist:
            physicalPath = menuobject.physicalPath
            if path_in is not None and physicalPath in path_in: in_path = 1
            else: in_path = 0
            submenu = None
            hasObjects = 0
            isabove = self.isAbove(physicalPath, object_path)
            issibling = self.isSibling(physicalPath, object_path)
            ischild = self.isChild(physicalPath, object_path)
            iscurrent = self.isCurrent(physicalPath, object_path)
            if not isabove and not iscurrent:
                if submenu or in_path:
                    submenu = self.getMenuMap(physicalPath, current_path=current_path, sort_on=sort_on, sort_order=sort_order, submenu=submenu-1, query=query)
                    hasObjects=len(submenu)
                elif in_path:
                    submenu = self.getMenuMap(physicalPath, current_path=current_path, path_in=path_in,sort_on=sort_on, sort_order=sort_order, query=query)
                    hasObjects=len(submenu)
                else:
                    hasObjects=len(self.getMenu(physicalPath, query=query))
            navigation_mapping.append({
                        'id': menuobject.id,
                        'title': menuobject.title or menuobject.navigation_title or menuobject.id,
                        'navigation_title': menuobject.navigation_title or menuobject.title or menuobject.id,
                        'physicalPath': physicalPath,
                        'virtualURL': menuobject.virtualURL,
                        'brain': menuobject,
                        'submenu': submenu,
                        'hasSubDocuments': hasObjects and not isabove,
                        'is_current': current_path == physicalPath,
                        'in_path': in_path,
                        'is_child': ischild,
                        'is_above': isabove,
                        'is_sibling': issibling,
                    })
        return navigation_mapping

    security.declareProtected(View, 'getNavigationContextMap')
    def getNavigationContextMap(self
                                , object_path=None
                                , start_path=None
                                , current_path=None
                                , sort_on=None
                                , sort_order=None
                                , submenu=None
                                , skip_root=None
                                , query=None
                                ):
        """
        DEPRECATED, It is expensive
        Renders a map of the navigation trail
        """
        if type(query)!=type({}): query={}
        here=None
        req=getattr(self, 'REQUEST', None)
        if req is not None: here=getattr(req, 'here', None)
        current_path=object_path
        if here: current_path=here.getPhysicalPath()
        path_in=self._calc_path_in(object_path)
        if start_path is None: start_path = ('',)
        if type(start_path)!=type(()): (start_path,) # Does this do any good really?
        return self.getMenuMap(start_path, current_path=current_path, path_in=path_in, sort_on=sort_on, sort_order=sort_order, skip_root=skip_root, query=query)


    security.declareProtected(AccessContentsInformation, 'get_object')
    def get_object(self, path, default=None, query=None):
        LOG('EasyNavigation', INFO, 'EasyNavigation:get_object', """EasyNavigation:get_object has been deprecated\n""")
        if type(query)!=type({}): query={}
        query['physicalPath'] = {'query':[path]}
        try:
            return self.searchResults(query)[0]
        except:
            return default

    # Extensive and slow version of Get Object Navigation Trail
    security.declareProtected(AccessContentsInformation, 'get_subobjects')
    def get_subobjects(self
                        , current_path
                        , object_path=None
                        , current_paths=None
                        , getsubmenu=None
                        , spec=None
                        , query=None
                      ):
        LOG('EasyNavigation', INFO, 'EasyNavigation:get_subobjects', """EasyNavigation:get_subobjects has been deprecated\n""")
        if type(query)!=type({}): query={}
        if spec is not None and type(spec) != type([]): spec = [spec]
        if type(current_path) != type([]): current_path = [current_path]

        query['virtualParentPath'] = {'query':current_path}
        if spec:
            query['meta_type'] = {'query':spec}
        menuobjectlist = self.searchResults(query)
        objects = []
        for menuobject in menuobjectlist:
            physicalPath = menuobject.physicalPath
            if physicalPath in current_paths: isInPath = 1
            else: isInPath = 0
            submenu = None
            hasObjects=0
            isabove = self.isAbove(physicalPath, object_path)
            issibling = self.isSibling(physicalPath, object_path)
            ischild = self.isChild(physicalPath, object_path)
            iscurrent = self.isCurrent(physicalPath, object_path)
            if not isabove and not iscurrent:
                if isInPath:
                    submenu = self.get_subobjects(physicalPath, object_path=object_path, current_paths=current_paths, getsubmenu=getsubmenu, spec=spec)
                    hasObjects=len(submenu)
                elif menuobject.hasSubDocuments and not isInPath and getsubmenu:
                    submenu = self.get_subobjects(physicalPath, object_path=object_path, current_paths=current_paths, getsubmenu=getsubmenu - 1, spec=spec)
                    hasObjects=len(submenu)
                else:
                    hasObjects=len(self.getMenu(physicalPath))
            objects.append({
                'id': menuobject.id,
                'title': menuobject.title or menuobject.navigation_title or menuobject.id,
                'navigation_title': menuobject.navigation_title or menuobject.title or menuobject.id,
                'physicalPath': physicalPath,
                'virtualURL': menuobject.virtualURL,
                'brain': menuobject,
                'submenu': submenu,
                'hasSubDocuments': hasObjects and not isabove,
                'is_current': object_path == physicalPath,
                'in_path': isInPath,
                'is_child': ischild,
                'is_above': isabove,
                'is_sibling': issibling,
                })
        return objects

    security.declareProtected(AccessContentsInformation, 'get_object_navigation_path')
    def get_object_navigation_path(self
                                    , object_path
                                    , root=None
                                    , getsubmenu=None
                                    , spec=None
                                    , query=None
                                  ):
        LOG('EasyNavigation', INFO, 'EasyNavigation:get_object_navigation_path', """EasyNavigation:get_object_navigation_path has been deprecated\n""")
        if type(query)!=type({}): query={}
        if spec is not None and type(spec) != type([]): spec = [spec]
        if root is None: root = self.getVirtualRootPath()
        current_paths = []
        path = ()
        for id in object_path:
            path = path + (id,)
            current_paths.append(path)
        query['physicalPath'] = {'query':[root]}
        if spec: query['meta_type'] = {'query':spec}
        menuobject = self.searchResults(query)[0]
        objects =[]
        physicalPath = menuobject.physicalPath
        if physicalPath in current_paths: isInPath = 1
        else: isInPath = 0
        isabove = self.isAbove(physicalPath, object_path)
        issibling = self.isSibling(physicalPath, object_path)
        ischild = self.isChild(physicalPath, object_path)
        iscurrent = self.isCurrent(physicalPath, object_path)
        hasObjects=0
        submenu=None
        if not isabove and not iscurrent:
            if menuobject.hasSubDocuments and isInPath:
                submenu = self.get_subobjects(physicalPath, object_path=object_path, current_paths=current_paths, getsubmenu=getsubmenu, spec=spec)
                hasObjects=len(submenu)
            elif menuobject.hasSubDocuments and not isInPath and getsubmenu:
                submenu = self.get_subobjects(physicalPath, object_path=object_path, current_paths=current_paths, getsubmenu=getsubmenu - 1, spec=spec)
                hasObjects=len(submenu)
            else:
                hasObjects=len(self.getMenu(physicalPath))
        objects.append({
            'id': menuobject.id,
            'title': menuobject.title or menuobject.navigation_title or menuobject.id,
            'navigation_title': menuobject.navigation_title or menuobject.title or menuobject.id,
            'physicalPath': physicalPath,
            'virtualURL': menuobject.virtualURL,
            'brain': menuobject,
            'submenu': submenu,
            'hasSubDocuments': hasObjects and not isabove,
            'is_current': object_path == physicalPath,
            'in_path': isInPath,
            'is_child': ischild,
            'is_above': isabove,
            'is_sibling': issibling,
            })
        return objects

    security.declareProtected(AccessContentsInformation, 'getHighestCountForMenu')
    def getHighestCountForMenu(self, navigation_menu):
        menuitems = self.getMenu(navigation_menu)
        if menuitems != []:
            lastmenuobject = menuitems[-1]
            try:
                return lastmenuobject.navigation_menus_data[navigation_menu][0] + 1
            except: pass
        return 0

    #RENDER FOLDER ICONS

    security.declareProtected(View, 'render_tree_icon_id')
    def render_tree_icon_id(self, open=0, hasSubDocuments=0, first=0, last=0):
        d = '1'
        if hasSubDocuments:
            if open:
                a = 'm'
            else:
                a = 'p'
        else:
            if first: a = '1'
            else: a = '0'
        if first: b = '0'
        else: b = '1'
        if last: c = '0'
        else: c = '1'
        return """%s%s%s%s""" % (a,b,c,d)

    security.declareProtected(View, 'render_tree_icon')
    def render_tree_icon(self, open=0, hasSubDocuments=0, first=0, last=0):
        id = self.render_tree_icon_id(open, hasSubDocuments, first, last)
        return """<img src="%s/misc_/EasyNavigation/%s" alt="" width="18" height="20" border="0">""" % (self.REQUEST['BASEPATH1'], id,)


    security.declareProtected(AccessContentsInformation, 'renderXmlTreeNode')
    def renderXmlTreeNode(self,brain,physpath,current_paths,fullmetadata=0,filter=None,easy_resources=None, isRoot=0):

        result = ""
        result = result + xmlObjectProperties(client=brain, current_paths=current_paths, physpath=physpath, fullmetadata=fullmetadata, isRoot=isRoot)

        if filter is None: filter = list(FILTERDEFAULT)

        if easy_resources is not None:
            pathTreeNodes = easy_resources.searchResults(physicalParentPath=[brain.physicalPath], meta_type=filter)
        else:
            pathTreeNodes = []

        # PATCH: 020124, Customer: Anders @ Connex
        # brain.hasSubDocuments only tests on EasyNavagation aware subdocumnets (not including EasyResources)
        # I have removed it to enable EasyFiles to be visible in the link dialog even
        # if there is no EasyDocument (et al) in the current folder.
        # This may have make the rendering of XML a bit slower (not sure really?)
        # if brain.hasSubDocuments and brain.physicalPath in current_paths:
        if brain.physicalPath in current_paths:
            pathTreeNodes = pathTreeNodes + self.searchResults(physicalParentPath=[brain.physicalPath], meta_type=filter)

        pathTreeNodes = list(pathTreeNodes)
        pathTreeNodes.sort(lambda x,y: cmp(x.id,y.id))

        if len(pathTreeNodes) > 0:
            result = result + xmlObjectsStart
            for treebrain in pathTreeNodes:
                if treebrain.physicalPath in current_paths or treebrain.physicalParentPath == physpath:
                    result = result + xmlObjectStart
                    result = result + self.renderXmlTreeNode(treebrain,physpath,current_paths,fullmetadata,easy_resources=easy_resources)
                    result = result + xmlObjectStop
            result = result + xmlObjectsStop
        return result

    security.declareProtected(AccessContentsInformation, 'xmlSubObjects')
    def xmlSubObjects(self, document_url, fullmetadata=0, filter=None, REQUEST=None, RESPONSE=None):
        """attw"""

        if document_url == "" or document_url[-1] != '/': document_url = document_url + '/'

        if filter is None: filter = list(FILTERDEFAULT)
        if type(filter) == type(()): filter = list(filter)
        if type(filter) == type(''): filter = [filter]
        if 'EasyRoot' not in filter: filter.append('EasyRoot')

        physpath = self.physicalParentPath
        current_paths = [physpath]

        for id in self.virtualURLToPhysicalPath(document_url)[len(physpath):]:
            if self.unrestrictedTraverse(physpath + (id,),None) is not None:
                physpath = physpath + (id,)
                current_paths.append(physpath)

        topdir = self.physicalParentPath

        pathTreeNodes = self.searchResults(physicalPath=[topdir], meta_type=filter)

        easy_resources = getattr(self,RESOURCE_CATALOG_ID,None)
        if easy_resources is not None:
            pathTreeNodes = pathTreeNodes + easy_resources.searchResults(physicalPath=[topdir], meta_type=filter)

        pathTreeNodes = list(pathTreeNodes)
        pathTreeNodes.sort(lambda x,y: cmp(x.id,y.id))

        result = xmlTreeStart
        result = result + xmlObjectsStart

        for treebrain in pathTreeNodes:
            if treebrain.physicalPath in current_paths:
                result = result + xmlObjectStart
                result = result + self.renderXmlTreeNode(treebrain,physpath,current_paths,fullmetadata,filter=filter,easy_resources=easy_resources, isRoot=1)
                result = result + xmlObjectStop
        result = result + xmlObjectsStop
        result = result + xmlTreeStop
        RESPONSE.setHeader('Content-Type', 'text/XML')
        return result


    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        #base=aq_base(self)
        ob_ver=getattr(self,'_object_version', 0.0)
        try: ob_ver=float(ob_ver)
        except: ob_ver=1.7

        changed, messages=upgrade_indexes(self, changed, DEFAULT_INDEXES, DEPRECATED_INDEXES, REQUIRED_INDEXES, DEFAULT_COLUMNS, DEPRECATED_COLUMNS, REQUIRED_COLUMNS)

        self._object_version = 1.8

        if REQUEST is not None:
            return "<h1>%s</h1>%s"%(changed and 'Changed' or 'Not Changed', ''.join(map(lambda x: '<p>%s</p>'%x, messages)),)
        return changed, messages


Globals.InitializeClass(EasyNavigation)



xmlTreeStart = """<?xml version="1.0" encoding="ISO-8859-1"?><body><tree>"""
xmlTreeStop =  """</tree></body>"""
xmlObjectsStart = """<objects>"""
xmlObjectsStop =  """</objects>"""
xmlObjectStart = """<object>"""
xmlObjectStop =  """</object>"""

xmlObjectProperties = HTML( """
<id><dtml-var id></id>
<display_id><dtml-var id></display_id>
<url><dtml-var virtualURL></url>
<title><![CDATA[<dtml-if title><dtml-var title><dtml-else><dtml-if navigation_title><dtml-var navigation_title><dtml-else><dtml-var id></dtml-if></dtml-if>]]></title>
<icon><dtml-var icon></icon>
<meta_type><dtml-var meta_type></meta_type>
<hasSubDocuments><dtml-var hasSubDocuments missing="0"></hasSubDocuments>
<isAnObjectManager><dtml-var isAnObjectManager missing="0"></isAnObjectManager>
<navigation_title><![CDATA[<dtml-if navigation_title><dtml-var navigation_title><dtml-else><dtml-var id></dtml-if>]]></navigation_title>
<parentURL><dtml-if "virtualParentPath is not _.None"><dtml-var "path2url(virtualParentPath)"></dtml-if></parentURL >
<isRoot><dtml-var isRoot missing="0"></isRoot>
<dtml-if fullmetadata>
    <effective_date><dtml-var effective_date></effective_date>
    <expiration_date><dtml-var expiration_date></expiration_date>
    <author><dtml-var author></author>
    <last_modified_by><dtml-var last_modified_by></last_modified_by>
    <workflowstate><dtml-var getWorkflowState></workflowstate>
    <workflowchangedby><dtml-var getWorkflowStateLastChangedBy></workflowchangedby>
    <workflowchangedate><dtml-var getWorkflowStateLastChangedDate></workflowchangedate>
</dtml-if>
<iscurrent><dtml-if "physicalPath == physpath">1<dtml-else>0</dtml-if></iscurrent>
<inpath><dtml-if "physicalPath in current_paths">1<dtml-else>0</dtml-if></inpath>
""", globals())




def manage_addEasyNavigation(self, REQUEST=None):
    """Add a EasyNavigation object"""
    c=EasyNavigation(NAVIGATION_CATALOG_ID, '', None, self)
    self._setObject(NAVIGATION_CATALOG_ID, c)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)




def initialize(context):
    """Initialize StyleManagers Product"""

    context.registerClass(
        EasyNavigation,
        permission=EasyNavigationServiceAdd,
        icon='icons/EasyNavigation.gif',
        constructors = ( manage_addEasyNavigation, manage_addEasyNavigation),
        meta_type = NAVIGATION_CATALOG_ID,
        category='Easy Publisher Management',

    )

    context.registerHelp()
    context.registerHelpTitle('Easy Help')



