# coding: iso-8859-1
# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyNewsService/EasyNewsService.py,v 1.3 2004/08/21 07:50:40 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__='''EasyNewsService product: EasyNewsService class.'''
__version__='1.5.7'

import Globals
from string import join, split
from types import StringType, DictType
from Globals import DTMLFile, HTML
import DateTime
from AccessControl import getSecurityManager
from AccessControl import ClassSecurityInfo

from Products.EasyCore.ZCatalog.ProtectedCatalog import ProtectedCatalog

from Products.EasyCore.UpgradeSupport.Utils import upgrade_indexes

from Products.EasyCore.VirtualPath import VirtualPathBase, mergePaths
from Products.EasyPresentationHelpers.EasyPresentationHelpers import EasyPresentationHelpers
from Products.EasyCore.EasyCorePermissions import View, AccessContentsInformation, EasyNewsServiceAdd, EasyViewArchived, EasyViewAhead
from Products.EasyCore.EasyNewsServiceConf import NEWS_SERVICE_ID, FILTERDEFAULT, EASYEDITABLEMETATYPES
from Products.EasyWorkflow.EasyWorkflowBase import wfDRAFT, wfSUBMITTED, wfDENIED, wfPUBLISHED, NullDate
from Products.EasyEditor.EasyPublisher import EasyPublisherBase

DEFAULT_INDEXES = {}
REQUIRED_INDEXES = {

          'id':'FieldIndex'
        , 'title':'TextIndex'
        , 'bobobase_modification_time':'FieldIndex'
        #, 'summary':'TextIndex'
        , 'path':'PathIndex'
        , 'meta_type':'FieldIndex'

        , 'effective_date':'FieldIndex'
        , 'expiration_date':'FieldIndex'
        , 'publish_date':'FieldIndex'
        , 'archive_date':'FieldIndex'
        , 'effective_year':'FieldIndex'
        , 'effective_month':'FieldIndex'
        , 'effective_month_name':'FieldIndex'
        , 'effective_day':'FieldIndex'
        , 'effective_weekday':'FieldIndex'
        , 'getWorkflowState':'FieldIndex'
        , 'author':'FieldIndex'
        , 'getVirtualRootPath':'FieldIndex'
        , 'virtualPath':'FieldIndex'
        , 'physicalPath':'FieldIndex'
        , 'virtualParentPath':'FieldIndex' #getVirtualParentPath
        , 'hasSubDocuments':'FieldIndex'
        , 'category':'KeywordIndex'
        , 'subject':'KeywordIndex'
        , 'getTemplateId':'FieldIndex'
        , 'physicalPathIndex':'PathIndex'
}
DEPRECATED_INDEXES=('id','title','meta_type','bobobase_modification_time','PrincipiaSearchSource','path')


DEFAULT_COLUMNS = ()
REQUIRED_COLUMNS=(
          'id'
        , 'title'
        , 'meta_type'
        , 'bobobase_modification_time'
        , 'summary'

        , 'creation_date'
        , 'effective_date'
        , 'expiration_date'
        , 'publish_date'
        , 'archive_date'
        , 'effective_year'
        , 'effective_month'
        , 'effective_month_name'
        , 'effective_day'
        , 'effective_weekday'
        , 'getWorkflowState'
        , 'author'
        , 'icon'
        , 'getVirtualRootPath'
        , 'virtualPath'
        , 'virtualURL'
        , 'physicalPath'
        , 'virtualParentPath' #getVirtualParentPath
        , 'hasSubDocuments'
        , 'category'
        , 'subject'
        , 'description'
        , 'navigation_title'
        , 'getTemplateId'
        , 'isAnObjectManager'
)
DEPRECATED_COLUMNS=('summary', 'bobobase_modification_time', 'meta_type', 'title', 'id')

class EasyNewsService(ProtectedCatalog, VirtualPathBase, EasyPresentationHelpers, EasyPublisherBase):
    """ EasyNewsService object
        The EasyNewsService Catalog is a ProtectedCatalog that is extended to work as a
        news service. "EasyNewsAware" objects are automatically indexed in the
        EasyNewsService, and functions to get a list of objects that are either
        entries in a menu or entries in a path.
    """
    meta_type = NEWS_SERVICE_ID


    security = ClassSecurityInfo()


    #manage_options = EasyNewsService.inheritedAttribute('manage_options')
    manage_options = ProtectedCatalog.manage_options


    #(
    #{'label': 'NewsServ',           # TAB: Contents
    # 'action': 'manage_workflow',
    # 'help': ('EasyNavigation','WorkflowOverview.stx')},
    #) + ProtectedCatalog.inheritedAttribute('manage_options')


    def __init__(self, id, title='', vocab_id=None, container=None):
        ProtectedCatalog.__init__(self, id, title, vocab_id, container)

        # More default fields and indexes:

        for id in DEPRECATED_INDEXES:
            self.delIndex(id, force=1, dont_raise=1)
        for id, type in REQUIRED_INDEXES.items():
            self.addIndex(id, type, undeletable=1)
        for id, type in DEFAULT_INDEXES.items():
            self.addIndex(id, type)

        for id in DEPRECATED_COLUMNS:
            self.delColumn(id, force=1, dont_raise=1)
        for id in REQUIRED_COLUMNS:
            self.addColumn(id, undeletable=1)
        for id in DEFAULT_COLUMNS:
            self.addColumn(id)



    security.declareProtected(View, 'getNewsMenu')
    def getNewsMenu(self
            , query=None
            , sort_on=None
            , sort_order=None
            , includepublished=1
            , includearchived=0
            , physicalPathIndex=None
            , **kw):
        """ Returns the list of menuentries for the menu navigation_menu.
        """
        caller = getattr(self, 'aq_parent', None)
        if caller:
            easyviewarchived = getSecurityManager().checkPermission(EasyViewArchived, caller)
            easyviewahead = getSecurityManager().checkPermission(EasyViewAhead, caller)
        else:
            easyviewarchived = 1
            easyviewahead = 0
        if query is None or type(query)!=DictType: query = {}

        if not easyviewahead or hasattr(self,'REQUEST') and hasattr(self.REQUEST,'anonymous'):
            query['getWorkflowState'] = {'query':wfPUBLISHED}
#        else:
#            #This is a workaround to get by the "can't sort without query problem" (this bug seems to be solved in Zope 2.4.x)
#            query['getWorkflowState'] = {'query':[wfDRAFT, wfSUBMITTED, wfDENIED, wfPUBLISHED]}

        if kw:
            query.update(kw)

        if physicalPathIndex and type(physicalPathIndex) == StringType:
            query['physicalPathIndex'] = {'query':physicalPathIndex }

        #Pre sort
        if sort_on is None: sort_on = 'publish_date'
        if sort_order is None: sort_order = 'reverse'
        if sort_on is not None and not query.has_key('sort_on') and not query.has_key('sort-on'):
            query['sort_on'] = sort_on
            query['sort-order'] = sort_order

        now = DateTime.now()
        #Don't show archive if use don't have EasyViewArchived permission, no use anyway because teh
        #probably don't have permission to view archived news items anyway.
        includearchived = includearchived and easyviewarchived
        #I'm not sure how to handle ViewAhead scenarios?


        if includepublished and includearchived:
            query['publish_date'] = {'query':now, 'range': 'max'}
        elif includepublished and not includearchived:
            query['publish_date'] = {'query':now, 'range': 'max'}
            query['expiration_date'] = {'query':now, 'range': 'min'}
        elif not includepublished and includearchived:
            query['publish_date'] = {'query':now, 'range': 'max'}
            query['expiration_date'] =  {'query':now, 'range': 'max'}
        elif not includepublished and not includearchived:
            return []

        #Search
        return self.searchResults(query)


    security.declareProtected(View, 'getYears')
    def getYears(self
            , showpublished=1
            , showarchived=None
            , **kw):
        years = list(self.uniqueValuesFor('effective_year'))
        years.sort()
        years.reverse()
        return years


    security.declareProtected(View, 'getMonthsOfYear')
    def getMonthsOfYear(self
            , showpublished=1
            , showarchived=None
            , text=None
            , **kw):
        eff_months = list(self.uniqueValuesFor('effective_month'))
        eff_months.sort()
        if text:
            if hasattr(self,'monthnames'):
                nom=self.monthnames
            else:
                nom=english_month_names
            return map(lambda x, nom=nom: nom[x-1], eff_months)
        return eff_months


    security.declareProtected(AccessContentsInformation, 'renderXmlTreeNode')
    def renderXmlTreeNode(self,brain,virtpath,current_paths,fullmetadata=0,filter=None,easy_resources=None):
        pass

    security.declareProtected(AccessContentsInformation, 'xmlSubObjects')
    def xmlSubObjects(self, document_url, root=None, fullmetadata=0, filter=None, REQUEST=None, RESPONSE=None):
        pass

    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        #base=aq_base(self)
        ob_ver=getattr(self,'_object_version', 0.0)
        try: ob_ver=float(ob_ver)
        except: ob_ver=1.7


        changed, messages=upgrade_indexes(self, changed, DEFAULT_INDEXES, DEPRECATED_INDEXES, REQUIRED_INDEXES, DEFAULT_COLUMNS, DEPRECATED_COLUMNS, REQUIRED_COLUMNS)

        self._object_version = 1.75

        if REQUEST is not None:
            return "<h1>%s</h1>%s"%(changed and 'Changed' or 'Not Changed', ''.join(map(lambda x: '<p>%s</p>'%x, messages)),)
        return changed, messages






Globals.InitializeClass(EasyNewsService)


# MIXIN DEFINITIONS ALSO IN EASYNAVIGATION
xmlTreeStart = """<?xml version="1.0" encoding="ISO-8859-1"?><body><tree>"""
xmlTreeStop =  """</tree></body>"""
xmlObjectsStart = """<objects>"""
xmlObjectsStop =  """</objects>"""
xmlObjectStart = """<object>"""
xmlObjectStop =  """</object>"""

xmlObjectProperties = HTML( """
<id><dtml-var id></id>
<url><dtml-var physicalURL></url>
<title><![CDATA[<dtml-if title><dtml-var title><dtml-else><dtml-if navigation_title><dtml-var navigation_title><dtml-else><dtml-var id></dtml-if></dtml-if>]]></title>
<icon><dtml-var icon></icon>
<meta_type><dtml-var meta_type></meta_type>
<hasSubDocuments><dtml-var hasSubDocuments missing="0"></hasSubDocuments>
<isAnObjectManager><dtml-var isAnObjectManager missing="0"></isAnObjectManager>
<navigation_title><![CDATA[<dtml-if navigation_title><dtml-var navigation_title><dtml-else><dtml-var id></dtml-if>]]></navigation_title>
<parentURL><dtml-if "virtualParentPath is not _.None"><dtml-var "path2root_safe_url(virtualParentPath)"></dtml-if></parentURL >
<dtml-if fullmetadata>
    <effective_date><dtml-var effective_date></effective_date>
    <expiration_date><dtml-var expiration_date></expiration_date>
    <author><dtml-var author></author>
    <last_modified_by><dtml-var last_modified_by></last_modified_by>
    <workflowstate><dtml-var getWorkflowState></workflowstate>
    <workflowchangedby><dtml-var getWorkflowStateLastChangedBy></workflowchangedby>
    <workflowchangedate><dtml-var getWorkflowStateLastChangedDate></workflowchangedate>
</dtml-if>
<iscurrent><dtml-if "physicalPath == virtpath">1<dtml-else>0</dtml-if></iscurrent>
<inpath><dtml-if "physicalPath in current_paths">1<dtml-else>0</dtml-if></inpath>
""", globals())

english_weekday_names = ['Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday']
swedish_weekday_names = ['Sndag','Mondag','Tisdag','Onsdag','Torsdag','Fredag','Lrdag']
names_of_weekday = {
    'default':english_weekday_names,
    'en':english_weekday_names,
    'sv':swedish_weekday_names,
}

english_month_names=['January','February','March','April','May','June','July',
                     'August', 'September', 'October', 'November', 'December']
swedish_month_names=['Januari','Februari','Mars','April','Maj','Juni','Juli',
                     'Augusti', 'September', 'October', 'November', 'December']
names_of_months = {
    'default':english_month_names,
    'en':english_month_names,
    'sv':swedish_month_names,
}


def manage_addEasyNewsService(self, REQUEST=None):
    """Add a EasyNewsService object"""
    c=EasyNewsService(NEWS_SERVICE_ID, '', None, self)
    self._setObject(NEWS_SERVICE_ID, c)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)




def initialize(context):
    """Initialize EasyNewsService Product"""

    context.registerClass(
        EasyNewsService,
        permission=EasyNewsServiceAdd,
        icon='icons/EasyNewsService.gif',
        constructors = ( manage_addEasyNewsService, manage_addEasyNewsService),
        meta_type = NEWS_SERVICE_ID,
        category='Easy Publisher Management',
    )


