# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyNotification/EasyNotification.py,v 1.2 2004/06/29 14:14:16 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyNotification Object"""
__version__='$Revision: 1.2 $'[11:-2]



import Globals
from AccessControl import ClassSecurityInfo
from OFS.Folder import Folder
from Globals import DTMLFile
from Acquisition import aq_base
from string import join
from Products.EasyCore.EasyCorePermissions import *
from OFS.ObjectManager import IFAwareObjectManager
from Products.EasyCore.interfaces.EasyNotification import EasyNotificationInfoInterface
from Products.EasyNotification.DefaultNotifications import send_error_message, notifiable_workflow_transitions
from Products.EasyNotification.EasyNotificationInfo import EasyNotificationInfo
from zLOG import LOG, INFO

from Products.EasyCore import MLOG_TAB


EASYNOTIFICATIONID = 'EasyNotification'


class EasyNotification(IFAwareObjectManager, Folder):
    """
    A Service for sending notification via e-mail.
    """
    security = ClassSecurityInfo()

    meta_type = EASYNOTIFICATIONID
    isPrincipiaFolderish=1


    manage_options=Folder.manage_options

    # This is here as a "private" meta type. It will show up in "all_meta_types",
    # which is necessary to be able to copy EasyObjects into this EasyObject, but
    # it will not appear in filtered_meta_types, and hence not in the add box.
    # This is all pretty much an undocemented and magical part of Zope.

    _product_meta_types = ({'name': EASYNOTIFICATIONID, 'meta_type':EASYNOTIFICATIONID, 'action': ''}, )
    _product_interfaces = (EasyNotificationInfoInterface, )

    def __init__(self, id, title=''):
        """initialise a new instance"""
        self.id = id
        self.title = title

    security.declarePrivate('getNotifiableWorkflowTransitions')
    def getNotifiableWorkflowTransitions(self):
        raise "Deprecated"
        return notifiable_workflow_transitions

    security.declarePrivate('getFilteredDefaultNotifications')
    def getFilteredDefaultNotifications(self):
        local_ids=self.objectIds(spec='EasyNotificationInfo')
        return map( lambda entry:
                    (entry[0], entry[1][0]),
                    filter( lambda entity, local_ids=local_ids:
                                entity[0] not in local_ids,
                                notifiable_workflow_transitions.items()))


    security.declarePrivate('getDefaultNotifications')
    def getDefaultNotifications(self):
        return map(lambda entry: (entry[0], entry[1][0]), notifiable_workflow_transitions.items())

    security.declarePrivate('getDefaultNotificationItems')
    def getDefaultNotificationItems(self):
        return getDefaultNotificationItems()

    security.declarePrivate('getNotifiableWorkflowTransitionIdsFor')
    def getNotifiableWorkflowTransitionIdsFor(self, object):
        return self.objectIds(spec='EasyNotificationInfo')

    security.declarePrivate('getNotifiableWorkflowTransitionItemsFor')
    def getNotifiableWorkflowTransitionItemsFor(self, object):
        return self.objectItems(spec='EasyNotificationInfo')


    security.declarePrivate('manage_notify')
    def manage_notify(self, notification, fromuser, tousers, namespace, context):

        easy_mail_service = getattr(self, 'EasyMailService', None)
        if easy_mail_service is None:
            LOG('EasyNotification', INFO, 'No EasyMailService installed')
            return

        notify_emails = ()
        fromuserdict = self.getUsersByIds(fromuser, context)
        touserdict = self.getUsersByIds(tousers, context)

        fromemail = None
        if fromuserdict.has_key(fromuser):
            try:
                fromemail = fromuserdict[fromuser].getEmail()
            except:
                fromemail = fromuser + ' NO E-MAIL ADDRESS'

        for name, user in touserdict.items():
            try:
                notify_emails =  notify_emails + (user.getEmail(),)
            except:
                LOG('EasyNotification', INFO, '', name + ' ' + user)

        if notify_emails:
            nwft = self.getNotificationByName(notification)
            if nwft is not None:
                namespace['from_email'] = fromemail
                namespace['to_emails'] = join(notify_emails, ', ')
                notification_message = nwft.render_message(context, namespace)
                notification_subject = nwft.render_subject(context, namespace)

                if fromemail is None or fromemail.strip() == '':
                    error_mail=send_error_message(error_message="USER %s HAS NO EMAIL ADRESS"%str(fromemail), from_addr=fromemail, to_addr=join(notify_emails,', '), subject=notification_subject, message=notification_message )
                    LOG('EasyNotification', INFO, 'manage_notify', error_mail)
                    return

                try:
                    easy_mail_service.send(notification_message, notify_emails, fromemail, notification_subject)
                except:
                    LOG('EasyNotification', INFO, 'manage_notify', send_error_message(error_message="SENDING MAIL FAILED", from_addr=fromemail, to_addr=join(notify_emails,', '), subject=notification_subject, message=notification_message ))

    def getNotificationByName(self, notification_name, default=None):
        if notification_name in self.objectIds(spec='EasyNotificationInfo'):
            return self._getOb(notification_name)
        return default


    #It should really be somewhere else, who's knows about Users Ids?
    security.declarePrivate('getUsersByIds')
    def getUsersByIds(self, userids, context=None):
        if context:
            item=context
        else:
            item=self
        dict={}
        if type(userids) not in (type(()), type([])):
            userids = (userids,)
        while len(dict.keys()) < len(userids):
            if hasattr(aq_base(item), 'acl_users') and \
               hasattr(item.acl_users, 'user_names'):
                for name in userids:
                    if not dict.has_key(name):
                        userobj = item.acl_users.getUser(name)
                        if userobj:
                            dict[name] = userobj
            if not hasattr(item, 'aq_parent'):
                break
            item=item.aq_parent
        return dict


    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        base=aq_base(self)
        ob_ver=getattr(self,'_object_version', 0.0)
        try: ob_ver=float(ob_ver)
        except: ob_ver=0.0
        if ob_ver<1.8:
            for nid, value in notifiable_workflow_transitions.items():
                if not hasattr(base, nid):
                    not_obj=EasyNotificationInfo(nid, value[0],value[0], value[1], value[2])
                    self._setObject(nid,not_obj)

        self._object_version=1.8

        if REQUEST is not None:
            return "<h1>Done</h1>"
        return changed

Globals.InitializeClass(EasyNotification)


def getDefaultNotificationItems():
    return notifiable_workflow_transitions.items()

#manage_addEasyNotificationForm=DTMLFile('dtml/addEasyNotification_form', globals())

def manage_addEasyNotification(self, REQUEST=None):
    """\
    Add EasyNotification service into the system.
    """

    id="EasyNotification"
    title=""

    ens=EasyNotification(id, title)

    self._setObject(id,ens)
    service=self._getOb(id)
    for nid, value in getDefaultNotificationItems():
        not_obj=EasyNotificationInfo(nid, value[0],value[0], value[1], value[2])
        #not_obj.manage_update()
        service._setObject(nid,not_obj)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_workspace')



def initialize(context):
    """Initialize EasyNotification Product"""


    context.registerClass(
        EasyNotification,
        meta_type = EasyNotification.meta_type,
        permission=EasyNotificationServiceAdd,
        constructors=(manage_addEasyNotification,
                      manage_addEasyNotification),
        icon='icons/EasyNotification.gif',
        category='Easy Publisher Management',
    )


