# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyNotification/EasyNotificationInfo.py,v 1.1 2004/06/28 10:05:47 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#


__doc__="""\
EasyNotificationInfo
"""
__version__='$Revision: 1.1 $'[11:-2]


import Globals
from Globals import DTMLFile, HTML
from Acquisition import aq_base
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from OFS.ObjectManager import BeforeDeleteException
from Products.EasyCore.EasyCorePermissions import *
from Products.EasyNotification.DefaultNotifications import notifiable_workflow_transitions
from Products.EasyCore.interfaces.EasyNotification import EasyNotificationInfoInterface


from Products.EasyCore import MLOG_TAB


class EasyNotificationInfo(PropertyManager, SimpleItem):
    """\
    A Information Object holding information about a certain notification type.
    """

    security = ClassSecurityInfo()

    meta_type="EasyNotificationInfo"

    manage_options=PropertyManager.manage_options + SimpleItem.manage_options

    title=''
    description=''
    subject=''
    message=''
    reply_to_address=''

    _properties=(
        {'id':'title','label':'Title','type':'string', 'mode': 'w', 'select_variable':''},
        {'id':'description','label':'Description','type':'string', 'mode': 'w', 'select_variable':''},
        {'id':'subject','label':'Subject','type':'text', 'mode': 'w', 'select_variable':''},
        {'id':'message','label':'Message','type':'text', 'mode': 'w', 'select_variable':''},
        )
        #{'id':'reply_to_address','label':'ReplyTo Address','type':'text', 'mode': 'w', 'select_variable':''},


    def __init__(self, id, title='', description='', subject='', message=''):
        self.id=self.__name__=id
        self.title=title
        self.description=description
        self.subject=subject
        self.message=message


    security.declareProtected(EasyNotificationManage, 'manage_update')
    def manage_update(self, title, description, subject, message, REQUEST=None):
        self.title=title
        self.description=description
        self.subject=subject
        self.message=message


    security.declarePrivate('render_subject')
    def render_subject(self, object, namespace):
        return HTML(self.subject, globals())(object, namespace)

    security.declarePrivate('render_message')
    def render_message(self, object, namespace):
        return HTML(self.message, globals())(object, namespace)


    #def manage_beforeDelete(self, item, container):
    #    if item.getId() in notifiable_workflow_transitions.keys():
    #        raise BeforeDeleteException


    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        base=aq_base(self)
        if REQUEST is not None:
            return "<h1>Done</h1>"
        return changed




Globals.InitializeClass(EasyNotificationInfo)


manage_addEasyNotificationInfoForm=DTMLFile('dtml/manage_addEasyNotificationInfoForm', globals())

def manage_addEasyNotificationInfo(self, type_of_notification, id='', title='', REQUEST=None):
    """\
    Add a EasyNotificationInfo into the EasyNotification object.
    """


    if type_of_notification == 'custom':
        object=EasyNotificationInfo(id, title)
    else:
        id=type_of_notification
        notification=notifiable_workflow_transitions.get(type_of_notification, None)
        if notification is None:
            raise ValueError, "There doesn't exist a default notification named %s" % type_of_notification
        object=EasyNotificationInfo(id, notification[0],notification[0], notification[1], notification[2])

    self._setObject(id, object)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_workspace')



def initialize(context):
    """Initialize EasyNotification Product"""


    context.registerClass(
        EasyNotificationInfo,
        permission=EasyNotificationManage,
        meta_type = EasyNotificationInfo.meta_type,
        constructors=(manage_addEasyNotificationInfoForm,
                      manage_addEasyNotificationInfo),
        icon='icons/EasyNotificationInfo.gif',
        interfaces=(EasyNotificationInfoInterface,),
        visibility = None,
    )


