# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyPresentationHelpers/EasyParameterTracker.py,v 1.2 2004/09/09 13:25:46 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#
__version__='$Revision: 1.2 $'[11:-2]
__doc__='''EasyParameterTracker product: EasyParameterTracker mixin class.'''


import Globals
from string import join, split, lower, count
from ExtensionClass import Base
from types import StringType, TupleType, ListType, IntType, LongType, FloatType

from DateTime import DateTime
from Globals import DTMLFile, HTML
from AccessControl import getSecurityManager
from AccessControl import ClassSecurityInfo
from zLOG import LOG, INFO

from Products.EasyCore.EasyCorePermissions import View, AccessContentsInformation
from Products.EasyCore.Utils.parse_parameters import fields2parameters, parameters2fields, type_to_string_mapping, type_defaults
from Products.EasyCore.EasyCorePermissions import setDefaultRoles
from Products.EasyPresentationHelpers.EasyPresentationHelpers import EasyPresentationHelpers

from ZPublisher.Converters import type_converters
from Products.Formulator.FieldRegistry import FieldRegistry
from Products.Formulator.Validator import ValidationError
from Acquisition import aq_base

from ZTUtils.Batch import Batch


class TypeConvertException(Exception):
    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, error_key, value, value_type):
        Exception.__init__(self, error_key)
        self.error_key = error_key
        self.value = value
        self.value_type = value_type
        self.error_text = 'TypeConvertion error when converting [%s] to the type [%s (%s)]' % (value,value_type,type(value_type))

TUPLETYPE = type(())
LISTTYPE = type([])
STRINGTYPE = type('')
INTTYPE = type(0)
LONGTYPE = type(0L)
FLOATTYPE = type(0.0)
TOKENSTYPE = "tokens"
LINESTYPE = "lines"

def list_type_convert(value_list, value_type):
    result = []
    if type(value_list) in (TUPLETYPE,LISTTYPE):
        for value in value_list:
            result.append(type_convert(value,value_type))
        if type(value_list)==TUPLETYPE:
            return tuple(result)
        return result
    raise TypeConvertException("ListTypeConvetionException",res, value_type)



def type_default(value_type):
    return type_defaults.get(value_type, None)

def guess_property_type(type_value):
    return type_to_string_mapping.get(type_value, None)


def type_convert(res, value_type):
    """
    Converts a value to the requested value_type

    If an empty value is submitted it with be returned
    as an empty value in the value_type type.

    If the value_type is 'tokens' or 'lines' the convertion
    too from ZPublisher is used to convert the value
    """
    second_value_type=None
    vtype = type(value_type)
    rtype = type(res)
    if vtype==STRINGTYPE and count(value_type,':'):
        value_types = split(value_type,':')
        value_type = value_types[-1]
        second_value_type=join(value_types[:-1],':')
    if value_type=='tuple':
        if rtype==LISTTYPE: return tuple(res)
        if rtype==TUPLETYPE: return res
        return (res,)
    elif value_type=='list':
        if rtype==TUPLETYPE: return list(res)
        elif rtype==LISTTYPE: return res
        return [res]
    elif value_type in type_converters.keys():
        if not res: return type_default(value_type)
        res = type_converters[value_type](res)
        if second_value_type:
            res = list_type_convert(res, second_value_type)
        return res
    elif vtype==type(res):
        return res
    elif vtype==TUPLETYPE and rtype==LISTTYPE:
        return tuple(res)
    elif vtype==TUPLETYPE and rtype!=LISTTYPE:
        if not res: return ()
        return (res,)
    elif vtype==LISTTYPE and rtype==TUPLETYPE:
        return list(res)
    elif vtype==LISTTYPE and rtype!=TUPLETYPE:
        if not res: return []
        return [res]
    elif vtype==STRINGTYPE and rtype in (TUPLETYPE,LISTTYPE):
        return join(res)
    elif vtype==STRINGTYPE:
        if not res: return ''
        return str(res)
    elif vtype==INTTYPE:
        return int(res)
    elif vtype==LONGTYPE:
        return long(res)
    elif vtype==FLOATTYPE:
        return float(res)
    raise TypeConvertException("TypeConvetionException",res, value_type)


EasyParameterTrackerAdd = 'Easy Parameter Tracker Add'
setDefaultRoles(EasyParameterTrackerAdd, ('Manager',))

FROM_DEFAULT = 0
FROM_REQUEST = 1
FROM_SESSION = 2
FROM_REQUEST_FIELD = 3
FROM_DEFAULT_EXPR = 4


#Change this to handle empty Trackers (e.g. track_fields should be
#removed it it's empty).
#This would let Trackers be used as base classes for hieracical
#objects. For instance I just added EasyParameterTracker to
#Templates, if track_fields was removed I could add it to
#EasyEditor and EasyDocument as well.
#The Tracker currently used would be the one closest in
#acqusition terms. -- Johan

from Products.EasyCore import MLOG_TAB

class Batches:

    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, search_result
                     , batch_size=None
                     , query_start=None
                     , max_batch_size=None
                     , before_orphans=None
                     , after_orphans=None
                     , orphan=0
                     , overlap=0
                     , quantizies=1
                     ):
        self.init(search_result
                    , batch_size=batch_size
                    , query_start=query_start
                    , max_batch_size=max_batch_size
                    , before_orphans=before_orphans
                    , after_orphans=after_orphans
                    , orphan=orphan
                    , overlap=overlap
                    , quantizies=quantizies)


    def init(self, search_result
                    , batch_size=None
                    , query_start=None
                    , max_batch_size=None
                    , before_orphans=None
                    , after_orphans=None
                    , orphan=0
                    , overlap=0
                    , quantizies=1
                ):

        batch_size=self.batch_size=self.calculate_batch_size(search_result, batch_size=batch_size, max_batch_size=max_batch_size)

        try: query_start=int(query_start)
        except: query_start=1
        if not query_start: query_start=1
        self.query_start=query_start

        search_length=self.search_length=len(search_result)

        #Post preparations
        search_batches = []
        overflow_branch=0

        previous = query_start - batch_size
        previous_size = batch_size
        if previous<1: previous = None

        next = query_start + batch_size
        next_size = batch_size
        if next>search_length: next = None

        number_of_batches = search_length/batch_size + 1
        rest = search_length%batch_size
        number_of_batches=number_of_batches+(not not rest)

        #query_start

        current_batch=query_start/batch_size + 1
        if quantizies:
            query_start=(current_batch-1)*batch_size+1

        if not before_orphans:
            before_orphans=0
            start_batch=1
        else:
            start_batch=max(current_batch-before_orphans,1)
        if not after_orphans:
            after_orphans=0
            end_batch=number_of_batches
        else:
            end_batch=min(current_batch+after_orphans,number_of_batches)


        self.before_orphans=before_orphans
        self.after_orphans=after_orphans
        self.current_batch=current_batch
        self.start_batch=start_batch
        self.end_batch=end_batch


        for batch_number in range(start_batch,end_batch):
            if batch_number==(end_batch-1) and rest:
                start_index = ((end_batch - 2) * batch_size) + 1
                stop_index = ((end_batch - 2) * batch_size) + rest
                search_batches.append(BatchInfo(batch_number, number_of_batches, start_index, stop_index, query_start, search_length, batch_size))
            else:
                start_index = (batch_number - 1) * batch_size + 1
                stop_index =  (batch_number - 1) * batch_size + batch_size
                search_batches.append(BatchInfo(batch_number, number_of_batches, start_index, stop_index, query_start, search_length, batch_size))

        #post set-up
        prev=None
        self.prev_batch=None
        self.next_batch=None
        for index in range(len(search_batches)):
            batch=search_batches[index]
            if prev:
                batch.setPrevSize(prev.size)
                if batch.is_in_batch:
                    self.prev_batch=prev
            if index+1<len(search_batches):
                batch.setNextSize(search_batches[index+1].size)
                if batch.is_in_batch:
                    self.next_batch=search_batches[index+1]
            prev=batch


        self.values=Batch(search_result, batch_size, start=(query_start-1), end=0, orphan=orphan, overlap=overlap)
        self.batches=search_batches


    def calculate_batch_size(self, search_result, batch_size=None, max_batch_size=None):
        try: batch_size=int(batch_size)
        except: batch_size=len(search_result)
        try: max_batch_size=int(max_batch_size)
        except: max_batch_size=None
        if max_batch_size and (batch_size>max_batch_size or batch_size==0):
            batch_size=max_batch_size
        return batch_size

    def calculate_index(self, index=1):
        return self.query_start+index-1

    def calculate_index_inv(self, index=1):
        return self.search_length-(self.query_start+index-2)


class BatchInfo:

    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, batch_number, number_of_batches, start_index, stop_index, query_start, search_length, batch_size, dummy=0, overflow=0):
        self.dummy=dummy
        self.batch_number = batch_number
        self.number_of_batches=number_of_batches
        self.start_index = start_index
        self.stop_index = stop_index
        self.start_index_inv = search_length-start_index+1
        self.stop_index_inv = search_length-stop_index+1
        self.is_in_batch = (query_start>=start_index and query_start<=stop_index)
        self.size = stop_index-start_index+1
        self.batch_size = batch_size
        self.search_length = search_length
        self.query_start = query_start
        self.next_size = 0
        self.prev_size = 0
        self.overflow=overflow

    def setNextSize(self, next_size=0):
        self.next_size = next_size

    def setPrevSize(self, prev_size=0):
        self.prev_size = prev_size

    def display_start(self):
        return str(self.start_index)

    def display_start_inv(self):
        return str(self.start_index_inv)

    def display_stop(self):
        return str(self.stop_index)

    def display_stop_inv(self):
        return str(self.stop_index_inv)

    def render(self, template='<dtml-var batch_number>', REQUEST=None):
        return HTML(template, globals())(self,REQUEST=REQUEST)


class EasyParameterTracker(EasyPresentationHelpers):
    """
    EasyParameterTracker mix-in class

    The EasyParameterTracker keeps track of parameters that
    the user might supply to an application.
    """

    security = ClassSecurityInfo()

    meta_type='EasyParameterTracker'
    error_fields=()

    FROM_DEFAULT = FROM_DEFAULT
    FROM_REQUEST = FROM_REQUEST
    FROM_SESSION = FROM_SESSION
    FROM_REQUEST_FIELD = FROM_REQUEST_FIELD
    FROM_DEFAULT_EXPR = FROM_DEFAULT_EXPR

    #Remove this and make track_fields optional (see comment above).
    #track_fields = ()
    manage_options = ({'label':'Tracker', 'action':'manage_parameter_tracker'},)

    def __init__(self, id, path=None, title='', timeout_mins=20, onstart=None, onend=None, track_fields=None):
        EasyParameterTracker.inheritedAttribute('__init__')(self, id=id, path=path, title=title, timeout_mins=timeout_mins, onstart=onstart, onend=onend)
        if track_fields:
            self.track_fields = track_fields
        self.error_fields=()

    security.declarePrivate('renderTrackFields')
    def renderTrackFields(self, include_errors=1):
        """parse the TrackFileds to a textarea:lines"""
        tf=()
        if hasattr(self, 'track_fields'):
            tf=self.track_fields
        tf=tf+self.error_fields
        if tf:
            return join(parameters2fields(tf),'\n')
        return ''

    security.declarePrivate('count_parameters')
    def count_parameters(self):
        """Returns the number of parameters plus 1"""
        result=0
        if hasattr(self, 'track_fields'):
            result=len(self.track_fields)
        result=result+len(self.error_fields) + 1
        return result

    security.declarePrivate('_manage_parameter_tracker')
    _manage_parameter_tracker = DTMLFile('dtml/manage_parameter_tracker', globals())

    security.declareProtected(EasyParameterTrackerAdd, 'manage_parameter_tracker')
    def manage_parameter_tracker(self, title=None, submit=None, track_fields=None, REQUEST=None):
        """Saves Tracking Parameters"""
        if title is not None: self.title = title

        if track_fields is not None:
            track_fields, error_fields = fields2parameters(track_fields)
            if track_fields: self.track_fields = track_fields
            elif hasattr(aq_base(self), 'track_fields'): delattr(self, 'track_fields')
            if error_fields: self.error_fields = error_fields

        if REQUEST is not None:
            manage_tabs_message = ""
            management_view = "Tracker"
            return self._manage_parameter_tracker(self,REQUEST,manage_tabs_message=manage_tabs_message,management_view=management_view)

    # Site Search API
    security.declareProtected(View, 'initTracking')
    def initTracking(self, **kw):
        """
        Init tracked parameters.

        **kw is default optional key:values
        """

        REQUEST = getattr(self,'REQUEST', None)
        if REQUEST:
            submitted_fields_ids = self.getSubmittedFieldIds(REQUEST=REQUEST)
            session = REQUEST.SESSION
            if REQUEST.has_key('reset_tracker') and REQUEST['reset_tracker']:
                session.invalidate()
            done = ()
            track_fields = getattr(self, 'track_fields', [])
            for parameter in track_fields:
                id = parameter['id']
                ptype = parameter['type']
                default = parameter['default']
                default_expr = 'default_' + id
                status_id = id+'_status'
                field_id = 'field_'+id
                done = done + (id,)
                if REQUEST.has_key(id):
                    value=self.type_convert(REQUEST[id], ptype)
                    REQUEST.set(id, value)
                    session.set(id, value)
                    REQUEST.set(status_id, FROM_REQUEST)
                elif REQUEST.has_key(field_id):
                    value=self.type_convert(REQUEST[field_id], ptype)
                    REQUEST.set(id, value)
                    session.set(id, value)
                    REQUEST.set(status_id, FROM_REQUEST_FIELD)
                elif id in submitted_fields_ids:
                    type_default_value=self.type_default(ptype)
                    REQUEST.set(id, type_default_value)
                    session.set(id, type_default_value)
                    REQUEST.set(status_id, FROM_REQUEST_FIELD)
                elif session.has_key(id):
                    REQUEST.set(id, session.get(id))
                    REQUEST.set(status_id, FROM_SESSION)
                else:
                    if kw.has_key(id):
                        REQUEST.set(id, kw[id])
                        REQUEST.set(status_id, FROM_DEFAULT)
                    elif hasattr(self,default_expr):
                        REQUEST.set(id, getattr(self, default_expr))
                        REQUEST.set(status_id, FROM_DEFAULT_EXPR)
                    else:
                        REQUEST.set(id, default)
                        REQUEST.set(status_id, FROM_DEFAULT)
            self._v_keyword_parameters = kw
            for key, default in kw.items():
                if type(default)==StringType and count(default,':'):
                    default = split(default,':')[0]
                    ptype = join(split(default,':')[1:],':')
                else:
                    ptype = self.guess_property_type(type(default))
                if key not in done:
                    if REQUEST.has_key(key):
                        value=self.type_convert(REQUEST[key], ptype)
                        REQUEST.set(key, value)
                        session.set(key, value)
                        REQUEST.set(key+'_status', FROM_REQUEST)
                    elif REQUEST.has_key('field_'+key):
                        value=self.type_convert(REQUEST['field_'+key], ptype)
                        REQUEST.set(key, value)
                        session.set(key, value)
                        REQUEST.set(key+'_status', FROM_REQUEST_FIELD)
                    elif key in submitted_fields_ids:
                        type_default_value=self.type_default(ptype)
                        REQUEST.set(key, type_default_value)
                        session.set(key, type_default_value)
                        REQUEST.set(key+'_status', FROM_REQUEST_FIELD)
                    elif session.has_key(key):
                        REQUEST.set(key, session.get(key))
                        REQUEST.set(key+'_status', FROM_SESSION)
                    else:
                        REQUEST.set(key, default)
                        REQUEST.set(key+'_status', FROM_DEFAULT)

    security.declareProtected(View, 'getParameters')
    def getParameters(self):
        """
        Returns the parameters that are being tracked.
        """
        parameters = self.getPredefinedParameters()
        parameters = parameters + self.getAdditionalParameters()
        return parameters

    security.declareProtected(View, 'getPredefinedParameters')
    def getPredefinedParameters(self):
        """
        Returns the predefined parameters that are being tracked.
        """
        if hasattr(self,'track_fields'):
            return map(lambda x: x['id'], self.track_fields)
        return []

    security.declareProtected(View, 'getAdditionalParameters')
    def getAdditionalParameters(self):
        """
        Returns the additional parameters that are being tracked.
        """
        parameters=[]
        if hasattr(aq_base(self), '_v_keyword_parameters'):
            parameters = parameters + aq_base(self)._v_keyword_parameters.keys()
        return parameters

    security.declareProtected(View, 'getDefaultValue')
    def getDefaultValue(self, id):
        """
        Returns the default value of a parameter
        """
        track_fields = getattr(self,'track_fields', [])
        parameters = map(lambda x: x['id'], track_fields)
        if id in parameters:
            parameters = filter(lambda x, id=id: x['id']==id, track_fields)
            return parameters[0]['default']

        if hasattr(aq_base(self), '_v_keyword_parameters'):
            if aq_base(self)._v_keyword_parameters.has_key(id):
                return aq_base(self)._v_keyword_parameters[id]
        return None

    security.declareProtected(View, 'getType')
    def getType(self, id):
        """
        Returns the type of a parameter
        """
        track_fields = getattr(self,'track_fields', [])
        parameters = map(lambda x: x['id'], track_fields)
        if id in parameters:
            parameters = filter(lambda x, id=id: x['id']==id, track_fields)
            return parameters[0]['type']
        if hasattr(aq_base(self), '_v_keyword_parameters'):
            if aq_base(self)._v_keyword_parameters.has_key(id):
                return self.guess_property_type(type(aq_base(self)._v_keyword_parameters[id]))
        return None

    security.declareProtected(AccessContentsInformation, 'manage_save')
    def manage_save(self, id, container, brain=None, REQUEST=None):
        """
        Saves the tracked parameters in an object in the container.

        Brain can be used to extend the object being safed.
        """
        pass

    security.declareProtected(AccessContentsInformation, 'manage_load')
    def manage_load(self, object, REQUEST=None):
        """
        Loads tracked parameters from an object.
        """
        pass

    security.declarePublic('renderField')
    def renderField(self, field_type, id, value_type='', validate=0, REQUEST=None, **kw):
        """
        Renders a field with a tracked value.

        Overrides the EasyPresentationHelpers renderField too add
        validation and use the Tracker to track the fields value.
        """
        field = apply(self._createField, (field_type, id), kw)
        if REQUEST is None:
            REQUEST = getattr(self, 'REQUEST', None)
        if validate and REQUEST:
            session = REQUEST.SESSION
            try:
                res = field.validate(REQUEST)
            except ValidationError, e:
                if session.has_key(id):
                    res = session.get(id)
                    REQUEST.set(id, res)
                    REQUEST.set(id+'_status', FROM_SESSION)
                    REQUEST.set('validate_flag', REQUEST.get('validate_flag', 0))
                else:
                    res = field.get_value('default')
                    REQUEST.set(id, res)
                    REQUEST.set(id+'_status', FROM_DEFAULT)
                    REQUEST.set('validate_flag', 1)
                REQUEST.set('%s_field_exception' % id, e)
            except Exception, e:
                raise Exception, e
            else:
                REQUEST.set(id, res)
                REQUEST.set(id+'_status', FROM_REQUEST)
                session.set(id, res)
                REQUEST.set('validate_flag', REQUEST.get('validate_flag', 0))
            return field.render(res) + submitted_fields_snippet % (id,field_type,type2name(value_type))

        elif REQUEST and not validate:
            session = REQUEST.SESSION
            res = REQUEST.get('field_%s' % id, None)
            submitted_fields = self.getSubmittedFieldIds(REQUEST=REQUEST)
            if id in submitted_fields:
                res = self.type_convert(res, value_type)
                session.set(id, res)
                REQUEST.set(id, res)
                REQUEST.set(id+'_status', FROM_REQUEST)
            else:
                if session.has_key(id):
                    res = session.get(id)
                    REQUEST.set(id, res)
                    REQUEST.set(id+'_status', FROM_SESSION)
                else:
                    res = field.get_value('default')
                    REQUEST.set(id, res)
                    REQUEST.set(id+'_status', FROM_DEFAULT)
            REQUEST.set('validate_flag', REQUEST.get('validate_flag', 0))
            return field.render(res) + submitted_fields_snippet % (id,field_type,type2name(value_type))
        REQUEST.set('validate_flag', REQUEST.get('validate_flag', 0))
        return field.render(default) + submitted_fields_snippet % (id,field_type,type2name(value_type))

    def submitted_fields_snippet(self, id, field_type, value_type):
        return submitted_fields_snippet % (id, field_type, value_type)

    security.declarePublic('getSubmittedFields')
    def getSubmittedFieldIds(self,REQUEST=None):
        REQUEST = getattr(self, 'REQUEST', REQUEST)
        if REQUEST is not None:
            sf = REQUEST.get('submitted_fields',[])
            return map(lambda x: split(x,':')[0], sf)
        return []

    security.declarePublic('getSubmittedFields')
    def getSubmittedFields(self,REQUEST=None):
        REQUEST = getattr(self, 'REQUEST', REQUEST)
        if REQUEST is not None:
            sf = REQUEST.get('submitted_fields',[])
            return map(lambda x: tuple(split(x,':')), sf)
        return []


    security.declarePublic('getFieldValue')
    def getFieldValue(self, id, default='', value_type='', field_type=None, validate=0, REQUEST=None, **kw):
        """
        Returns the field value from REQUEST.form if it exists.

        Converts the value to the value_type.
        Optionally validates the value with the field validator
        """
        if REQUEST is None: REQUEST = getattr(self, 'REQUEST', None)
        if field_type and validate and REQUEST:
            field = apply(self._createField, (field_type, id), kw)
            try:
                res = field.validate(REQUEST)
            except ValidationError, e:
                session = REQUEST.SESSION
                if session.has_key(id):
                    res = session.get(id)
                else:
                    res = default
            except Exception, e:
                raise Exception, e

        elif REQUEST and not validate:
            res = REQUEST.get('field_%s' % id, None)
            submitted_fields = self.getSubmittedFieldIds(REQUEST=REQUEST)
            if id in submitted_fields:
                res = self.type_convert(res, value_type)
            else:
                session = REQUEST.SESSION
                if session.has_key(id):
                    res = session.get(id)
                else:
                    res = default
        return res

    security.declarePublic('list_type_convert')
    def list_type_convert(self, res, value_type):
        return list_type_convert(res, value_type)

    security.declarePublic('type_convert')
    def type_convert(self, res, value_type):
        return type_convert(res, value_type)

    security.declarePublic('type_default')
    def type_default(self, value_type):
        return type_default(value_type)

    security.declarePublic('guess_property_type')
    def guess_property_type(self, value_type):
        return guess_property_type(value_type)

    security.declarePublic('calculate_batch_size')
    def calculate_batch_size(self, search_result, batch_size=None, max_batch_size=None):
        try: batch_size=int(batch_size)
        except: batch_size=len(search_result)
        try: max_batch_size=int(max_batch_size)
        except: max_batch_size=None
        if max_batch_size and (batch_size>max_batch_size or batch_size==0):
            batch_size=max_batch_size
        return batch_size

    security.declarePublic('create_batches')
    def create_batches(self
                         , search_result
                         , batch_size=None
                         , query_start=None
                         , max_batch_size=None
                         , before_orphans=None
                         , after_orphans=None
                         , quantizies=1
                     ):

        return Batches(search_result, batch_size=batch_size, query_start=query_start, max_batch_size=max_batch_size, before_orphans=before_orphans, after_orphans=after_orphans, quantizies=quantizies)


Globals.InitializeClass(EasyParameterTracker)


class NoBrain: pass

class FieldSheet(Base):

    def __init__(self, fields, category):
        self.fields=fields
        self.category=category

    def validate(self, id):
        field = self.fields.get(id, None)
        if field:
            if kw:
                field_values= field.values
                for key,value in kw.items():
                    if field_values.has_key(key):
                        field_values[key]=value
                field.values=field_values
            return field.render(value)
        return

    def renderField(self, id , value=None, default=None, **kw):
        field = self.fields.get(id, None)
        if field:
            if kw:
                field_values= field.values
                for key,value in kw.items():
                    if field_values.has_key(key):
                        field_values[key]=value
                field.values=field_values
            return field.render(value)
        return ''

    def getFieldById(self, id, marker=None):
        return self.fields.get(id, marker)



class TrackedPropertyFactory(Base):
    """
    Defines Properties/Fields that should be tracked,
    cataloged, validated and/or rendered.

    """
    def __init__(self):
        self.categories = {}
        self.indexes = {}
        self.fields = {}
        self.data = {}
        self.compiled_indexes={}

    def registerProperty(self, **kw):
        id = kw.get('id',None)
        if id is None: raise 'KeyException', 'No Id submitted.'
        if id in self.data.keys():
            LOG('EasyParameterTracker',
                INFO,
                'Duplicate id [%s] registered in TrackedPropertySheet::registerProperty'%(id,)
               )
        self.data[id] = kw
        category = kw.get('category',None)
        if category is None:
            category='default'
        if not self.categories.has_key(category):
            self.categories[category]=()
        self.categories[category]=self.categories[category] + (id,)
        if kw.has_key('field_type'):
            field = apply(FieldRegistry.get_field_class(kw['field_type']), (id,), kw)
            self.fields[id]=field
        for index_id in kw.keys():
            index = self.indexes.get(index_id,())
            index = index + (id,)
            self.indexes[index_id] = index


    def getRecordsByIndex(self, index, record_ids=None):
        if not record_ids:
            record_ids=None
        if type(record_ids)==type(()) and len(record_ids)==1:
            record_ids=record_ids[0]
        rindexes = self.indexes.get(index,[])
        records = []
        for id in rindexes:
            record = self.data[id]
            if record_ids is None:
                pass
            elif type(record_ids)==type(''):
                record=record[record_ids]
            else:
                tuplized = ()
                for id in record_ids:
                    tuplized = tuplized + (record[id],)
                record = tuplized
            records.append(record)
        return records


    def getRecordsByTrueIndex(self, index, record_ids=None):
        if not record_ids:
            record_ids=None
        if type(record_ids)==type(()) and len(record_ids)==1:
            record_ids=record_ids[0]
        rindexes = self.indexes.get(index,[])
        records = []
        for id in rindexes:
            record = self.data[id]
            if record.get(index, 0):
                if type(record_ids)==type(''):
                    record=record[record_ids]
                elif type(record_ids)==type(()):
                    tuplized = ()
                    for id in record_ids:
                        tuplized = tuplized + (record[id],)
                    record = tuplized
                records.append(record)
        return records

    def getRecordsByIndexesMatching(self, record_ids=None, **kw):
        if not record_ids: record_ids=None
        if type(record_ids)==type(()) and len(record_ids)==1:
            record_ids=record_ids[0]
        indexes = kw.keys()
        all_ids=()
        for index in indexes:
            all_ids=all_ids+self.indexes.get(index,())
        records = []
        for id in all_ids:
            record = self.data[id]
            include_this_record=1
            for index in indexes:
                if not (record.has_key(index) and record.get(index)==kw[index]):
                    include_this_record=0
                    break
            if include_this_record:
                if type(record_ids)==type(''):
                    record=record[record_ids]
                elif type(record_ids)==type(()):
                    tuplized = ()
                    for id in record_ids:
                        tuplized = tuplized + (record[id],)
                    record = tuplized
                records.append(record)
        return records

    def getZCatalogColumns(self):
        zindexes = []
        for id in self.indexes.get('column',[]):
            record = self.data[id]
            if record.get('column',0):
                zindexes.append(id)
        return zindexes

    def getZCatalogIndexes(self):
        zindexes = []
        for id in self.indexes.get('index',[]):
            index = self.data[id].get('index',0)
            if index:
                zindexes.append((id,index))
        return zindexes

    def getFieldsMap(self):
        return self.fields

    def getFieldsByCategory(self,category):
        fields_group=[]
        for id in self.categories.get(category,[]):
            if self.fields.has_key(id):
                fields_group.append(self.fields[id])
        return fields_group

    def createFieldSheet(self,brain=NoBrain):

        class mybrains(brain, FieldSheet):
            pass

        return mybrains()

    def getDatabaseConnections(self):
        unique_databases = []
        for record in self.getRecordsByTrueIndex('dbConnection'):
            if record['dbConnection'] not in unique_databases:
                unique_databases.append(record['dbConnection'])
        return unique_databases

    def getDatabaseTablesDict(self):
        database_tables = {}
        for database in self.getDatabaseConnections():
            tables_names = []
            for record in self.getRecordsByIndexesMatching(dbConnection=database):
                if record['dbTable'] not in tables_names:
                    tables_names.append(record['dbTable'])
            database_tables[database] = tables_names
        return database_tables

    def getDatabaseTablesForDB(self, database):
        unique_tables = []
        for record in self.getRecordsByTrueIndex('dbTable'):
            if record['dbTable'] not in unique_tables and record.get('dbConnection', None) == database:
                unique_tables.append(record['dbTable'])
        return unique_tables

    def getTableColumnsDict(self):
        database_tables = {}
        for database in self.getDatabaseConnections():
            tables_names = []
            for record in self.getRecordsByIndexesMatching(dbConnection=database):
                if record['dbTable'] not in tables_names:
                    tables_names.append(record['dbTable'])
            database_tables[database] = tables_names
        return database_tables


    def getDatabaseTablesAndColumns(self):
        unique_tables = {}
        for database in self.getDatabaseConnections():
            unique_columns = {}
            for tablename in self.getDatabaseTablesForDB(database):
                columns = []
                columns_names = []
                for record in self.getRecordsByIndexesMatching(dbTable=tablename):
                    if record['dbColumn'] not in columns:
                        columns.append(record['dbColumn'])
                        columns_names.append(record)
                unique_columns[tablename] = columns_names
            unique_tables[database] = unique_columns
        return unique_tables


SESSION_ID_MGR = 'session_id_mgr'

submitted_fields_snippet = """<input type="hidden" name="submitted_fields:string:list" value="%s:%s:%s">"""

manage_EasyParameterTrackerAddForm = DTMLFile('dtml/manage_EasyParameterTrackerAddForm', globals())

def type2name(value_type):
    if value_type in type_converters.keys(): return value_type
    if type(value_type)==STRINGTYPE : return 'str'
    if type(value_type)==LISTTYPE : return 'list'
    if type(value_type)==TUPLETYPE : return 'tuple'
    if type(value_type)==INTTYPE : return 'int'
    if type(value_type)==LONGTYPE : return 'long'
    if type(value_type)==FLOATTYPE : return 'float'
    return 'str'

#Depecated
def name2type(typename):
    if typename=='str': return ''
    #if typename=='string': return ''
    if typename=='list': return []
    if typename=='tuple': return ()
    if typename=='lines': return []
    if typename=='tokens': return ()
    if typename=='int': return 0
    if typename=='long': return 0L
    if typename=='float': return 0.0
    return ''



def manage_EasyParameterTrackerAdd(self, id, title='', submit='', REQUEST=None):
    """Add a EasyParameterTracker object"""

    checkPermission=getSecurityManager().checkPermission

    tracker=EasyParameterTracker(id, title)
    self._setObject(id, tracker)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_workspace')


def initialize(context):
    """Initialize EasyParameterTracker Product"""

    context.registerClass(
        EasyParameterTracker,
        permission=EasyParameterTrackerAdd,
        icon='icons/EasyParameterTracker.gif',
        constructors = ( manage_EasyParameterTrackerAddForm, manage_EasyParameterTrackerAdd),
        meta_type = EasyParameterTracker.meta_type
    )




