# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyWorkflow/EasyPropertyDiff.py,v 1.1 2004/06/28 10:05:49 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""EasyPublishSync mixin class"""
__version__='$Revision: 1.1 $'[11:-2]

import Globals
from Acquisition import Implicit, aq_base, aq_get
from types import DictType, TupleType, ListType, StringType
from Products.EasyCore.VirtualPathHelpers import path2url, url2path, comparePaths
from ComputedAttribute import ComputedAttribute

class PropertyInfo:

    def __init__(self, id='', property=None, object=None):
        if type(property)!=type({}): property={}
        self.id=id or ''
        self.type=property.get('type', None)
        self.label=property.get('label', None)
        self.value=getattr(object, id, None)

    def getLabel(self):
        return self.label or self.id

    def __getitem__(self, key, default=None):
        if key in ('type','value','id','label'):
            return getattr(self, key, default)

    def getValue(self, default=None):
        return self.value or default

    def exists(self):
        return self.value is not None

MISSINGPROPERTYINFO=PropertyInfo()


class PropertySheetInfo:

    def __init__(self, id='', properties=None, object=None, ignore=('creation_date','contributors','language')):
        self.id=id or ''
        self.ignore=ignore
        self.properties={}
        if properties is not None:
            for property in properties.propertyMap():
                id=property['id']
                if id not in ignore:
                    self.properties[id]=PropertyInfo(id, property, object)

    def get(self, key, default=None):
        return self.properties.get(key, default)

    def getPropertiesKeys(self):
        return self.properties.keys()

    def mergePropertiesKeys(self, keys=[]):
        for key in self.properties.keys():
            if key not in keys: keys.append(key)
        return keys

    def getPropertiesItems(self):
        return self.properties.items()

MISSINGPROPERTYSHEETINFO=PropertySheetInfo()




class PropertySheetDiff(Implicit):
    """\
    PropertySheetDiff is the result of an Diff operation.

    PropertySheetDiff takes three arguments at initiation:
        local, remote, old_remote, which are ObjectInfo instances

    """

    __allow_access_to_unprotected_subobjects__=1


    def __init__(self, id, local, remote, old_remote):
        self.id=id or ''
        self.local =local
        self.remote=remote
        self.old_remote=old_remote
        self.diff_sheet={}


    def getDiffPropertySheet(self, local_obj, ignore=('creation_date','contributors','language')):
        remote_obj=local_obj.getMasterDocument()
        return self.diffPropertySheet(local_obj, remote_obj, ignore=ignore)


    def diffPropertySheet(self, local_obj, remote_obj, ignore=('creation_date','contributors','language')):
        diff_sheet={}

        diff_sheet_keys=self.local.mergePropertiesKeys()
        diff_sheet_keys=self.remote.mergePropertiesKeys(diff_sheet_keys)
        diff_sheet_keys=self.old_remote.mergePropertiesKeys(diff_sheet_keys)

        for id in diff_sheet_keys:
            if id not in ignore:
                diff_sheet[id]=PropertyDiff( id
                                , self.local.get(id, MISSINGPROPERTYINFO)
                                , self.remote.get(id, MISSINGPROPERTYINFO)
                                , self.old_remote.get(id, MISSINGPROPERTYINFO)
                              )

        return diff_sheet.items()



class PropertyDiff(Implicit):
    """\
    PropertyDiff is the result of an Diff operation.

    PropertyDiff takes three arguments at initiation:
        local, remote, old_remote, which are ObjectInfo instances

    """

    __allow_access_to_unprotected_subobjects__=1

    def __init__(self, id, local, remote, old_remote):
        self.id=id or ''
        self.local=local
        self.remote=remote
        self.old_remote=old_remote

    def getId(self):
        return self.id

    def getLabel(self):
        prop = self.local or self.remote or self.old_remote
        return prop.getLabel() or self.id

    def _hasPropertyValueChanged(self):
        oldvalue=getattr(self.old_remote, 'value', None)
        currentvalue=getattr(self.remote, 'value', None)
        if oldvalue!=currentvalue: return 1
        return 0
    hasPropertyValueChanged=ComputedAttribute(_hasPropertyValueChanged, 0)
    isDifferent=ComputedAttribute(_hasPropertyValueChanged, 0)

    def _haspropertyTypeChanged(self):
        return self.remote.type!=self.old_remote.type
    haspropertyTypeChanged=ComputedAttribute(_haspropertyTypeChanged, 0)

    def getPropertyTypeRemote(self):
        return getattr(self.remote, 'type', None)

    def getLocalProperty(self):
        return self.local

    def getRemoteProperty(self):
        return self.remote

    def getOldRemoteProperty(self):
        return self.old_remote

    def existsLocally(self):
        exists=getattr(self.local,'exists', 0)
        if callable(exists): exists=exists()
        return exists

    def existsRemotely(self):
        exists=getattr(self.remote,'exists', 0)
        if callable(exists): exists=exists()
        return exists

