# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyWorkflow/EasyPublishable.py,v 1.1 2004/06/28 10:05:50 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

#

__doc__="""Publishable mixin class"""
__version__='$Revision: 1.1 $'[11:-2]


import Globals
from Globals import DTMLFile, HTML
from types import StringType
import DateTime
from ExtensionClass import Base
from AccessControl import getSecurityManager, ClassSecurityInfo
from Acquisition import aq_base

from Products.EasyCore.EasyCorePermissions import *
from Products.EasyCore.ManageOptionBase import ManageOptionBase
from Products.EasyCore.BodyTabs import InitializeBodyTabs, BodyTabs
from Products.EasyPresentationHelpers.EasyPresentationHelpers import EasyPresentationHelpers
from Products.EasyNotification.EasyNotifiable import EasyNotifiable

from HistoryTracker import HistoryTracker
from EasyWorkflowBase import *

from Products.EasyWorkflow.EasySync import EasySyncAware

from zLOG import LOG

from Products.EasyCore import MLOG_TAB

_marker=[]

def safe_call(self, names, default=''):
    value=_marker
    if type(names)==StringType: names=(names,)
    for name in names:
        if hasattr(self, name):
            value=getattr(self, name)
        elif hasattr(self, name.lower()):
            value=getattr(self, name.lower())
        elif hasattr(self, name.upper()):
            value=getattr(self, name.upper())
    if value is _marker: value=default
    if callable(value):
        value=value()
    return value

class Publishable(ManageOptionBase, EasySyncAware, WorkflowBase, EasyNotifiable, EasyPresentationHelpers, BodyTabs):
    """Publishable adds effective_date, expiration_date and a Publish manage tab """

    security = ClassSecurityInfo()

    _object_version_workflow = '1.0' # This gives unupgraded objects a version number.

    _manage_options_publish = (
            {'label':'Publish',
             'action':'manage_publishform',
             #'help':('EasyWorkflow','Icons_Description.stx')
            },
        )


    def __init__(self):
        EasySyncAware.__init__(self)
        self._object_version_workflow = '1.1'
        self.resetPublishHistory()


    security.declareProtected(EasyViewAhead, 'preview_html')
    def preview_html(self, client=None, REQUEST=None, viewname=None):
        """Displayes a non-published object"""
        return self.index_html(client, REQUEST, viewname)


    security.declareProtected(ViewManagementScreens, 'manage_options_publish')
    def manage_options_publish(self):
        options = ()
        if hasattr(self, 'EasyNavigation'):
            options = self._manage_options_publish
        if hasattr(self, 'EasyNotification'):
            options = options + EasyNotifiable.manage_options
        return options

    use_manage_options = ((None,manage_options_publish,None),)

    security.declareProtected(EasyWorkflowView, 'manage_publishform')
    manage_publishform = DTMLFile('dtml/manage_publishform', globals()
                                 , body_tab={'label':'Publish','order':1, 'management_view':'Publish'}
                                 , management_view='Publish'
                                 )

    security.declareProtected(EasyWorkflowView, 'manage_show_workflow_history')
    manage_show_workflow_history = DTMLFile('dtml/manage_show_workflow_history', globals()
                                     , body_tab={'label':'History','order':2, 'management_view':'Publish'}
                                     , management_view='Publish'
                                 )


    security.declareProtected(EasyWorkflowView, 'snippet_default_transition')
    snippet_default_transition = HTML("""\
        <table cellspacing="0" cellpadding="2" border="0">
        <tr>
          <td width="150"><strong class="tab-small"><tml-var lable_text></strong></td>
          <td align=left>
            <input type="hidden" name="transitions.effective_date:records" value="&dtml-effective_date;">
            <input type="hidden" name="transitions.expiration_date:records" value="&dtml-expiration_date;">
            <input type="hidden" name="transitions.old_state:records" value="&dtml-getWorkflowState;">
            <input type="hidden" name="transitions.new_state:records" value="&dtml-new_state;">
            <input type="hidden" name="transitions.text:records" value="&dtml-action_text;">
            <input type="submit" name="transition" value="&dtml-action_text;">
          </td>
        </tr>
        </table>
        """, globals())


    security.declareProtected(EasyWorkflowView, 'snippet_show_users')
    snippet_show_users = DTMLFile('dtml/snippet_show_users', globals())

    security.declareProtected(EasyWorkflowView, 'snippet_set_publish_date')
    snippet_set_publish_date = DTMLFile('dtml/snippet_set_publish_date', globals())

    security.declareProtected(EasyWorkflowView, 'snippet_set_arcive_date')
    snippet_set_arcive_date = DTMLFile('dtml/snippet_set_arcive_date', globals())

    security.declareProtected(EasyWorkflowView, 'snippet_change_author')
    snippet_change_author = DTMLFile('dtml/snippet_change_author', globals())


    security.declareProtected(EasyWorkflowManage, 'resetPublishHistory')
    def resetPublishHistory(self):
        user = getSecurityManager().getUser().getId()
        self.effective_date = NullDate
        self.effective_date_set_by = user
        self.expiration_date = NullDate
        self.expiration_date_set_by = user
        self.creation_date = DateTime.now()
        self.last_modified_by = user
        self.created_by = user
        self.approved_by = ""
        self._workflow_history = HistoryTracker( Comment="Object Created.",
                                                 User=user,
                                                 Time=self.creation_date,
                                                 OldState="",
                                                 State=wfDRAFT
                                               )
        # workflow_history is a tuple: comment, userid, time, effective_date, expiration_date
        self.publish_history = []

    security.declareProtected(EasyWorkflowView, 'getPublishIcon')
    def getPublishIcon(self, size=16):
        return self.getPublishIconForState(self.getPublishState(), size=size)

    security.declareProtected(EasyWorkflowView, 'getPublishStateMessage')
    def getPublishStateMessage(self):
        return self.getPublishStateMessageForState(self.getPublishState())

    security.declareProtected(EasyWorkflowView, 'getPublishState')
    def getPublishState(self):
        if (self.effective_date!=NullDate and self.effective_date>DateTime.now()) or (not self.getWorkflowState() == wfPUBLISHED):
            return psUNPUBLISHED
        if self.expiration_date!=NullDate and self.expiration_date < DateTime.now():
            return psARCHIVED
        return psPUBLISHED

    security.declarePublic('isLocked')
    def isLocked(self):
        state=self.getWorkflowState()
        pstate=self.getPublishState()
        checkPermission=getSecurityManager().checkPermission
        if state in (wfDRAFT, wfDENIED) and checkPermission(EasyWorkflowEditDraft, self):
            return 0
        elif pstate==psPUBLISHED and state ==wfPUBLISHED and checkPermission(EasyWorkflowEditPublished, self):
            return 0
        elif state ==wfSUBMITTED and checkPermission(EasyWorkflowEditSubmitted, self):
            return 0
        elif pstate==psARCHIVED and checkPermission(EasyWorkflowEditArchived, self):
            return 0
        return 1

    security.declarePublic('isPublished')
    def isPublished(self):
        state=self.getWorkflowState()
        pstate=self.getPublishState()
        if pstate==psPUBLISHED and state==wfPUBLISHED:
            return 1
        return 0

    security.declarePublic('isArchived')
    def isArchived(self):
        pstate=self.getPublishState()
        if pstate==psARCHIVED:
            return 1
        return 0

    security.declarePublic('getLockedIcon')
    def getLockedIcon(self):
        if self.isLocked():
            state=self.getWorkflowState()
            pstate=self.getPublishState()
            return """<img src="misc_/EasyWorkflow/lock_by_workflow_icon" border="0" """\
                   """title="Locked by workflow [%s/%s]" alt="%s/%s" width="10" height="13"/>"""  % (state,pstate,state,pstate)
        return ''

    security.declareProtected(EasyWorkflowView, 'getExpirationDateSetBy')
    def getExpirationDateSetBy(self):
        if self.publish_history == []:
            return 'Nobody'
        return self.publish_history[-1][1]


    security.declareProtected(EasyWorkflowView, 'getWorkflowIcon')
    def getWorkflowIcon(self, size=16):
        return self.getWorkflowIconForState(self.getWorkflowState(), size=size)

    security.declareProtected(EasyWorkflowView, 'getWorkflowStateMessage')
    def getWorkflowStateMessage(self):
        return self.getWorkflowMessageForState(self.getWorkflowState())

    security.declareProtected(EasyWorkflowView, 'getWorkflowStateName')
    def getWorkflowStateName(self):
        return self.getWorkflowNameForState(self.getWorkflowState())

    security.declareProtected(EasyWorkflowView, 'getWorkflowState')
    def getWorkflowState(self):
        """Returns the field 'State' in the last entry of the workflow history"""
        return self._workflow_history.getLastEntryField('State')

    security.declareProtected(EasyWorkflowView, 'getWorkflowStateLastChangedBy')
    def getWorkflowStateLastChangedBy(self):
        return self._workflow_history.getLastEntryField('User')

    security.declareProtected(EasyWorkflowView, 'getWorkflowStateLastChangedDate')
    def getWorkflowStateLastChangedDate(self):
        return self._workflow_history.getLastEntryField('Time')

    security.declareProtected(EasyWorkflowView, 'getTransitions')
    def getTransitions(self, old_state=None):
        transitions = []
        if old_state is None: old_state = self.getWorkflowState()
        for transition in wf_transitions[old_state]:
            if self._checkPermission(transition['permissions'], transition['permissions_operator']):
                transitions.append(transition)
        return transitions

    security.declareProtected(EasyWorkflowView, 'getWorkflowStateLastChangeComment')
    def getWorkflowStateLastChangeComment(self):
        return self._workflow_history.getLastEntryField('Comment')

    security.declareProtected(EasyWorkflowView, 'getWorkflowLastEntry')
    def getWorkflowLastEntry(self):
        return self._workflow_history.getLastEntry()

    security.declareProtected(EasyWorkflowView, 'getApprovedBy')
    def getApprovedBy(self):
        if not hasattr(self, 'approved_by'):
            self.approved_by = ""
        return self.approved_by

    #TODO: Should return the sum of publishdate and workflow state

    security.declareProtected(EasyWorkflowView, 'getLatestEffectiveDate')
    def getLatestEffectiveDate():
        return self.effective_date

    security.declareProtected(EasyWorkflowView, 'getLatestExpirationDate')
    def getLatestExpirationDate():
        return self.expiration_date

    security.declareProtected(EasyWorkflowView, 'getLatestPublishDateChangedBy')
    def getLatestPublishDateChangedBy():
        if not hasattr(self, 'publish_history') or  self.publish_history==[]:
            self.publish_history=[]
            return "Nobody"
        return self.publish_history[-1][1]


    #render methods
    security.declareProtected(EasyWorkflowView, 'showWorkflow')
    def showWorkflow(self,client=None,REQUEST=None):
        """Render default_tranisition snippets"""
        result = ""
        for transition in self.getTransitions():
            if self._checkPermission(transition['permissions'],transition['permissions_operator']):
                result = result + apply(self.snippet_default_transition.__call__, (self, REQUEST), transition)
        return result

    #management methods
    security.declareProtected(EasyWorkflowChangeDate, 'manage_change_publish_dates')
    def manage_change_publish_dates(self, transition, transitions, comment="", REQUEST=None):
        """Change the publishing dates for an object."""
        for t in transitions:
            if t['text'] == transition:
                self.changePublishState(t['effective_date'], t['expiration_date'], comment)
                if REQUEST:
                    message="Publish data changed."
                    return self.manage_publishform(
                        self,
                        REQUEST,
                        manage_tabs_message=message,
                        management_view='Publish')
        if hasattr(aq_base(self),'index_object'): self.index_object()

####
# These simple workflow state changers are *NOT* made for usage through the Zope Management Interface,
# But instead for usage when you want access to management tools from the webpages themselves.
####

    security.declareProtected(EasyWorkflowView, 'manage_submit')
    def manage_submit(self, comment='', REQUEST=None):
        """Change Workflow State"""
        self.doTransition(self.getWorkflowState(), wfSUBMITTED, comment, self.effective_date, self.expiration_date)
        if REQUEST is not None:
            try:    url=self.DestinationURL()
            except: url=REQUEST['URL1']
            REQUEST.RESPONSE.redirect(url)

    security.declareProtected(EasyWorkflowView, 'manage_accept')
    def manage_accept(self, comment='', REQUEST=None):
        """Change Workflow State"""
        self.doTransition(self.getWorkflowState(), wfPUBLISHED, comment, self.effective_date, self.expiration_date)
        if REQUEST is not None:
            try:    url=self.DestinationURL()
            except: url=REQUEST['URL1']
            REQUEST.RESPONSE.redirect(url)

    security.declareProtected(EasyWorkflowView, 'manage_deny')
    def manage_deny(self, comment='', REQUEST=None):
        """Change Workflow State"""
        self.doTransition(self.getWorkflowState(), wfDENIED, comment, self.effective_date, self.expiration_date)
        if REQUEST is not None:
            try:    url=self.DestinationURL()
            except: url=REQUEST['URL1']
            REQUEST.RESPONSE.redirect(url)

    security.declareProtected(EasyWorkflowView, 'manage_publish')
    def manage_publish(self, comment='', REQUEST=None):
        """Change Workflow State"""
        self.doTransition(self.getWorkflowState(), wfPUBLISHED, comment, self.effective_date, self.expiration_date)
        if REQUEST is not None:
            try:    url=self.DestinationURL()
            except: url=REQUEST['URL1']
            REQUEST.RESPONSE.redirect(url)

    security.declareProtected(EasyWorkflowView, 'manage_revoke')
    def manage_revoke(self, comment='', REQUEST=None):
        """Change Workflow State"""
        self.doTransition(self.getWorkflowState(), wfDENIED, comment, self.effective_date, self.expiration_date)
        if REQUEST is not None:
            try:    url=self.DestinationURL()
            except: url=REQUEST['URL1']
            REQUEST.RESPONSE.redirect(url)


####
# And this one is used by the ZMI Workflow pages.
####
    security.declareProtected(EasyWorkflowView, 'manage_changeWorkflowState')
    def manage_changeWorkflowState(self, transition, transitions, comment='', user='', recursive=0, REQUEST=None):
        """Change Workflow State"""
        if transition == 'Delegate To':
            self.manage_changeOwnership(user=user, comment=comment, recursive=recursive)
            message="Author has changed."
        elif transition == 'Change Expiration Date' or transition == 'Change Effective Date':
            self.manage_change_publish_dates(transition, transitions, comment)
            message="Publish data changed."
        else:
            message=""
            for t in transitions:
                if t['text'] == transition:
                    self.doTransition(self.getWorkflowState(), t['new_state'], comment, t['effective_date'], t['expiration_date'])
                    message="Workflow state has changed."

        if REQUEST:
            return self.manage_publishform(
                    self,
                    REQUEST,
                    manage_tabs_message=message,
                    management_view='Publish')

    #State changing methods

    security.declarePrivate('doTransition')
    def doTransition(self, old_state, new_state, comment, effective_date=None, expiration_date=None):
        for current in wf_transitions[old_state]:
            if current['new_state'] == new_state:
                if not self._checkPermission(current['permissions'],current['permissions_operator']):
                    raise 'Unauthorized', (
                          'You are not authorized to change workflow from %s to %s.' % (old_state,new_state)
                          )
                try:
                    return apply(current['transition_method'], (self, comment, effective_date, expiration_date))
                finally:
                    if hasattr(aq_base(self),'index_object'): self.index_object()
        return 0

    security.declarePrivate('doTransition')
    def changePublishState(self, effective_date=DONTCHANGEDATE, expiration_date=DONTCHANGEDATE, comment=''):
        if effective_date:
            try: effective_date = DateTime.DateTime(effective_date)
            except: effective_date = DONTCHANGEDATE
        else: effective_date = NullDate
        if expiration_date:
            try: expiration_date = DateTime.DateTime(expiration_date)
            except: expiration_date = DONTCHANGEDATE
        else: expiration_date = NullDate

        #Fix None problem and clean-up old stuff
        if self.effective_date is None: self.effective_date=NullDate
        if self.expiration_date is None: self.expiration_date=NullDate
        if not hasattr(self,'publish_history'): self.publish_history = []

        if self.expiration_date!=expiration_date and expiration_date!=DONTCHANGEDATE:
            ph = self.publish_history
            ph.append(
                (
                    comment,
                    getSecurityManager().getUser().getId(),
                    DateTime.now(),
                    self.effective_date,
                    expiration_date
                ))
            self.publish_history = ph
            self.expiration_date = expiration_date
            self.reindex_object()

        if self.effective_date!=effective_date and effective_date!=DONTCHANGEDATE:
            ph = self.publish_history
            ph.append(
                (
                    comment,
                    getSecurityManager().getUser().getId(),
                    DateTime.now(),
                    effective_date,
                    self.expiration_date
                ))
            self.publish_history = ph
            self.effective_date = effective_date
            self.reindex_object()

        if self.publish_history!=[]:
            return self.publish_history[-1]

    security.declarePrivate('changeWorkflowState')
    def changeWorkflowState(self, new_state, comment=''):
        old_state = self.getWorkflowState()
        user = getSecurityManager().getUser().getId()
        transition = self.getWorkflowTransitionFor(old_state, new_state, None)
        if transition is not None:
            if self._checkPermission(transition['permissions'], transition['permissions_operator']):
                if new_state==wfPUBLISHED:
                    self.approved_by = user
                old_author = self.author
                self._workflow_history.addHistoryEntry( Comment=comment,
                                                        User=user,
                                                        Time=DateTime.now(),
                                                        OldState=old_state,
                                                        State=new_state,
                                                        OldAuthor=old_author,
                                                        Author=old_author
                                                      )
                for notification in self.getNotificationsForTransition(old_state, new_state):
                    notify_users = self.getUserIdToNotify(notification)
                    self.notifyOnWorkflowChange(notification, notify_users, self._workflow_history.getLastEntry())
                if hasattr(self, 'doNotifyOnMasterWorkflowChange'):
                    self.doNotifyOnMasterWorkflowChange(old_state, new_state, self._workflow_history.getLastEntry())
            else:
                raise 'Unauthorized', (
                    'You are not authorized to change workflow <em>%s</em> to <em>%s</em> for object <em>%s</em>.' % (old_state, new_state, self.id)
                  )
        #If the transition is none, most likely is was indexed incorrectly.
        # So we reindex even if we didn't change anything.
        self.reindex_object()
        return self._workflow_history.getLastEntry()


    security.declarePrivate('notifyOnWorkflowChange')
    def notifyOnWorkflowChange(self, notification, notify_users, history_entry):
        fromuser = history_entry['User']
        not_ns= {   'id':self.id,
                    'title':safe_call(self,'Title'),
                    'description':safe_call(self,'Description'),
                    'subject': ', '.join(safe_call(self,'Subject')),
                    'date': history_entry['Time'],
                    'changed_by': history_entry['User'],
                    'new_state':history_entry['State'],
                    'old_state':history_entry['OldState'],
                    'absolute_url': self.absolute_url(),
                    'comment': history_entry['Comment'],
                }
        self.manage_notify(notification, fromuser, notify_users,  not_ns)


    #Change Ownership

    security.declarePrivate('notifyOnOwnershipChange')
    def notifyOnOwnershipChange(self, notification, fromuser, notify_users, history_entry):
        not_ns= {   'id':self.id,
                    'title': safe_call(self,'Title'),
                    'description': safe_call(self,'Description'),
                    'subject': ', '.join(safe_call(self,'Subject')),
                    'date': history_entry['Time'],
                    'changed_by': fromuser,
                    'new_state':history_entry['State'],
                    'old_state':history_entry['OldState'],
                    'new_author': history_entry['OldAuthor'],
                    'old_author': history_entry['Author'],
                    'absolute_url': self.absolute_url(),
                    'comment': history_entry['Comment'],
                }
        self.manage_notify(notification, fromuser, notify_users,  not_ns)


    security.declareProtected(EasyOwnershipChange, 'manage_changeOwnership')
    def manage_changeOwnership(self, user, comment='', recursive=0, REQUEST=None, RESPONSE=None):
        """
        Change user if the new user has permission to edit or submit a document.
        """
        userobj = self.getUserById(user)
        old_author = self.author
        new_author = user
        if userobj is None:
            message="Can't find a user with the id: %s. The Author is still %s." % (new_author, old_author)
        elif userobj is not None and userobj.has_permission(EasyDocumentChange, self) and \
                userobj.has_permission(EasyWorkflowSubmit, self):
            delegator = getSecurityManager().getUser().getId()
            new_state = old_state = self.getWorkflowState()
            comment_append = "Author changed from [%s] to [%s] by [%s]\n\n" % (old_author,new_author,delegator)
            comment  = comment_append + comment
            self._workflow_history.addHistoryEntry( Comment=comment,
                                                    User=delegator,
                                                    Time=DateTime.now(),
                                                    OldState=old_state,
                                                    State=new_state,
                                                    OldAuthor=old_author,
                                                    Author=new_author
                                                  )
            self.changeOwnership(userobj, recursive)

            notification = 'notify_delegate'
            notify_users = self.getUserIdToNotify(notification)
            if not new_author in notify_users:
                notify_users.append(new_author)
            self.notifyOnOwnershipChange(notification, delegator, notify_users, self._workflow_history.getLastEntry())
            if hasattr(aq_base(self),'index_object'): self.index_object()

            message="The Author for the document has been changed from %s to %s." % (old_author,new_author)
        else:
            message="The user %s doesn't have permission to be set as Author for the document. The Author is still %s." % (new_author, old_author)

        if REQUEST and RESPONSE:
            RESPONSE.redirect('%s/manage_publishform?manage_tabs_message=%s' % (REQUEST.URL1, message))

    security.declareProtected(EasyWorkflowView, 'showChangeOwnership')
    def showChangeOwnership(self, client=None, REQUEST=None):
        checkPermission=getSecurityManager().checkPermission
        if checkPermission(EasyOwnershipChange, self):
            userids=map(lambda user: (user,user), self.get_valid_userids_for_role('Author'))
            return self.snippet_change_author(currentuser=self.author, userids=userids)
        return ""

    security.declareProtected(EasyWorkflowView, 'showChangeDates')
    def showChangeDates(self, client=None, REQUEST=None):
        checkPermission=getSecurityManager().checkPermission
        if checkPermission(EasyWorkflowChangeDate, self):
            return self.snippet_set_publish_date(REQUEST) + self.snippet_set_arcive_date(REQUEST)
        return ""

    security.declareProtected('Easy Upgrade Manage', 'upgradeWorkflow')
    def upgradeWorkflow(self, REQUEST=None):
        if hasattr(aq_base(self), 'workflow_history'):
            if hasattr(aq_base(self), '_workflow_history'):
                delattr(self, 'workflow_history')
                self._object_version_workflow = '1.1'
                return '[workflow_history] deleted (HistoryTracker already existed).'
            else:
                self._workflow_history = HistoryTracker()
                for entry in self.workflow_history:
                    if len(entry) == 7:
                        self._workflow_history.addHistoryEntry( Comment=entry[0],
                                                                User=entry[1],
                                                                Time=entry[2],
                                                                OldState=entry[3],
                                                                State=entry[4],
                                                                OldAuthor=entry[5],
                                                                Author=entry[6]
                                                              )
                    else:
                        old_author = self.author
                        self._workflow_history.addHistoryEntry( Comment=entry[0],
                                                                User=entry[1],
                                                                Time=entry[2],
                                                                OldState=entry[3],
                                                                State=entry[4],
                                                                OldAuthor=old_author,
                                                                Author=old_author
                                                              )
                self._object_version_workflow = '1.1'
                delattr(self, 'workflow_history')
            return '[workflow_history] (List) converted to [_workflow_history]  (HistoryTracker)'

        return 'No upgrade'


    security.declareProtected(EasyWorkflowView, 'getWorkflowHistory')
    def getWorkflowHistory(self):
        return self._workflow_history



Globals.InitializeClass(Publishable)
InitializeBodyTabs(Publishable)


