# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/EasyWorkflow/EasyWorkflowManager.py,v 1.2 2004/06/29 14:14:16 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

__doc__='''EasyWorkflowManager'''
__version__='1.8'

import Globals
from string import join, split

from Globals import DTMLFile
import DateTime

from AccessControl import ClassSecurityInfo
from Acquisition import aq_base
from OFS.SimpleItem import SimpleItem

from Products.EasyWorkflow.EasyWorkflow import WorkflowManagement
from Products.EasyCore.VirtualPath import VirtualPathBase
from Products.EasyPresentationHelpers.EasyPresentationHelpers import EasyPresentationHelpers
from Products.EasyCore.EasyCorePermissions import EasyTemplateChange
from webdav.WriteLockInterface import WriteLockInterface
#from PropertyTextParser import PropertyTextParser, PropertyTextGenerator
from Products.EasyCore.PropertyTextParser import PropertyTextParser, PropertyTextGenerator
from zLOG import LOG, INFO



EASY_WORKFLOW_MANAGER='EasyWorkflowManager'

from Products.EasyCore.EasyCorePermissions import setDefaultRoles
EasyWorkflowManagerAdd = 'Easy WorkflowManager Add'
setDefaultRoles(EasyWorkflowManagerAdd, ('Manager',))

class EasyWorkflowManager(VirtualPathBase, WorkflowManagement,  EasyPresentationHelpers, SimpleItem):
    """ EasyWorkflow

    """
    meta_type = EASY_WORKFLOW_MANAGER
    content_type="plain/text"

    _v_errors=''
    __implements__ = (WriteLockInterface,)

    _hidden_properties = (
                            {   'id':'title',
                                'label':'Title',
                                'type': 'string',
                            },
                        )

    isPrincipiaFolderish=1

    security = ClassSecurityInfo()

    manage_options = WorkflowManagement.manage_options + SimpleItem.manage_options



    def __init__(self, id, title=''):
        self.id=self.__name__=id
        self.title=title


    #WebDAV
    def read(self):
        error=''
        try:
            generator = PropertyTextGenerator()
            generator.renderFromInstance(self, self._hidden_properties)
            if self._v_errors:
                return "%s\n%s" %(self._v_errors, generator.getCode())
            return generator.getCode()
        except:
            try:
                error="# ERROR: %s: %s\n# \n" % sys.exc_info()[:2]
                generator_errors='\n'.join(map(lambda err: "# " + str(err), generator.getErrors()))
                if self._v_errors:
                    return error + "%s\n%s" % (self._v_errors, generator_errors)
                return local_errors
            except:
                pass
        return error + self._v_errors


    def munge(self, body):
        self._v_errors = ''
        try:
            parser = PropertyTextParser()
            parser.parseToInstance(self, self._hidden_properties, body)
        except:
            error="# ERROR: %s: %s\n# \n" % sys.exc_info()[:2]
            self._v_errors = error + '\n'.join(map(lambda err: "# " + str(err), parser.getErrors()))


    security.declareProtected('View management screens', 'document_src')
    def document_src(self, REQUEST=None, RESPONSE=None):
        """Return unprocessed document source."""
        if RESPONSE is not None:
            RESPONSE.setHeader('Content-Type', 'text/plain')
        return self.read()

    ## Protocol handlers
    security.declareProtected(EasyTemplateChange, 'PUT')
    def PUT(self, REQUEST, RESPONSE):
        """Handle HTTP PUT requests."""
        self.dav__init(REQUEST, RESPONSE)
        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        body=REQUEST.get('BODY', '')
        self.munge(body)
        RESPONSE.setStatus(204)
        return RESPONSE

    def get_size(self):
        return len(self.read())

    def content_type(self):
        return self.content_type

    security.declareProtected('FTP access', 'manage_FTPget')
    def manage_FTPget(self):
        "Get source for FTP download"
        return self.read()

    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        return changed

Globals.InitializeClass(EasyWorkflowManager)



def manage_addEasyWorkflowManager(self, title='', REQUEST=None):
    """Add a EasyWorkflowManager object"""
    c=EasyWorkflowManager(EASY_WORKFLOW_MANAGER, title)
    self._setObject(EASY_WORKFLOW_MANAGER, c)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)




def initialize(context):
    """Initialize StyleManagers Product"""

    context.registerClass(
        EasyWorkflowManager,
        permission=EasyWorkflowManagerAdd,
        icon='icons/EasyWorkflowManager.gif',
        constructors = ( manage_addEasyWorkflowManager, manage_addEasyWorkflowManager),
        meta_type = EASY_WORKFLOW_MANAGER,
        category='Easy Publisher Management',
    )




