#!/usr/local/bin/python

import urllib
import urllib2
import httplib
import mimetypes
import threading
import time
import random
import sys
import os

from utils import post_multipart

WRITERATIO = 0.1
FILESPATH = '/usr/local/zope/opt/FileCacheManager-HEAD/tests/files'
TESTFILES = { 'small.pdf'  : os.path.join(FILESPATH, 'small.pdf')
            , 'medium.pdf' : os.path.join(FILESPATH, 'medium.pdf')
            , 'large.pdf'  : os.path.join(FILESPATH, 'large.pdf')
            }
TESTFILESIZES = { 'small.pdf'  : os.stat(TESTFILES.get('small.pdf'))[6]
                , 'medium.pdf' : os.stat(TESTFILES.get('medium.pdf'))[6]
                , 'large.pdf'  : os.stat(TESTFILES.get('large.pdf'))[6]
                }
SERVERS   = ( 'localhost:10080',
#            , 'localhost:20080'
#            , 'localhost:30080'
#            , 'localhost:40080'
#            , 'localhost:50080'
            )
FOLDERS   = ('folder1', 'folder2', 'folder3', 'folder4', 'folder5')
FILES     = ('file1', 'file2', 'file3')
METHODS   = ('index_html', 'manage_edit')
PATHS     = []

for server in SERVERS:
    for folder in FOLDERS:
        for file in FILES:
            PATHS.append('http://%s/fcmrig/%s/%s' % (server, folder, file))

def pickUrl():
    # Pick whether to read or write based on the WRITERATIO
    write_randvalue = random.random()
    if write_randvalue <= WRITERATIO:
        method = 'manage_edit'
    else:
        method = 'index_html'

    return ( random.choice(PATHS)
           , method
           , random.choice(TESTFILES.keys())
           )


def openUrl(url, method, note, file):
    if method == 'index_html':
        print '.',
        sys.stdout.flush()
        request = urllib2.Request(url)
        try:
            response = urllib2.urlopen(request)
            body = response.read()

            assert len(body) in TESTFILESIZES.values()

        except urllib2.HTTPError, error:
            print error
        except urllib2.URLError, error:
            print error
        except AssertionError:
            print 'Problem file: %s (%d long)' % (url , len(body))
            sys.stdout.flush()

    elif method == 'manage_edit':
        print ':',
        sys.stdout.flush()
        # We have to create a special request
        # Ugh, have to hack apart the URL to get at its components
        host, basefolder, folder, fname = url[7:].split('/')
        urlpath1 = '/%s/%s/%s/manage_upload' % (basefolder, folder, fname)
        upload_data = open(TESTFILES.get(file), 'rb').read()
        httplib_resp = post_multipart( host
                                     , urlpath1
                                     , ()
                                     , ( ('file', file, upload_data), )
                                     )
        status = int(httplib_resp.status)
        if status == 200:
            print '!',
            sys.stdout.flush()
        else:
            print status, httplib_resp.reason, httplib_resp.read()
            sys.stdout.flush()


class DetachedRequest(threading.Thread):
    def __init__(self, url, note):
        self.url = url[0]
        self.method = url[1]
        self.note = note
        self.file = url[2]
        self.isDone = 0
        threading.Thread.__init__(self)

    def run(self):
        begin = time.time()
        time.sleep(random.uniform(0,0.25))
        self.note['detachedSleepfor'] = time.time() - begin
        openUrl(self.url, self.method, self.note, self.file)
        self.isDone = 1

    
class Browser(threading.Thread):

    def __init__(self, sleepiness, browserThreads, iterations):
        self.sleepiness = sleepiness
        self.browserThreads = browserThreads
        self.iterations = iterations
        self.isDone = 0
        threading.Thread.__init__(self)

    def run(self):
        requests = []

        for i in range(self.iterations):
            # Each loop simulates a browser session - several page requests
            url = pickUrl()
            sleepfor = 0
            note = { 'sleepiness' : self.sleepiness
                   , 'sleepfor'   : sleepfor
                   }

            for reference in range(random.randint(0,self.browserThreads)):
                req = DetachedRequest(pickUrl(), note)
                requests.append(req)
                req.start()

            # Sleep for a bit. Remembing just how long we intented to sleep
            # for so that we can report this if needed.
            begin = time.time()
            time.sleep(random.uniform(0, self.sleepiness))
            sleepfor = time.time() - begin
            url = pickUrl()

        while requests:
            for request in requests:
                if request.isDone:
                    print ',',
                    sys.stdout.flush()
                    requests.remove(request)
                time.sleep(0.1)

        self.isDone = 1

            
def start(browserCount, sleepiness, browserThreads=4, iterations=10):
    browsers = []
    start_time = time.time()

    for browser in range(browserCount):
        browser = Browser(sleepiness, browserThreads, iterations)
        browsers.append(browser)
        browser.start()

    while browsers:
        for browser in browsers:
            if browser.isDone:
                browsers.remove(browser)
            time.sleep(0.1)


    end_time = time.time()
    duration = end_time - start_time

    print '\n%d browsers, %d browser threads, %d iterations, %d sleepiness' % (
            browserCount, browserThreads, iterations, sleepiness)
    print 'Write Ratio:', WRITERATIO
    print 'Took %.2f seconds' % duration

# Kick off a mini colony of robots. With a range of sleepiness.
start(5,2,4,10)
