# Copyright (C) 2003 by Dr. Dieter Maurer, Eichendorffstr. 23, D-66386 St. Ingbert, Germany
# see "LICENSE.txt" for details
#       $Id: KeywordIndex.py,v 1.1 2004/09/25 10:55:08 johanc Exp $
'''Managable KeywordIndex.'''

from sys import maxint

from BTrees.OOBTree import difference, OOSet, union

from ManagableIndex import ManagableIndex, addForm

class KeywordIndex(ManagableIndex):
  '''a managable 'KeywordIndex'.'''
  meta_type= 'Managable KeywordIndex'

  Combiners= ManagableIndex.Combiners + ('union',)
  CombineType= Combiners[-1]

  def _createDefaultValueProvider(self):
    ManagableIndex._createDefaultValueProvider(self)
    # add (value) normalizer to let it behave identical to
    # a standard Zope KeywordIndex
    vp= self.objectValues()[0]
    setattr(vp, vp.NormalizerProperty, 'python: hasattr(value,"capitalize") and (value,) or value')

  def _indexValue(self,documentId,val,threshold):
    if not threshold: threshold= maxint
    n= i= 0; T= get_transaction()
    for v in val.keys():
      self._insert(v,documentId)
      n+=1; i+=1
      if i == threshold: T.commit(1); i= 0
    return n

  def _unindexValue(self,documentId,val):
    for v in val.keys():
      self._remove(v,documentId)

  def _update(self,documentId,val,oldval,threshold):
    add= difference(val,oldval)
    rem= difference(oldval,val)
    if add: self._indexValue(documentId,add,threshold)
    if rem: self._unindexValue(documentId,rem)
    # optimize transaction size by not writing _unindex bucket
    oldval.clear(); oldval.update(val)
    return len(add),

  def _equalValues(self,val1,val2):
    if val1 == val2: return 1
    if val1 is None or val2 is None: return 0
    return tuple(val1.keys()) == tuple(val2.keys())

  def _combine_union(self,values,object):
    if not values: return
    set= None
    for v in values:
      sv= self._standardizeValue(v,object)
      if not sv: continue
      set= union(set,sv)
    return set

  def _standardizeValue(self,value,object):
    '''convert to a set of standardized terms.'''
    if not value: return
    set= OOSet([st for st in [self._standardizeTerm(t,object) for t in value] if st is not None])
    return set or None


def addKeywordIndexForm(self):
  '''add KeywordIndex form.'''
  return addForm.__of__(self)(
    type= KeywordIndex.meta_type,
    description= '''A KeywordIndex indexes an object under a set of terms.''',
    action= 'addIndex',
    )
