# $Header: /usr/custom/cvs/public/CollibertyEasyPublisher/Products/ObjectRepository/ObjectRepository.py,v 1.1 2004/06/28 10:07:06 johanc Exp $
#
# Copyright (C) 2001-2003, Johan Carlsson, Lennart Regebro, Colliberty.
# This software is licensed under the Easy Publisher Shared License 2.0 and/or
# the Easy Publisher Enterprise License 2.0 and/or the Easy Publisher Developers License 2.0
# and/or the Easy Publisher Personal Developers License 2.0.
# The licensee may not give, sell, sub-license, rent, lease, lend or in any other
# way distribute any portion of the Software or Documentation to anyone.
# For details about the licenses contact Colliberty at info@colliberty.com
#

import Globals
from Globals import DTMLFile
from AccessControl import ClassSecurityInfo
from types import StringType, ListType, TupleType

from Products.ZCatalog.Catalog import CatalogError, Catalog, safe_callable
import Record
import Acquisition
from Products.ZCatalog.CatalogBrains import NoBrainer
from Products.ZCatalog.ZCatalog import ZCatalog
from Products.ZCatalog.Lazy import LazyMap
from Products.EasyCore.GUID import GUIDGenerator

from Products.ObjectRepository.ObjectDatabase import ObjectDatabase


ObjectRepositoryAdd='Object Repository Add'
ObjectRepositoryManage='Object Repository Manage'

_marker=[]

class Revision:
    def __init__(self):
        pass

class AbstractRevisionBrain(Record.Record, Acquisition.Implicit):
    pass



class ObjectRepository(ZCatalog):
    """Object Repository """

    security = ClassSecurityInfo()

    meta_type = "ObjectRepository"

    threshold=10000
    _v_total=0
    _v_transaction = None
    title = ''


    def __init__(self, id, title='', vocab_id=None, container=None):
        self.id = id

        # ZCatalog no longer cares about vocabularies
        # so the vocab_id argument is ignored (Casey)

        if container is not None:
            self=self.__of__(container)
        self.id=id
        self.title=title

        # vocabulary and vocab_id are left for backwards
        # compatibility only, they are not used anymore
        self.vocabulary = None
        self.vocab_id = ''

        self.threshold = 10000
        self._v_total = 0

        self._catalog = ObjectDatabase()
        self._guid_generator=GUIDGenerator()


    def __len__(self):
        return len(self._catalog)


    #manage_options = ({'label':'Contents', 'action':'manage_main',},) + ZCatalog.manage_options[1:]

    security.declareProtected(ObjectRepositoryManage, 'manage_main')
    manage_main = DTMLFile('dtml/manage_object_repository', globals())

    security.declarePrivate('getGUID')
    def createGUID(self):
        return self._guid_generator.createGUID()

    security.declarePrivate('resolve_path')
    def resolve_path(self, path):
        """
        Attempt to resolve a url into an object in the Zope
        namespace. The url may be absolute or a catalog path
        style url. If no object is found, None is returned.
        No exceptions are raised.
        """
        if type(path) in (ListType,TupleType): path=path[0]
        try: return self.unrestrictedTraverse(path)
        except: pass

    security.declarePrivate('resolve_url')
    def resolve_url(self, path, REQUEST):
        """
        Attempt to resolve a url into an object in the Zope
        namespace. The url may be absolute or a catalog path
        style url. If no object is found, None is returned.
        No exceptions are raised.
        """
        if type(path) in (ListType,TupleType): path=path[0]
        script=REQUEST.script
        if path.find(script) != 0:
            path='%s/%s' % (script, path)
        try: return REQUEST.resolve_url(path)
        except: pass


    security.declarePrivate('catalog_object')
    def catalog_object(self, obj, uid=None, idxs=[]):
        """ wrapper around catalog """
        if isinstance(uid, TupleType):
            old_language = obj.getCurrentLanguage(default=None)
            obj.changeCurrentVersion(DRAFT_VER, uid[1])
            self._catalog.catalogObject(obj, uid, None, idxs)
            obj.changeCurrentVersion(DRAFT_VER, old_language)
        else:
            old_language = obj.getCurrentLanguage(default=None)
            url = obj.physicalURL
            for language in obj.getLanguages():
                obj.changeCurrentVersion(DRAFT_VER, language)

                r=self._catalog.catalogObject(obj, (url, language), None, idxs)
            obj.changeCurrentVersion(DRAFT_VER, old_language)


        # None passed in to catalogObject as third argument indicates
        # that we shouldn't try to commit subtransactions within any
        # indexing code.  We throw away the result of the call to
        # catalogObject (which is a word count), because it's
        # worthless to us here.

        if self.threshold is not None:
            # figure out whether or not to commit a subtransaction.
            t = id(get_transaction())
            if t != self._v_transaction:
                self._v_total = 0
            self._v_transaction = t
            self._v_total = self._v_total + 1
            # increment the _v_total counter for this thread only and get
            # a reference to the current transaction.
            # the _v_total counter is zeroed if we notice that we're in
            # a different transaction than the last one that came by.
            # self.threshold represents the number of times that
            # catalog_object needs to be called in order for the catalog
            # to commit a subtransaction.  The semantics here mean that
            # we should commit a subtransaction if our threshhold is
            # exceeded within the boundaries of the current transaction.
            if self._v_total > self.threshold:
                get_transaction().commit(1)
                self._p_jar.cacheFullSweep(3)
                self._v_total = 0

    def uncatalog_object(self, uid):
        """ wrapper around catalog """
        if isinstance(uid, TupleType):
            self._catalog.uncatalogObject(uid)
        else:
            old_language = obj.getCurrentLanguage(default=None)
            url = obj.physicalURL
            for language in obj.getLanguages():
                obj.changeCurrentVersion(DRAFT_VER, language)
                r=self._catalog.uncatalogObject((url, language))
            obj.changeCurrentVersion(DRAFT_VER, old_language)

    security.declareProtected(ObjectRepositoryManage, 'getpath')
    def getpath(self, rid):
        """Return the path to a cataloged object given a 'data_record_id_'
        """
        path=self._catalog.paths[rid]
        if type(path) in (ListType,TupleType): path=path[0]
        return path

    security.declareProtected(ObjectRepositoryManage, 'getlanguage')
    def getlanguage(self, rid):
        """Return the path to a cataloged object given a 'data_record_id_'
        """
        path=self._catalog.paths[rid]
        if type(path) in (ListType,TupleType): path=path[1]
        return path

    security.declareProtected(ObjectRepositoryManage, 'getuid')
    def getuid(self, rid):
        """Return the uid (or path) use to a cataloged object given a 'data_record_id_'
        """
        path=self._catalog.paths[rid]
        return path


    security.declareProtected(ObjectRepositoryManage, 'getrid')
    def getrid(self, path, default=None):
        """Return 'data_record_id_' the to a cataloged object given a 'path'
        """
        if type(path) in (ListType,TupleType): path=path[0]
        return self._catalog.uids.get(path, default)

    security.declareProtected(ObjectRepositoryManage,'manage_catalogView')
    manage_catalogView = DTMLFile('dtml/catalogView',globals())

    security.declareProtected(ObjectRepositoryManage, 'searchResults')
    def searchResults(self, REQUEST=None, used=None, **kw):
        """
        Search the catalog according to the ZTables search interface.
        Search terms can be passed in the REQUEST or as keyword
        arguments.
        """
        #set AbstractMultiBrain
        self._catalog.useBrains(AbstractMultiBrain)

        return apply(self._catalog.searchResults, (REQUEST,used), kw)

    __call__=searchResults

    def manage_catalogObject(self, REQUEST, RESPONSE, URL1, urls=[]):
        """ index Zope object(s) that 'urls' point to """

        for url in urls:
            lang, url = url.split(':',1)
            obj = self.resolve_path(url)
            if not obj:
                obj = self.resolve_url(url, REQUEST)
            if obj is not None:
                self.catalog_object(obj, (url, lang))

        return self.manage_catalogView(self,REQUEST,manage_tabs_message='Object Cataloged')


    def manage_uncatalogObject(self, REQUEST, RESPONSE, URL1, urls=None):
        """ removes Zope object(s) 'urls' from catalog """

        for url in urls:
            lang, url = url.split(':',1)
            self.uncatalog_object((url, lang))

        return self.manage_catalogView(self,REQUEST,manage_tabs_message='Object Uncataloged')

    def refreshCatalog(self, clear=0):
        """ re-index everything we can find """

        cat = self._catalog
        paths = cat.paths.values()
        if clear:
            paths = tuple(paths)
            cat.clear()

        for p in paths:
            obj = self.resolve_path(p)
            if not obj:
                obj = self.resolve_url(p, self.REQUEST)
            if obj is not None:
                self.catalog_object(obj, p)



    security.declareProtected(ObjectRepositoryManage, 'hasObject')
    def hasObject(self, id):
        """Indicates whether the folder has an item by ID.
        """
        return self._tree.has_key(id)


    #security.declareProtected(ObjectRepositoryManage,
    #                          'objectIds')
    #def objectIds(self, spec=None):
    #    # Returns a list of subobject ids of the current object.
    #    # If 'spec' is specified, returns objects whose meta_type
    #    # matches 'spec'.
    #    if spec is not None:
    #        if isinstance(spec, StringType):
    #            spec = [spec]
    #        mti = self._mt_index
    #        set = None
    #        for meta_type in spec:
    #            ids = mti.get(meta_type, None)
    #            if ids is not None:
    #                set = union(set, ids)
    #        if set is None:
    #            return ()
    #        else:
    #            return set.keys()
    #    else:
    #        return self._tree.keys()

    ##ObjectManagerInterface
    #security.declareProtected(ObjectRepositoryManage,
    #                          'objectValues')
    #def objectValues(self, spec=None):
    #    # Returns a list of actual subobjects of the current object.
    #    # If 'spec' is specified, returns only objects whose meta_type
    #    # match 'spec'.
    #    return LazyMap(self._getOb, self.objectIds(spec))


    #security.declareProtected(ObjectRepositoryManage,
    #                          'objectItems')
    #def objectItems(self, spec=None):
    #    # Returns a list of (id, subobject) tuples of the current object.
    #    # If 'spec' is specified, returns only objects whose meta_type match
    #    # 'spec'
    #    return LazyMap(lambda id, _getOb=self._getOb: (id, _getOb(id)),
    #                   self.objectIds(spec))


    #security.declareProtected(ObjectRepositoryManage,
    #                          'objectMap')
    #def objectMap(self):
    #    # Returns a tuple of mappings containing subobject meta-data.
    #    return LazyMap(lambda (k, v):
    #                   {'id': k, 'meta_type': getattr(v, 'meta_type', None)},
    #                   self._tree.items(), self._count())


    #SQL Interface
    security.declareProtected(ObjectRepositoryManage,'select')
    def select(self):
        pass
        #_p_deactivate

    security.declareProtected(ObjectRepositoryManage,'update')
    def update(self): pass

    security.declareProtected(ObjectRepositoryManage,'insert')
    def insert(self): pass

    security.declareProtected(ObjectRepositoryManage,'create')
    def create(self): pass

    security.declareProtected(ObjectRepositoryManage,'alter')
    def alter(self): pass


    def _getObject(self, id, default=_marker):
        """\
        Return the named object from the folder.
        """
        tree = self._tree
        ob = tree.get(id, _marker)
        if ob is not _marker:
            return ob
        elif default is not _marker:
            return default
        raise KeyError, "The item %s was not found." % id

    def _setObject(self, id, object):
        """\
        Store the named object in the folder.
        """
        tree = self._tree
        if tree.has_key(id):
            raise KeyError, 'There is already an item named "%s".' % id
        tree[id] = object
        self._count.change(1)
        # Update the meta type index.
        mti = self._mt_index
        meta_type = getattr(object, 'meta_type', None)
        if meta_type is not None:
            ids = mti.get(meta_type, None)
            if ids is None:
                ids = OIBTree()
                mti[meta_type] = ids
            ids[id] = 1


    def _delObject(self, id):
        """\
        Remove the named object from the folder.
        """
        tree = self._tree
        meta_type = getattr(tree[id], 'meta_type', None)
        del tree[id]
        self._count.change(-1)
        # Update the meta type index.
        if meta_type is not None:
            mti = self._mt_index
            ids = mti.get(meta_type, None)
            if ids is not None and ids.has_key(id):
                del ids[id]
                if not ids:
                    # Removed the last object of this meta_type.
                    # Prune the index.
                    del mti[meta_type]


    #UPGRADE METHOD
    security.declareProtected('Easy Upgrade Manage', 'upgradehook')
    def upgradehook(self, REQUEST=None, changed=0):
        """ """
        return changed

Globals.InitializeClass(ObjectRepository)

manage_addObjectRepositoryForm = DTMLFile('dtml/manage_add_object_repository', globals())

def manage_addObjectRepository(self, id, title='', vocab_id=None, REQUEST=None):
    """Adds a new ObjectRepository object with id *id*.
    """
    id = str(id)
    ob = ObjectRepository(id, title, vocab_id, self)
    ob.title = str(title)
    self._setObject(id, ob)
    ob = self._getOb(id)
    if REQUEST is not None:
        return dispatcher.manage_main(self, REQUEST, update_menu=1)



def initialize(context):
    """Initialize ObjectRepository Product"""

    context.registerClass (
            ObjectRepository,
            permission=ObjectRepositoryAdd,
            icon='icons/ObjectRepository.gif',
            constructors=(manage_addObjectRepositoryForm,
                          manage_addObjectRepository),
             meta_type = ObjectRepository.meta_type
            )

