# DocFinderTab 0.5.2
# (c) 2001-2004, Stefan H. Holek, stefan@epy.co.at
# http://zope.org/Members/shh/DocFinderTab
# License: ZPL
# Zope: 2.3-2.7

# Monkey patch wrapper around Dieter Maurer's DocFinder product
# http://www.dieter.handshake.de/pyprojects/zope/DocFinder.html
# Adds a Doc tab to all Zope objects

# Thanks to Dieter for his input on how to make this work with
# PythonScripts and ExternalMethods, and for writing DocFinder
# in the first place.

__doc__ = 'Add a Doc tab to all Zope objects'
__version__ = '0.5.2'

__refresh_module__ = 0

try:

    from Globals import DTMLFile
    from App.Management import Tabs
    from OFS.SimpleItem import Item
    from Products.DocFinderTab.analyse import DocFinder
    from AccessControl import getSecurityManager
    from AccessControl.PermissionRole import PermissionRole
    from Products.DocFinderTab.Permissions import ViewDocPermission, ViewDocDefaultRoles

    def filtered_manage_options(self, REQUEST=None):
        # Append a Doc tab to an object's management tabs
        tabs = self._old_filtered_manage_options(REQUEST)
        secman = getSecurityManager()
        if len(tabs) and secman.checkPermission(ViewDocPermission, self.this()):
            tabs.append( {'label': 'Doc',
                          'action': 'showDocumentation',
                          'help': ('DocFinderTab', 'README.stx')} )
        return tabs

    Tabs._old_filtered_manage_options = Tabs.filtered_manage_options
    Tabs.filtered_manage_options = filtered_manage_options

    showDocumentation = DTMLFile('dtml/showDocumentation', globals())

    def analyseDocumentation(self, object, type='scripter', filter=''):
        return DocFinder(object, type, filter)

    ViewDocRoles = PermissionRole(ViewDocPermission, ViewDocDefaultRoles)

    Item.showDocumentation = showDocumentation
    Item.showDocumentation__roles__ = ViewDocRoles
    Item.analyseDocumentation = analyseDocumentation
    Item.analyseDocumentation__roles__ = ViewDocRoles


    #def show_doc_header(self, *arg, **kw):
    #    print "show_doc_header", arg, kw
    show_doc_header= DTMLFile('dtml/show_doc_header', globals())
    Item.show_doc_header = show_doc_header
    Item.show_doc_header__roles__ = ViewDocRoles

    #def show_doc_footer(self, *arg, **kw):
    #    print "show_doc_footer", arg, kw
    show_doc_footer= DTMLFile('dtml/show_doc_footer', globals())
    Item.show_doc_footer = show_doc_footer
    Item.show_doc_footer__roles__ = ViewDocRoles

    #def show_doc_leaves(self, *arg, **kw):
    #    print "show_doc_leaves", arg, kw
    show_doc_leaves= DTMLFile('dtml/show_doc_leaves', globals())
    Item.show_doc_leaves = show_doc_leaves
    Item.show_doc_leaves__roles__ = ViewDocRoles

    show_doc_expand= DTMLFile('dtml/show_doc_expand', globals())
    Item.show_doc_expand = show_doc_expand
    Item.show_doc_expand__roles__ = ViewDocRoles


    from zLOG import LOG, INFO
    LOG('DocFinderTab', INFO, 'Applied patch version %s.' % __version__)

except:

    import traceback
    traceback.print_exc()

