import Globals, ExtensionClass, os
from Globals import package_home
from OFS.misc_ import p_
from OFS.DTMLMethod import DTMLMethod
from OFS.Image import Image
from OFS.Folder import Folder
from OFS.SimpleItem import SimpleItem
from Acquisition import aq_get, aq_parent, aq_base
from AccessControl import ClassSecurityInfo
from string import split, replace, capwords
from Products.EasyPresentationHelpers.EasyParameterTracker import EasyParameterTracker

class WizardBase(ExtensionClass.Base):

    security = ClassSecurityInfo()
    is_folder=1
    is_external=0
    target='manage_main'

    security.declarePublic('addObject')
    def addObject(self, ob):
        self.objects.append(ob.id)
        setattr(self, ob.id, ob)
        return getattr(self, ob.id)

    security.declarePublic('hasObjects')
    def hasObjects(self):
        return len(self.objects)

    security.declarePublic('getObject')
    def getObject(self, id):
        ob = getattr(self,id, None)
        if ob:
            return getattr(self,id).__of__(self)

    security.declarePublic('getObjects')
    def getObjects(self):
        objects=[]
        for id in self.objects:
            if hasattr(self, id):
                objects.append(getattr(self,id).__of__(self))
        return objects

    security.declarePublic('index_html')
    def index_html(self, client=None, REQUEST=None, **kw):
        """Render the page/object with a template."""
        template = getattr(self, self.templateid, None)
        if template:
            return apply(template, (self, REQUEST), kw)
        return "<h1>Can't find Template: %s</h1>" %(self.templateid,)

    security.declarePublic('is_current')
    def is_current(self, id):
        return self.id==id

    security.declarePublic('is_expanded')
    def is_expanded(self, wmenu=''):
        wmenu = split(wmenu,'/')
        geturl = split(self.getUrl(),'/')
        return wmenu[0:len(geturl)]==geturl[0:len(geturl)]

    security.declarePublic('getUrl')
    def getUrl(self,remove=0):
        url=''
        if hasattr(self,'aq_parent'):
            url=getattr(self.aq_parent,'getUrl','')
            if callable(url): url=url()
        if remove: return url
        return url + "/" +self.id

    security.declarePublic('getIcon')
    def getIcon(self):
        if hasattr(self.aq_base,'icon'):
            return self.icon

    security.declarePublic('getNavPath')
    def getNavPath(self, stl='color:#000;'):
        path = split(self.getUrl(),'/')
        while len(path) and path[0]=='': path=path[1:]
        r=''
        u='/'
        for id in path:
            u=u+id+'/'
            r=r+'/ <a href="%s" style="%s">%s</a>&nbsp;' %(u,stl,capwords(replace(id,'_',' ')))
        return r

Globals.InitializeClass(WizardBase)

class Wizard(Folder, WizardBase, EasyParameterTracker):

    meta_type='Wizard'

    is_folder=0
    is_external=0

    security = ClassSecurityInfo()

    track_fields = ()

    def __init__(self, id, title='', short_title='', description='', templateid='', target='manage_main', icon=None, lpath=None, disable_attribute=None):
        self.id=id
        self.title=title
        self.short_title=short_title
        self.description=description
        self.templateid=templateid
        self.disable_attribute=disable_attribute
        self.target=target
        self.objects=[]
        if lpath:
            path = os.path.join(package_home(globals()), lpath + '.dtml')
            f = open(path,'r')
            file = f.read()
            f.close()
            self.content=DTMLMethod(file, __name__='content')
        if icon:
            path = os.path.join(package_home(globals()), icon)
            f = open(path,'rb')
            file = f.read()
            f.close()
            self.icon=Image('icon',title,'')
            self.icon.manage_upload(file)
            self.icon__roles__=None

    security.declarePrivate('appendWizardPage')
    def appendWizardPage(self, id, title='', short_title='', description='', templateid='', lpath='', next_handler=None, prev_handler=None):
        if not hasattr(self, id):
            wp=WizardPage(id, title, short_title, description, templateid, lpath, next_handler, prev_handler)
            self.objects.append(id)
            setattr(self, id, wp)
            return
        raise KeyError

    security.declarePublic('index_html')
    def index_html(self,client=None,REQUEST=None, **kw):
        """Redirect the the first PAge of the wizard."""
        rattr=1
        for dattr in self.disable_attribute:
            attr = getattr(self,dattr,None)
            if callable(attr): attr = attr()
            if not attr:
                rattr=0
                break;
        if not rattr and len(self.objects)>0:
            base_url="/"+self.absolute_url(relative=1)+"/"
            REQUEST.RESPONSE.redirect(base_url + self.objects[0])
        else:
            template = getattr(self, self.templateid, None)
            if template:
                return apply(template, (self, REQUEST), kw)
            return "<h1>Can't find Template: %s</h1>" %(self.templateid,)

    security.declarePublic('getPrevious')
    def getPrevious(self, id):
        try:
            i= self.objects.index(id)
            if i>0: return self.objects[i-1]
        except:
            return ''

    security.declarePublic('getPreviousObj')
    def getPreviousObj(self, id, default=None):
        try:
            i= self.objects.index(id)
            if i>0:
                prev = self.objects[i-1]
                return getattr(self,prev,default)
        except:
            return default

    security.declarePublic('getNext')
    def getNext(self,id):
        try:
            i= self.objects.index(id)
            return self.objects[i+1]
        except:
            return ''

    security.declarePublic('getNextObj')
    def getNextObj(self,id,default=None):
        try:
            i= self.objects.index(id)
            next = self.objects[i+1]
            return getattr(self,next,default)
        except:
            return default

    security.declarePublic('getOrder')
    def getOrder(self, id):
        order='middle'
        plen = len(self.objects)
        i = self.objects.index(id)
        if i == plen-1: order='last'
        if i == plen-2: order='nextlast'
        if i == 0: order='first'
        return order


Globals.InitializeClass(Wizard)


class WizardPage(Folder):

    meta_type='WizardPage'

    security = ClassSecurityInfo()
    is_folder=0
    is_external=0

    def __init__(self, id, title='', short_title='', description='', templateid='', lpath='', next_handler=None, prev_handler=None, icon=None,  target='manage_main', disable_attribute=None):
        self.id=id
        self.title=title
        self.short_title=short_title
        self.description=description
        self.templateid=templateid
        self.disable_attribute=disable_attribute
        self.target=target
        self.next_handler=next_handler
        self.prev_handler=prev_handler
        if lpath:
            path = os.path.join(package_home(globals()), lpath + '.dtml')
            f = open(path,'r')
            file = f.read()
            f.close()
            self.content=DTMLMethod(file, __name__='content')
        if icon:
            path = os.path.join(package_home(globals()), icon)
            f = open(path,'rb')
            file = f.read()
            f.close()
            self.icon=Image('icon',title,'')
            self.icon.manage_upload(file)
            self.icon__roles__=None

    security.declarePublic('index_html')
    def index_html(self, client=None, REQUEST=None, **kw):
        """Render the page/object with a template."""
        template = getattr(self, self.templateid, None)
        if template:
            #return template(self, REQUEST=REQUEST)
            return apply(template, (self, REQUEST), kw)
        return "<h1>Can't find Template: %s</h1>" %(self.templateid,)

    def move_next(self, currentid, client=None, REQUEST=None):
        """
        Execute the next handler and render the next page,
        or render the current page again with the result attribute set
        to the next handlers result code.
        """
        obj = getattr(self.aq_parent,currentid,self)
        if hasattr(obj,'next_handler') and obj.next_handler:
            res = apply(obj.next_handler, (obj, client, REQUEST))
        else: res=0
        next = obj.getNextObj(currentid)
        if not res and next:
            return next.index_html(client=client,REQUEST=REQUEST)
        return obj.index_html(client=client,REQUEST=REQUEST,result=res)

    def move_prev(self, currentid, client=None,REQUEST=None):
        """
        Execute the prev handler and render the previous page,
        or render the current page again with the result attribute set
        to the prev handlers result code.
        """
        obj = getattr(self.aq_parent,currentid,self)
        if hasattr(obj,'prev_handler') and obj.prev_handler:
            res = apply(obj.prev_handler, (obj, client, REQUEST))
        else: res=0
        prev = obj.getPreviousObj(currentid)
        if not res and prev:
            return prev.index_html(client=client,REQUEST=REQUEST)
        return obj.index_html(client=client,REQUEST=REQUEST,result=res)

    security.declarePublic('getPrevious')
    def getPrevious(self):
        if self.getOrder()!='last':
            return self.aq_parent.getPrevious(self.id)
        return ''

    security.declarePublic('getNext')
    def getNext(self):
        return self.aq_parent.getNext(self.id)

    security.declarePublic('getOrder')
    def getOrder(self):
        return self.aq_parent.getOrder(self.id)

    security.declarePublic('is_current')
    def is_current(self, id):
        return self.id == id


Globals.InitializeClass(WizardPage)


class WizardFolder(Folder, WizardBase):

    security = ClassSecurityInfo()
    meta_type='WizardFolder'

    def __init__(self, id, title='', short_title='', description='', templateid='', target='manage_main', lpath=None, icon=None, disable_attribute=None):
        self.id=id
        self.title=title
        self.short_title=short_title
        self.description=description
        self.target=target
        self.disable_attribute=disable_attribute
        self.templateid=templateid
        self.objects = []
        if lpath:
            path = os.path.join(package_home(globals()), lpath + '.dtml')
            f = open(path,'r')
            file = f.read()
            f.close()
            self.content=DTMLMethod(file, __name__='content')
        if icon:
            path = os.path.join(package_home(globals()), icon)
            f = open(path,'rb')
            file = f.read()
            f.close()
            self.icon=Image('icon',title,'')
            self.icon.manage_upload(file)
            self.icon__roles__=None


Globals.InitializeClass(WizardFolder)

class WizardExternalLink(Folder):

    security = ClassSecurityInfo()
    meta_type='WizardExternalLink'
    is_folder=0
    is_external=1

    def __init__(self, id, title='', short_title='', description='', target='_blank',  url='', icon=None, disable_attribute=None):
        self.id=id
        self.title=title
        self.short_title=short_title
        self.description=description
        self.disable_attribute=disable_attribute
        self.target=target
        self.url=url
        if icon:
            path = os.path.join(package_home(globals()), icon)
            f = open(path,'rb')
            file = f.read()
            f.close()
            self.icon=Image('icon',title,'')
            self.icon.manage_upload(file)
            self.icon__roles__=None

    def getUrl(self,remove=0):
        return self.url

    security.declarePublic('is_current')
    def is_current(self, id):
        return self.id==id

    security.declarePublic('is_expanded')
    def is_expanded(self, wmenu=''):
        return 0

    security.declarePublic('getIcon')
    def getIcon(self):
        if hasattr(self.aq_base,'icon'):
            return self.icon

Globals.InitializeClass(WizardExternalLink)


class WizardRegistry(SimpleItem):

    meta_type='WizardRegistry'

    security = ClassSecurityInfo()

    object_ids = []
    disable_attributes = []

    security.declarePublic('addObject')
    def addObject(self, ob, disable_attribute=None):
        self.object_ids.append(ob.id)
        if disable_attribute:
            if type(disable_attribute) not in (type(()),type([])): disable_attribute=(disable_attribute,)
            for dattr in disable_attribute:
                self.disable_attributes.append((dattr, ob.id))
        return ob

    security.declarePublic('getObjects')
    def getObjects(self):
        disable=[]
        for attr_id, id in self.disable_attributes:
            attr = getattr(self,attr_id,None)
            if callable(attr): attr = attr()
            if attr: disable.append(id)
        return filter(lambda id, disable=disable: id not in disable, self.object_ids)

Globals.InitializeClass(WizardRegistry)

